// ArboView.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "Muzip.h"
#include "MuzipArc.h"
#include "MuzipDoc.h"
#include "ArboView.h"
#include "MuzipView.h"
#include "ApercuView.h"
#include "Interf.h"

#include <Pile.h>

#include "dlgConfirm.h"
#include "dlgExtraire.h"
#include "dlgProprietes.h"

#include "Options.h"


IMPLEMENT_DYNCREATE(CArboView, CTreeView)
BEGIN_MESSAGE_MAP(CArboView, CTreeView)
	//{{AFX_MSG_MAP(CArboView)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_NOTIFY_REFLECT   (NM_RCLICK,                 OnRClick)
	ON_NOTIFY_REFLECT   (TVN_GETDISPINFO,           OnGetDispInfo)
	ON_NOTIFY_REFLECT   (TVN_KEYDOWN,               OnKeyDown)
	ON_NOTIFY_REFLECT   (TVN_SELCHANGED,            OnSelChanged)
	ON_NOTIFY_REFLECT   (TVN_ITEMEXPANDING,         OnItemExpanding)
	ON_NOTIFY_REFLECT   (TVN_BEGINDRAG,             OnBeginDrag)
	ON_NOTIFY_REFLECT   (TVN_BEGINLABELEDIT,        OnBeginLabelEdit)
	ON_NOTIFY_REFLECT   (TVN_ENDLABELEDIT,          OnEndLabelEdit)
	ON_COMMAND          (ID_ARCHIVE_MOINS,          OnArchiveMoins)
	ON_COMMAND          (ID_ARCHIVE_EXTRAIRE,       OnArchiveExtraire)
	ON_COMMAND          (ID_ARCHIVE_RENOMMER,       OnArchiveRenommer)
	ON_COMMAND          (ID_ARCHIVE_PROPRIETES,     OnArchiveProprietes)
	ON_COMMAND          (ID_ARCHIVE_TESTER,         OnArchiveTester)
	ON_COMMAND          (ID_ARCHIVE_NOUVEAUDOSSIER, OnArchiveNouveauDossier)
	ON_UPDATE_COMMAND_UI(ID_ARCHIVE_MOINS,          OnUpdateFautSel)
	ON_UPDATE_COMMAND_UI(ID_ARCHIVE_RENOMMER,       OnUpdateArchiveRenommer)
	ON_UPDATE_COMMAND_UI(ID_ARCHIVE_EXTRAIRE,       OnUpdateFautSel)
	ON_UPDATE_COMMAND_UI(ID_ARCHIVE_NOUVEAUDOSSIER, OnUpdateArchiveNouveauDossier)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


int CArboView::s_iRepClosed;
int CArboView::s_iRepOpen;


// Cration
int CArboView::OnCreate(CREATESTRUCT* pcs)
{
	m_ptrv = &GetTreeCtrl();
	m_fRenommage = false;
	
	pcs->style |= TVS_HASBUTTONS | TVS_HASLINES | TVS_LINESATROOT |
		TVS_EDITLABELS | TVS_SHOWSELALWAYS;
	if (CTreeView::OnCreate(pcs)==-1)  return -1;
	
	// Affecter la liste d'images systme  l'arborescence
	TreeView_SetImageList(*m_ptrv, oApp.s_himlSystem, TVSIL_NORMAL);
	
	// Enregistrer la cible de glisser/dplacer
	m_mdt.m_pDoc = GetDocument();
	m_mdt.Register(m_ptrv);
	
	// Passer la vue au document
	GetDocument()->m_ptrv = this;
	
	return 0;
}

// Destruction
void CArboView::OnDestroy()
{
	TreeView_SetImageList(*m_ptrv, NULL, TVSIL_NORMAL);
	
	CTreeView::OnDestroy();
	GetDocument()->m_ptrv = NULL;
}


// Mise  jour
void CArboView::OnUpdate(CView*, LPARAM, CObject*)
{
	CMuzipDoc *const pDoc = GetDocument();
	CMuzipArc *const pArc = pDoc->m_pArc;
	
	CDossier *const pCur = pDoc->m_pCur;
	
	// Vider l'arborescence et commencer  la remplir
	m_ptrv->DeleteAllItems();
	pArc->trvLister();
	
	// Ouvrir les items l'tant prcdemment,
	// grce au flag CDossier::m_fOuvert
	CPileCh<HTREEITEM> pilItem;
	pilItem.Push(m_ptrv->GetRootItem());
	while (pilItem.CanPop()) {
		HTREEITEM hti = pilItem.Pop();
		while (hti) {
			const CDossier *const pDossier = (CDossier*)m_ptrv->GetItemData(hti);
			if (pDossier->m_fOuvert) {
				m_ptrv->Expand(hti, TVE_EXPAND);
				pilItem.Push(m_ptrv->GetChildItem(hti));
			}
			if (pDossier==pCur)  m_ptrv->SelectItem(hti);
			hti = m_ptrv->GetNextSiblingItem(hti);
		}
	}
}


// Demande d'informations
void CArboView::OnGetDispInfo(NMTVDISPINFO* pNMDI, LRESULT*)
{
	if (pNMDI->item.mask & TVIF_TEXT)
		lstrcpy(pNMDI->item.pszText, ((CDossier*)pNMDI->item.lParam)->m_pszNom);
	if (pNMDI->item.mask & TVIF_CHILDREN)
		pNMDI->item.cChildren = ToBool(((CDossier*)pNMDI->item.lParam)->m_lstDossier.GetNb());
}

// Touche enfonce
void CArboView::OnKeyDown(NMTVKEYDOWN* pNMKD, LRESULT*)
{
	if (pNMKD->wVKey==VK_DELETE)  OnArchiveMoins();
}

// Slection modifie
void CArboView::OnSelChanged(NMTREEVIEW* pNMTRV, LRESULT*)
{
	CMuzipDoc *const pDoc = GetDocument();
	
	pDoc->m_pCur = (CDossier*)pNMTRV->itemNew.lParam;
	pDoc->m_plst->OnUpdate(NULL,NULL,NULL);
	pDoc->m_pArc->m_pi->OnDossierChange();
}

// Expansion d'item
#pragma warning (disable: 4706)
void CArboView::OnItemExpanding(NMTREEVIEW* pNMTRV, LRESULT* pResult)
{
	*pResult = FALSE;   // Ne pas annuler
	
	if (((CDossier*)pNMTRV->itemNew.lParam)->m_fOuvert = (pNMTRV->action==TVE_EXPAND)) {
		// Lister les sous-dossiers si ncessaire
		if (!m_ptrv->GetChildItem(pNMTRV->itemNew.hItem))
			((CDossier*)pNMTRV->itemNew.lParam)->trvLister(pNMTRV->itemNew.hItem);
	}
}
#pragma warning (default: 4706)


// Renommage - dbut
void CArboView::OnBeginLabelEdit(NMTVDISPINFO* pNMDI, LRESULT* pResult)
{
	m_fRenommage = (CDossier*)pNMDI->item.lParam!=GetDocument()->m_pArc;
	*pResult = !m_fRenommage;
}

// Renommage - fin
void CArboView::OnEndLabelEdit(NMTVDISPINFO* pNMDI, LRESULT* pResult)
{
	m_fRenommage = false;
	
	CDossier *pDossier = (CDossier*)pNMDI->item.lParam;
	
	// Si on a annul le renommage, ne rien faire
	const LPTSTR pszNom = pNMDI->item.pszText;
	*pResult = (pszNom) ? pDossier->Renommer(pszNom) : FALSE;
	
	if (!*pResult) {
		if (!*pDossier->m_pszNom)
			// Cration de dossier annule: supprimer le dossier
			pDossier->Supprimer();
	}else{
		// Ajout OK: il faut retrier le contenu du dossier parent
		// Il faut POSTER le message pour qu'il soit excut
		// APRS l'ajout de l'item
		// C'est pour cela que la mthode SortChildren() n'est pas utilise
		pDossier = pDossier->m_pParent;
		m_ptrv->PostMessage(TVM_SORTCHILDREN, 0,
			(pDossier) ? (LPARAM)pDossier->m_hti : NULL);
	}
}


// Glisser-dplacer
void CArboView::OnBeginDrag(NMTREEVIEW* pNMTRV, LRESULT*)
{
	// Construire la source de donnes OLE
	CMuzipDataSource mds;
	mds.m_pDossier = (CDossier*)pNMTRV->itemNew.lParam;
	mds.DelayRenderData(CF_HDROP);
	mds.Init(GetDocument());
	
	// Effectuer le glisser-dplacer
	if (mds.DoDragDrop()==DROPEFFECT_MOVE &&
	    CMuzipDataSource::s_pmds) {
		const HWND hwndFocus = ::GetFocus();
		
		// Si CMuzipDataSource::s_pmds==NULL,
		// le dplacement a dj t effectu
		m_ptrv->SelectItem(pNMTRV->itemNew.hItem);
		oApp.s_nConfirm--;
		OnArchiveMoins();
		oApp.s_nConfirm++;
		
		::SetFocus(hwndFocus);
	}
}


// Clic droit
void CArboView::OnRClick(NMHDR*, LRESULT*)
{
	HTREEITEM hti;
	
	// Rcuprer l'item cliqu
	CPoint pt;
	GetCursorPos(&pt);
	CPoint ptTemp = pt;
	ScreenToClient(&ptTemp);
	hti = m_ptrv->HitTest(ptTemp);
	if (!hti)  return;
	m_ptrv->SelectItem(hti);
	
	// Crer le menu contextuel du dossier
	CMenu mnu;
	mnu.LoadMenu(IDM_CONTEXT);
	CMenu *const pMenu = mnu.GetSubMenu(1);
	pMenu->TrackPopupMenu(
		TPM_LEFTALIGN | TPM_RIGHTBUTTON,
		pt.x,pt.y, this);
}


// Supprimer
void CArboView::OnArchiveMoins()
{
	if (!CDlgConfirm::GetReponse(e_opt.m_fConfirmArchiveMoins, DEM_ARCHIVEMOINS))
		return;
	
	CMuzipDoc *const pDoc = GetDocument();
	
	// Vider la liste de droite
	pDoc->m_plst->m_plst->DeleteAllItems();
	
	// Supprimer le dossier, et son parent s'il est vide
	CDossier *const pDossier = (CDossier*)m_ptrv->GetItemData(m_ptrv->GetSelectedItem());
	if (e_opt.m_fGeneralSupprDossierVide)
		pDossier->SupprimerEtParentSiVide();
	else
		pDossier->Supprimer();
	
	pDoc->m_pArc->FautReconstruire(ToBool(e_opt.m_fCompRmMove));
	pDoc->m_plst->OnUpdate(NULL,NULL,NULL);
}

// Extraire
void CArboView::OnArchiveExtraire()
{
	CDlgExtraire dlgExtraire(GetDocument()->m_pArc);
	if (dlgExtraire.DoModal()==IDCANCEL)  return;
	
	// Dcompresser le dossier
	((CDossier*)m_ptrv->GetItemData(m_ptrv->GetSelectedItem()))->
		DossierExtraire(dlgExtraire.m_sDest, &oApp.s_ext);
}

// Renommer
void CArboView::OnArchiveRenommer()
{
	m_ptrv->EditLabel(m_ptrv->GetSelectedItem());
}
void CArboView::OnUpdateArchiveRenommer(CCmdUI* pCmdUI)
{
	// Interdire le renommage du dossier racine
	const HTREEITEM hti = m_ptrv->GetSelectedItem();
	pCmdUI->Enable(hti &&
		(CDossier*)m_ptrv->GetItemData(hti)!=(CDossier*)GetDocument());
}

// Proprits
void CArboView::OnArchiveProprietes()
{
	CDlgProprietes((CDossier*)m_ptrv->GetItemData(m_ptrv->GetSelectedItem()), this).DoModal();
}

// Tester
void CArboView::OnArchiveTester()
{
	const HTREEITEM hti = m_ptrv->GetSelectedItem();
	
	CLstFichier lstFichier;
	((hti)
			? ((CDossier*)m_ptrv->GetItemData(hti))
			: GetDocument()->m_pArc)
		->ListerFichier(lstFichier);
	GetDocument()->m_pArc->ArchiveTester(lstFichier);
}


// Nouveau dossier
void CArboView::OnArchiveNouveauDossier()
{
	ArchiveNouveauDossier((CDossier*)m_ptrv->GetItemData(m_ptrv->GetSelectedItem()));
}
void CArboView::OnUpdateArchiveNouveauDossier(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(!m_fRenommage && m_ptrv->GetSelectedItem() &&
		(GetDocument()->m_pArc->GetCap() & capArbo));
}
void CArboView::ArchiveNouveauDossier(CDossier* pDossier)
{
	CDossier *const pnewDossier = new CDossier(pDossier, _T(""));
	pDossier->m_lstDossier.AddFin(pnewDossier);
	pnewDossier->trvLister();
	m_ptrv->EnsureVisible(pnewDossier->m_hti);
	m_ptrv->SelectItem   (pnewDossier->m_hti);
	OnArchiveRenommer();
}


// MAJ: Un dossier tre slectionn
void CArboView::OnUpdateFautSel(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(!m_fRenommage && m_ptrv->GetSelectedItem());
}


// Glisser/dplacer - Dplacement
DROPEFFECT CArboView::OnDragOver(COleDataObject* pDataObject, DWORD dwKeyState, CPoint pt)
{
	HTREEITEM hti = m_ptrv->HitTest(pt);
	if (!hti)  hti = GetDocument()->m_pArc->m_hti;
	m_ptrv->SelectDropTarget(hti);
	
	// Conserver le dossier cible
	GetDocument()->m_pDragD = (CDossier*)m_ptrv->GetItemData(hti);
	
	return m_mdt.OnDragOver(pDataObject, dwKeyState, pt);
}

// Glisser/dplacer - Lcher
BOOL CArboView::OnDrop(COleDataObject* pDataObject, DROPEFFECT dropEffect, CPoint pt)
{
	// Changer de dossier courant
	HTREEITEM hti = m_ptrv->HitTest(pt);
	if (!hti)  hti = GetDocument()->m_pArc->m_hti;
	m_ptrv->SelectItem(hti);
	
	const BOOL fRet = m_mdt.OnDrop(pDataObject, dropEffect, pt);
	
	m_ptrv->SelectDropTarget(NULL);
	GetDocument()->m_pDragD = NULL;
	
	GetParentFrame()->ActivateFrame(SW_SHOW);
	return fRet;
}

// Glisser/dplacer - Quitter
void CArboView::OnDragLeave()
{
	m_ptrv->SelectDropTarget(NULL);
	GetDocument()->m_pDragD = NULL;
	CTreeView::OnDragLeave();
}
