// Dossier.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif
#include "Muzip.h"
#include "Dossier.h"

#include "MuzipArc.h"
#include "Interf.h"
#include "ArboView.h"

#include "Decomp.h"

#include "IOFichier.h"
#include "Options.h"

#include "Zip/Zip.h"
#include "Zip/ZipFichier.h"

#include "dlgLister.h"
#include "dlgExtraire.h"

#ifdef UNICODE
#include <GUnicode.h>
#endif


RECHOPTIONS CDossier::s_rch;
DWORD       CDossier::s_mskAttrO;
DWORD       CDossier::s_mskAttrN;
CDlgLister* CDossier::s_pdlgLister;


// Constructeur
CDossier::CDossier(CDossier* pParent, LPCTSTR pszNom)
	: m_lstDossier(true), m_lstFichier(true)
{
	m_pParent = pParent;
	m_pszNom  = newDup(pszNom);
	m_pszPath = NULL;
	m_pArc    = (m_pParent) ? m_pParent->m_pArc : (CMuzipArc*)this;
	m_fOuvert = false;
	MAJPath();
}


// Compter les fichiers du dossier et de ses sous-dossiers
UINT CDossier::GetNbFichier() const
{
	UINT nbFichier = 0;
	POS pos;
	
	// Compter les fichiers non-ignors
	pos = m_lstFichier.GetPosDeb();
	while (pos)
		nbFichier += !m_lstFichier.GetSuiv(pos)->m_fIgnorer;
	
	// Poursuivre dans les sous-dossiers
	pos = m_lstDossier.GetPosDeb();
	while (pos)
		nbFichier += m_lstDossier.GetSuiv(pos)->GetNbFichier();
	
	return nbFichier;
}

// Compter les sous-dossiers du dossier et ceux de ses sous-dossiers
UINT CDossier::GetNbDossier() const
{
	UINT nbDossier = m_lstDossier.GetNb();
	POS pos = m_lstDossier.GetPosDeb();
	while (pos)
		nbDossier += m_lstDossier.GetSuiv(pos)->GetNbDossier();
	return nbDossier;
}

// Lister les fichiers
void CDossier::ListerFichier(CLstFichier& lstFichier) const
{
	lstFichier.AddFin(m_lstFichier);
	POS pos = m_lstDossier.GetPosDeb();
	while (pos)
		m_lstDossier.GetSuiv(pos)->ListerFichier(lstFichier);
}

// Mettre  jour le chemin d'accs du dossier et de ses sous-dossiers
void CDossier::MAJPath()
{
	// Dtruire l'ancien chemin d'accs
	delete [] m_pszPath;
	
	if (m_pParent) {
		// Composer le nouveau
		size_t lenParent = lstrlen(m_pParent->m_pszPath);
		if (lenParent) {
			// Le parent n'est pas la racine
			m_pszPath = new TCHAR[lenParent + lstrlen(m_pszNom)+2];
			lstrcpy(m_pszPath, m_pParent->m_pszPath);
			m_pszPath[lenParent++] = '\\';
			lstrcpy(m_pszPath+lenParent, m_pszNom);
		}else
			// Le parent est la racine
			m_pszPath = newDup(m_pszNom);
	}else
		// Dossier racine: pas de chemin d'accs
		*(m_pszPath = new TCHAR[1]) = 0;
	
	// Poursuivre dans les sous-dossiers
	POS pos = m_lstDossier.GetPosDeb();
	while (pos)
		m_lstDossier.GetSuiv(pos)->MAJPath();
}

// Mettre tous les fichiers du dossier  "non-trouv"
void CDossier::SetFichierNonTrouve()
{
	POS pos;
	
	// Traiter les fichiers
	pos = m_lstFichier.GetPosDeb();
	while (pos)
		m_lstFichier.GetSuiv(pos)->m_fTrouve = false;
	
	// Traiter les sous-dossiers
	pos = m_lstDossier.GetPosDeb();
	while (pos)
		m_lstDossier.GetSuiv(pos)->SupprimerFichierNonTrouve();
}


// Supprimer le dossier
void CDossier::Supprimer()
{
	m_pArc->m_pi->OnDossierSupprimer(this);
	
	// Supprimer fichiers et sous-dossiers
	m_lstFichier.DeleteAll();
	m_lstDossier.DeleteAll();
	
	if (m_pParent)
		// Enlever le dossier de la liste de son parent
		// et le dtruire en mme temps
		m_pParent->m_lstDossier.DeleteAt(
			m_pParent->m_lstDossier.Find(this));
}

// Supprimer tous les sous-dossiers vides
// puis le dossier lui-mme s'il est vide
// Retourner TRUE si le dossier a pu tre vid entirement et donc supprim
bool CDossier::SupprimerSiVide()
{
	// Traiter les sous-dossiers
	POS pos = m_lstDossier.GetPosDeb();
	while (pos)
		m_lstDossier.GetSuiv(pos)->SupprimerSiVide();
	
	// Il ne doit rester ni fichiers ni dossier
	VERIF(m_lstFichier.GetNb()+m_lstDossier.GetNb()==0);
	
	// Supprimer le dossier, et indiquer qu'il est vide en retournant TRUE
	Supprimer();
	return true;
}

// Supprimer le dossier, et ses parents s'ils sont vides
void CDossier::SupprimerEtParentSiVide()
{
	// Remonter l'arborescence tant que le parent
	// n'a aucun fichier mais un seul sous-dossier (le fils)
	CDossier *pDossier = this;
	CDossier *pParent;
	for (;;) {
		pParent = pDossier->m_pParent;
		if (!pParent ||
		     pParent->m_lstFichier.GetNb() ||
		     pParent->m_lstDossier.GetNb()>1)
			break;
		pDossier = pParent;
	}
	
	// Supprimer le dernier dossier trouv
	pDossier->Supprimer();
}

// Supprimer les fichiers non-trouvs du dossier
// et de ses sous-dossiers
void CDossier::SupprimerFichierNonTrouve()
{
	POS pos;
	
	// Traiter les fichiers
	pos = m_lstFichier.GetPosDeb();
	while (pos) {
		const POS posSauve = pos;
		if (!m_lstFichier.GetSuiv(pos)->m_fTrouve)
			m_lstFichier.DeleteAt(posSauve);
	}
	
	// Traiter les sous-dossiers
	pos = m_lstDossier.GetPosDeb();
	while (pos)
		m_lstDossier.GetSuiv(pos)->SupprimerFichierNonTrouve();
}

// Renommer le dossier
bool CDossier::Renommer(LPTSTR pszNom)
{
	// S'assurer que le nom est correct
	VERIF(NomCorriger(pszNom));
	
	// Vrifier qu'il n'y a pas de doublons avec les dossiers
	// de mme rang dans l'arborescence
	CLstDossier &lstDossier = m_pParent->m_lstDossier;
	POS pos = lstDossier.GetPosDeb();
	while (pos) {
		CDossier *const pDossier = lstDossier.GetSuiv(pos);
		if (!lstrcmpi(pszNom, pDossier->m_pszNom)) {
			if (pDossier==this)  return true;
			// Doublon
			AfxFormatString2(e_sMsg, ERR_RENOMDOUBLOND, pszNom,pDossier->m_pszNom);
			AfxMessageBox(e_sMsg);
			return false;
		}
	}
	
	// Pas de doublons: on peut renommer
	delete [] m_pszNom;
	m_pszNom = newDup(pszNom);
	// Mettre  jour le chemin d'accs du dossier
	// et de ses sous-dossiers
	MAJPath();
	
	// Reconstruire l'archive
	bool fOK = false;
	if (!m_pArc->m_pi->IsModified() && (m_pArc->GetCap() & capRapideRenommer)) {
		// Dplacement rapide permis: l'effectuer pour chaque fichier
		if (m_pArc->WriteOpen()) {
			fOK = RenommerSuite();
			m_pArc->WriteClose();
		}
	}
	if (!fOK)  m_pArc->FautReconstruire(ToBool(e_opt.m_fCompRmMove));
	return true;
}

// Avertir les fichiers du renommage
bool CDossier::RenommerSuite()
{
	POS pos;
	
	// Traiter les fichiers
	pos = m_lstFichier.GetPosDeb();
	while (pos)
		VERIF(m_lstFichier.GetSuiv(pos)->OnRenommer());
	
	// Traiter les sous-dossiers
	pos = m_lstDossier.GetPosDeb();
	while (pos)
		VERIF(m_lstDossier.GetSuiv(pos)->RenommerSuite());
	
	return true;
}


// Effectuer la recherche oApp.s_rch
void CDossier::Rechercher()
{
	s_rch = oApp.s_rch;
	
	s_mskAttrO =
	s_mskAttrN = 0;
	if      (s_rch.AT_nR==0)  s_mskAttrN |= FILE_ATTRIBUTE_READONLY;
	else if (s_rch.AT_nR==1)  s_mskAttrO |= FILE_ATTRIBUTE_READONLY;
	if      (s_rch.AT_nH==0)  s_mskAttrN |= FILE_ATTRIBUTE_HIDDEN;
	else if (s_rch.AT_nH==1)  s_mskAttrO |= FILE_ATTRIBUTE_HIDDEN;
	if      (s_rch.AT_nS==0)  s_mskAttrN |= FILE_ATTRIBUTE_SYSTEM;
	else if (s_rch.AT_nS==1)  s_mskAttrO |= FILE_ATTRIBUTE_SYSTEM;
	if      (s_rch.AT_nA==0)  s_mskAttrN |= FILE_ATTRIBUTE_ARCHIVE;
	else if (s_rch.AT_nA==1)  s_mskAttrO |= FILE_ATTRIBUTE_ARCHIVE;
	
	if (!s_rch.DO_sContenu.IsEmpty()) {
		// Prparer la recherche
		
		DECOMP::e_lenRech = lstrlen(s_rch.DO_sContenu);
#ifdef UNICODE
		DECOMP::e_pszRech = new char[DECOMP::e_lenRech+1];
		strtoA(DECOMP::e_pszRech, s_rch.DO_sContenu, DECOMP::e_lenRech+1);
#else
		DECOMP::e_pszRech = (LPTSTR)(LPCTSTR)s_rch.DO_sContenu;
#endif
		DECOMP::e_pfr = &m_pArc->m_fr;
	}
	
	RechercherSuite();
	
#ifdef UNICODE
	if (!s_rch.DO_sContenu.IsEmpty())
		// Nettoyer la recherche
		delete [] DECOMP::e_pszRech;
#endif
}

// Effectuer la recherche (fonction interne et rcursive)
void CDossier::RechercherSuite()
{
	POS pos;
	
	pos = m_lstFichier.GetPosDeb();
	while (pos) {
		bool fTrouve = false;
		CFichier *const pFichier = m_lstFichier.GetSuiv(pos);
		
		// Spcification
		if (!PathMatchSpec(pFichier->m_pszNom, s_rch.DO_sFiltre))  goto PasTrouve;
		
		// Attributs
		if (s_mskAttrO && !(pFichier->m_dwAttr & s_mskAttrO))  goto PasTrouve;
		if (s_mskAttrN &&  (pFichier->m_dwAttr & s_mskAttrN))  goto PasTrouve;
		
		// Taille
		if (s_rch.TA_nActive) {
			const DWORD lenAjuste = pFichier->m_dwTailleN>>(s_rch.TA_nUnite*10);
			if       (s_rch.TA_nOp==0)   {  // ==
				if (s_rch.TA_uTaille!=lenAjuste)  goto PasTrouve;
			}else if (s_rch.TA_nOp==1)   {  // >=
				if (s_rch.TA_uTaille>=lenAjuste)  goto PasTrouve;
			}else /* (s_rch.TA_nOp==2)*/ {  // <=
				if (s_rch.TA_uTaille<=lenAjuste)  goto PasTrouve;
			}
		}
		
		// Date
		if (s_rch.DA_rdM.fActive &&
		   !s_rch.DA_rdM.Correspond(pFichier->m_dtM))   goto PasTrouve;
		
		// Contenu (en dernier car a prend du temps)
		if (!s_rch.DO_sContenu.IsEmpty()) {
			DECOMP::e_dm      = dmRechercher;
			DECOMP::e_fTrouve = false;
			DECOMP::e_lenMem  = 0;
			pFichier->Extraire(NULL);
			DECOMP::e_dm = dmNormal;
			fTrouve = DECOMP::e_fTrouve;
		}else
			fTrouve = true;
		
PasTrouve:
		pFichier->m_fRech = fTrouve;
	}
	
	// Poursuivre la recherche dans les sous-dossiers
	pos = m_lstDossier.GetPosDeb();
	while (pos)
		m_lstDossier.GetSuiv(pos)->RechercherSuite();
}

// Poursuivre la recherche
void CDossier::RechercherSuivant(int nItem) const
{
	m_pArc->m_pi->RechercherSuivant(this, nItem);
}

// Lister le contenu du dossier
bool CDossier::Lister(int nProf) const
{
	POS pos;
	
	if (this!=m_pArc) {
		// Afficher le nom du dossier
		s_pdlgLister->AddDeb(nProf);
		s_pdlgLister->Lister(this);
		s_pdlgLister->AddFin();
		nProf++;
	}
	
	pos = m_lstFichier.GetPosDeb();
	while (pos) {
		s_pdlgLister->AddDeb(nProf);
		VERIF(s_pdlgLister->Lister(m_lstFichier.GetSuiv(pos)));
		s_pdlgLister->AddFin();
	}
	
	pos = m_lstDossier.GetPosDeb();
	while (pos) {
		VERIF(m_lstDossier.GetSuiv(pos)->Lister(nProf));
	}
	
	return true;
}

// Lister le dossier dans l'arborescence
void CDossier::trvLister() const
{
	TVINSERTSTRUCT tvis;
	tvis.hParent             = (m_pParent) ? m_pParent->m_hti : NULL;
	tvis.hInsertAfter        = TVI_SORT;
	tvis.item.mask           = TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE | TVIF_PARAM | TVIF_CHILDREN;
	tvis.item.iImage         = CArboView::s_iRepClosed;
	tvis.item.iSelectedImage = CArboView::s_iRepOpen;
	tvis.item.state          = 0;
	tvis.item.stateMask      = 0;
	tvis.item.cChildren      = I_CHILDRENCALLBACK;
	tvis.item.pszText        = m_pszNom;
	tvis.item.lParam         = (LPARAM)this;
	trvLister(m_pArc->m_pi->InsertItem(&tvis));
}

// Lister le contenu du dossier dans l'arborescence
// d'aprs son handle d'item
void CDossier::trvLister(HTREEITEM hti) const
{
	m_hti = hti;
	if (!m_fOuvert)  return;
	
	// Lister les sous-dossiers
	POS pos = m_lstDossier.GetPosDeb();
	while (pos)
		m_lstDossier.GetSuiv(pos)->trvLister();
}


// Copier l'entre d'un fichier  un autre
bool CDossier::FichierCopier(CFileRead& fr, CFileWrite& fw)
{
	POS pos = m_lstFichier.GetPosDeb();
	while (pos)
		VERIF(m_lstFichier.GetSuiv(pos)->FichierCopier(fr, fw));
	
	pos = m_lstDossier.GetPosDeb();
	while (pos)
		VERIF(m_lstDossier.GetSuiv(pos)->FichierCopier(fr, fw));
	
	return true;
}

// crire le central
bool CDossier::ZipPutCentral(CFileWrite& fw)
{
	POS pos = m_lstFichier.GetPosDeb();
	while (pos) {
		CZipFichier *const pzf = (CZipFichier*)m_lstFichier.GetSuiv(pos);
		if (pzf->m_fIgnorer)  continue;
		oApp.SetProgress(STA_TERMINER, pzf->m_pszNom,
			CFichier::s_nFichier++, CFichier::s_nbFichier);
		
		VERIF(oApp.UpdateProgressStatus()!=prsAnnuler);
		
		pzf->m_offCentral = e_lenEcrit;
		VERIF(pzf->PutCentral(fw));
		e_lenEcrit += sizeof(HEADERCENSIG)+pzf->m_lenNom+pzf->m_lenExtraC+pzf->m_lenCom;
		
		oApp.SetStatusAvance(CFichier::s_nFichier);
	}
	
	pos = m_lstDossier.GetPosDeb();
	while (pos)
		VERIF(m_lstDossier.GetSuiv(pos)->ZipPutCentral(fw));
	
	return true;
}


// Commencer une dcompression de dossier entier
bool CDossier::DossierExtraire(LPCTSTR pszRep, const EXTRACTION* pext)
{
	DECOMP::e_pfr = &m_pArc->m_fr;
	
	if (!pext)  pext = &oApp.s_ext;
	if (pext->m_fArbo) {
		// Respecter l'arborescence
		
		// Rcuprer les fichiers
		CLstFichierExt lstFichierExt(true);
		CLstStr1 lstRep(true);
		DossierExtraireSuite(lstFichierExt, lstRep, pszRep);
		
		// Les extraire
		return m_pArc->ArchiveExtraire(lstFichierExt, pext);
	}
	
	VERIF(DossierCreer(pszRep));
	
	// Extraire tous les fichiers au mme endroit
	CLstFichier lstFichier;
	ListerFichier(lstFichier);
	return m_pArc->ArchiveExtraire(lstFichier, pszRep, pext);
}

// Dcompresser le dossier entier
void CDossier::DossierExtraireSuite(CLstFichierExt& lstFichierExt, CLstStr1& lstRep, LPCTSTR pszRep)
{
	if (!DossierCreer(pszRep))  return;
	
	// Allouer une nouvelle chane et y placer
	// le dossier d'extraction, avec un anti-slash final
	LPTSTR pszDest = new TCHAR[MAX_PATH];
	TCHAR *pcFin = pszDest;
	{
		const TCHAR *pcIn = pszRep;
		while (*pcIn)  *pcFin++ = *pcIn++;
	}
	if (pcFin>pszDest && pcFin[-1]!='\\')  *pcFin++ = '\\';
	lstRep.AddDeb(pszDest);
	
	
	POS pos;
	
	// Dcompresser les fichiers
	pos = m_lstFichier.GetPosDeb();
	while (pos) {
		FICHIEREXT *const pFichierExt = new FICHIEREXT;
		pFichierExt->pFichier = m_lstFichier.GetSuiv(pos);
		pFichierExt->pszDest  = pszDest;
		pFichierExt->pcFin    = pcFin;
		lstFichierExt.AddDeb(pFichierExt);
	}
	
	// Dcompresser les sous-dossiers
	pos = m_lstDossier.GetPosDeb();
	while (pos) {
		CDossier *const pDossier = m_lstDossier.GetSuiv(pos);
		lstrcpy(pcFin, pDossier->m_pszNom);
		pDossier->DossierExtraireSuite(lstFichierExt, lstRep, pszDest);
	}
}
