// GZip/GZipArc.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "../Muzip.h"
#include "../Muzip2.h"
#include "GZipArc.h"
#include "GZipFichier.h"
#include "GZip.h"
#include "../Interf.h"

#include "../FichierPlus.h"

#include "../Z/Fen.h"
#include "../Decomp.h"

#include "../Options.h"
#include "../Exceptions.h"

#include "../dlgConfirm.h"
#include "../dlgNomFichier.h"

#include <GUnicode.h>


// Capacits
int CGZipArc::GetCap() const
{
	return capNiveau | capRapideRenommer | capRapideModifProp |
		capdtM | capMaxUnFichier;
}


// Ouverture
bool CGZipArc::OpenDocument()
{
	// Lire l'en-tte
	GZIPHEADER gzh;
	VERIF(m_fr.Read(&gzh, sizeof(gzh)));
	
	// Vrifier l'ID, la mthode et les flags
	if (gzh.wID     !=gzipID    ||
	    gzh.nMethode!=zmDeflate ||
	  ((gzh.nFlags  & gzfReserve) && !e_opt.m_nReparer)) {
		CMuzipException(ERR_FORMATGZIP).ReportError();
		return false;
	}
	
	BYTE b;
	DWORD off = sizeof(gzh);   // Dbut du premier fichier
	
	if (gzh.nFlags & gzfExtra) {
		// Sauter le champ extra
		
		// Lire la longueur du champ extra
		WORD len;
		VERIF(m_fr.Read(len));
		off += len;
		
		// Lire le champ extra lui-mme
		VERIF(m_fr.Seek((long)len, SEEK_CUR));
	}
	
	char  pszNom[_MAX_FNAME] = "";
	char  pszCom[1024] = "";
	int   lenNom = 0;
	int   lenCom = 0;
	char *pc;
	
	if (gzh.nFlags & gzfNom) {
		// Lire le nom original
		pc = pszNom;
		do{
			VERIF(m_fr.Read(b));
			*pc++ = (char)b;
			lenNom++;
		}while (b);
	}
	if (gzh.nFlags & gzfComment) {
		// Lire les commentaires
		pc = pszCom;
		do{
			VERIF(m_fr.Read(b));
			*pc++ = b;
			lenCom++;
		}while (b);
	}
	if (gzh.nFlags & gzfCRC) {
		// Lire le CRC de l'en-tte
		VERIF(m_fr.Read(m_nCRCEnTete));
	}
	
	// Lire l'en-tte final
	DWORD adw[2];
	VERIF(m_fr.Seek(-(long)sizeof(adw), FILE_END));
	VERIF(m_fr.Read(adw, sizeof(adw)));
	m_nCRC = adw[1];
	
	off += lenNom + lenCom;
	
	// Garder les commentaires
	if (!lenCom)
		m_pszCom = NULL;
	else{
		m_pszCom = new TCHAR[lenCom];
		strfromA(m_pszCom, pszCom, lenCom);
	}
	
	// Rcuprer le nom du fichier
	TCHAR pszNomTemp[_MAX_FNAME];
	if (!lenNom) {
		// Demander le nom du fichier
		
		// Rechercher une ventuelle double-extension
		// Les extensions .tgz et .taz correspondent  .gz.tar
		lstrcpy(pszNomTemp, PathFindFileName(m_fr.GetPath()));
		TCHAR *pcExt = PathFindExtension(m_fr.GetPath());
		if (!lstrcmpi(pcExt, _T(".tgz")) ||
		    !lstrcmpi(pcExt, _T(".taz")))
			PathRenameExtension(pszNomTemp, _T(".tar"));
		else{
			PathRemoveExtension(pszNomTemp);
			pcExt = PathFindExtension(pszNomTemp);
			if (!*pcExt) {
				// Aucune extension spcifie: il faut renommer
				// Afficher la bote de dialogue de renommage
				lstrcpy(pcExt, _T("."));
				if (!oApp.s_fPasNommer)  CDlgNomFichier(pszNomTemp).DoModal();
				oApp.s_fPasNommer = false;
			}
		}
	}else
		strfromA(pszNomTemp, pszNom, _MAX_FNAME);
	
	// Crer l'unique fichier de l'archive
	CGZipFichier *const pgzf = new CGZipFichier;
	pgzf->m_pArc      = this;
	pgzf->m_pDossier  = this;
	pgzf->m_pszNom    = newDup(pszNomTemp);
	pgzf->m_dwTailleC = GetFileSize(m_fr, NULL) - off-sizeof(adw);
	pgzf->m_dwTailleN = adw[1];
	pgzf->m_dwAttr    = 0;
	pgzf->SetUnixDateM(gzh.dtM);
	pgzf->MAJ();
	pgzf->m_off       = off;
	m_lstFichier.AddDeb(pgzf);
	
	// Si ce fichier est de type Tar, demander la dcompression
	// immdiate dans un dossier temporaire
	if (!lstrcmpi(PathFindExtension(pszNomTemp), _T(".tar")) &&
	     CDlgConfirm::GetReponse(e_opt.m_fConfirmDecompTar, DEM_DECOMPTAR)) {
		TCHAR pszDossierTemp[MAX_PATH];
		e_opt.GetDossierTemp(pszDossierTemp);
		LPTSTR pszDest;
		if (ReadOpen()) {
			DECOMP::e_pfr = &m_fr;
			const bool fOK = pgzf->ArchiveExtraireSuite(pszDossierTemp, &pszDest);
			oApp.s_fMsg = true;
			if (fOK) {
				ReadClose();
				oApp.OpenDocumentFile(pszDest);
				m_pi->SetFautFermer();
				new CSurvDecomp(NULL, pszDest, pgzf, this);
				delete [] pszDest;
				return true;
			}
		}
	}
	
	// Afficher les commentaires s'il y en a
	// et si l'option est coche
	if (e_opt.m_fGeneralCommentaire && m_pszCom)
		oApp.m_pMainWnd->PostMessage(WM_COMMAND, ID_ARCHIVE_COMMENT, 0);
	
	Rechercher();
	return true;
}


// Ajouter un fichier  la liste
CFichier* CGZipArc::NewFichier(LPCTSTR pszFichier, const WIN32_FIND_DATA& wfd)
{
	CGZipFichier *const pgzf = new CGZipFichier;
	pgzf->NewFichier(pszFichier, wfd);
	pgzf->m_pArc     = this;
	pgzf->m_pDossier = this;
	return pgzf;
}


// Ajouter des fichiers  l'archive
bool CGZipArc::ArchivePlusUn(CFichierPlus* pFichierPlus, CFileRead& fr, CFileWrite& fw)
{
	CGZipFichier *const pgzf = (CGZipFichier*)pFichierPlus->m_pFichier;
	
	// crire l'en-tte
	VERIF(pgzf->PutHeader(fw));
	
	// Compresser et crire le fichier
	oApp.SetProgress(STA_ARCHIVEPLUS, pgzf->m_pszNomES);
	DWORD nCRC;
	pgzf->m_dwTailleC = CZArc::Compresser(pgzf, fr, fw, &nCRC,
		pFichierPlus->m_ppi->CO_nNiveau,zmDeflate);
	
	// crire l'en-tte final
	const DWORD aCRC[] = { nCRC, e_lenIn };
	return fw.Write(aCRC, sizeof(aCRC));
}
