// IOFichier.cpp
//
// Des fonctions utilitaires pour grer les fichiers et les dossiers
// affichant les messages d'erreurs appropris quand il y a lieu


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "Muzip.h"
#include "IOFichier.h"

#include "Options.h"
#include "dlgConfirm.h"

#include <FileBuf.h>


static bool DossierSupprimerInterne(bool fQueVide, LPTSTR pszPath);


// Crer un fichier en grant les messages d'erreur
bool FichierCreer(LPCTSTR pszFichier)
{
	if (PathFileExists(pszFichier))  return true;
	
	CFileWrite fw;
	VERIF(fw.Open(pszFichier));
	fw.Close();
	return true;
}

// Crer un fichier et demander confirmation en cas d'crasement
bool FichierEcraser(LPCTSTR pszFichier)
{
	if (PathFileExists(pszFichier)) {
		AfxFormatString1(e_sMsg, DEM_ECRASER, pszFichier);
		VERIF(CDlgConfirm::GetReponse(e_opt.m_fConfirmEcraser, 0, cfmNonDefaut));
		SetFileAttributes(pszFichier, 0);
	}
	
	CFileWrite fw;
	VERIF(fw.Open(pszFichier));
	fw.Close();
	return true;
}



// Supprimer un fichier en grant les messages d'erreur
bool FichierSupprimer(LPCTSTR pszFichier, bool fPeutCorbeille)
{
	if (!PathFileExists(pszFichier))  return true;
	
	// Rajouter un second nul final
	TCHAR pszFichier2[MAX_PATH+1];
	lstrcpy(pszFichier2, pszFichier);
	pszFichier2[lstrlen(pszFichier2)+1] = 0;
	
	// Supprimer le fichier
	SHFILEOPSTRUCT shfo;
	shfo.hwnd   = *oApp.m_pMainWnd;
	shfo.wFunc  = FO_DELETE;
	shfo.pFrom  = pszFichier2;
	shfo.pTo    = NULL;
	shfo.fFlags = (fPeutCorbeille && e_opt.m_fGeneralCorbeille)
		? (FILEOP_FLAGS)(FOF_SILENT | FOF_NOCONFIRMATION | FOF_ALLOWUNDO)
		: (FILEOP_FLAGS)(FOF_SILENT | FOF_NOCONFIRMATION);
	return !SHFileOperation(&shfo) && !shfo.fAnyOperationsAborted;
}

// Dplacer un fichier
// Supporte les changement de volumes
bool FichierDeplacer(LPCTSTR pszDe, LPCTSTR pszVers)
{
	// Rajouter un second nul final
	TCHAR pszDe2[MAX_PATH+1];
	lstrcpy(pszDe2, pszDe);
	pszDe2[lstrlen(pszDe2)+1] = 0;
	
	// Dplacer le fichier le fichier
	SHFILEOPSTRUCT shfo;
	shfo.hwnd   = *oApp.m_pMainWnd;
	shfo.wFunc  = FO_MOVE;
	shfo.pFrom  = pszDe2;
	shfo.pTo    = pszVers;
	shfo.fFlags = FOF_SILENT | FOF_NOERRORUI | FOF_NOCONFIRMATION;
	return !SHFileOperation(&shfo) && !shfo.fAnyOperationsAborted;
}


// Crer un dossier en grant les messages d'erreur
bool DossierCreer(LPCTSTR pszDossier)
{
	// Formatter le message d'erreur ventuel
	AfxFormatString1(e_sMsg, ERR_DOSSIERCREER, pszDossier);
	
	// Rcuprer le chemin d'accs complet
	TCHAR pszTemp[MAX_PATH];
	while (!GetFullPathName(pszDossier, nbArray(pszTemp), pszTemp, NULL)) {
		if (AfxMessageBox(e_sMsg, MB_RETRYCANCEL | MB_ICONEXCLAMATION)==IDCANCEL)
			return false;
	}
	
	TCHAR *pc = PathSkipRoot(pszTemp);
	VERIF(*pc);
	
	for (;;) {
		
		// Passer  la partie suivante
		bool fFin = (*pc++==0);
		if (!fFin) {
			while (*pc && *pc!='\\')  pc++;
			fFin = (*pc==0);
			*pc = 0;
		}
		
		// Crer la partie
		while (!CreateDirectory(pszTemp, NULL)) {
			if (GetLastError()==ERROR_ALREADY_EXISTS)  break;
			if (AfxMessageBox(e_sMsg, MB_RETRYCANCEL | MB_ICONEXCLAMATION)==IDCANCEL)
				return false;
		}
		
		if (fFin)  return true;
		*pc = '\\';
	}
}


// Dtruire un dossier et son contenu
bool DossierSupprimer(LPCTSTR pszDossier, bool fQueVide)
{
	if (!PathFileExists(pszDossier))  return true;
	
	TCHAR pszPath[MAX_PATH];
	lstrcpy(pszPath, pszDossier);
	return DossierSupprimerInterne(fQueVide, pszPath);
}
bool DossierSupprimerInterne(bool fQueVide, LPTSTR pszPath)
{
	bool fOK = true;
	
	PathAddBackslash(pszPath);
	TCHAR *const pcPathFin = pszPath+lstrlen(pszPath);
	
	// Effectuer la recherche
	WIN32_FIND_DATA wfd;
	lstrcpy(pcPathFin, _T("*.*"));
	HANDLE hff = ::FindFirstFile(pszPath, &wfd);
	VERIF(hff!=INVALID_HANDLE_VALUE);
	do{
		
		// Ignorer les dossiers . et ..
		if ( wfd.cFileName[0]=='.' &&
		   (!wfd.cFileName[1] ||
		   ( wfd.cFileName[1]=='.' && !wfd.cFileName[2])))
			continue;
		
		lstrcpy(pcPathFin, wfd.cFileName);
		if (wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
			// Sous-dossier
			fOK &= DossierSupprimerInterne(fQueVide, pszPath);
			lstrcpy(pcPathFin, wfd.cFileName);
		}else{
			// Fichier
			if (fQueVide || !DeleteFile(pszPath))  fOK = false;
		}
		
	}while (::FindNextFile(hff, &wfd));
	::FindClose(hff);
	
	// Supprimer le dossier lui-mme
	*pcPathFin = 0;
	PathRemoveBackslash(pszPath);
	if (fOK && RemoveDirectory(pszPath))
		SHChangeNotify(SHCNE_RMDIR, SHCNF_PATH | SHCNF_FLUSHNOWAIT, pszPath, 0);
	else
		fOK = false;
	
	return fOK;
}

// Indiquer si le nom pass (sans chemin d'accs) est valide
// Enlever les espaces des deux cts du nom
bool NomCorriger(LPTSTR pszNom)
{
	// Supprimer les espaces avant/aprs
	TCHAR *pcIn  = pszNom;
	TCHAR *pcOut = pszNom;
	while (isspace(*pcIn))  *pcIn++;
	if (*pcIn) {
		lstrcpy(pcOut, pcIn);
		pcOut = pcOut+lstrlen(pcOut);
		while (isspace(pcOut[-1]))  pcOut--;
	}
	*pcOut = 0;
	
	if (!*pszNom) {
		// Nom vide
		AfxMessageBox(ERR_NOMVIDE);
		return false;
	}
	
	// Vrifier l'absence de caractres interdits
	for (const TCHAR *pc = pszNom; *pc; pc++) {
		if (*pc=='\\' || *pc=='/' || *pc==':' || *pc=='*' || *pc=='?'||
				*pc=='\"' || *pc=='<' || *pc=='>' || *pc=='|') {
			AfxMessageBox(ERR_NOMINVALIDE);
			return false;
		}
	}
	
	return true;
}
