// MainFrame.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "AppResource.h"
#include "Muzip2.h"
#include "MainFrame.h"

#include "MuzipDoc.h"
#include "MuzipArc.h"

#include "Options.h"

#include <SysObject.h>


const UINT MSG_GETARCHIVEINFO = RegisterWindowMessage(_T("MuzipGetArchiveInfo"));


BEGIN_MESSAGE_MAP(CMainFrame, CMDIFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_SETTINGCHANGE()
	ON_WM_SYSCOMMAND()
	ON_WM_INITMENUPOPUP()
	ON_NOTIFY           (NM_CLICK, AFX_IDW_STATUS_BAR, OnIndicatorClick)
	ON_COMMAND          (ID_VIEW_TOOLBAR,              OnViewToolbar)
	ON_COMMAND          (ID_VIEW_APERCUAUTO,           OnViewApercuAuto)
	ON_UPDATE_COMMAND_UI(ID_VIEW_TOOLBAR,              OnUpdateViewToolbar)
	ON_UPDATE_COMMAND_UI(ID_VIEW_APERCUAUTO,           OnUpdateViewApercuAuto)
	ON_UPDATE_COMMAND_UI(ID_INDICATOR_SEL,             OnUpdateIndicatorEtat)
	ON_UPDATE_COMMAND_UI(ID_INDICATOR_COMMENT,         OnUpdateIndicatorComment)
	ON_UPDATE_COMMAND_UI(ID_INDICATOR_TOTAL,           OnUpdateIndicatorEtat)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


// Liste des barres-d'outils (une par menu)
static struct
{
	UINT    nID;      // ID de la barre-d'outils
	LPCTSTR pszReg;   // Cl d'enregistrement dans le registre
	
} const s_aToolBar[] =
{
	{ IDR_MAINFRAME,  _T("BarreOutilsPrincipale") },
	{ IDR_MUZIPTYPE,  _T("BarreOutilsArchive") },
	{ IDR_SCRIPTTYPE, _T("BarreOutilsScript") },
};


// Indicateurs sur la barre d'tat
static const UINT s_StatusBar[] =
{
	ID_SEPARATOR,
	ID_INDICATOR_SEL,
	ID_INDICATOR_TOTAL,
	ID_INDICATOR_COMMENT,
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
};


// Cration
int CMainFrame::OnCreate(CREATESTRUCT* pcs)
{
	// Activer le toujours visible si ncessaire
	if (e_opt.m_fGeneralTjVisible)
		pcs->dwExStyle |= WS_EX_TOPMOST;
	
	if (CMDIFrameWnd::OnCreate(pcs)==-1)  return -1;
	
	// Sous-classer la fentre cliente
	m_wndClient.SubclassWindow(m_hWndMDIClient);
	
	// Barres d'outils
	for (int i=0; i<nbArray(s_aToolBar); i++) {
		if (!m_wndToolBar[i].Create(this, WS_CHILD |
		     CBRS_TOP | CBRS_TOOLTIPS | CBRS_SIZE_DYNAMIC, s_aToolBar[i].nID) ||
		    !m_wndToolBar[i].LoadToolBar(s_aToolBar[i].nID))  return -1;
		m_wndToolBar[i].SetEditable();
		m_wndToolBar[i].FromReg(s_aToolBar[i].pszReg, false);
	}
	
	m_wndToolBar[m_nToolBar = 0].ShowWindow(SW_SHOW);
	
	// Barre d'tat
	m_fComment = false;
	if (!m_wndStatusBar.CreateEx(this, SBT_TOOLTIPS) ||
	    !m_wndStatusBar.SetIndicators(s_StatusBar,
	    sizeof(s_StatusBar)/sizeof(UINT)))  return -1;
	
	// Initialiser l'icne des commentaires:
	// dfinir sa largeur, enlever le texte, et lui affecter une bulle d'aide
	// On n'enlve pas rellement le texte mais on le remplace par des espaces,
	// comme a l'info bulle pourra tre affiche mme s'il n'y a pas
	// l'icne (selon le MSDN, l'info bulle ne peut tre affiche que s'il y
	// a une icne ou du texte qui dborde du panel: ici on met plein d'espaces
	// pour tre sr que a sort des 16+1 pixels disponibles)
	const int nPane = m_wndStatusBar.CommandToIndex(ID_INDICATOR_COMMENT);
	m_wndStatusBar.SetPaneInfo(nPane, ID_INDICATOR_COMMENT,
		m_wndStatusBar.GetPaneStyle(nPane), 16+1);
	m_wndStatusBar.SetPaneText(nPane, _T("       "), FALSE);
	m_wndStatusBar.GetStatusBarCtrl().SetTipText(nPane,
		CStringRes(ID_INDICATOR_COMMENT));
	
	
	// Menu systme: rajouter un item "Toujours visible"
	ASSERT((IDS_SCTOUJOURSVISIBLE & 0xFFF0)==IDS_SCTOUJOURSVISIBLE);
	ASSERT( IDS_SCTOUJOURSVISIBLE < 0xF000);
	const CString s = MAKEINTRESOURCE(IDS_SCTOUJOURSVISIBLE);
	CMenu *const pMenu = GetSystemMenu(FALSE);
	pMenu->AppendMenu(MF_SEPARATOR);
	pMenu->AppendMenu(MF_STRING, IDS_SCTOUJOURSVISIBLE, _tcschr(s, '\n')+1);
	
	return 0;
}


// Destruction
void CMainFrame::OnDestroy()
{
	// Enregistrer l'tat des barres-d'outils
	for (int i=0; i<nbArray(s_aToolBar); i++)
		m_wndToolBar[i].ToReg(s_aToolBar[i].pszReg, false);
	
	CMDIFrameWnd::OnDestroy();
}


// Changement de paramtre systme
void CMainFrame::OnSettingChange(UINT uFlags, LPCTSTR lpszSection)
{
	CMDIFrameWnd::OnSettingChange(uFlags, lpszSection);
	
	// Mettre  jour les couleurs systme
	// ainsi que les options (ncessaire si on se base
	// sur les paramtres de Windows)
	CSysObject::MAJ();
	e_opt.MAJAffichageStyleEx();
	e_opt.Appliquer();
}


// Commande systme
void CMainFrame::OnSysCommand(UINT nID, LPARAM lParam)
{
	if ((nID & 0xFFF0)==IDS_SCTOUJOURSVISIBLE) {
		// Grer la commande "Toujours visible"
		const bool fTjVisible = !ToBool(GetExStyle() & WS_EX_TOPMOST);
		e_opt.m_fGeneralTjVisible = fTjVisible;
		SetWindowPos((fTjVisible)
			? &wndTopMost : &wndNoTopMost, 0,0, 0,0, SWP_NOMOVE | SWP_NOSIZE);
		
	}else
		CMDIFrameWnd::OnSysCommand(nID, lParam);
}

// Menu droul
void CMainFrame::OnInitMenuPopup(CMenu* pPopupMenu, UINT nIndex, BOOL bSysMenu)
{
	CMDIFrameWnd::OnInitMenuPopup(pPopupMenu, nIndex, bSysMenu);
	
	// Cocher/dcocher la commande "Toujours visible"
	if (bSysMenu)
		pPopupMenu->CheckMenuItem(IDS_SCTOUJOURSVISIBLE,
			(GetExStyle() & WS_EX_TOPMOST)
				? MF_BYCOMMAND | MF_CHECKED
				: MF_BYCOMMAND);
}


// Barre d'tat - aucun texte
void CMainFrame::OnUpdateIndicatorEtat(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(FALSE);   // pCmdUI->SetText(NULL) en plus rapide
}

// Barre d'tat - Icne commentaire
void CMainFrame::OnIndicatorClick(NMHDR*, LRESULT*)
{
	// Le panneau cliqu doit tre celui de l'icne
	CPoint pt;
	GetCursorPos(&pt);
	m_wndStatusBar.ScreenToClient(&pt);
	CStatusBarCtrl &sb = m_wndStatusBar.GetStatusBarCtrl();
	const int nPane = m_wndStatusBar.CommandToIndex(ID_INDICATOR_COMMENT);
	CRect rc;
	sb.GetRect(nPane, rc);
	if (!rc.PtInRect(pt))  return;
	
	// Le document actif doit tre une archive
	CFrameWnd *const pFrame = GetActiveFrame();
	if (!pFrame)  return;
	CView     *const pView = pFrame->GetActiveView();
	CMuzipDoc *const pDoc = DYNAMIC_DOWNCAST(CMuzipDoc, pFrame->GetActiveDocument());
	if (pView && pDoc && (pDoc->m_pArc->GetCap() & capCommentaire))
		pView->PostMessage(WM_COMMAND, ID_ARCHIVE_COMMENT);
}
void CMainFrame::OnUpdateIndicatorComment(CCmdUI* pCmdUI)
{
	if (m_fComment) {
		m_fComment = false;
		m_wndStatusBar.GetStatusBarCtrl().SetIcon(
			m_wndStatusBar.CommandToIndex(pCmdUI->m_nID), NULL);
	}
}


// Dfinir l'affichage de l'icne Commentaire
void CMainFrame::SetComment(bool fComment)
{
	if (m_fComment==fComment)  return;
	
	static HICON s_hIcon = NULL;
	if (!s_hIcon)
		s_hIcon = (HICON)::LoadImage(AfxGetResourceHandle(),
			MAKEINTRESOURCE(IDI_COMMENT), IMAGE_ICON, 16,16, 0);
	
	m_fComment = fComment;
	m_wndStatusBar.GetStatusBarCtrl().SetIcon(
		m_wndStatusBar.CommandToIndex(ID_INDICATOR_COMMENT),
		(m_fComment) ? s_hIcon : NULL);
}


// Charger la bonne barre d'outils selon le type du document actif
void CMainFrame::SwitchToolBar()
{
	// Rechercher l'ID du document actif
	// (IDR_MAINFRAME s'il n'y a aucun document)
	UINT nID = IDR_MAINFRAME;
	CMDIChildWnd *const pFrame = MDIGetActive();
	if (pFrame) {
		CDocument *const pDoc = pFrame->GetActiveDocument();
		nID = ((CMuzipDocTemplate*)pDoc->GetDocTemplate())->GetResourceID();
	}
	
	// En dduire l'index de la barre d'outils  afficher
	int nToolBar;
	for (nToolBar=0; s_aToolBar[nToolBar].nID!=nID; nToolBar++);
	if (m_nToolBar==nToolBar)  return;   // Aucun changement
	
	// Masquer l'ancienne barre d'outils et afficher la nouvelle
	m_wndToolBar[m_nToolBar].ShowWindow(SW_HIDE);
	m_nToolBar = nToolBar;
	m_wndToolBar[m_nToolBar].ShowWindow(SW_SHOW);
	RecalcLayout();
}

// Afficher ou masquer la barre d'outils
void CMainFrame::OnViewToolbar()
{
	m_wndToolBar[m_nToolBar].ShowWindow(
		(m_wndToolBar[m_nToolBar].IsWindowVisible()) ? SW_HIDE : SW_SHOW);
	RecalcLayout();
}
void CMainFrame::OnUpdateViewToolbar(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(m_wndToolBar[m_nToolBar].IsWindowVisible());
}

// Aperu automatique
void CMainFrame::OnViewApercuAuto()
{
	e_opt.m_fApercuMAJAuto = !e_opt.m_fApercuMAJAuto;
}
void CMainFrame::OnUpdateViewApercuAuto(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(e_opt.m_fApercuMAJAuto);
}
