// Muzip.h


#ifndef _MUZIP_H_
#define _MUZIP_H_
#if _MSC_VER>1000
#pragma once
#endif
#ifndef __AFXWIN_H__
#error Inclure 'StdAfx.h' avant d'inclure ce fichier
#endif

#include "AppResource.h"
#include "Global.h"

#include "PlusItem.h"


class CMuzipArc;
class CDlgProgress;

class CMuzipCmdLine;
class CMuzipDocTemplate;


const LPCTSTR pszExtDefaut = _T("zip");


enum PRGSTATUS
{
	prsOK,
	prsIgnorer,
	prsAnnuler,
	prsErreur,
};


enum EXTRACTION_ACTION
{
	exaAddMAJ,
	exaMAJ,
	exaAdd,
};

struct ASSOC;

struct EXTRACTION;


// Fichier  extraire
struct FICHIEREXT
{
	CFichier *pFichier;
	LPTSTR    pszDest;
	TCHAR    *pcFin;
};
typedef CListePtr1<FICHIEREXT*> CLstFichierExt;


class CSurvDecomp
{
public:  // Fonctions
	
	CSurvDecomp(HANDLE hProcess, LPCTSTR pszFichier,
		const CFichier* pFichier, CMuzipArc* pArc);
	
	~CSurvDecomp();
	
	void Main();
	void VerifModif();
	void VerifAttribut();
	
	
	static HANDLE s_hEvent;   // Pour stopper prcThreadTous
	static DWORD WINAPI prcThreadTous(void* pv);
	
	
public:  // Donnes
	
	enum{ etModif=1, etAttr=2, etFini=4, etClose=8 };
	
	LPTSTR          m_pszFichier;   // Fichier dcompress
	const CFichier *m_pFichier;     // Fichier  surveiller
	CMuzipArc      *m_pArc;         // Document du fichier
	HANDLE          m_hProcess;     // Processus lanc
	
	HANDLE          m_hf;           // Handle du fichier
	FILETIME        m_ftModif;      // Date de modification initiale
	DWORD           m_dwAttr;       // Attributs initiaux
	int             m_et;           // tat du fichier
	HANDLE          m_hThread;      // Thread de surveillance
	HANDLE          m_hEvent;       // Pour stopper prcThread
	static DWORD WINAPI prcThread(CSurvDecomp* psd);
};
typedef CListePtr1<CSurvDecomp*> CLstSurvDecomp;



class CApp : public CWinApp
{
	friend class CDlgOptions;
	
public:  // Fonctions
	
	CApp();
	~CApp();
	
	CDocument* FindDocument(LPCTSTR pszFichier);
	
	bool ShowFileOpenDialog(CFileDialog& dlg, bool fOuvrir=true);
	
	void DoEvents();
	
	void SetMessageText(UINT    nID);
	void SetMessageText(LPCTSTR psz);
	void SetProgressFichier(LPCTSTR pszFichier);
	void SetProgress(UINT nID, LPCTSTR pszFichier);
	void SetProgress(UINT nID, LPCTSTR pszFichier, UINT iCur, UINT iTotal);
	void InitStatusAvance (const CMuzipArc* pArc=NULL, UINT nIDSTitre=0);
	void ShowStatusAvance (DWORD nMax, bool fPeutIgnorer=false);
	void ShowStatusAvance2(DWORD nMax);
	void HideStatusAvance();
	void HideStatusAvance2();
	void SetStatusAvance (DWORD n);
	void AddStatusAvance (DWORD n=1);
	void AddStatusAvance2(DWORD n=1);
	PRGSTATUS GetProgressStatus();
	PRGSTATUS UpdateProgressStatus();
	void      ResetProgressStatus();
	
	bool ProcessShellCommand(CMuzipCmdLine& cmdInfo);
	
	static void Register();
	static void Unregister();
	static void RegisterDLL(bool fRegister);
	
	bool DemanderEncoreErreur(POS pos);
	
	
private:  // Interne
	
	static bool s_fErreurFatale;
	
	// Paramtres de la barre de progression
	static DWORD s_pos,  s_max;
	static DWORD s_pos2, s_max2;
	
	
public:  // Donnes
	
	static bool       s_fMsg;       // TRUE s'il faut afficher les AfxMessageBox
	static CLIPFORMAT s_cfMuzip;    // Format de presse-papiers
	static int        s_nConfirm;   // Apparition des messages de confirmation
	                                // (activ si >0, dsactiv si <=0)
	static bool       s_fPasNommer; // TRUE s'il ne faut pas demander le nom des GZip
	
	static CLstSurvDecomp s_lstSurvDecomp;   // Fichiers dcompresss  surveiller
	static CLstStr1       s_lstDelDossier;   // Dossiers  supprimer  la sortie
	
	static ARCHIVEPLUS  s_arPlus;  // Options d'ajout par dfaut
	static CPlusItem    s_pi;      // Options de compression par dfaut
	static EXTRACTION   s_ext;     // Options d'extraction
	static RECHOPTIONS  s_rch;     // Options de recherche
	static ASSOC       *s_aAssoc;  // Associations
	
	// Bote de dialogue d'avancement
	static CDlgProgress *s_pdlgProgress;   // Pointeur
	static UINT          s_ndlgProgress;   // Compteur d'affichage (0=masqu)
	static CString       s_sdlgProgress;   // Titre
	
	static CFont s_fntVoir;     // Police du visualiseur interne
	void SetFontVoir(const LOGFONT* plf);
	
	static HIMAGELIST s_himlSystem;   // Liste d'images systme
	
	// Modle de documents des script
	static CMuzipDocTemplate *s_pmdtScript;
	
	
public:  // Statique
	
	static bool s_fOuvreVisible;
	static bool s_fOccupe;        // TRUE si une tche est en cours
	
	
public:  // Hritage
	
	//{{AFX_VIRTUAL(CApp)
	public:
	virtual BOOL InitInstance();
	virtual int  ExitInstance();
	virtual BOOL OnIdle(LONG lCount);
	virtual BOOL OnDDECommand(LPTSTR pszCommand);
	virtual int  DoMessageBox(LPCTSTR lpszPrompt, UINT nType, UINT nIDPrompt);
	//}}AFX_VIRTUAL
	
	
public:  // Messages
	
	//{{AFX_MSG(CApp)
	afx_msg void OnAppAbout();
	afx_msg void OnFileNew();
	afx_msg void OnFileNewScript();
	afx_msg void OnFileReparer();
	afx_msg void OnFileOptions();
	afx_msg void OnFileOpen();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};


extern CApp oApp;


//{{AFX_INSERT_LOCATION}}
#endif
