// MuzipArc.cpp


#include "StdAfx.h"
#include "Interract.h"   // Ici car utilise new
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "Muzip.h"
#include "Muzip2.h"
#include "MuzipDoc.h"
#include "MuzipArc.h"
#include "Interf.h"

#include "PlusItem.h"
#include "FichierPlus.h"
#include "Recherche.h"

#include "Decomp.h"

#include "dlgConfirm.h"
#include "dlgExtraire.h"
#include "dlgPlusItem.h"

#include "Reg.h"
#include "IOFichier.h"
#include "Exceptions.h"

#include "MuzipView.h"
#include "MuzipSfx/MuzipSfx.h"


CLstFichierPlus CMuzipArc::s_lstFichierPlus;

CLstMuzipArc e_lstArc;
CInterract   e_oInterract;



// Constructeur
CMuzipArc::CMuzipArc(CInterf* pi)
	: CDossier(NULL, CStringRes(IDS_RACINE)), m_pi(pi)
{
	m_nUse        = 1;
	m_fRechFiltre = false;
	m_fRm         = false;
	m_fOuvert     = true;   // Dossier racine ouvert
	m_pi->SetArc(this);
	m_fr.SetPeutFermer(false);
	
	// Ajouter l'archive  la liste
	e_lstArc.AddDeb(this);
}

// Destructeur
CMuzipArc::~CMuzipArc()
{
	// Enlever l'archive de la liste
	e_lstArc.RemoveAt(e_lstArc.Find(this));
	
	delete m_pi;
}

// Instanciation
void CMuzipArc::AddRef(CMuzipDoc* pDoc)
{
	m_nUse++;
	if (pDoc && !m_pi->IsDoc()) {
		// Passer l'archive en mode document
		delete m_pi;
		m_pi = new CInterfDoc(pDoc);
	}
}

// Libration
void CMuzipArc::Release(bool fDoc)
{
	if (!--m_nUse) {
		CloseDocument();
		delete this;
		return;
	}
	if (fDoc) {
		// Passer l'archive en mode sans document
		delete m_pi;
		m_pi = new CInterf;
	}
}


// Ouvrir une archive
bool CMuzipArc::OpenDocument(LPCTSTR pszFichier)
{
	// Ouvrir le fichier et ignorer les fichiers vides
	CloseDocument();
	m_pi->SetModifiedFlag(FALSE);
	oApp.DoEvents();
	m_fr.Close();
	
	// Ouvrir l'archive
	if (!m_fr.Open(pszFichier))  return FALSE;
	
	bool fRet = true;
	if (GetFileSize(m_fr, NULL)) {
		// Si elle a une taille nulle, ne pas aller plus loin
		
		// Effectuer l'ouverture en grant le sablier et la barre d'tat
		CWaitCursor wc;
		AfxFormatString1(e_sMsg, STA_ARCHIVEOPEN, pszFichier);
		oApp.SetMessageText(e_sMsg);
		fRet = OpenDocument();
		oApp.SetMessageText(AFX_IDS_IDLEMESSAGE);
		
		m_fr.SetVerbeux();
	}
	
	if (fRet)  m_sPath = pszFichier;
	
	return fRet;
}


// Fermer le document
void CMuzipArc::CloseDocument()
{
	m_fr.Close();
}


// Reconstruire l'archive
void CMuzipArc::Reconstruire()
{
	m_pi->SetModifiedFlag(FALSE);
}

// Excut aprs une commande demandant
// reconstruction de l'archive
void CMuzipArc::FautReconstruire(bool fRm)
{
	// Recommencer la recherche
	Rechercher();
	
	if (m_fRm)  goto Manuel;
	
	// Uniquement si l'archive est trop grosse
	if (GetFileSize(m_fr, NULL)>>20 < e_opt.m_nCompRmTaille)
		goto Auto;
	
	if (fRm)  goto Manuel;
	
	// Vrifier que l'archive n'est pas sur un support amovible
	if (e_opt.m_fCompRmAmovible) {
		TCHAR pszDir[MAX_PATH];
		lstrcpy(pszDir, m_sPath);
		PathStripToRoot(pszDir);
		if (GetDriveType(pszDir)==DRIVE_REMOVABLE)
			goto Manuel;
	}
	
	
Auto:
	
	// Reconstruire directement
	Reconstruire();
	return;
	
	
Manuel:
	m_fRm = true;
	
	// Positionner le flag de modification
	m_pi->SetModifiedFlag();
}


// Ouvrir l'archive en mode lecture
bool CMuzipArc::ReadOpen()
{
	return (m_fr!=INVALID_HANDLE_VALUE || m_fr.Open(m_sPath));
}

// Fermer l'archive ouverte en mode lecture
void CMuzipArc::ReadClose()
{
	m_fr.Close();
}


// Ouvrir l'archive en mode criture
bool CMuzipArc::WriteOpen()
{
	ASSERT(CFichier::s_fw==INVALID_HANDLE_VALUE);
	
	ReadClose();
	if (CFichier::s_fw.Open(m_sPath, 0, OPEN_EXISTING))  return true;
	
	ReadOpen();
	return false;
}

// Fermer l'archive ouverte en mode criture
void CMuzipArc::WriteClose()
{
	CFichier::s_fw.Close();
	ReadOpen();
}


// Commencer l'ajout de fichiers  l'archive
void CMuzipArc::ArchivePlus(CArchivePlus& arPlus)
{
	m_pi->DeleteAll();
	m_pi->SetModifiedFlag(FALSE);
	
	POS pos;
	CRecherche rech(this, arPlus);
	
	CMuzipOp mop(this, STA_ARCHIVEPLUST);
	
	if (arPlus.m_nSup==supTout) {
		// Vider l'archive
		oApp.SetMessageText(STA_ARCHIVEPLUSSUPTOUT);
		Supprimer();
	}
	
	oApp.SetMessageText(STA_RECHERCHE);
	
	// Mettre tous les fichiers en "non-trouv"
	SetFichierNonTrouve();
	
	// Effectuer les recherches
	bool fOK = true;
	oApp.ShowStatusAvance(arPlus.GetNb());
	int nRecherche = 0;
	pos = arPlus.GetPosDeb();
	while (pos) {
		CPlusItem *const ppi = arPlus.GetSuiv(pos);
		if (!rech.Rechercher(ppi)) {
			oApp.ResetProgressStatus();
			fOK = false;
			break;
		}
		oApp.SetStatusAvance(++nRecherche);
	}
	rech.Finaliser(s_lstFichierPlus);
	if (!fOK) {
		oApp.HideStatusAvance();
		goto Fin;
	}
	
	if (arPlus.m_nSup==supRecherche) {
		// Supprimer tous les fichiers non-trouvs
		oApp.SetStatusAvance(0);
		oApp.SetMessageText(STA_ARCHIVEPLUSSUPRECHERCHE);
		SupprimerFichierNonTrouve();
	}
	
	oApp.HideStatusAvance();
	
	if (s_lstFichierPlus.GetNb())
		// Ne faire quelque chose que si au moins un fichier
		// doit tre ajout  l'archive
		ArchivePlus();
	
Fin:
	
	// Dtruire les fichiers restants (dont les fichiers ignors)
	pos = s_lstFichierPlus.GetPosDeb();
	while (pos) {
		CFichierPlus *const pFichierPlus = s_lstFichierPlus.GetSuiv(pos);
		CFichier *pFichier;
		
		pFichier = pFichierPlus->m_pFichier;
		if (pFichier) {
			delete [] pFichier->m_pszNomES;
			delete    pFichier;
		}
	}
	s_lstFichierPlus.DeleteAll();
	
	m_pi->UpdateAll();
}

// Ajouter des fichiers  une archive
// Par dfaut on suppose que l'archive ne peut contenir
// qu'un seul fichier car c'est la mthode par dfaut
// Pour grer plusieurs fichiers, la classe C*Arc doit implmenter
// cette mthode
// La suite est dans ArchivePlusUn() pour la compression proprement dite
bool CMuzipArc::ArchivePlus()
{
	// Rechercher le premier fichier  compresser
	CFichierPlus *pFichierPlus;
	POS pos = s_lstFichierPlus.GetPosDeb();
	do{
		if (!pos) {
			pFichierPlus = NULL;
			break;
		}
		pFichierPlus = s_lstFichierPlus.GetSuiv(pos);
	}while (!pFichierPlus->m_pFichier->m_fFautComp);
	if (!pFichierPlus)  return false;
	
	// Rechercher le suivant pour regarder
	// s'il y a plusieurs fichiers  compresser
	while (pos) {
		if (s_lstFichierPlus.GetSuiv(pos)->m_pFichier->m_fFautComp) {
			// S'il faut compresser plusieurs fichiers,
			// afficher un avertissement
			AfxFormatString1(e_sMsg, DEM_PLUSPLUSIEURS,
				pFichierPlus->m_pFichier->m_pszNomES);
			if (!CDlgConfirm::GetReponse(e_opt.m_fConfirmPlusPlusieurs))
				return false;
			break;
		}
	}
	
	// Supprimer tous les fichiers
	m_lstFichier.DeleteAll();
	
	CFichier *const pFichier = pFichierPlus->m_pFichier;
	
	if (!pFichier->CompresseArchive()) {
		oApp.ShowStatusAvance(pFichier->m_dwTailleN);
		
		CFileWrite fw;
		CFileRead  fr;
		
		// Ouvrir l'archive en mode criture
		ReadClose();
		if (!fw.Open(m_sPath))  goto Fin;
		
		// Ouvrir le fichier en lecture
		if (!fr.Open(pFichier->m_pszNomES))  goto Erreur;
		
		// Remplir la classe du fichier
		{
			WIN32_FIND_DATA wfd;
			const HANDLE hff = ::FindFirstFile(pFichier->m_pszNomES, &wfd);
			if (!hff) {
				CMuzipException(ERR_READ, pFichier->m_pszNomES).ReportError();
				goto Erreur;
			}
			::FindClose(hff);
			
			// Rcuprer la taille et la date de modification
			pFichier->m_dwTailleN = wfd.nFileSizeLow;
			pFichier->m_dtM       = wfd.ftCreationTime;
		}
		
		if (!ArchivePlusUn(pFichierPlus, fr, fw))  goto Erreur;
		
		// Finaliser le CFichierPlus et le supprimer de la liste
		pFichierPlus->Finaliser();
		s_lstFichierPlus.DeleteAt(s_lstFichierPlus.Find(pFichierPlus));
		
		
Erreur:
		fr.Close();
		fw.Close();
		
Fin:
		oApp.SetStatusAvance(1);
		oApp.HideStatusAvance();
		ReadOpen();
	}
	
	return false;   // Car on finalise manuellement
}

// Grer les erreurs  la compression
PRGSTATUS CMuzipArc::ErreurArchivePlus()
{
	const PRGSTATUS prs = oApp.GetProgressStatus();
	return (prs==prsOK) ? prsErreur : prs;
}

// Grer la progression de la compression
// (ignorance de fichier, annulation de compression, etc.)
// Retourner FALSE s'il faut stopper la compression
bool CMuzipArc::ProgressArchivePlus(CFichier* pFichier, PRGSTATUS prs, POS pos, DWORD offOld)
{
	if (prs==prsOK)  return true;   // Pas d'erreur
	
	bool fContinuer = true;
	
	const bool fPeutAnnuler = (prs!=prsErreur);
	if (prs==prsErreur && !oApp.DemanderEncoreErreur(pos))
		prs = prsAnnuler;
	
	if (prs==prsAnnuler) {
		const UINT ret = AfxMessageBox(
			(fPeutAnnuler) ? DEM_ANNULERCOMP1 : DEM_ANNULERCOMP2,
			(fPeutAnnuler)
				? MB_YESNOCANCEL | MB_ICONQUESTION
				: MB_YESNO       | MB_ICONQUESTION);
		if (ret==IDCANCEL)  return true;   // Continuer sans rien stopper ni annuler
		
		if (ret==IDYES)  return false;   // Annulation complte
		
		// ret==IDNO: ignorer tous les fichiers restants
		fContinuer = false;
		while (pos)
			s_lstFichierPlus.GetSuiv(pos)->m_pFichier->m_fIgnorer = true;
	}
	
	// Remettre l'tat de progression  zro
	// Si prs==prsIgnorer, il ne faut ignorer qu'un fichier
	// Si prs==prsAnnuler, il ne faut pas annuler la finalisation de l'archive
	// car on est en mode stop de compression (et non annulation de compression)
	oApp.ResetProgressStatus();
	
	// prs==prsIgnorer ou prs==prsAnnuler: ignorer le fichier courant
	IgnorerFichier(offOld, pFichier);
	
	return fContinuer;
}

// Finaliser l'ajout
void CMuzipArc::Finaliser()
{
	// Finaliser les fichiers
	POS pos = s_lstFichierPlus.GetPosDeb();
	while (pos) {
		const POS posSauve = pos;
		if (s_lstFichierPlus.GetSuiv(pos)->Finaliser())
			s_lstFichierPlus.DeleteAt(posSauve);
	}
}

// Ignorer un fichier
void CMuzipArc::IgnorerFichier(DWORD offOld, CFichier* pFichier)
{
	// Retourner en arrire dans le fichier
	e_fw.Seek(offOld, FILE_BEGIN);
	SetEndOfFile(e_fw);
	
	// Marquer le fichier comme ignor
	// (ne pas le compter dans la finalisation de l'archive)
	pFichier->m_fIgnorer = true;
}


// Extraire
bool CMuzipArc::ArchiveExtraire(CLstFichierExt& lstFichierExt, const EXTRACTION* pext)
{
	POS pos;
	
	// Appliquer le filtre
	if (pext &&
	   (pext->m_nAction!=exaAddMAJ ||
	    pext->m_fEcraseDate)) {
		
		// Enlever de la liste les fichiers  ne pas extraire
		pos = lstFichierExt.GetPosDeb();
		while (pos) {
			const POS posSauve = pos;
			FICHIEREXT *const pFichierExt = lstFichierExt.GetSuiv(pos);
			
			// Composer le nom du fichier aprs dcompression
			lstrcpy(pFichierExt->pcFin, pFichierExt->pFichier->m_pszNom);
			
			WIN32_FIND_DATA wfd;
			const HANDLE hff = FindFirstFile(pFichierExt->pszDest, &wfd);
			const bool fExiste = (hff!=INVALID_HANDLE_VALUE);
			if (fExiste)  FindClose(hff);
			
			// Action rductrice
			// Le fichier doit exister en mode Mise  jour,
			// et ne pas exister en mode Ajouter
			if (pext->m_nAction!=exaAddMAJ &&
			    fExiste!=(pext->m_nAction==exaMAJ))
				goto Supprimer;
			
			// N'craser que les fichiers plus anciens
			// Il faut que le fichier existe et que la date soit valide
			if (fExiste && pext->m_fEcraseDate &&
			   (wfd.ftLastWriteTime.dwLowDateTime || wfd.ftLastWriteTime.dwHighDateTime)) {
				if (pFichierExt->pFichier->m_dtM>CTempsDHS(wfd.ftLastWriteTime))
					goto Supprimer;
			}
			
			continue;
			
Supprimer:
			lstFichierExt.DeleteAt(posSauve);
		}
	}
	
	// Il doit rester au moins un fichier
	VERIF(lstFichierExt.GetNb()>0);
	
	CMuzipOp mop(this, STA_ARCHIVEEXTRAIRET);
	
	// Calculer la taille totale des fichiers en lecture
	DWORD dwTotal = 0;
	pos = lstFichierExt.GetPosDeb();
	while (pos)
		dwTotal += lstFichierExt.GetSuiv(pos)->pFichier->m_dwTailleC;
	oApp.ShowStatusAvance(dwTotal);
	
	// Extraire chaque fichier
	const bool fEcraseDem = (pext && pext->m_fEcraseDem);
	UINT iFichier = 0;
	pos = lstFichierExt.GetPosDeb();
	while (pos && oApp.UpdateProgressStatus()!=prsAnnuler) {
		FICHIEREXT *const pFichierExt = lstFichierExt.GetSuiv(pos);
		oApp.SetProgress(STA_ARCHIVEEXTRAIRE, pFichierExt->pFichier->m_pszNom,
			iFichier++, lstFichierExt.GetNb());
		
		lstrcpy(pFichierExt->pcFin, pFichierExt->pFichier->m_pszNom);
		if (fEcraseDem && PathFileExists(pFichierExt->pszDest)) {
			// Le fichier existe
			// Demander confirmation pour l'craser
			AfxFormatString1(e_sMsg, DEM_ECRASER, pFichierExt->pszDest);
			const int nRet = AfxMessageBox(e_sMsg, MB_YESNOCANCEL | MB_ICONQUESTION);
			if (nRet==IDCANCEL)  break;
			if (nRet==IDNO)      continue;
		}
		pFichierExt->pFichier->Extraire(pFichierExt->pszDest);
	}
	oApp.HideStatusAvance();
	
	return true;
}

// Extraire une liste de fichiers
bool CMuzipArc::ArchiveExtraire(CLstFichier& lstFichier, LPCTSTR pszRep, const EXTRACTION* pext)
{
	const bool fPerso = ToBool(pszRep && *pszRep);
	
	CDlgExtraire dlgExtraire(this);
	if (!fPerso) {
		VERIF(dlgExtraire.DoModal()!=IDCANCEL);
		pszRep = dlgExtraire.m_sDest;
		pext = &oApp.s_ext;
	}
	
	// Construire la liste de FICHIEREXT
	TCHAR pszDest[MAX_PATH];
	TCHAR *pcFin = pszDest;
	{
		const TCHAR *pcIn  = pszRep;
		while (*pcIn)  *pcFin++ = *pcIn++;
		if (pcFin>pszDest && pcFin[-1]!='\\')  *pcFin++ = '\\';
	}
	
	CLstFichierExt lstFichierExt(true);
	POS pos = lstFichier.GetPosDeb();
	while (pos) {
		FICHIEREXT *const pFichierExt = new FICHIEREXT;
		pFichierExt->pFichier = lstFichier.GetSuiv(pos);
		pFichierExt->pszDest  = pszDest;
		pFichierExt->pcFin    = pcFin;
		lstFichierExt.AddDeb(pFichierExt);
	}
	
	// Dcompresser cette liste
	return ArchiveExtraire(lstFichierExt, (pext) ? pext : &oApp.s_ext);
}

// Tester
void CMuzipArc::ArchiveTester(CLstFichier& lstFichier)
{
	CMuzipOp mop(this, STA_ARCHIVETESTERT);
	
	DECOMP::e_dm  = dmTester;
	DECOMP::e_pfr = &m_fr;
	
	// Tester chaque fichier
	UINT iFichier = 0;
	POS pos = lstFichier.GetPosDeb();
	while (pos && oApp.UpdateProgressStatus()!=prsAnnuler) {
		CFichier *const pFichier = lstFichier.GetSuiv(pos);
		oApp.SetProgress(STA_ARCHIVETESTER, pFichier->m_pszNom,
			iFichier++, lstFichier.GetNb());
		
		pFichier->Extraire(NULL);
		
		oApp.SetStatusAvance(iFichier);
	}
	
	DECOMP::e_dm = dmNormal;
}

// Supprimer
void CMuzipArc::ArchiveMoins(CLstFichier& lstFichier)
{
	if (!lstFichier.GetNb())  return;
	
	{
		CMuzipOp mop(this, STA_ARCHIVEMOINST);
		
		// Enlever les fichiers de la liste
		oApp.ShowStatusAvance(lstFichier.GetNb());
		UINT iFichier = 0;
		POS pos = lstFichier.GetPosDeb();
		while (pos) {
			CFichier *const pFichier = lstFichier.GetSuiv(pos);
			oApp.SetProgress(STA_ARCHIVEMOINS, pFichier->m_pszNom,
				iFichier++, lstFichier.GetNb());
			
			pFichier->Supprimer(false);
			oApp.SetStatusAvance(iFichier);
		}
		
		if (e_opt.m_fGeneralSupprDossierVide) {
			// Supprimer les dossiers vides
			oApp.SetMessageText(STA_SUPPRDOSSIERVIDE);
			SupprimerSiVide();
		}
		
		oApp.HideStatusAvance();
	}
	FautReconstruire();
}

// Crer un auto-extractible
void CMuzipArc::ArchiveSfx(LPCTSTR pszEXE, LPCTSTR pszDest, bool fEcraser)
{
	const HINSTANCE hInstRes = AfxGetResourceHandle();
	
	// Charger la ressource contenant l'auto-extractible
	const HRSRC   hRSrc   = FindResource(hInstRes,
		MAKEINTRESOURCE(GetSfxID()), _T("SFX"));
	if (!hRSrc)  return;
	const HGLOBAL hGlobal = LoadResource(hInstRes, hRSrc);
	if (!hGlobal)  return;
	void *const pv        = LockResource(hGlobal);
	if (!pv)  return;
	
	bool fOK = false;
	
	// Initialiser le footer
	SFXFOOTER sfx;
	sfx.id       = idSFXFooter;
	sfx.dwOff    = SizeofResource(hInstRes, hRSrc);
	sfx.dwLen    = GetFileSize(m_fr, NULL);
	lstrcpy(sfx.pszDest, pszDest);
	sfx.fEcraser = fEcraser;
	
	CFileWrite fw;
	if (!fw.Open(pszEXE))  return;
	
	// crire l'excutable
	if (!fw.Write(pv, sfx.dwOff))  goto Fin;
	
	// crire l'archive
	m_fr.Seek(0, FILE_BEGIN);
	if (!fw.WriteFrom(m_fr, sfx.dwLen))  goto Fin;
	
	// crire le footer
	if (!fw.Write(&sfx, sizeof(sfx)))  goto Fin;
	
	fOK = true;
	
Fin:
	fw.Close();
	if (!fOK)  DeleteFile(pszEXE);
}


// Convertir l'archive
void CMuzipArc::Convertir(LPCTSTR pszFichier)
{
	// Ouvrir le fichier de la nouvelle archive...
	CMuzipDoc *const pDoc = DYNAMIC_DOWNCAST(CMuzipDoc, oApp.OpenDocumentFile(pszFichier));
	if (pDoc) {
		// ...et y compresser l'archive originale
		
		CMuzipArc *const pArc = pDoc->m_pArc;
		
		CArchivePlus arPlus;
		CPlusItem pi;
		pi.SetDefautExt();
		if (CDlgPlusItem(pArc, pi, pArc->m_sMdp, false).DoModal()==IDCANCEL)
			pArc->m_pi->CloseDocument();
		else{
			pi.DO_sFiltre = m_sPath;
			arPlus.AddDeb(&pi);
			
			// Effectuer la compression
			ReadClose();
			pArc->ArchivePlus(arPlus);
			m_pi->CloseDocument();
		}
	}
	
	DeleteFile(pszFichier);
}


// Retourner le dossier d'un fichier, en le crant si ncessaire
CDossier* CMuzipArc::GetDossier(LPCTSTR psz, bool fDossier)
{
	// Dupliquer le chemin d'accs pass
	// S'il s'agit d'un dossier, rajouter un antislash  la fin
	LPTSTR pszPath = new TCHAR[lstrlen(psz)+2];
	lstrcpy(pszPath, psz);
	if (fDossier)  PathAddBackslash(pszPath);
	
	TCHAR *pcSep;
	
	// Convertir les '\' en '/'
	for (pcSep=pszPath;; pcSep++) {
		pcSep = _tcschr(pcSep, '\\');
		if (!pcSep)  break;
		*pcSep = '/';
	}
	
	// Commencer  la premire partie
	pcSep = _tcschr(pszPath, '/');
	if (!pcSep) {
		delete [] pszPath;
		return this;
	}
	
	// - Isoler chaque partie du chemin d'accs
	//   et les rajouter  la liste si ncessaire
	// - Garder le sous-dossier de dernier niveau
	bool fExiste = true;
	LPTSTR pcNom = pszPath;
	CLstDossier *plstDossier = &m_lstDossier;
	CDossier *pDossierCur = this;
	do{
		*pcSep = 0;
		if (fExiste) {
			POS pos = plstDossier->GetPosDeb();
			while (pos) {
				CDossier *const pDossier = plstDossier->GetSuiv(pos);
				if (!lstrcmpi(pcNom, pDossier->m_pszNom)) {
					// Le dossier existe: OK
					pDossierCur = pDossier;
					goto SuitePartie;
				}
			}
		}
		// Le dossier n'existe pas: le crer
		fExiste = false;
		pDossierCur = new CDossier(pDossierCur, pcNom);
		plstDossier->AddFin(pDossierCur);
SuitePartie:
		plstDossier = &pDossierCur->m_lstDossier;
		// Passer  la partie suivante
		pcNom = ++pcSep;
		pcSep = _tcschr(pcSep, '/');
	}while (pcSep);
	
	delete [] pszPath;
	return pDossierCur;
}


// Rcuprer des informations sur l'archive
void CMuzipArc::GetArchiveInfo(CMuzipCmdLine& cmdInfo)
{
	CInterract *const pInterract = cmdInfo.m_pInterract;
	CLstFichier lstFichier;
	POS pos;
	ListerFichier(lstFichier);
	
	// Calculer la taille totale de l'archive non-compresse
	DWORD dwTailleC = 0;
	DWORD dwTailleN = 0;
	pos = lstFichier.GetPosDeb();
	while (pos) {
		CFichier *const pFichier = lstFichier.GetSuiv(pos);
		dwTailleC += pFichier->m_dwTailleC;
		dwTailleN += pFichier->m_dwTailleN;
	}
	
	// crire les valeurs
	pInterract->nbFichier = (int)lstFichier.GetNb();
	pInterract->nbDossier = (int)GetNbDossier();
	pInterract->dwTailleC = dwTailleC;
	pInterract->dwTailleN = dwTailleN;
	
	// Avertir l'appelant
	// On peut s'occuper de la liste plus tard,
	// alors autant en profiter pour ne pas risquer de dpasser
	// le dlai fix par MuzipExt.dll
	OnGetArchiveInfoOK(pInterract);
	
	if (IsWindow(cmdInfo.m_hwndLst)) {
		CListCtrl lst;
		lst.Attach(cmdInfo.m_hwndLst);
		
		// Initialiser la liste
		CMuzipView::InitListCtrl(lst, pInterract);
		
		// Ajouter les items
		pos = lstFichier.GetPosDeb();
		int nItem = 0;
		while (pos && IsWindow(lst))
			lstFichier.GetSuiv(pos)->lstLister(lst, nItem++, pInterract);
		lst.Detach();
	}
}

// La rcupration d'informations est termine (russie ou non)
void CMuzipArc::OnGetArchiveInfoOK(const CInterract* pInterract)
{
	// Formatter le nom de l'vnement
	TCHAR pszEvent[nbArray(pszEventFormat)+2];
	wsprintf(pszEvent, pszEventFormat, pInterract->m_nUnique);
	
	// L'ouvrir, le positionner, et le fermer
	const HANDLE hEvent = OpenEvent(EVENT_MODIFY_STATE, FALSE, pszEvent);
	if (hEvent) {
		PulseEvent (hEvent);
		CloseHandle(hEvent);
	}
}


// Ouvrir l'archive, crer un fichier temporaire
// et y copier les entres de la liste
bool CMuzipArc::OpenArchiveTemp(CFileWrite& fw, LPTSTR pszTemp)
{
	// Rcuprer le nom du fichier temporaire
	TCHAR pszPath[MAX_PATH];
	e_opt.GetDossierDrag(pszPath);
	GetTempFileName(pszPath, _T("MUZ"), 0, pszTemp);
	
	// Aller au dbut de l'archive
	VERIF(m_fr.Seek(0, FILE_BEGIN));
	
	// Ouvrir le fichier temporaire
	VERIF(fw.Open(pszTemp));
	
	// Ouvrir l'ancienne archive pour crire les anciens fichiers
	CFichier::s_nbFichier = GetNbFichier();
	CFichier::s_nFichier  = 0;
	oApp.ShowStatusAvance(CFichier::s_nbFichier);
	const bool fOK = FichierCopier(m_fr, fw);
	CFichier::s_nbFichier = 0;
	if (!fOK)  fw.Close();
	oApp.HideStatusAvance();
	
	return fOK;
}

// Fermer une archive temporaire
bool CMuzipArc::CloseArchiveTemp(CFileWrite& fw, LPCTSTR pszTemp)
{
	fw.Close();
	
	// Effacer l'ancienne archive
	ReadClose();
	if (!FichierSupprimer(m_sPath, false)) {
ErreurRemplacer:
		AfxMessageBox(ERR_REMPLACER);
		ReadOpen();
		return false;
	}
	
	// Le remplacer par le fichier temporaire
	while (!FichierDeplacer(pszTemp, m_sPath)) {
		AfxFormatString2(e_sMsg, ERR_RENOMMER, pszTemp, m_sPath);
		if (AfxMessageBox(e_sMsg, MB_RETRYCANCEL | MB_ICONEXCLAMATION)==IDCANCEL)
			goto ErreurRemplacer;
	}
	
	return ReadOpen();
}



// Ouvrir une archive
CMuzipArc* OpenArchive(LPCTSTR pszFichier, CMuzipDoc* pDoc)
{
	// Rcuprer le chemin complet
	TCHAR pszPath[MAX_PATH];
	if (!GetFullPathName(pszFichier, nbArray(pszPath),pszPath, NULL))
		*pszPath = 0;
	
	CMuzipArc *pArc;
	
	// Regarder si l'archive est dj ouverte
	{
		POS pos = e_lstArc.GetPosDeb();
		while (pos) {
			pArc = e_lstArc.GetSuiv(pos);
			if (pArc->m_sPath==pszPath) {
				pArc->AddRef(pDoc);
				return pArc;
			}
		}
	}
	
	// Tester l'ouverture du fichier
	CFileRead fr;
	if (!fr.Open(pszPath))  return NULL;
	fr.Close();
	
	// Ouvrir l'archive
	// Chercher d'abord le meilleur document
	pArc = NULL;
	POSITION pos = oApp.GetFirstDocTemplatePosition();
	while (pos) {
		CMuzipDocTemplate *const pTemplate = (CMuzipDocTemplate*)oApp.GetNextDocTemplate(pos);
		
		CDocument *pDocMatch;
		if (pTemplate->MatchDocType(pszPath, pDocMatch)==CDocTemplate::yesAttemptNative) {
			// Template trouv !
			// Vrifier qu'il ne s'agit pas du template des scripts
			if (pTemplate==oApp.s_pmdtScript)  break;
			
			// Ouvrir l'archive
			pArc = pTemplate->CreateArchive((pDoc) ? new CInterfDoc(pDoc) : new CInterf);
			if (!pArc->OpenDocument(pszPath)) {
				// Erreur  l'ouverture: librer l'archive
				pArc->Release(false);
				pArc = NULL;
			}
			return pArc;
		}
	}
	
	// Erreur: aucun format ne correspond
	AfxFormatString1(e_sMsg, ERR_FORMAT, pszPath);
	AfxMessageBox(e_sMsg);
	
	return NULL;
}


// Initialiser la progression principale
// avec la taille totale des fichiers
void CMuzipArc::ShowStatusAvanceFichier(bool fPeutIgnorer)
{
	DWORD dwTailleTotal = 0;
	
	POS pos = s_lstFichierPlus.GetPosDeb();
	while (pos)
		dwTailleTotal += s_lstFichierPlus.GetSuiv(pos)->m_pFichier->m_dwTailleN;
	
	oApp.ShowStatusAvance(dwTailleTotal, fPeutIgnorer);
}
