// MuzipArc.h


#ifndef _MUZIPARC_H_
#define _MUZIPARC_H_
#if _MSC_VER>1000
#pragma once
#endif

#include "Global.h"
#include "Fichier.h"
#include "Dossier.h"

#include <FileBuf.h>


enum
{
	capNiveau          = 0x001,
	capCommentaire     = 0x002,
	capArbo            = 0x004,
	capAttribut        = 0x008,
	capCrypte          = 0x010,
	capRapideRenommer  = 0x020,
	capRapideModifProp = 0x040,
	capSfx             = 0x080,
	capdtM             = 0x100,
	capMaxUnFichier    = 0x200,
};

enum PRGSTATUS;

struct EXTRACTION;

class CMuzipDoc;
class CInterf;
class CPlusItem;
class CArchivePlus;
class CMuzipCmdLine;
class CInterract;

class CFichierPlus;
typedef CListePtr1<CFichierPlus*> CLstFichierPlus;


class CMuzipArc : public CDossier
{
	friend class CFichier;
	
public:  // Fonctions
	
	void AddRef (CMuzipDoc* pDoc=NULL);
	void Release(bool fDoc);
	
	bool OpenDocument(LPCTSTR pszFichier);
	
	void FautReconstruire(bool fRm=false);
	
	bool ReadOpen();
	void ReadClose();
	bool WriteOpen();
	void WriteClose();
	
	void ArchivePlus    (CArchivePlus& arPlus);
	bool ProgressArchivePlus(CFichier* pFichier, PRGSTATUS prs, POS pos, DWORD offOld);
	PRGSTATUS ErreurArchivePlus();
	void Finaliser      ();
	bool ArchiveExtraire(CLstFichierExt& lstFichierExt, const EXTRACTION* pext);
	bool ArchiveExtraire(CLstFichier& lstFichier, LPCTSTR pszRep=NULL, const EXTRACTION* pext=NULL);
	void ArchiveTester  (CLstFichier& lstFichier);
	void ArchiveMoins   (CLstFichier& lstFichier);
	void ArchiveSfx     (LPCTSTR pszEXE, LPCTSTR pszDest, bool fEcraser);
	
	void Convertir(LPCTSTR pszFichier);
	
	CDossier* GetDossier(LPCTSTR psz, bool fDossier=true);
	
	void GetArchiveInfo(CMuzipCmdLine& cmdInfo);
	static void OnGetArchiveInfoOK(const CInterract* pInterract);
	
	
public:  // Hritage
	
	virtual int  GetCap()   const = 0;
	virtual UINT GetSfxID() const { return 0; }
	
	virtual bool OpenDocument() = 0;
	virtual void CloseDocument();
	
	virtual void Reconstruire();
	
	virtual bool OpenArchiveTemp (CFileWrite& fw, LPTSTR  pszTemp);
	virtual bool CloseArchiveTemp(CFileWrite& fw, LPCTSTR pszTemp);
	
	virtual CFichier* NewFichier (LPCTSTR pszFichier, const WIN32_FIND_DATA& wfd) = 0;
	
	virtual void SetComment(LPCTSTR psz) { psz; }
	virtual LPCTSTR GetComment() const { return NULL; }
	
	
protected:  // Interne
	
	virtual bool ArchivePlus();
	virtual bool ArchivePlusUn(CFichierPlus*, CFileRead&, CFileWrite&) { return false; }
	
	virtual void IgnorerFichier(DWORD offOld, CFichier* pFichier);
	
	
	int m_nUse;   // Compteur d'utilisation
	CMuzipArc(CInterf* pi);
	virtual ~CMuzipArc();
	
	
	static void ShowStatusAvanceFichier(bool fPeutIgnorer=false);
	
	
public:  // Donnes
	
	CInterf    *m_pi;      // Interface
	CString     m_sPath;   // Chemin d'accs de l'archive
	
	CString     m_sMdp;    // Mot de passe
	CFileRead   m_fr;      // Fichier en mode lecture
	
	static CLstFichierPlus s_lstFichierPlus;   // Liste des fichiers  ajouter
	
	bool m_fRechFiltre;      // Filtrer recherche
	bool m_fRm;              // Reconstruction manuelle
};
typedef CListe1<CMuzipArc*> CLstMuzipArc;


extern CLstMuzipArc e_lstArc;
extern CInterract   e_oInterract;


CMuzipArc* OpenArchive(LPCTSTR pszFichier, CMuzipDoc* pDoc=NULL);


#endif
