// MuzipSfx.cpp


#include "StdAfx.h"
#include "Resource.h"
#include "MuzipSfx.h"
#include "ZipFichier.h"

#include "../CRC32.h"
#include "../Z/Cons.h"
#include "../Z/Crypt.h"

#include "../Z/UnZ.h"
using namespace UNZ;

#include "../Decomp.h"
using namespace DECOMP;


CFileWrite e_fw;


// Pour gagner un peu de place sur l'excutable
// en vitant d'avoir les StringTable en double,
// on place les chanes directement dans le code
static const LPCTSTR s_pszStringTable[] =
{
#if   defined(FRANCAIS)
	_T("Dcompresser dans le dossier :"),
	_T("Erreur  l'ouverture du fichier %s\n%s"),
	_T("Erreur de lecture pour le fichier %s\n%s"),
	_T("Erreur d'criture pour le fichier %s\n%s"),
	_T("Le fichier %s existe dj. Voulez-vous l'craser ?"),
	_T("Impossible de crer le dossier %s."),
	_T("Mot de passe incorrect pour le fichier %s."),
	_T("Erreur  la dcompression de %s."),
#elif defined(ANGLAIS)
	_T("Uncompress to folder:"),
	_T("Error when opening %s\n%s"),
	_T("Reading error in %s\n%s"),
	_T("Write error in %s\n%s"),
	_T("File %s already exists. Overwrite it?"),
	_T("Unable to create the folder %s."),
	_T("Invalid password for %s."),
	_T("Error when decompressing %s."),
#endif
};

#define LoadStaticString(nID)  s_pszStringTable[(nID)-1]


TCHAR e_psz[bufFormat+MAX_PATH];
TCHAR e_pszMdp[101];
BYTE  e_abBuf[sizeof(HEADERCENSIG)];


// Pour rduire la taille de l'excutable,
// on n'utilise pas la bibliothque standard C
// Il faut donc implmenter quelques fonctions de base

static HANDLE s_hHeap;

// Allouer
void* operator new(size_t len)
{
	return HeapAlloc(s_hHeap, 0, len);
}
// Librer
void operator delete(void* pv)
{
	HeapFree(s_hHeap, 0, pv);
}



#ifdef _DEBUG
#define ASSERT(x) \
	while (!(x)) { \
		TCHAR psz[128]; \
		wsprintf(psz, _T("Assert fail\n%s\nLine %d"), __FILE__, __LINE__); \
		MessageBox(NULL, psz, NULL, MB_OK); \
		break; \
	}
#else
#define ASSERT(x)  (void)0
#endif


#include <FileBuf.h>

#include "../Zip/Zip.h"


const TCHAR pszApp[] = _T("Muzip");


static HINSTANCE s_hInst;

static SFXFOOTER s_sfx;
static CFileRead s_fr;   // Fichier d'entre (contenant l'archive)
static CFileRead s_fw;   // Fichier de sortie
static bool      s_fOutOK;


static void Decompresser();
static bool UnZip(LPCTSTR pszFichier, LPCTSTR pszFichierOut);
static bool DossierCreer(LPCTSTR pszDossier);

static BOOL CALLBACK prcMain  (HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
static BOOL CALLBACK prcMdp   (HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
static int  CALLBACK prcBrowse(HWND hWnd, UINT uMsg, LPARAM lParam, LPARAM lpData);


void WINAPI WinMainEntry();

// Optimisations diverses en mode Release
// Code spcifique  VC++
// On n'a pas besoin des arguments de WinMain, et la lib Microsoft
// fait un tas de choses dont on n'a pas besoin mais qui prennent de la place
// Conclusion: on dfinit notre propre point d'entre
#ifdef _DEBUG
int APIENTRY WinMain(HINSTANCE, HINSTANCE, LPSTR, int)
{
	WinMainEntry();
	return 0;
}
#else
// 1) Gagner un peu de place en fusionnant deux segments
// 2) Dfinir la fonction constituant le point d'entre
#pragma comment(linker,"/merge:.CRT=.data")
#pragma comment(linker,"/entry:WinMainEntry")
#endif


// Dmarrage
#pragma warning (disable: 4102)
void WINAPI WinMainEntry()
{
	s_hInst = GetModuleHandle(NULL);
	s_hHeap = GetProcessHeap();
	
	
#ifdef CREER_SFX
	// Ouvrir le fichier
	
	if (!s_fr.Open(_T("Muzip.zip"), FILE_SHARE_READ))  return;
	
	{
		// Copier l'EXE
		CFileRead fr;
		if (!fr.Open(_T("G:\\Programmes\\Muzip\\MuzipSfx\\Release\\MuzipSfx.exe"), FILE_SHARE_READ))  return;
		
		CFileWrite fw;
		fw.Open(_T("Auto.exe"));
		
		s_sfx.id       = idSFXFooter;
		s_sfx.dwOff    = GetFileSize(  fr, NULL);
		s_sfx.dwLen    = GetFileSize(s_fr, NULL);
		lstrcpy(s_sfx.pszDest, _T("C:\\Windows\\Bureau\\Essai"));
		s_sfx.fEcraser = false;
		
		fw.WriteFrom(fr, GetFileSize(fr, NULL));
		fr.Close();
		
		// Copier l'archive
		fw.WriteFrom(s_fr, GetFileSize(s_fr, NULL));
		s_fr.Close();
		
		// Copier l'header
		fw.Write(&s_sfx, sizeof(s_sfx));
		fw.Close();
	}
	return;
	
#endif
	
	
#ifdef TESTER_EXTERIEUR
	
	// Copier l'EXE
	if (!s_fr.Open(_T("Muzip.zip"), FILE_SHARE_READ))  return;
	
	s_sfx.id       = idSFXFooter;
	s_sfx.dwOff    = 0;
	s_sfx.dwLen    = GetFileSize(s_fr, NULL);
	lstrcpy(s_sfx.pszDest, _T("C:\\Windows\\Bureau\\Essai"));
	s_sfx.fEcraser = false;
	
#else
	
	// Ouvrir l'auto-extractible
	TCHAR pszEXE[MAX_PATH];
	GetModuleFileName(NULL, pszEXE, nbArray(pszEXE));
	if (!s_fr.Open(pszEXE, FILE_SHARE_READ))  goto Fin;
	
	// Rcuprer l'en-tte
	if (!s_fr.Seek(-(long)sizeof(s_sfx), FILE_END))  goto Fin;
	if (!s_fr.Read(&s_sfx, sizeof(s_sfx)))  goto Fin;
	
#endif
	
	// Vrifier l'ID
	if (s_sfx.id!=idSFXFooter)  return;
	
	e_pfr = &s_fr;
	
	// Le dossier d'extraction par dfaut est le dossier temporaire
	// si aucun chemin d'accs n'est prcis
	if (!*s_sfx.pszDest) {
		GetTempPath(nbArray(s_sfx.pszDest), s_sfx.pszDest);
		PathRemoveBackslash(s_sfx.pszDest);
	}
	
	// Afficher la bote de dialogue
	if (DialogBox(s_hInst, MAKEINTRESOURCE(IDD_MAIN), NULL, prcMain)==IDOK) {
		RemplirCRC();
		InflateInit();
		Decompresser();
		InflateClose();
	}
	
Fin:
	s_fr.Close();
}
#pragma warning (default: 4102)


// Afficher une bote de message
int AfxMessageBox(UINT nID, UINT nType, UINT)
{
	return AfxMessageBox(LoadStaticString(nID), nType);
}
int AfxMessageBox(LPCTSTR pszText, UINT nType, UINT)
{
	if (nType==MB_OK)  nType = MB_ICONEXCLAMATION;
	return MessageBox(GetActiveWindow(), pszText, pszApp, nType);
}


// Formatter dans e_psz
void Formatter(UINT nIDFormat, ...)
{
	// Effectuer le formattage
	va_list val;
	va_start(val, nIDFormat);
	wvsprintf(e_psz, LoadStaticString(nIDFormat), val);
	va_end(val);
}


// Demander le mot de passe
bool DemanderMDP()
{
	return DialogBox(s_hInst, MAKEINTRESOURCE(IDD_MDP), NULL, prcMdp)==IDOK;
}



// Procdure de la bote de dialogue principale
BOOL CALLBACK prcMain(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM)
{
	switch (uMsg) {
		
		// Initialisation
		case WM_INITDIALOG:
			// Optimisation TRS subtile: toutes les fonctions importes de l'API
			// de Windows ont leur nom marqu quelque part dans l'excutable.
			// Or utiliser SetDlgItemText() et CheckDlgButton() demanderait la
			// prsence de DEUX fonctions, alors qu'une seule suffit:
			// SendDlgItemMessage(). Cela gagne de la place et permet  peu
			// de choses prs de gagner de prcieux ko  cause de l'alignement
			// de l'EXE sur 4ko.
			SendDlgItemMessage(hDlg, IDCTXT_DEST,    WM_SETTEXT, 0, (LPARAM)s_sfx.pszDest);
			SendDlgItemMessage(hDlg, IDCCHK_ECRASER, BM_SETCHECK, s_sfx.fEcraser,0);
			// SetDlgItemText(hDlg, IDCTXT_DEST,    s_sfx.pszDest);
			// CheckDlgButton(hDlg, IDCCHK_ECRASER, s_sfx.fEcraser);
			return TRUE;
		
		// Commande
		case WM_COMMAND:
			switch (LOWORD(wParam)) {
				
				// OK
				case IDOK:
					// GetDlgItemText(hDlg, IDCTXT_DEST, s_sfx.pszDest, nbArray(s_sfx.pszDest));
					// s_sfx.fEcraser = ToBool(IsDlgButtonChecked(hDlg, IDCCHK_ECRASER));
					SendDlgItemMessage(hDlg, IDCTXT_DEST, WM_GETTEXT, nbArray(s_sfx.pszDest), (LPARAM)s_sfx.pszDest);
					s_sfx.fEcraser = ToBool(SendDlgItemMessage(hDlg, IDCCHK_ECRASER, BM_GETCHECK, 0,0));
				
				// Annuler
				case IDCANCEL:
					EndDialog(hDlg, LOWORD(wParam));
					break;
				
				// Parcourir
				case IDCCMD_PARCOURIR:
					{
						// Rcuprer le chemin courant
						// GetDlgItemText(hDlg, IDCTXT_DEST, s_sfx.pszDest, nbArray(s_sfx.pszDest));
						SendDlgItemMessage(hDlg, IDCTXT_DEST, WM_GETTEXT, nbArray(s_sfx.pszDest), (LPARAM)s_sfx.pszDest);
						
						// Afficher la bote de dialogue de slection de dossier
						BROWSEINFO bi;
						bi.hwndOwner      = hDlg;
						bi.pidlRoot       = NULL;
						bi.pszDisplayName = NULL;
						bi.lpszTitle      = LoadStaticString(IDS_PARCOURIR);
						bi.ulFlags        = BIF_RETURNONLYFSDIRS;
						bi.lpfn           = prcBrowse;
						const LPITEMIDLIST pidl = SHBrowseForFolder(&bi);
						if (pidl) {
							
							// Rcuprer le nouveau dossier et l'afficher
							SHGetPathFromIDList(pidl, s_sfx.pszDest);
							SetDlgItemText(hDlg, IDCTXT_DEST, s_sfx.pszDest);
							
							// Librer le PIDL
							IMalloc *pMalloc;
							if (SUCCEEDED(SHGetMalloc(&pMalloc))) {
								pMalloc->Free(pidl);
								pMalloc->Release();
							}
						}
					}
					break;
				
				// Lancer Muzip
				case IDCCMD_OUVRIR:
					ShellExecute(hDlg, NULL, _T("Muzip.exe"), NULL, NULL, SW_SHOWDEFAULT);
					break;
			}
			break;
	}
	
	return FALSE;
}


// Procdure de la bote de dialogue de demande du mot de passe
BOOL CALLBACK prcMdp(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM)
{
	switch (uMsg) {
		
		// Initialisation
		case WM_INITDIALOG:
			//SetDlgItemText(hDlg, IDCTXT, e_pszMdp,nbArray(e_pszMdp));
			return TRUE;
		
		// Commande
		case WM_COMMAND:
			switch (LOWORD(wParam)) {
				
				// OK
				case IDOK:
					// GetDlgItemText(hDlg, IDCTXT, e_pszMdp,nbArray(e_pszMdp));
					SendDlgItemMessage(hDlg, IDCTXT, WM_GETTEXT, nbArray(e_pszMdp), (LPARAM)e_pszMdp);
				
				// Annuler
				case IDCANCEL:
					EndDialog(hDlg, LOWORD(wParam));
					break;
			}
			break;
	}
	
	return FALSE;
}


// Hook pour SHBrowseForFolder()
int CALLBACK prcBrowse(HWND hWnd, UINT uMsg, LPARAM, LPARAM)
{
	if (uMsg==BFFM_INITIALIZED)
		// Dfinir le dossier initial
		PostMessage(hWnd, BFFM_SETSELECTION, TRUE, (LPARAM)s_sfx.pszDest);
	
	return 0;
}



// Crer un dossier
bool DossierCreer(LPCTSTR pszDossier)
{
	// Prparer un ventuel message d'erreur
	Formatter(ERR_DOSSIERCREER, pszDossier);
	
	// Rcuprer le chemin d'accs complet
	TCHAR pszTemp[MAX_PATH];
	GetFullPathName(pszDossier, nbArray(pszTemp), pszTemp, NULL);
	while (!GetFullPathName(pszDossier, nbArray(pszTemp), pszTemp, NULL)) {
		if (AfxMessageBox(e_psz, MB_RETRYCANCEL | MB_ICONEXCLAMATION)==IDCANCEL)
			return false;
	}
	
	TCHAR *pc = PathSkipRoot(pszTemp);
	VERIF(*pc);
	
	for (;;) {
		
		// Passer  la partie suivante
		bool fFin = (*pc++==0);
		if (!fFin) {
			while (*pc && *pc!='\\')  pc++;
			fFin = (*pc==0);
			*pc = 0;
		}
		
		// Crer la partie
		while (!CreateDirectory(pszTemp, NULL)) {
			if (GetLastError()==ERROR_ALREADY_EXISTS)  break;
			if (AfxMessageBox(e_psz, MB_RETRYCANCEL | MB_ICONEXCLAMATION)==IDCANCEL)
				return false;
		}
		
		if (fFin)  return true;
		*pc = '\\';
	}
}


// Dcompresser l'archive dans le dossier s_sfx.pszDest
void Decompresser()
{
	DWORD m_offDebut = 0;
	
	BYTE *const abBufTemp = new BYTE[4096+4];
	BYTE *pb = &abBufTemp[4096];
	pb[1] = 0;
	pb[2] = 0;
	pb[3] = 0;
	
	bool fOK;
	
	// Rechercher la signature finale  partir de la fin,
	// en lisant des tranches de 4ko  la fois
	// On doit en effet ignorer ce qu'il y a aprs la fin des structures Zip,
	// et ne pas prsumer que les structures Zip se terminent avec le fichier
	//  la fin du processus, m_offDebut contiendra la position de l'en-tte final
	// (contradictoire avec le nom de la variable, mais tant pis!)
	{
		size_t len;   // Utilis plus tard, mais les goto demandent une dclaration avant
		s_fr.SetVerbeux(false);
		fOK = s_fr.Seek(-4096-(long)sizeof(SFXFOOTER), FILE_END);
		s_fr.SetVerbeux();
		if (fOK) {
			m_offDebut = s_fr.Tell();
			for (;;) {
				abBufTemp[4096] = pb[1];
				abBufTemp[4097] = pb[2];
				abBufTemp[4098] = pb[3];
				
				s_fr.Read(abBufTemp, 4096);
				s_fr.Seek(-2*4096, FILE_CURRENT);
				pb = &abBufTemp[4095];
				
				do{
					// Rechercher la signature finale
					if (*(DWORD*)pb==sigFin)  goto FinTrouveOK;
				}while (--pb>=abBufTemp);
				if (m_offDebut<4096)  break;
				m_offDebut -= 4096;
			}
			if (!m_offDebut)  return;
			s_fr.Seek(0, FILE_BEGIN);
			len = s_fr.ReadLen(abBufTemp, m_offDebut);
			
		}else{
			// Lire le maximum d'octets (4ko - 1 octet)
			len = s_fr.ReadLen(abBufTemp, 4096-1);
			m_offDebut = 0;
		}
		
		// Rechercher la signature finale dans les tous-premiers octets,
		// car soit le fichier faisait moins de 4ko,
		// soit la signature se trouve dans les 4095 premiers octets (= 4ko - 1octet)
		
		abBufTemp[len  ] = pb[1];
		abBufTemp[len+1] = pb[2];
		abBufTemp[len+2] = pb[3];
		pb = &abBufTemp[len-1];
		do{
			if (*(DWORD*)pb==sigFin)  goto FinTrouveOK;
		}while (--pb>=abBufTemp);
		
		// Erreur: Signature finale non-trouve
		delete [] abBufTemp;
		return;
		
FinTrouveOK:  // Fin du central trouve
		delete [] abBufTemp;
	}
	
	// Signature finale trouve
	// Calculer la position de la fin du central, et y aller
	m_offDebut += (DWORD)(pb - abBufTemp);
	s_fr.Seek(m_offDebut+lenSig, FILE_BEGIN);
	
	// Lire l'en-tte final
	if (!s_fr.Read(e_abBuf, sizeof(HEADERFIN)))  return;
	
	WORD  len;
	LPSTR psz;
	
	
	// Lire les commentaires
	len = phFin->lenCom;
	if (len) {
		psz = new char[len];
		if (!s_fr.Read(psz, len)) {
			delete [] psz;
			return;
		}
		delete [] psz;
	}
	
	// On prsume que le central prcde immdiatement l'en-tte final
	DWORD m_offCen = m_offDebut - phFin->nTailleT;
	if (!s_fr.Seek(m_offCen, FILE_BEGIN))  return;
	
	// Lire le central
	// La partie des en-ttes centraux qui doit tre commune
	// avec les en-ttes locaux est lue brute, pour une comparaison future
	// (cela demande que l'offset de m_pbExtra dans CZipFichier soit suprieur
	// ou gal  LOCHEAD)
	DWORD dwReste = s_sfx.dwLen - (m_offCen-s_sfx.dwOff);
	while (dwReste>=sizeof(HEADERCENSIG)) {
		
		// Lire la signature: ce doit tre la signature des en-ttes centraux
		if (!s_fr.Read(e_abBuf, sizeof(HEADERCENSIG)))  return;
		if (phCenSig->sig!=sigCen)  break;
		
		TCHAR pszFichier[MAX_PATH], pszFichierOut[MAX_PATH];
		
		// Lire le nom du fichier
		len = phCenSig->lenNom;
		if (!len)  return;
		psz = new char[len+1];
		if (!s_fr.Read(psz, len)) {
			delete [] psz;
			return;
		}
		psz[len] = 0;
		OemToChar(psz, pszFichier);
		delete [] psz;
		
		// Convertir les '/' en '\'
		TCHAR *pc = pszFichier;
		while (*pc) {
			if (*pc=='/')  *pc = '\\';
			pc++;
		}
		// Composer le nom du fichier cible
		// et dcompresser le fichier
		PathCombine(pszFichierOut, s_sfx.pszDest, pszFichier);
		const DWORD pos = s_fr.Tell();
		if (!UnZip(pszFichier, pszFichierOut))  return;
		if (!s_fr.Seek((long)pos, FILE_BEGIN))  return;
		dwReste = s_sfx.dwLen - (pos-s_sfx.dwOff);
		
		// Sauter le champ extra et les commentaires
		if (!s_fr.Seek((long)(phCenSig->lenExtraC+phCenSig->lenCom), FILE_CURRENT))
			return;
	}
}


// Dcompresser un fichier
// Retourner FALSE s'il faut tout arrter (et non pas si le fichier
// n'a pas pu tre dcompress)
bool UnZip(LPCTSTR pszFichier, LPCTSTR pszFichierOut)
{
	// Regarder si le fichier existe dj
	if (!s_sfx.fEcraser && PathFileExists(pszFichierOut)) {
		// Confirmer l'crasement
		
		Formatter(DEM_ECRASER, pszFichierOut);
		switch (AfxMessageBox(e_psz, MB_YESNOCANCEL | MB_ICONQUESTION)) {
			case IDYES:     break;
			case IDNO:      return true;
			case IDCANCEL:  return false;
		}
	}
	
	// Crer le dossier du fichier
	TCHAR pszDossier[MAX_PATH];
	lstrcpy(pszDossier, pszFichierOut);
	PathRemoveFileSpec(pszDossier);
	if (DossierCreer(pszDossier)) {
		phCenSig->off += s_sfx.dwOff;
		// Extraire le fichier
		if (!ExtraireZip(pszFichier, pszFichierOut))
			// Supprimer le fichier en cas d'chec
			DeleteFile(pszFichierOut);
	}
	return true;
}




// Implmenter une version minimaliste de CFileRead/Write
// gagne quelques ko sur l'excutable


DWORD CFileBuf::s_dwES;


const DWORD SEEK_ERR = 0xFFFFFFFF;


//*** CFileBuf

// Fermer le fichier
void CFileBuf::Close()
{
	CloseHandle(m_hf);
	m_hf = INVALID_HANDLE_VALUE;
}

// Fermer le fichier en catastrophe
void CFileBuf::CloseErreur()
{
	if (!m_fPeutFermer)  return;
	CloseHandle(m_hf);
	m_hf = INVALID_HANDLE_VALUE;
}



// Afficher la dernire erreur
int CFileBuf::Erreur(UINT nID, UINT nFlags)
{
	// Rcuprer le message d'erreur de Windows
	LPTSTR pszMessage;
	FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
		NULL, GetLastError(), MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
		(LPTSTR)&pszMessage, 0, NULL);
	
	// Formatter et afficher
	Formatter(nID, m_pszFichier, pszMessage);
	LocalFree(pszMessage);
	return AfxMessageBox(e_psz, nFlags | MB_ICONEXCLAMATION);
}


//*** CFileRead

// Ouvrir le fichier
bool CFileRead::Open(LPCTSTR pszFichier,
	DWORD dwShare, DWORD dwCreate, DWORD dwFlags)
{
#ifdef ASSERT
	ASSERT(m_fAttach || m_hf==INVALID_HANDLE_VALUE);
#endif
	
	lstrcpy(m_pszFichier, pszFichier);
	m_len     =
	m_posCur  = 0;
	m_fEOF    = false;
#ifdef _DEBUG
	m_fAttach = false;
#endif
	
	for (;;) {
		m_hf = CreateFile(pszFichier, GENERIC_READ,
			dwShare, NULL, dwCreate, dwFlags, NULL);
		if (m_hf!=INVALID_HANDLE_VALUE)  return true;
		VERIF(m_fVerbeux && Erreur(ERR_OPEN, MB_RETRYCANCEL)==IDRETRY);
	}
}


// Lire des octets dans le fichier
bool CFileRead::Read(void* pBuf, DWORD len)
{
#ifdef ASSERT
	ASSERT(m_hf!=INVALID_HANDLE_VALUE);
#endif
	
	return ReadProtect((BYTE*)pBuf, len,len);
}


// Lire des octets dans le fichier
// et retourner le nombre d'octets lus
DWORD CFileRead::ReadLen(void* pBuf, DWORD len)
{
#ifdef ASSERT
	ASSERT(m_hf!=INVALID_HANDLE_VALUE);
#endif
	
	ReadProtect((BYTE*)pBuf, len,0);
	return s_dwES;
}


// Lire un octet dans le fichier
bool CFileRead::Read(BYTE& b)
{
	return Read((BYTE*)&b, sizeof(b));
}

// Lire un mot dans le fichier
bool CFileRead::Read(WORD& w)
{
	return Read((BYTE*)&w, sizeof(w));
}


// Rcuprer la position courante
DWORD CFileRead::Tell() const
{
	return SetFilePointer(m_hf, 0,NULL, FILE_CURRENT);
}

// Dplacer le pointeur de lecture
bool CFileRead::Seek(long lMove, DWORD dwMethode)
{
#ifdef ASSERT
	ASSERT(m_hf!=INVALID_HANDLE_VALUE);
#endif
	
	while (SetFilePointer(m_hf, lMove,NULL, dwMethode)==SEEK_ERR) {
		VERIF(m_fVerbeux);
		if (Erreur(ERR_READ, MB_RETRYCANCEL)!=IDRETRY) {
			CloseErreur();
			return false;
		}
	}
	return true;
}

// Lecture protge: gestion des erreurs, affichage d'une bote de dialogue
bool CFileRead::ReadProtect(BYTE* pbBuf, DWORD len, DWORD lenFaut)
{
	if (!len)  return true;
	
	DWORD dwES;
	s_dwES = 0;
	
	for (;;) {
		
		// Lire
		const BOOL bOK = ReadFile(m_hf, pbBuf, len-s_dwES, &dwES, NULL);
		pbBuf  += dwES;
		s_dwES += dwES;
		
		if (s_dwES==len)  return true;   // OK, tout termin
		
		if (bOK) {
			// Fin du fichier atteinte
			if (s_dwES>=lenFaut)  return true;   // On a lu le minimum requis
			
			// Afficher le message d'erreur
			m_fEOF = true;
			if (m_fVerbeux) {
				SetLastError(ERROR_HANDLE_EOF);
				Erreur(ERR_READ, MB_OK);
				CloseErreur();
			}
			return false;
		}
		
		// Demander s'il faut recommencer
		VERIF(m_fVerbeux);
		if (Erreur(ERR_READ, MB_RETRYCANCEL)!=IDRETRY) {
			CloseErreur();
			return false;
		}
	}
}



//*** CFileWrite

// Ouvrir le fichier
bool CFileWrite::Open(LPCTSTR pszFichier,
	DWORD dwShare, DWORD dwCreate, DWORD dwFlags)
{
#ifdef ASSERT
	ASSERT(m_fAttach || m_hf==INVALID_HANDLE_VALUE);
#endif
	
	lstrcpy(m_pszFichier, pszFichier);
	m_len     = 0;
#ifdef _DEBUG
	m_fAttach = false;
#endif
	
	for (;;) {
		m_hf = CreateFile(pszFichier, GENERIC_WRITE,
			dwShare, NULL, dwCreate, dwFlags, NULL);
		if (m_hf!=INVALID_HANDLE_VALUE)  return true;
		VERIF(m_fVerbeux && Erreur(ERR_OPEN, MB_RETRYCANCEL)==IDRETRY);
	}
}

// Fermer le fichier
void CFileWrite::Close()
{
	if (m_hf!=INVALID_HANDLE_VALUE) {
		CloseHandle(m_hf);
		m_hf = INVALID_HANDLE_VALUE;
	}
}


// crire des octets dans le fichier
bool CFileWrite::Write(const void* pBuf, DWORD len)
{
#ifdef ASSERT
	ASSERT(m_hf!=INVALID_HANDLE_VALUE);
#endif
	
	return WriteProtect((const BYTE*)pBuf, len);
}


// Dplacer le pointeur de lecture
bool CFileWrite::Seek(long lMove, DWORD dwMethode)
{
#ifdef ASSERT
	ASSERT(m_hf!=INVALID_HANDLE_VALUE);
#endif
	
	while (SetFilePointer(m_hf, lMove,NULL, dwMethode)==SEEK_ERR) {
		VERIF(m_fVerbeux);
		if (Erreur(ERR_WRITE, MB_RETRYCANCEL)!=IDRETRY) {
			CloseErreur();
			return false;
		}
	}
	return true;
}


// crit des octets provenant d'un fichier en lecture
#ifdef _DEBUG
bool CFileWrite::WriteFrom(CFileRead& fr, DWORD len)
{
	if (!len)  return true;
	
	while (len>0) {
		DWORD lenES = (len<lenFileBuf) ? len : lenFileBuf;
		if (!fr.ReadProtect(m_abBuf, lenES,lenES)) {
			CloseErreur();
			return false;
		}
		if (!WriteProtect(m_abBuf, lenES)) {
			fr.CloseErreur();
			return false;
		}
		len -= lenES;
	}
	
	return true;
}
#endif


// criture protge: gestion des erreurs, affichage d'une bote de dialogue
bool CFileWrite::WriteProtect(const BYTE* pbBuf, DWORD len)
{
	DWORD dwES;
	s_dwES = 0;
	for (;;) {
		
		// crire
		WriteFile(m_hf, pbBuf, len-s_dwES, &dwES, NULL);
		pbBuf  +=   dwES;
		s_dwES +=   dwES;
		
		if (s_dwES==len)  return true;   // OK, tout termin
		
		// Demander s'il faut recommencer
		VERIF(m_fVerbeux);
		if (Erreur(ERR_READ, MB_RETRYCANCEL)!=IDRETRY) {
			CloseErreur();
			return false;
		}
	}
}
