// Options.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "Muzip.h"
#include "Options.h"

#include "IOFichier.h"

#include "MuzipDoc.h"
#include "MuzipView.h"
#include "ApercuView.h"

#include "Reg.h"
const LPCTSTR eszGeneralOptions = _T("Options");


COptions e_opt;


struct OPTIONS
{
	#define dclValeur(type, nom, defaut)  type m_##nom;
	ManipulerValeur(dclValeur)
	#undef  dclValeur
	
	#define dclChaine(nom, defaut)        int m_len##nom;
	ManipulerChaine(dclChaine)
	#undef  dclChaine
};


// Charger les options
void COptions::Load()
{
	UINT len;
	BYTE *pb;
	
	// Options gnrales
	if (oApp.GetProfileBinary(kszGeneral, eszGeneralOptions, &pb, &len)) {
		OPTIONS &opt = *(OPTIONS*)pb;
		#define _opt  opt   // Demand par la macro TotalLen
		if (len>=sizeof(OPTIONS) &&
		    len==sizeof(OPTIONS)+TotalLen) {
		#undef _opt
			
			// Charger les valeurs
			#define ChargerOption(type, nom, defaut)   m_##nom = opt.m_##nom;
			ManipulerValeur(ChargerOption)
			#undef ChargerOption
			
			// Charger les chanes
			TCHAR *pc = (TCHAR*)&opt + sizeof(OPTIONS);
			#define ChargerChaine(nom, defaut) \
				memcpy(m_s##nom.GetBuffer(opt.m_len##nom), pc, opt.m_len##nom); \
				m_s##nom.ReleaseBuffer(opt.m_len##nom); \
				pc += opt.m_len##nom;
			ManipulerChaine(ChargerChaine)
			#undef ChargerChaine
			
		}else{
			// Taille incorrecte
			delete [] pb;
			pb = NULL;
		}
	}
	if (pb)
		delete [] pb;
	else{
		// Options par dfaut
		
		#define SetMembresDefaut(type, nom, defaut)  m_##nom = defaut;
		ManipulerNomPage(SetMembresDefaut)
		#undef SetMembresDefaut
	}
	MAJAffichageStyleEx();
	MAJPasCompExt();
}


// Sauver les options
void COptions::Save()
{
	// Rcuprer les longueurs des chanes
	int len = sizeof(OPTIONS);
	#define GetLenChaine(nom, defaut) \
		const int len##nom = m_s##nom.GetLength(); \
		len += len##nom;
	ManipulerChaine(GetLenChaine)
	#undef GetLenChaine
	
	OPTIONS *const popt = (OPTIONS*)new BYTE[len];
	
	// Copier les valeurs
	#define CopierValeur(type, nom, defaut)   popt->m_##nom = m_##nom;
	ManipulerValeur(CopierValeur)
	#undef CopierValeur
	
	// Copier les chanes
	TCHAR *pc = (TCHAR*)popt + sizeof(OPTIONS);
	#define CopierChaine(nom, defaut) \
		popt->m_len##nom = len##nom; \
		memcpy(pc, m_s##nom, len##nom); \
		pc += len##nom;
	ManipulerChaine(CopierChaine)
	#undef CopierChaine
	
	// Enregistrer le tout
	oApp.WriteProfileBinary(kszGeneral, eszGeneralOptions, (BYTE*)popt, len);
	
	delete [] popt;
}


// Appliquer les options
void COptions::Appliquer()
{
	// Mettre  jour le style des listes
	// et les aperus
	POSITION pos = oApp.GetFirstDocTemplatePosition();
	while (pos) {
		CDocTemplate *const pTemplate = oApp.GetNextDocTemplate(pos);
		if (pTemplate==(CDocTemplate*)oApp.s_pmdtScript)  continue;
		POSITION pos2 = pTemplate->GetFirstDocPosition();
		while (pos2) {
			CMuzipDoc *const pDoc = (CMuzipDoc*)pTemplate->GetNextDoc(pos2);
			pDoc->m_plst->m_plst->SetExtendedStyle(m_nAffichageStyleEx);
			pDoc->m_plst->OnUpdate(NULL,NULL,NULL);
			if (pDoc->m_papr && pDoc->m_papr->m_pApercu)
				pDoc->m_papr->m_pApercu->MAJ();
		}
	}
	
	// Recrer le thread de surveillance gnrale
	// car il utilise GetDossierTemp()
	PulseEvent(CSurvDecomp::s_hEvent);
	DWORD idThread;
	CloseHandle(::CreateThread(NULL, 0, CSurvDecomp::prcThreadTous, NULL, 0, &idThread));
}

// Mettre  jour le style des listes d'aprs les paramtres
void COptions::MAJAffichageStyleEx()
{
	int afs  = afsClicDeux;
	int afss = afssJamais;
	
	SHELLFLAGSTATE sfs;
	sfs.fWin95Classic = TRUE;
	
	// Excuter SHGetSettings
	const HINSTANCE hShell32 = LoadLibrary(_T("Shell32.dll"));
	if (hShell32) {
		typedef void (WINAPI *PFNSHGETSETTINGSPROC)(SHELLFLAGSTATE* psfs, DWORD dwMask);
		const PFNSHGETSETTINGSPROC pfnSHGetSettings =
			(PFNSHGETSETTINGSPROC)GetProcAddress(hShell32, "SHGetSettings");
		if (pfnSHGetSettings)
			pfnSHGetSettings(&sfs, SSF_DOUBLECLICKINWEBVIEW | SSF_WIN95CLASSIC);
		FreeLibrary(hShell32);
	}
	
	if (!sfs.fWin95Classic && !sfs.fDoubleClickInWebView) {
		// Pas de clic pour slectionner
		// Rcuprer la valeur IconUnderline, premier octet:
		// 03 - toujours soulign (comme navigateur)
		// 02 - soulign sous pointeur
		
		afs  = afsClicUn;
		afss = afssPointe;
		
		HKEY hKey;
		if (RegOpenKeyEx(HKEY_CURRENT_USER, _T("Software\\Microsoft\\Windows\\CurrentVersion\\Explorer"), 0, KEY_ALL_ACCESS, &hKey)==ERROR_SUCCESS) {
			DWORD len;
			if (RegQueryValueEx(hKey, _T("IconUnderline"), 0, NULL, NULL, &len)==ERROR_SUCCESS && len>=1) {
				// La valeur existe, on a rcuprer sa longueur
				BYTE *const pb = new BYTE[len];
				if (RegQueryValueEx(hKey, _T("IconUnderline"), 0, NULL, pb, &len)==ERROR_SUCCESS &&
				    pb[0]==0x03)
					afss = afssToujours;
				delete [] pb;
			}
			RegCloseKey(hKey);
		}
	}
	
	// craser les options de Windows si ncessaire
	if (m_nAffichageSouris!=afsWindows) {
		afs = m_nAffichageSouris;
		if (m_nAffichageSouris==afsClicUn)
			// Simple clic
			afss = m_nAffichageSouris1;
	}
	
	// Option par dfaut pour le simple clic
	if (m_nAffichageSouris!=afsClicUn)
		m_nAffichageSouris1 = afss;
	
	if (afs==afsClicUn) {
		// Simple clic
		
		if      (afss==afssJamais)
			m_nAffichageStyleEx = LVS_EX_ONECLICKACTIVATE | LVS_EX_TRACKSELECT;
		else if (afss==afssPointe)
			m_nAffichageStyleEx = LVS_EX_ONECLICKACTIVATE | LVS_EX_TRACKSELECT | LVS_EX_ONECLICKACTIVATE | LVS_EX_UNDERLINEHOT;
		else // (afss==afssToujours)
			m_nAffichageStyleEx = LVS_EX_ONECLICKACTIVATE | LVS_EX_TRACKSELECT | LVS_EX_ONECLICKACTIVATE | LVS_EX_UNDERLINECOLD;
		
	}else // (afs==afsClicDeux)
		// Double clic
		m_nAffichageStyleEx = 0;
	
	if (m_fAffichageGrille)    m_nAffichageStyleEx |= LVS_EX_GRIDLINES;
	if (m_fAffichageSelLigne)  m_nAffichageStyleEx |= LVS_EX_FULLROWSELECT;
}

// Mettre  jour la liste des extensions  ne pas compresser
void COptions::MAJPasCompExt()
{
	m_lstPasCompExt.DeleteAll();
	
	const TCHAR *pc = m_sCompPasCompExt;
	for (;;) {
		// Rcuprer le sparateur suivant (';' ou fin de chane)
		const TCHAR *pcSep = _tcschr(pc+1, ';');
		if (!pcSep)  pcSep = pc+lstrlen(pc);
		
		// Ajouter l'extension  la liste si elle n'est pas vide
		if (pcSep!=pc) {
			const LPTSTR pszExt = new TCHAR[pcSep-pc+1];
			lstrcpyn(pszExt, pc, pcSep-pc);
			pszExt[pcSep-pc] = 0;
			CharLower(pszExt);
			m_lstPasCompExt.AddFin(pszExt);
		}
		
		// Passer  l'extension suivante
		if (!*pcSep)  break;
		pc = pcSep+1;
	}
}

// Rcuprer le nom du dossier temporaire
bool COptions::GetDossierTemp(LPTSTR pszDossier) const
{
	if (m_sDossiersTemp==opdtAuto) {
		GetTempPath(MAX_PATH, pszDossier);
		lstrcat(pszDossier, _T("Muzip"));
	}else
		lstrcpy(pszDossier, m_sDossiersTemp);
	
	return DossierCreer(pszDossier);
}

// Rcuprer le nom du dossier temporaire de glisser-dplacer
// et vider le contenu du dossier
bool COptions::GetDossierDrag(LPTSTR pszDossier) const
{
	if (m_sDossiersDrag==opdtAuto) {
		GetTempPath(MAX_PATH, pszDossier);
		lstrcat(pszDossier, _T("MuzipDrag"));
	}else
		lstrcpy(pszDossier, m_sDossiersDrag);
	
	// Dtruire et recrer le dossier
	DossierSupprimer(pszDossier);
	return DossierCreer(pszDossier);
}
