// Recherche.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "Muzip.h"
#include "Recherche.h"

#include "Dossier.h"
#include "Fichier.h"
#include "PlusItem.h"
#include "FichierPlus.h"
#include "MuzipArc.h"

#include "Exceptions.h"

#include <GUnicode.h>


// Constructeur
CRecherche::CRecherche(CMuzipArc* pArc, CArchivePlus& arPlus)
	: m_pArc(pArc), m_arPlus(arPlus), m_pfpDeb(NULL)
{
	m_pParent = m_pArc->GetDossier(arPlus.m_sDossier);
}


// Commencer une recherche
// Retourner TRUE pour annuler la recherche
bool CRecherche::Rechercher(CPlusItem* ppi)
{
	m_ppi = ppi;
	
	// Composer les masques d'attributs
	m_mskAttrO =
	m_mskAttrN = 0;
	if      (m_ppi->AT_nR==0)  m_mskAttrN |= FILE_ATTRIBUTE_READONLY;
	else if (m_ppi->AT_nR==1)  m_mskAttrO |= FILE_ATTRIBUTE_READONLY;
	if      (m_ppi->AT_nH==0)  m_mskAttrN |= FILE_ATTRIBUTE_HIDDEN;
	else if (m_ppi->AT_nH==1)  m_mskAttrO |= FILE_ATTRIBUTE_HIDDEN;
	if      (m_ppi->AT_nS==0)  m_mskAttrN |= FILE_ATTRIBUTE_SYSTEM;
	else if (m_ppi->AT_nS==1)  m_mskAttrO |= FILE_ATTRIBUTE_SYSTEM;
	if      (m_ppi->AT_nA==0)  m_mskAttrN |= FILE_ATTRIBUTE_ARCHIVE;
	else if (m_ppi->AT_nA==1)  m_mskAttrO |= FILE_ATTRIBUTE_ARCHIVE;
	m_mskAttrOPasSup = (m_ppi->CO_fArchiveQue)
		? FILE_ATTRIBUTE_ARCHIVE : 0;
	
	bool fOK = true;
	
	// Effectuer chaque recherche, chacune tant spare
	// de la suivante par un point-virgule ';'
	TCHAR pszFiltre[MAX_PATH*64];
	lstrcpy(pszFiltre, ppi->DO_sFiltre);
	TCHAR *pcDeb = pszFiltre;
	TCHAR pszPartie[MAX_PATH];
	TCHAR pszPath  [MAX_PATH];
	for (;;) {
		TCHAR *const pcSep = _tcschr(pcDeb, ';');
		if (pcSep)  *pcSep = 0;
		lstrcpy(pszPartie, pcDeb);
		CDossier *pParent = m_pParent;
		
		// Regarder s'il s'agit d'un fichier,
		// auquel cas pas besoin d'extraire de spcification et autres
		const DWORD dwAttr = GetFileAttributes(pszPartie);
		const bool fVraieRecherche =
			dwAttr==(DWORD)-1 ||
			(dwAttr & FILE_ATTRIBUTE_DIRECTORY);   // TRUE si dossier ou wildcards
		
		if (fVraieRecherche) {
			// Ce n'est pas un fichier seul:
			// il s'agit d'une vraie recherche
			
			// Extraire la spcification
			TCHAR *pcPartieSepFin = _tcsrchr(pszPartie, '\\');
			if (!pcPartieSepFin)  pcPartieSepFin = pszPartie-1;
			lstrcpy(m_pszSpec, ++pcPartieSepFin);
			*pcPartieSepFin = 0;
			
			if (dwAttr!=(DWORD)-1 && (dwAttr & FILE_ATTRIBUTE_DIRECTORY)) {
				// La recherche porte sur un dossier sans spcification
				
				TCHAR pszParent[MAX_PATH];
				const TCHAR *pcIn     = m_pParent->m_pszPath;
				      TCHAR *pcParent = pszParent;
				while (*pcIn)  *pcParent++ = *pcIn++;
				if (pcParent>pszParent && pcParent[-1]!='/')  *pcParent++ = '/';
				lstrcpy(pcParent, m_pszSpec);
				
				// Rajouter le slash final
				while (*pcParent)  pcParent++;
				*pcParent++ = '/';
				*pcParent   = 0;
				pParent = m_pArc->GetDossier(pszParent, false);
				
				// Prendre "*.*" comme spcification
				lstrcpy(pcPartieSepFin, m_pszSpec);
				while (*pcPartieSepFin)  pcPartieSepFin++;
				*pcPartieSepFin++ = '\\';
				*pcPartieSepFin   = 0;
				lstrcpy(m_pszSpec,   _T("*.*"));
			}
		}else
			// Fichier seul: la spcification est le fichier lui-mme
			lstrcpy(m_pszSpec, PathFindFileName(pszPartie));
		
		
		//*** Former le chemin d'accs du dossier de destination
		
		// Copier le nom du dossier parent
		const TCHAR *pcIn  = pParent->m_pszPath;
		      TCHAR *pcOut = pszPath;
		while (*pcIn) {
			*pcOut++ = (*pcIn=='\\') ? '/' : *pcIn;
			pcIn++;
		}
		
		if (ppi->CO_nArbo==arboTout) {
			// Garder le chemin d'accs complet:
			// il faut rajouter le chemin d'accs du fichier
			
			// Ne pas inclure la spcification de lecteur
			pcIn = PathSkipRoot(pszPartie);
			if (pcIn) {
				// Rajouter un premier sparateur
				*pcOut++ = '/';
				// Puis copier le chemin du fichier
				while (*pcIn) {
					*pcOut++ = (*pcIn=='\\') ? '/' : *pcIn;
					pcIn++;
				}
			}
		}
		
		// Enlever les ventuels sparateurs finaux
		while (pcOut>pszPath && pcOut[-1]=='/')  pcOut--;
		
		*pcOut = 0;   // C'est termin !
		
		CDossier *const pParentTemp = m_pParent;
		m_pParent = pParent;
		if (!Rechercher(pszPartie, pszPath, fVraieRecherche)) {
			fOK = false;
			break;
		}
		m_pParent = pParentTemp;
		
		if (!pcSep)  break;
		pcDeb = pcSep+1;
	}
	
	return fOK;
}

// Suite d'une recherche (fonction rcursive)
// Retourner FALSE pour annuler la recherche
bool CRecherche::Rechercher(LPTSTR pszParent, LPTSTR pszPath, bool fVraieRecherche)
{
	// Dossier d'ajout
	// On ne le recherche pas ds le dbut car si aucun fichier
	// ne correspond  la spcificiation, le dossier va tre affich vide
	CDossier *pDossierParent = (m_ppi->CO_nArbo==arboNon) ? m_pParent : NULL;
	
	TCHAR *pcPathFin, *pcParentFin;
	TCHAR pszRecherche[MAX_PATH];
	
	pcPathFin = pszPath + lstrlen(pszPath);
	if (pcPathFin>pszPath && pcPathFin[-1]!='/') {
		*pcPathFin++ = '/';
		*pcPathFin   = 0;
	}
	
	if (fVraieRecherche) {
		// Spcification
		
		// Rajouter un anti-slash  pszParent si ncessaire
		// et rcuprer la position de fin de la chane
		size_t len = lstrlen(pszParent);
		if (len>0 && pszParent[len-1]!='\\') {
			pszParent[len++] = '\\';
			pszParent[len  ] = 0;
		}
		pcParentFin = pszParent+len;
		
		// Lancer la recherche
		lstrcpy(pszRecherche,     pszParent);
		lstrcpy(pszRecherche+len, _T("*.*"));
		
	}else{
		// Fichier seul
		
		pcParentFin = pcPathFin;
		lstrcpy(pszRecherche, pszParent);
	}
	
	WIN32_FIND_DATA wfd;
	const HANDLE hff = ::FindFirstFile(pszRecherche, &wfd);
	if (hff==INVALID_HANDLE_VALUE) {
		CMuzipException(ERR_OPEN, pszRecherche).ReportError();
		return true;
	}
	
	bool fOK = true;
	do{
		if (oApp.UpdateProgressStatus()==prsAnnuler) {
			fOK = false;
			break;
		}
		
		if (wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
			// Rpertoire
			
			if (!fVraieRecherche || !m_ppi->OP_fSous)  continue;
			if ((wfd.dwFileAttributes & FILE_ATTRIBUTE_HIDDEN) &&
			    !m_ppi->OP_fCache)
				continue;
		
			// Ignorer les dossiers . et ..
			if ( wfd.cFileName[0]=='.' &&
			   (!wfd.cFileName[1] ||
			   ( wfd.cFileName[1]=='.' && !wfd.cFileName[2])))
				continue;
			
			// Poursuivre la recherche dans le sous-dossier
			lstrcpy(pcParentFin, wfd.cFileName);
			lstrcpy(pcPathFin,   wfd.cFileName);
			Rechercher(pszParent, pszPath, fVraieRecherche);
			
		}else{
			// Fichier
			
			if (!PathMatchSpec(wfd.cFileName, m_pszSpec))  continue;
			
			// Attributs
			if (m_mskAttrO && !(wfd.dwFileAttributes & m_mskAttrO))  continue;
			if (m_mskAttrN &&  (wfd.dwFileAttributes & m_mskAttrN))  continue;
			
			// Taille
			if (m_ppi->TA_nActive) {
				const DWORD lenAjuste = wfd.nFileSizeLow>>(m_ppi->TA_nUnite*10);
				if       (m_ppi->TA_nOp==0)   {  // ==
					if (m_ppi->TA_uTaille!=lenAjuste)  continue;
				}else if (m_ppi->TA_nOp==1)   {  // >=
					if (m_ppi->TA_uTaille>=lenAjuste)  continue;
				}else /* (m_ppi->TA_nOp==2)*/ {  // <=
					if (m_ppi->TA_uTaille<=lenAjuste)  continue;
				}
			}
			
			// Date
			if (m_ppi->DA_rdC.fActive &&
			   !m_ppi->DA_rdC.Correspond(wfd.ftCreationTime))    continue;
			if (m_ppi->DA_rdM.fActive &&
			   !m_ppi->DA_rdM.Correspond(wfd.ftLastWriteTime))   continue;
			
			// Le fichier correspond au filtre
			// Composer le chemin d'accs
			lstrcpy(pcParentFin, wfd.cFileName);
			
			// Rcuprer le dossier du fichier si ncessaire
			if (!pDossierParent) {
				*pcPathFin = 0;
				pDossierParent = m_pArc->GetDossier(pszPath, false);
			}
			
			// Regarder si le fichier est dj dans la liste
			// des fichiers  ajouter
#if 0   //$$$
			POS pos = m_lstFichierPlus.GetPosDeb();
			while (pos) {
				const POS posSauve = pos;
				CFichierPlus *const pFichierPlus = m_lstFichierPlus.GetSuiv(pos);
				if (!lstrcmpi(pszParent, pFichierPlus->m_pFichier->m_pszNomES)) {
					// Les fichiers sont les mmes: supprimer l'ancien
					delete [] pFichierPlus->m_pFichier->m_pszNomES;
					delete    pFichierPlus->m_pFichier;
					m_lstFichierPlus.DeleteAt(posSauve);
					break;
				}
			}
#else   //$$$
			CFichierPlus **ppfpCur = &m_pfpDeb;
			while (*ppfpCur) {
				CFichierPlus *const pfpCur = *ppfpCur;
				const int iDiff = lstrcmpi(pszParent, pfpCur->m_pFichier->m_pszNomES);
				if (!iDiff)  break;
				ppfpCur = (iDiff<0) ? &pfpCur->m_pfpGauche : &pfpCur->m_pfpDroite;
			}
#endif  //$$$
			
			CFichierPlus *const pfpCur = *ppfpCur;
			
			// Regarder si le fichier est prsent dans l'archive
			CFichier *pFichierOld;
			POS posFichierOld;
			if (pfpCur) {
				// Dj trouv
				pFichierOld   = pfpCur->m_pFichierOld;
				posFichierOld = pfpCur->m_posFichierOld;
			}else{
				CLstFichier &lstFichier = pDossierParent->m_lstFichier;
				POS pos = lstFichier.GetPosDeb();
				do{
					posFichierOld = pos;
					if (!pos) {
						pFichierOld = NULL;
						break;
					}
					
					// Continuer tant que le fichier n'est pas trouv
					pFichierOld = lstFichier.GetSuiv(pos);
				}while (lstrcmpi(wfd.cFileName, pFichierOld->m_pszNom));
			}
			
			// Nouveaut du fichier
			if (pFichierOld && (m_pArc->GetCap() & capdtM) &&
			   pFichierOld->m_dtM >= wfd.ftLastWriteTime)
				continue;
			
			if (m_ppi->CO_nAction==coaMoins) {
				// Exclure de la recherche
Supprimer:
				if (pfpCur) {
					delete [] pfpCur->m_pFichier->m_pszNomES;
					delete    pfpCur->m_pFichier;
					delete    pfpCur;
					*ppfpCur = NULL;
				}
			}
			
			bool fFautComp = true;
			
			//*** Ici posFichierOld et pFichierOld contiennent
			//*** l'index du fichier et le fichier lui-mme
			//*** dans l'archive active
			
			if (m_ppi->CO_nAction==coaPlusMAJ) {
				// Ajouter et mettre  jour
				
				if (posFichierOld &&
				   (m_mskAttrOPasSup!=(wfd.dwFileAttributes & m_mskAttrOPasSup)))
					goto PasComp;
				if (m_mskAttrOPasSup!=(wfd.dwFileAttributes & m_mskAttrOPasSup))
					goto Supprimer;
				
			}else if (m_ppi->CO_nAction==coaMAJ) {
				// Mettre  jour
				
				if (!posFichierOld)  goto Supprimer;
				if (m_mskAttrOPasSup!=(wfd.dwFileAttributes & m_mskAttrOPasSup))
					goto PasComp;
				
			}else if (m_ppi->CO_nAction==coaPlus) {
				// Ajouter
				
				if (posFichierOld) {
PasComp:
					pFichierOld->m_fTrouve = true;
					if (m_arPlus.m_nSup==supTout)  goto Supprimer;
				}
				
				if (m_mskAttrOPasSup!=(wfd.dwFileAttributes & m_mskAttrOPasSup))
					goto Supprimer;
				
			}else /* (m_ppi->CO_nAction==coaPasComp)*/ {
				// Ne pas compresser
				
				if (posFichierOld)  pFichierOld->m_fTrouve = true;
				fFautComp = false;
			}
			
			// Ajouter le fichier  la liste
#if 0   //$$$
			m_lstFichierPlus.AddDeb(new CFichierPlus(
				pFichier, pFichierOld,posFichierOld, pDossierParent, m_ppi));
#else   //$$$
			
			if (pfpCur) {
				// Modifier l'entre existente
				
				pfpCur->m_pFichier->m_fFautComp = fFautComp;
				pfpCur->m_ppi = m_ppi;
				
			}else{
				// Ajouter une nouvelle entre
				CFichier *const pFichier = m_pArc->NewFichier(pszParent, wfd);
				pFichier->m_fFautComp = fFautComp;
				*ppfpCur = new CFichierPlus(
					pFichier, pFichierOld,posFichierOld, pDossierParent, m_ppi);
			}
#endif  //$$$
		}
		
	}while (::FindNextFile(hff, &wfd));
	
	::FindClose(hff);
	return fOK;
}


// Finaliser la recherche en rcuprant la liste des fichiers
void CRecherche::Finaliser(CLstFichierPlus& lstFichierPlus)
{
	m_pfpDeb->RemplirListe(lstFichierPlus);
}
