// Tar/TarArc.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "../Muzip.h"
#include "../Muzip2.h"
#include "../TarDoc.h"
#include "TarArc.h"
#include "TarFichier.h"
#include "Tar.h"

#include "../FichierPlus.h"

#include "../Options.h"
#include "../Exceptions.h"

#include <GUnicode.h>


// Capacits
int CTarArc::GetCap() const
{
	return capArbo | capRapideRenommer | capRapideModifProp | capdtM;
}


// Ouverture
bool CTarArc::OpenDocument()
{
	DWORD off = 0;
	TARRECORD tr;
	DWORD dw;
	TCHAR pszPath[nbArray(tr.th.pszPath)];
	for (;;) {
		m_fr.SetVerbeux(false);
		const bool fOK = m_fr.Read(&tr, sizeof(tr));
		m_fr.SetVerbeux();
		if (!fOK || !tr.th.pszPath[0])  break;
		
		// Rcuprer le checksum
		int i;
		for (i=0; i<sizeof(tr.th.pszCheckSum); i++) {
			if (tr.th.pszCheckSum[i]!=' ') {
				DWORD dwCheckSum = 0;
				for (;;) {
					const char c = tr.th.pszCheckSum[i];
					if (!c || c==' ')  break;
					if (i==8 || c<'0' || c>'7') {
						if (e_opt.m_nReparer<repAltere)  goto ErreurFormat;
						break;
					}
					
					// Enregistrer le nouveau chiffre
					dwCheckSum = (dwCheckSum*8) + c-'0';
					i++;
				}
				// Valider le checksum
				if (e_opt.m_nReparer<=repPresumer) {
					memset(tr.th.pszCheckSum, cCheckSumVide, sizeof(tr.th.pszCheckSum));
					for (dw=0, i=sizeof(TARHEADER); i; dw += tr.pb[--i]);
					if (dwCheckSum!=dw)
						goto ErreurFormat;
				}
				break;
			}
		}
		
		// Identifier le champ magique GNU
		const bool fGNU = !lstrcmpA(tr.th.pszMagic, pszMagicGNU);
		
		// Valider le type du fichier
		if (e_opt.m_nReparer==repNon) {
			if (tr.th.cType!=tartNormalUnix && (tr.th.cType<'0' || tr.th.cType>'7')) {
				if (!fGNU ||
				   (tr.th.cType!=tartRepListe &&
				    tr.th.cType!=tartMultiVol &&
				    tr.th.cType!=tartHeaderVol))
					goto ErreurFormat;
			}
		}
		
		off += sizeof(tr);
		
		// Rcuprer la taille
		DWORD dwTaille;
		if (!GetOctal12(tr.th.taille, dwTaille))  break;
		
		switch (tr.th.cType) {
			
			// Fichier normal
			case tartNormalUnix:
			case tartNormal:
			case tartContigu:
				break;
			
			// Fichier sur rparti sur plusieurs volumes
			case tartMultiVol:
				break;
			
			// Fichier  sauter
			default:
				goto Suite;
		}
		
		{
			// Convertir le chemin d'accs et rcuprer le nom du fichier
			strfromA(pszPath, tr.th.pszPath, nbArray(pszPath));
			const LPTSTR pcFichier = PathFindFileName(pszPath);
			if (!pcFichier) {
				if (e_opt.m_nReparer<=repPresumer)  goto ErreurFormat;
				goto Suite;
			}
			
			CTarFichier *const ptf = new CTarFichier;
			ptf->m_pszNom    = newDup(pcFichier);
			time_t dtM;
			if (GetOctal12(tr.th.dtM, (DWORD&)dtM))  ptf->m_dtM = dtM;
			ptf->m_dwTailleC =
			ptf->m_dwTailleN = dwTaille;
			ptf->m_dwFlags   = 0;
			ptf->m_dwAttr    = 0;
			ptf->m_off       = off;
			ptf->MAJ();
			
			// Ajouter le fichier  son dossier
			ptf->m_pArc      = this;
			(ptf->m_pDossier = GetDossier(pszPath, false))->m_lstFichier.AddFin(ptf);
		}
		
Suite:
		
		// Sauter le fichier
		if (dwTaille) {
			// Arrondir dwTaille  sizeof(tr) prs
			if (dwTaille & sizeof(tr)-1)
				dwTaille = (dwTaille | (sizeof(tr)-1)) + 1;
			// Effectuer le dplacement
			if (!m_fr.Seek((long)dwTaille, FILE_CURRENT))  goto Erreur;
			off += dwTaille;
		}
	}
	
	Rechercher();
	return true;
	
ErreurFormat:
	CMuzipException(ERR_FORMATTAR).ReportError();
Erreur:
	m_lstFichier.DeleteAll();
	m_lstDossier.DeleteAll();
	return false;
}


// Reconstruire l'archive
void CTarArc::Reconstruire()
{
	if (!ReadOpen())  return;
	
	CMuzipOp mop;
	
	TCHAR pszTemp[MAX_PATH];
	if (OpenArchiveTemp (e_fw, pszTemp) &&
	    CloseArchiveTemp(e_fw, pszTemp))
		CMuzipArc::Reconstruire();
}


// Initialiser la cration d'une archive temporaire
bool CTarArc::OpenArchiveTemp(CFileWrite& fw, LPTSTR pszTemp)
{
	CTarFichier::s_off = 0;
	return CMuzipArc::OpenArchiveTemp(fw, pszTemp);
}


// Ajouter un fichier  la liste
CFichier* CTarArc::NewFichier(LPCTSTR pszFichier, const WIN32_FIND_DATA& wfd)
{
	CTarFichier *const pf = new CTarFichier;
	pf->NewFichier(pszFichier, wfd);
	
	pf->m_pDossier = this;
	pf->m_pArc     = this;
	
	return pf;
}


// Ajouter des fichiers  l'archive
bool CTarArc::ArchivePlus()
{
	// Crer un fichier temporaire de stockage et l'ouvrir
	TCHAR pszTemp[MAX_PATH];
	VERIF(OpenArchiveTemp(e_fw, pszTemp));
	
	CTarFichier::s_fSauterHeader = false;
	
	ShowStatusAvanceFichier(true);
	UINT iFichier = 0;
	POS pos = s_lstFichierPlus.GetPosDeb();
	while (pos) {
		CFichierPlus *const pFichierPlus = s_lstFichierPlus.GetSuiv(pos);
		CTarFichier  *const ptf = (CTarFichier*)pFichierPlus->m_pFichier;
		oApp.SetProgress(STA_ARCHIVEPLUS, ptf->m_pszNomES,
			iFichier++, s_lstFichierPlus.GetNb());
		
		PRGSTATUS prs = prsOK;
		
		// Si le fichier est l'archive elle-mme,
		// afficher un message d'erreur et passer au fichier suivant
		const DWORD offOld = e_fw.Tell();
		if (ptf->CompresseArchive())
			prs = prsErreur;
		else{
			if (ptf->m_fFautComp) {
				
				// Ouvrir le fichier en vue de lecture
				CFileRead fr;
				if (!fr.Open(ptf->m_pszNomES, FILE_SHARE_READ))
					prs = prsErreur;
				else{
					
					// Remplir la classe CTarFichier:
					WIN32_FIND_DATA wfd;
					const HANDLE hff = ::FindFirstFile(ptf->m_pszNomES, &wfd);
					if (!hff) {
						CMuzipException(ERR_READ, ptf->m_pszNomES).ReportError();
						prs = prsErreur;
					}else{
						::FindClose(hff);
						
						// Rcuprer la taille et la date de modification
						ptf->m_dwTailleC =
						ptf->m_dwTailleN = wfd.nFileSizeLow;
						ptf->m_dtM       = wfd.ftCreationTime;
						
						// crire le fichier aprs son en-tte
						if (!ptf->FichierCopier(fr, e_fw))
							prs = ErreurArchivePlus();
					}
					fr.Close();
				}
			}
		}
		
		if (!ProgressArchivePlus(ptf, prs, pos, offOld))  break;
	}
	oApp.s_fMsg = true;
	oApp.HideStatusAvance2();
	oApp.HideStatusAvance();
	
	CTarFichier::s_fSauterHeader = true;
	
	Finaliser();
	CloseArchiveTemp(e_fw, pszTemp);
	return true;
}



// Traduire une chane de 11 caractres d'ASCII octal termine par un espace
// en un DWORD, et retourner TRUE si succs
bool CTarArc::GetOctal12(LPCSTR psz, DWORD& dw)
{
	bool fDeb = true;
	UINT64 ddw = 0;
	for (int i=0;; i++) {
		const char c = psz[i];
		if (c==' ' || !c) {
			if (c && fDeb)  continue;
			dw = (DWORD)ddw;
			return true;
		}else
			fDeb = false;
		
		if (i==12 || c<'0' || c>'7')  return false;
		
		// Enregistrer le nouveau chiffre
		ddw = (ddw*8) + c-'0';
	}
}

