// Tar/TarFichier.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "../Muzip.h"
#include "../Dossier.h"
#include "TarFichier.h"
#include "Tar.h"

#include "../Decomp.h"
using namespace DECOMP;

#include "../Exceptions.h"

#include <FileBuf.h>
#include <GUnicode.h>


DWORD CTarFichier::s_off;
bool  CTarFichier::s_fSauterHeader = true;


// Copier l'entre d'un fichier  un autre
// * s_nbFichier
// * s_nFichier
bool CTarFichier::FichierCopier(CFileRead& fr, CFileWrite& fw)
{
	if (s_nbFichier)  VERIF(CFichier::FichierCopier(fr, fw));
	
	VERIF(PutHeader(fw));
	if (s_fSauterHeader)  VERIF(fr.Seek(sizeof(TARRECORD), FILE_CURRENT));
	
	// crire le fichier
	oApp.ShowStatusAvance2(m_dwTailleC);
	VERIF(CFichier::FichierCopier(fr, fw, m_dwTailleC, true));
	if (s_nbFichier)  oApp.AddStatusAvance();
	
	s_off += sizeof(TARRECORD);
	m_off  = s_off;
	s_off += m_dwTailleC;
	
	// Complter avec des caractres nuls
	DWORD dw = m_dwTailleC & (sizeof(TARRECORD)-1);
	if (dw) {
		BYTE abBuf[sizeof(TARRECORD)-1];
		dw = sizeof(TARRECORD)-dw;
		s_off += dw;
		ZeroMemory(abBuf, dw);
		VERIF(fw.Write(abBuf, dw) && fr.Seek(dw, FILE_CURRENT));
	}
	return true;
}


// Extraire le fichier (spcifique  l'extraction de dossiers)
bool CTarFichier::ExtraireInterne(LPCTSTR pszDest)
{
	e_pfr->Seek(m_off, FILE_BEGIN);
	
	// Cas spcial ici: le fichier Tar n'est pas compress, il est donc
	// bien plus rapide d'utiliser WriteFrom() plutt que DECOMP::Flush()
	// Par contre il faut utiliser DECOMP::Flush() pour la recherche,
	// et lire par petits bouts pour le test
	
	if (e_dm==dmNormal) {
		// Mode rel (sortie sur fichier)
		
		// Crer le fichier (sans attributs car on ne les connait pas)
		SetFileAttributes(pszDest, 0);
		VERIF(e_fw.Open(pszDest));
		
		// Dfinir la date de modification du fichier
		const FILETIME ftM = m_dtM;
		SetFileTime(e_fw, NULL, NULL, &ftM);
	}
	
	const UINT lenBuf = 0x1000;
	BYTE *abBuf = new BYTE[lenBuf];
	
	DWORD dwES;
	DWORD dwReste = m_dwTailleC;
	while (dwReste>=lenBuf) {
		if (!::ReadFile(*e_pfr, abBuf, lenBuf, &dwES, NULL)) {
ErreurIn:
			delete [] abBuf;
			CMuzipException(*e_pfr).ReportError();
			return false;
		}
		OnRead(lenBuf);
		
		if (e_dm!=dmTester && !Flush(abBuf, lenBuf))  goto Fin;
		
		dwReste -= lenBuf;
	}
	if (dwReste && !::ReadFile(*e_pfr, abBuf, dwReste, &dwES, NULL))
		goto ErreurIn;
	if (e_dm==dmRechercher)  Flush(abBuf, dwReste);
	
Fin:
	delete [] abBuf;
	
	e_fw.Close();
	
	return true;
}


// crire l'en-tte du fichier
bool CTarFichier::PutHeader(CFileWrite& fw)
{
	TARRECORD tr;
	ZeroMemory(&tr, sizeof(tr));
	
	if (*m_pDossier->m_pszPath) {
		// Convertir le chemin d'accs: transformer les \ en /
		
		// Copier le chemin d'accs
		const size_t lenPath = lstrlen(m_pDossier->m_pszPath)+1;
		strtoA(tr.th.pszPath, m_pDossier->m_pszPath, MAX_PATH);
		// Convertir les \ en /
		char *pcSep = tr.th.pszPath;
		for (;;) {
			pcSep = strchr(pcSep, '\\');
			if (!pcSep)  break;
			*pcSep++ = '/';
		}
		
		// Copier le nom du fichier
		tr.th.pszPath[lenPath-1] = '/';
		strtoA(tr.th.pszPath+lenPath, m_pszNom, MAX_PATH);
		
	}else
		// Recopier la chane
		strtoA(tr.th.pszPath, m_pszNom, nbArray(tr.th.pszPath));
	
	// Remplir la structure d'en-tte
	const char pszVide8[8] = "0000000";
	lstrcpyA  (tr.th.mode,        pszVide8);
	lstrcpyA  (tr.th.idUser,      pszVide8);
	lstrcpyA  (tr.th.idGroup,     pszVide8);
	SetOctal12(tr.th.taille,      m_dwTailleC);
	SetOctal12(tr.th.dtM,         (DWORD)(time_t)tr.th.dtM);
	memset    (tr.th.pszCheckSum, cCheckSumVide, sizeof(tr.th.pszCheckSum));
	tr.th.cType          = tartNormal;
	/*
	tr.th.pszMagic   [0] = 0;
	tr.th.pszNomUser [0] = 0;
	tr.th.pszNomGroup[0] = 0;
	*/
	lstrcpyA  (tr.th.devMajor,    pszVide8);
	lstrcpyA  (tr.th.devMinor,    pszVide8);
	
	// Calculer puis crire le checksum
	int i;
	DWORD dw;
	for (dw=0, i=sizeof(TARHEADER); i; dw += tr.pb[--i]);
	tr.th.pszCheckSum[7] = 0;
	for (i=nbArray(tr.th.pszCheckSum)-2; i>=0; i--) {
		tr.th.pszCheckSum[i] = (char)('0' + (dw & 7));
		dw /= 8;
	}
	
	// crire l'en-tte
	return fw.Write(&tr, sizeof(tr));
}


// Traduire une chane de 11 caractres d'ASCII octal termine par un espace
// en un DWORD, et retourner TRUE si succs
void CTarFichier::SetOctal12(LPSTR psz, DWORD dw)
{
	psz[11] = 0;
	for (int i=10; i>=0; i--) {
		psz[i] = (char)('0' + (dw & 7));
		dw /= 8;
	}
}


// Renommage
bool CTarFichier::OnRenommer()
{
	// crire l'en-tte
	return s_fw.Seek(m_off-sizeof(TARRECORD), FILE_BEGIN) && PutHeader(s_fw);
}

// Modification des proprits
bool CTarFichier::OnModifProp()
{
	return OnRenommer();
}
