// Uue/UueArc.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "../Muzip.h"
#include "../Muzip2.h"
#include "UueArc.h"
#include "UueFichier.h"
#include "Uue.h"

#include "../FichierPlus.h"

#include "../Exceptions.h"

#include <GUnicode.h>


static const char aEncoder[0x40] =
{
	'`', '!', '"', '#', '$', '%', '&', '\'', '(', ')', '*', '+', ',',  '-', '.', '/',
	'0', '1', '2', '3', '4', '5', '6', '7',  '8', '9', ':', ';', '<',  '=', '>', '?',
	'@', 'A', 'B', 'C', 'D', 'E', 'F', 'G',  'H', 'I', 'J', 'K', 'L',  'M', 'N', 'O',
	'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W',  'X', 'Y', 'Z', '[', '\\', ']', '^', '_',
};


// Capacits
int CUueArc::GetCap() const
{
	return capMaxUnFichier;
}


// Ouverture
bool CUueArc::OpenDocument()
{
	m_fr.SetVerbeux(false);
	
	int i;
	BYTE b;
	
	// Rechercher le dbut du fichier (marqueur 'begin')
	const int lenMarqueur = 5;
	const BYTE abMarqueur[] = "begin";
	for (;;) {
		for (i=0;; i++) {
			if (i==lenMarqueur) {
				// Aprs 'begin' on doit avoir un caractre d'espacement
				if (!m_fr.Read(b))  goto ErreurFormat;
				if (isspace(b))  goto Trouve;
				break;
			}
			if (!m_fr.Read(b)) {
ErreurFormat:
				CMuzipException(ERR_FORMATUUE, m_fr.GetPath()).ReportError();
				return false;
			}
			if (b!=abMarqueur[i])  break;
		}
		
		// Atteindre la fin de la ligne
		while (b!='\r' && b!='\n')
			if (!m_fr.Read(b))  goto ErreurFormat;
	}
	
Trouve:
	
	// Conserver l'offset de l'en-tte
	const DWORD off = m_fr.Tell()-6;
	
	// Sauter les droits d'accs
	for (;;) {
		if (!m_fr.Read(b))  goto ErreurFormat;
		if (b=='\r' || b=='\n')  goto ErreurFormat;
		if (!isspace(b))  break;
	}
	const BYTE bAccesUser = b;
	for (i=0; i<3; i++) {
		if (b<'0' || b>'7')  goto ErreurFormat;
		if (!m_fr.Read(b))  goto ErreurFormat;
	}
	
	// Lire le nom du fichier
	for (;;) {
		if (b=='\r' || b=='\n')  goto ErreurFormat;
		if (!isspace(b))  break;
		if (!m_fr.Read(b))  goto ErreurFormat;
	}
	TCHAR pszNom[MAX_PATH], *pc=pszNom;
	for (;;) {
		*pc++ = (TCHAR)b;
		if (!m_fr.Read(b))  goto ErreurFormat;
		if (b=='\r' || b=='\n')  break;
	}
	*pc = 0;
	
	CUueFichier *const puf = new CUueFichier;
	puf->m_pArc      = this;
	puf->m_pDossier  = this;
	puf->m_pszNom    = newDup(pszNom);
	puf->m_dwTailleC = GetFileSize(m_fr, NULL) - off - 6;// `\nend\n
	puf->m_dwTailleN = (puf->m_dwTailleC/4)*3;
	puf->m_dwAttr    = (bAccesUser & 2) ? 0 : FILE_ATTRIBUTE_READONLY;
	
	// Rcuprer la date de modification
	FILETIME ftM;
	GetFileTime(m_fr, NULL, NULL, &ftM);
	puf->m_dtM = ftM;
	
	puf->MAJ();
	puf->m_off = off;
	
	m_lstFichier.AddDeb(puf);
	
	return true;
}


// Ajouter un fichier  la liste
CFichier* CUueArc::NewFichier(LPCTSTR pszFichier, const WIN32_FIND_DATA& wfd)
{
	CUueFichier *const pf = new CUueFichier;
	pf->NewFichier(pszFichier, wfd);
	pf->m_pDossier = this;
	pf->m_pArc     = this;
	return pf;
}


// Ajouter un fichier  l'archive
bool CUueArc::ArchivePlusUn(CFichierPlus* pFichierPlus, CFileRead& fr, CFileWrite& fw)
{
	CUueFichier *const puf = (CUueFichier*)pFichierPlus->m_pFichier;
	
	// En-tte
	char psz[10+MAX_PATH+1] = "begin 600 ";
	strtoA(psz+10, PathFindFileName(puf->m_pszNomES), MAX_PATH);
	lstrcatA(psz, "\r\n");
	VERIF(fw.Write(psz));
	
	puf->m_off       = fr.Tell();
	puf->m_dwTailleC = lstrlenA(psz);
	
	// Rcuprer la taille du fichier d'entre
	// pour l'affichage de l'avancement
	oApp.SetProgress(STA_ARCHIVEPLUS, puf->m_pszNomES);
	
	puf->m_dwTailleC = 0;
	BYTE abIn  [  maxLigneD  ];
	char pszOut[1+maxLigneC+2];
	for (;;) {
		fr.SetVerbeux(false);
		const DWORD lenLigne = fr.ReadLen(abIn, maxLigneD);
		if (!lenLigne)  break;
		fr.SetVerbeux();
		
		oApp.AddStatusAvance(lenLigne);
		
		// crire la longueur de la ligne
		char *pcOut = pszOut;
		*pcOut++ = aEncoder[lenLigne];
		
		// Encoder la ligne elle-mme
		const BYTE *pbIn = abIn;
		for (DWORD i=0; i<lenLigne; i+=3,pbIn+=3) {
			// Encoder les 3 octets courants
			
			*pcOut++ = aEncoder[  pbIn[0]        >>2  ];
			*pcOut++ = aEncoder[((pbIn[0] & 0x03)<<4) |
			                     (pbIn[1]        >>4) ];
			*pcOut++ = aEncoder[((pbIn[1] & 0x0F)<<2) |
			                     (pbIn[2]        >>6) ];
			*pcOut++ = aEncoder[  pbIn[2] & 0x3F      ];
		}
		*pcOut++ = '\r';
		*pcOut++ = '\n';
		*pcOut   = 0;
		
		// crire la ligne
		VERIF(fw.Write(pszOut));
		puf->m_dwTailleC += lstrlenA(pszOut);
	}
	
	// Fin du fichier
	VERIF(fw.Write(pszUueEnd) && fw.Write("\r\n"));
	puf->m_dwTailleC += 5;
	
	return true;
}
