// Z/Arbre.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "../CodeRapide.h"
#include "../Muzip.h"
#include "ZArc.h"

#include "Fen.h"
#include "Huffman.h"

#include "../Zip/Zip.h"


// Tailles des tampon de correspondances pour les littraux/longueurs
// et les distances. On trouve 4 raisons pour limiter lenBufL  64ko:
// - les frquences peuvent tre mises dans des compteurs 16 bits.
// - si la compression choue pour le premier bloc, toutes les donnes
//   d'entre restent dans la fentre, on peut donc toujours mettre
//   un bloc stock mme si l'entre vient de l'entre standard
//   (cela peut aussi tre effectu sur tous les blocs si lenBufL<32ko)
// - si la compression choue pour un fichier plus petit que 64ko, on peut
//   mme mettre un fichier stock au lieu d'un bloc stock (ce qui
//   conomise 5 octets).
// - crer de nouveau arbres Huffman moins frquemment ne donne pas une
//   adaptation rapide aux changements dans les statistiques des donnes
//   d'entre (prenez par exemple un fichier binaire commenant par un
//   code difficilement compressible suivi d'une table de chanes
//   facilement compactable). Des tampons plus petits donne une adaptation
//   rapide mais on bien sr l'inconvnient d'avoir  transmettre des
//   arbres plus souvent.
// Le code courant est gnral et permet lenBufD<lenBufL (pour
// conomiser de la mmoire au prix d'une moins bonne compression).
// Certaines optimisations seraient envosageables si lenBufD==lenBufL.

const UINT lenBufL = 0x8000;
const UINT lenBufD = lenBufL;



// Index dans le tas du noeud le moins frquent dans l'arbre Huffman
const UINT nPlusPetit = 1;

// Codes Huffman: rpter la longueur bit prcdente:
const int rep3_6    = 16;   // de 3  6 fois (2 bits pour le nombre de rptitions)
const int rep3_10   = 17;   // de 3  10 fois (2 bits pour le nombre de rptitions)
const int rep11_138 = 18;   // de 11  138 fois (7 bits pour le nombre de rptitions)


// Trois types de bloc
enum
{
	bltStocke,
	bltStatique,
	bltDynamique
};


// Tableau de bits indiquant pour chaque valeur de s_bufL
// si c'est est un littral ou une longueur
static BYTE s_fBuf[(lenBufL/8)];


// Les bits sont remplis dans les flags  partir du bit 0
// (le moins significatif)
// Note: ces valeurs sont redondantes dans le code et ne tirent
// donc pas avantage du fait que lenBufD==lenBufL
static WORD s_bufD[0x8000];   // Tampon pour les distances
static BYTE s_bufL[0x8000];   // Tampon pour les littraux/longueurs
static UINT s_derD;          // Index courant dans s_bufD
static UINT s_derL;          // Index courant dans s_bufL
static UINT s_derFlags;      // Index courant dans s_fBuf
static BYTE s_nFlags;        // Flags courants pas encore placs dans s_fBuf
static BYTE s_nFlagsBit;     // Bit courant dans s_nFlags

#ifdef _DEBUG
static DWORD s_lenIn;
#endif
static DWORD s_lenOut;

static WORD     *s_pnAttr;
static ZMETHODE *s_pnMethode;


static DWORD s_lenDyn;   // Longueur de bit du bloc courant avec arbres optimaux
static DWORD s_lenSta;   // Longueur de bit du bloc courant avec arbres statiques


// Arbres statiques
static const CTDATA s_sTreeL[nbcodeL+2] =
{
	{ 12, 8}, {140, 8}, { 76, 8}, {204, 8}, { 44, 8},
	{172, 8}, {108, 8}, {236, 8}, { 28, 8}, {156, 8},
	{ 92, 8}, {220, 8}, { 60, 8}, {188, 8}, {124, 8},
	{252, 8}, {  2, 8}, {130, 8}, { 66, 8}, {194, 8},
	{ 34, 8}, {162, 8}, { 98, 8}, {226, 8}, { 18, 8},
	{146, 8}, { 82, 8}, {210, 8}, { 50, 8}, {178, 8},
	{114, 8}, {242, 8}, { 10, 8}, {138, 8}, { 74, 8},
	{202, 8}, { 42, 8}, {170, 8}, {106, 8}, {234, 8},
	{ 26, 8}, {154, 8}, { 90, 8}, {218, 8}, { 58, 8},
	{186, 8}, {122, 8}, {250, 8}, {  6, 8}, {134, 8},
	{ 70, 8}, {198, 8}, { 38, 8}, {166, 8}, {102, 8},
	{230, 8}, { 22, 8}, {150, 8}, { 86, 8}, {214, 8},
	{ 54, 8}, {182, 8}, {118, 8}, {246, 8}, { 14, 8},
	{142, 8}, { 78, 8}, {206, 8}, { 46, 8}, {174, 8},
	{110, 8}, {238, 8}, { 30, 8}, {158, 8}, { 94, 8},
	{222, 8}, { 62, 8}, {190, 8}, {126, 8}, {254, 8},
	{  1, 8}, {129, 8}, { 65, 8}, {193, 8}, { 33, 8},
	{161, 8}, { 97, 8}, {225, 8}, { 17, 8}, {145, 8},
	{ 81, 8}, {209, 8}, { 49, 8}, {177, 8}, {113, 8},
	{241, 8}, {  9, 8}, {137, 8}, { 73, 8}, {201, 8},
	{ 41, 8}, {169, 8}, {105, 8}, {233, 8}, { 25, 8},
	{153, 8}, { 89, 8}, {217, 8}, { 57, 8}, {185, 8},
	{121, 8}, {249, 8}, {  5, 8}, {133, 8}, { 69, 8},
	{197, 8}, { 37, 8}, {165, 8}, {101, 8}, {229, 8},
	{ 21, 8}, {149, 8}, { 85, 8}, {213, 8}, { 53, 8},
	{181, 8}, {117, 8}, {245, 8}, { 13, 8}, {141, 8},
	{ 77, 8}, {205, 8}, { 45, 8}, {173, 8}, {109, 8},
	{237, 8}, { 29, 8}, {157, 8}, { 93, 8}, {221, 8},
	{ 61, 8}, {189, 8}, {125, 8}, {253, 8}, { 19, 9},
	{275, 9}, {147, 9}, {403, 9}, { 83, 9}, {339, 9},
	{211, 9}, {467, 9}, { 51, 9}, {307, 9}, {179, 9},
	{435, 9}, {115, 9}, {371, 9}, {243, 9}, {499, 9},
	{ 11, 9}, {267, 9}, {139, 9}, {395, 9}, { 75, 9},
	{331, 9}, {203, 9}, {459, 9}, { 43, 9}, {299, 9},
	{171, 9}, {427, 9}, {107, 9}, {363, 9}, {235, 9},
	{491, 9}, { 27, 9}, {283, 9}, {155, 9}, {411, 9},
	{ 91, 9}, {347, 9}, {219, 9}, {475, 9}, { 59, 9},
	{315, 9}, {187, 9}, {443, 9}, {123, 9}, {379, 9},
	{251, 9}, {507, 9}, {  7, 9}, {263, 9}, {135, 9},
	{391, 9}, { 71, 9}, {327, 9}, {199, 9}, {455, 9},
	{ 39, 9}, {295, 9}, {167, 9}, {423, 9}, {103, 9},
	{359, 9}, {231, 9}, {487, 9}, { 23, 9}, {279, 9},
	{151, 9}, {407, 9}, { 87, 9}, {343, 9}, {215, 9},
	{471, 9}, { 55, 9}, {311, 9}, {183, 9}, {439, 9},
	{119, 9}, {375, 9}, {247, 9}, {503, 9}, { 15, 9},
	{271, 9}, {143, 9}, {399, 9}, { 79, 9}, {335, 9},
	{207, 9}, {463, 9}, { 47, 9}, {303, 9}, {175, 9},
	{431, 9}, {111, 9}, {367, 9}, {239, 9}, {495, 9},
	{ 31, 9}, {287, 9}, {159, 9}, {415, 9}, { 95, 9},
	{351, 9}, {223, 9}, {479, 9}, { 63, 9}, {319, 9},
	{191, 9}, {447, 9}, {127, 9}, {383, 9}, {255, 9},
	{511, 9}, {  0, 7}, { 64, 7}, { 32, 7}, { 96, 7},
	{ 16, 7}, { 80, 7}, { 48, 7}, {112, 7}, {  8, 7},
	{ 72, 7}, { 40, 7}, {104, 7}, { 24, 7}, { 88, 7},
	{ 56, 7}, {120, 7}, {  4, 7}, { 68, 7}, { 36, 7},
	{100, 7}, { 20, 7}, { 84, 7}, { 52, 7}, {116, 7},
	{  3, 8}, {131, 8}, { 67, 8}, {195, 8}, { 35, 8},
	{163, 8}, { 99, 8}, {227, 8},
};

static const CTDATA s_sTreeD[nbcodeD] =
{
	{ 0, 5}, {16, 5}, { 8, 5}, {24, 5}, { 4, 5},
	{20, 5}, {12, 5}, {28, 5}, { 2, 5}, {18, 5},
	{10, 5}, {26, 5}, { 6, 5}, {22, 5}, {14, 5},
	{30, 5}, { 1, 5}, {17, 5}, { 9, 5}, {25, 5},
	{ 5, 5}, {21, 5}, {13, 5}, {29, 5}, { 3, 5},
	{19, 5}, {11, 5}, {27, 5}, { 7, 5}, {23, 5},
};

// Arbres dynamiques
const UINT lenTas = 2*nbcodeL+1;      // Taille maximum du tas
static CTDATA s_dTreeL [lenTas];         // Littraux et longueurs
static CTDATA s_dTreeD [2*nbcodeD+1];    // Distances
static CTDATA s_dTreeBL[2*nbcodeBL+1];   // Longueurs bit


// Nombre de codes  chaque longueur bit pour un code optimal
static WORD s_anbBL[maxBits+1] = { 0,0,0,0,0,0,0,24,152,112,0,0,0,0,0 };


// Description des arbres
static TREEDESC s_descL  = { s_dTreeL, s_sTreeL, e_aExtraL,  nbOctet+1, nbcodeL,  maxBits,    0 };
static TREEDESC s_descD  = { s_dTreeD, s_sTreeD, e_aExtraD,  0,         nbcodeD,  maxBits,    0 };
static TREEDESC s_descBL = { s_dTreeBL, NULL,    e_aExtraBL, 0,         nbcodeBL, maxBitsLen, 0 };

// Premire longueur/distance normalise pour chaque code
// (0=minMatch)
static const int s_debLen[nbcodeLen] =
{
	0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 12, 14, 16, 20, 24, 28, 32, 40, 48, 56,
	64, 80, 96, 112, 128, 160, 192, 224, 0
};

static const int s_debD[nbcodeD] =
{
	   0,    1,    2,    3,    4,    6,    8,    12,    16,    24,
	  32,   48,   64,   96,  128,  192,  256,   384,   512,   768,
	1024, 1536, 2048, 3072, 4096, 6144, 8192, 12288, 16384, 24576
};


// Codes de longueur pour chaque longueur de correspondance
// Le tableau est normalis (0=minMatch)
static const BYTE s_aCodeLen[maxMatch-minMatch+1] =
{
	 0,  1,  2,  3,  4,  5,  6,  7,  8,  8,  9,  9, 10, 10, 11, 11, 12, 12, 12, 12,
	13, 13, 13, 13, 14, 14, 14, 14, 15, 15, 15, 15, 16, 16, 16, 16, 16, 16, 16, 16,
	17, 17, 17, 17, 17, 17, 17, 17, 18, 18, 18, 18, 18, 18, 18, 18, 19, 19, 19, 19,
	19, 19, 19, 19, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20,
	21, 21, 21, 21, 21, 21, 21, 21, 21, 21, 21, 21, 21, 21, 21, 21, 22, 22, 22, 22,
	22, 22, 22, 22, 22, 22, 22, 22, 22, 22, 22, 22, 23, 23, 23, 23, 23, 23, 23, 23,
	23, 23, 23, 23, 23, 23, 23, 23, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
	24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
	25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25,
	25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 26, 26, 26, 26, 26, 26, 26, 26,
	26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26,
	26, 26, 26, 26, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
	27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 28
};

// Codes de distance
// Les 256 premires valeurs correspondent aux distances 3~258,
// les 256 dernires aux 8 bits de poids fort des 15 bits de distance
static const BYTE s_aCodeDist[512] =
{
	 0,  1,  2,  3,  4,  4,  5,  5,  6,  6,  6,  6,  7,  7,  7,  7,  8,  8,  8,  8,
	 8,  8,  8,  8,  9,  9,  9,  9,  9,  9,  9,  9, 10, 10, 10, 10, 10, 10, 10, 10,
	10, 10, 10, 10, 10, 10, 10, 10, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11,
	11, 11, 11, 11, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12,
	12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 13, 13, 13, 13,
	13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13,
	13, 13, 13, 13, 13, 13, 13, 13, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14,
	14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14,
	14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14,
	14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 15, 15, 15, 15, 15, 15, 15, 15,
	15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15,
	15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15,
	15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15,  0,  0, 16, 17,
	18, 18, 19, 19, 20, 20, 20, 20, 21, 21, 21, 21, 22, 22, 22, 22, 22, 22, 22, 22,
	23, 23, 23, 23, 23, 23, 23, 23, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
	24, 24, 24, 24, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25,
	26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26,
	26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 27, 27, 27, 27, 27, 27, 27, 27,
	27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
	27, 27, 27, 27, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28,
	28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28,
	28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28,
	28, 28, 28, 28, 28, 28, 28, 28, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29,
	29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29,
	29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29,
	29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29
};


// Envoyer le code nCode  l'arbre pTree
#define CodeEnvoyer(nCode, pTree) \
	biEnvoyer(pTree[nCode].Code, pTree[nCode].Len)

// Mapper une distance en code de distance
// nDist = distance-1
// s_aCodeDist[256] et s_aCodeDist[257] ne sont jamais utiliss
#define GetCodeDist(nDist) \
	((nDist)<256 ? s_aCodeDist[nDist] : s_aCodeDist[256+((nDist)>>7)])


static void trBlocInit();
static void trTreeScan(CTDATA* pTree, int maxCode);
static void trTreeBuild(TREEDESC& desc);


// Allouer le tampon de correspondance, initialiser les diffrentes
// tables et sauver la position des attributs internes et de la mthode
void CZArc::ctInit(WORD* pnAttr, ZMETHODE* pnMethode)
{
	s_pnAttr    = pnAttr;
	s_pnMethode = pnMethode;
#ifdef _DEBUG
	s_lenIn     = 0;
#endif
	s_lenOut    = 0;
	
	// Initialiser le premier bloc du premier fichier
	trBlocInit();
}


// Initialiser un nouveau bloc
void trBlocInit()
{
	// Initialiser les arbres
	int n;
	for (n=0; n<nbcodeL;  s_dTreeL [n++].Freq = 0);
	for (n=0; n<nbcodeD;  s_dTreeD [n++].Freq = 0);
	for (n=0; n<nbcodeBL; s_dTreeBL[n++].Freq = 0);
	s_dTreeL[finBloc].Freq = 1;
	
	s_lenDyn    =
	s_lenSta    =
	s_derL      =
	s_derD      =
	s_derFlags  = 0;
	s_nFlags    = 0;
	s_nFlagsBit = 1;
}


// Envoyer un arbre des littraux ou des distance sous forme compresss
// en utilisant les codes de s_dTreeBL
// pTree   - arbre  examiner
// maxCode - son plus grand code de frquence non-nulle
void CZArc::trTreeEnvoyer(CTDATA* pTree, int maxCode)
{
	int maxNb, minNb;
	int lenSuiv = pTree[0].Len;
	if (lenSuiv) {
		maxNb =   7;
		minNb =   4;
	}else{
		maxNb = 138;
		minNb =   3;
	}
	
	int nb = 0;
	int lenPrec = -1;
	for (int n=0; n<=maxCode; n++) {
		int lenCur = lenSuiv;
		lenSuiv = pTree[n+1].Len;
		
		if (++nb<maxNb && lenCur==lenSuiv)  continue;
		
		if (nb<minNb) {
			do{
				CodeEnvoyer(lenCur, s_dTreeBL);
			}while (--nb);
		}else if (lenCur) {
			if (lenCur!=lenPrec) {
				CodeEnvoyer(lenCur, s_dTreeBL);
				nb--;
			}
			ASSERT(nb>=3 && nb<=6); // 3_6?
			CodeEnvoyer(rep3_6, s_dTreeBL);
			biEnvoyer(nb-3, 2);
		
		}else if (nb <= 10){
			CodeEnvoyer(rep3_10, s_dTreeBL);
			biEnvoyer(nb-3, 3);
		}else{
			CodeEnvoyer(rep11_138, s_dTreeBL);
			biEnvoyer(nb-11, 7);
		}
		nb = 0;
		lenPrec = lenCur;
		if (lenSuiv==0) {
			maxNb = 138;
			minNb = 3;
		}else if (lenCur==lenSuiv) {
			maxNb = 6;
			minNb = 3;
		}else{
			maxNb = 7;
			minNb = 4;
		}
	}
}

// Construire l'arbre Huffman des longueur bit
// Retourner l'index dans e_aOrdreBL du dernier code de longueur bit  envoyer
static int trTreeBuildBL()
{
	// Dterminer les frquence des longueurs bit pour les arbres
	// des littraux et des distances
	trTreeScan(s_dTreeL, s_descL.maxCode);
	trTreeScan(s_dTreeD, s_descD.maxCode);
	
	// Construire l'arbre des longueur bit
	trTreeBuild(s_descBL);
	
	// Maintenant s_lenDyn contient la longueur des reprsentations des arbres
	// except les longueurs des codes de longueur bit et les 5+5+4 bits
	// des nombres
	
	// Dterminer le nombre de code de longueur bit  envoyer
	// Le format PKZIP en demande au moins 4 (le fichier texte dit 3,
	// mais la valeur actuelle est 4)
	int maxnBL;
	for (maxnBL=nbcodeBL-1; maxnBL>=3; maxnBL--) {
		if (s_dTreeBL[e_aOrdreBL[maxnBL]].Len)  break;
	}
	
	// Mettre  jour s_lenDyn pour inclure l'arbre des longueur bit
	// et les nombres
	s_lenDyn += 3*(maxnBL+1) + 5+5+4;
	
	return maxnBL;
}


// Envoyer l'en-tte d'un bloc en utilisant des arbres Huffman dynamiques:
// les nombres, les longueurs des codes de longueur bit, l'arbre des littraux
// et celui des distances
// codeL>=257, codeD>=1, codeBL>=5
void CZArc::trTreeEnvoyerTout(int codeL, int codeD, int codeBL)
{
	ASSERT(codeL>=    257 && codeD>=      1 && codeBL>=       4); // Pas assez de codes
	ASSERT(codeL<=nbcodeL && codeD<=nbcodeD && codeBL<=nbcodeBL); // Trop de codes
	biEnvoyer(codeL-257, 5);
	
	// Pas +255 comme explicit par PKZIP 1.93a, -256 dans 2.04c ou -3
	biEnvoyer(codeD-1,   5);
	biEnvoyer(codeBL-4,  4);
	for (int nBL=0; nBL<codeBL; nBL++)
		biEnvoyer(s_dTreeBL[e_aOrdreBL[nBL]].Len, 3);
	
	// Envoyer l'arbre des littraux et celui des distances
	trTreeEnvoyer(s_dTreeL, codeL-1);
	trTreeEnvoyer(s_dTreeD, codeD-1);
}


// Dterminer le meilleur codage pour le bloc courant:
// arbres dynamiques, statiques, ou stockage, envoyer le bloc cod
// dans le fichier Zip, et retourner la longueur totale compresse
// pour le fichier
// pbBuf - bloc d'entre, ou NULL si trop vieux
// len   - longueur du bloc d'entre
// fEOF  - TRUE s'il s'agit du dernier bloc du fichier
DWORD CZArc::trBlocFlush(BYTE* pbBuf, DWORD len, bool fEOF)
{
	DWORD lenDynO, lenStaO;   // s_lenDyn et s_lenSta en octets
	int   maxnBL;             // Index du dernier code de longueur bit
	                          // de frquence non-nulle
	
	// Sauver les flags des 8 derniers items
	s_fBuf[s_derFlags] = s_nFlags;
	
	// Vrifier si le fichier est ASCII ou binaire
	if (s_pnAttr && *s_pnAttr==attInconnu) {
		UINT nbAscii   = 0;
		UINT nbBinaire = 0;
		int n = 0;
		while (n<7)       nbBinaire += s_dTreeL[n++].Freq;
		while (n<128)     nbAscii   += s_dTreeL[n++].Freq;
		while (n<nbOctet) nbBinaire += s_dTreeL[n++].Freq;
		*s_pnAttr = nbBinaire > (nbAscii>>2) ? attBinaire : attASCII;
	}
	
	// Construire les arbres des littraux et des distances
	trTreeBuild(s_descL);
	trTreeBuild(s_descD);
	
	// A ce point, s_lenDyn et s_lenSta contiennent les longueur bit totales
	// des donnes du bloc compress, reprsentations des arbres exclues
	
	// Construire l'arbre de longueur bit pour les deux arbres prcdents,
	// et rcuprer l'index dans e_aOrdreBL du dernier code de longueur bit
	//  envoyer
	maxnBL = trTreeBuildBL();
	
	// Dterminer le meilleur codage
	// Calculer d'abord la longueur du bloc en octets
	lenDynO = (s_lenDyn+3+7)>>3;
	lenStaO = (s_lenSta+3+7)>>3;
#ifdef _DEBUG
	s_lenIn += len;
#endif
	
	if (lenStaO<=lenDynO)  lenDynO = lenStaO;
	
#ifndef    PGP
	// PGP ne supporte pas les blocs stocks
	// Si la compression choue et s'il s'agit du premier et dernier bloc,
	// le fichier tout entier est transform en fichier stock
#ifdef FORCE_METHOD
	if (e_nNiveau==1 && fEOF && !s_lenOut) { // Forcer le fichier stock
#else
	if (len<=lenDynO && fEOF && !s_lenOut) {
#endif
		// Comme lenBufL<=2*lenFen, les donnes d'entre doivent tre ici
		
		trBlocCopier(pbBuf, (UINT)len, false);   // Sans en-tte
		s_lenOut = len<<3;
		*s_pnMethode = zmStore;
		
	}else
#endif // !PGP
#ifdef FORCE_METHOD
	if (e_nNiveau==2 && pbBuf) {   // Forcer le bloc stock
#else
	if (len+4<=lenDynO && pbBuf) {
		// 4: deux mots pour les longueurs
#endif
		// Le test pbBuf!=NULL n'est ncessaire que si lenBufL>lenFen
		// Sinon on ne peut pas avoir trait plus que lenFen octets d'entre
		// depuis le flush du dernier bloc, car la compression aurait russi
		// Si lenBufL<=lenFen, il n'est pas trop tard pour transformer
		// un bloc en bloc stock
		
		// Envoyer le type du blc
		biEnvoyer((bltStocke<<1)+fEOF, 3);
		s_lenOut = (s_lenOut + 3 + 7) & ~7L;
		s_lenOut += (len + 4) << 3;
		
		// Avec en-tte
		trBlocCopier(pbBuf, (UINT)len, true);
#ifdef FORCE_METHOD
	}else if (e_nNiveau==3) {  // Forcer les arbres statiques
#else
	}else if (lenStaO==lenDynO) {
#endif
		biEnvoyer((bltStatique<<1)+fEOF, 3);
		trBlocCompresser((CTDATA*)s_sTreeL, (CTDATA*)s_sTreeD);
		s_lenOut += 3+s_lenSta;
	}else{
		biEnvoyer((bltDynamique<<1)+fEOF, 3);
		trTreeEnvoyerTout(s_descL.maxCode+1, s_descD.maxCode+1, maxnBL+1);
		trBlocCompresser(s_dTreeL, s_dTreeD);
		s_lenOut += 3+s_lenDyn;
	}
	ASSERT(s_lenOut==e_lenOut); // Taille compresse incorrecte
	trBlocInit();
	
	if (fEOF) {
		ASSERT(s_lenIn==e_lenIn);   // Taille d'entre incorrecte
		biFlushOctet();
		s_lenOut += 7;   // Aligner sur un octet
	}
	
	return s_lenOut>>3;
}


// Sauver les informations de correspondance et compter les nombres
// de frquences
// Retourner TRUE si le bloc courant doit tre flush
// nDist - distance de la chane correspondante
// lc    - longueur de correspondance-minMatch ou caractre
//         non-correspondant (si nDist==0)
bool CZArc::ctCompter(int nDist, int lc)
{
	s_bufL[s_derL++] = (UCHAR)lc;
	if (!nDist)
		// lc est un caractre non-correspondant
		s_dTreeL[lc].Freq++;
	else{
		// Ici, lc = LongueurCorrespondance-minMatch
		// nDist = DistanceCorrespondance-1
		nDist--;
		ASSERT((WORD)nDist<(WORD)maxDist &&
		       (WORD)lc<=(WORD)(maxMatch-minMatch) &&
		       (WORD)GetCodeDist(nDist)<(WORD)nbcodeD); // ctCompter: mauvaise correspondance
		
		s_dTreeL[s_aCodeLen[lc]+nbOctet+1].Freq++;
		s_dTreeD[GetCodeDist(nDist)].Freq++;
		
		s_bufD[s_derD++] = (WORD)nDist;
		s_nFlags |= s_nFlagsBit;
	}
	s_nFlagsBit <<= 1;
	
	// Mettre les flags en sortie s'ils remplissent un octet
	if ((s_derL & 7)==0) {
		s_fBuf[s_derFlags++] = s_nFlags;
		s_nFlags = 0;
		s_nFlagsBit = 1;
	}
	// Essayer de deviner s'il est profitable d'arrter
	// le bloc courant ici
	if (oApp.s_pi.CO_nNiveau>2 && !(s_derL & 0xFFF)) {
		// Calculer une limite suprieure pour la longueur compresse
		DWORD lenOut = (DWORD)s_derL*8;
		DWORD lenIn  = (DWORD)e_offStr-e_debBloc;
		for (int dCode=0; dCode<nbcodeD; dCode++)
			lenOut += (DWORD)s_dTreeD[dCode].Freq*(5L+e_aExtraD[dCode]);
		lenOut >>= 3;
		
		if (s_derD<s_derL/2 && lenOut<lenIn/2)  return true;
	}
	
	// Eviter l'galit avec lenBufL  cause du wraparound
	//  64ko sur les machines 16bits et car les blocs stocks
	// sont restreints  64ko-1 octet
	return (s_derL==lenBufL-1 || s_derD==lenBufD);
}


// Envoyer le bloc de donnes compresses en utilisant les arbres Huffman donns
void CZArc::trBlocCompresser(CTDATA* pTreeL, CTDATA* pTreeD)
{
	UINT nDist;        // Distance de la chane correspondante
	int  lc;           // Longueur de correspondance ou caractre non-correspondant (si nDist==0)
	UINT nL = 0;       // Index courant dans s_bufL
	UINT nD = 0;       // Index courant dans s_bufD
	UINT nF = 0;       // Index courant dans s_fBuf
	BYTE nFlags = 0;   // Flags courants
	UINT nCode;        // Code  envoyer
	int  lenExtra;     // Nombre de bits extra  envoyer
	
	if (s_derL) {
		do{
			if (!(nL & 7))  nFlags = s_fBuf[nF++];
			lc = s_bufL[nL++];
			if (!(nFlags & locfCrypte)) {
				// Envoyer un littral
				CodeEnvoyer(lc, pTreeL);
			}else{
				
				// Ici, lc = LongueurCorrespondance-minMatch
				nCode = s_aCodeLen[lc];
				
				// Envoyer le code de longueur
				CodeEnvoyer(nCode+nbOctet+1, pTreeL);
				lenExtra = e_aExtraL[nCode];
				if (lenExtra) {
					// Envoyer les bits extra de longueur
					lc -= s_debLen[nCode];
					biEnvoyer(lc, lenExtra);
				}
				nDist = s_bufD[nD++];
				
				// Ici, nDist = DistanceCorrespondance-1
				nCode = GetCodeDist(nDist);
				ASSERT(nCode<nbcodeD);   // Mauvais nbcodeD
				
				// Envoyer le code de distance
				CodeEnvoyer(nCode, pTreeD);
				lenExtra = e_aExtraD[nCode];
				if (lenExtra) {
					// Envoyer les bits extra de distance
					nDist -= s_debD[nCode];
					biEnvoyer(nDist, lenExtra);
				}
			}
			nFlags >>= 1;
		}while (nL<s_derL);
	}
	
	CodeEnvoyer(finBloc, pTreeL);
}


// Examiner un arbre de littraux ou de distances pour dterminer
// les frquences des codes dans l'arbre des longueur bit
// Mettre  jour s_lenDyn pour prendre en compte le nombre de rptitions
// (La contribution des codes de longueur bit sera ajoute plus tard,
// pendant la construction de s_dTreeBL)
// pTree   - arbre  examiner
// maxCode - son plus grand code de frquence non-nulle
#pragma warning (disable: 4244)
void trTreeScan(CTDATA* pTree, int maxCode)
{
	int maxNb, minNb;
	int lenSuiv = pTree[0].Len;
	if (lenSuiv) {
		maxNb =   7;
		minNb =   4;
	}else{
		maxNb = 138;
		minNb =   3;
	}
	
	pTree[maxCode+1].Len = (WORD)-1;   // Garder
	
	int nb = 0;
	int lenPrec = -1;
	for (int n=0; n<=maxCode; n++) {
		int lenCur = lenSuiv;
		lenSuiv = pTree[n+1].Len;
		
		if (++nb<maxNb && lenCur==lenSuiv)  continue;
		if (nb<minNb)
			s_dTreeBL[lenCur].Freq += nb;
		else if (lenCur) {
			if (lenCur!=lenPrec) s_dTreeBL[lenCur].Freq++;
			s_dTreeBL[rep3_6].Freq++;
		}else if (nb<=10)
			s_dTreeBL[rep3_10].Freq++;
		else
			s_dTreeBL[rep11_138].Freq++;
		
		nb      = 0;
		lenPrec = lenCur;
		if       (lenSuiv==0) {
			maxNb = 138;
			minNb = 3;
		}else if (lenSuiv==lenCur) {
			maxNb = 6;
			minNb = 3;
		}else{
			maxNb = 7;
			minNb = 4;
		}
	}
}
#pragma warning (default: 4244)


// Les fils de aTas[n] sont aTas[2*n] et aTas[2*n+1]
// aTas[0] n'est pas utilis
// Le mme tableau de tas est utilis pour construire tous les arbres
static int e_aTas[2*nbcodeL+1];   // Tas utilis pour construire les arbres Huffman
static int e_lenTas;              // Nombre d'lments dans le tas
static int e_maxTas;              // Element de plus grande frquence

// Profondeur de chaque sous-arbre
static BYTE s_anProfond[2*nbcodeL+1];


static void trTasMAJ (CTDATA* pTree, int nParent);
static void trGenCode(CTDATA* pTree, int maxCode);
static void trGenBL  (TREEDESC& desc);



// Restaurer l'intgrit du tas en descendant dans l'arbre  partir d'un noeud,
// et en changeant chaque noeud avec le plus petit de ses deux enfants
// On s'arrte lorsque le parent est plus petit que ses deux enfants
void trTasMAJ(CTDATA* pTree, int nParent)
{
	// Comparer deux sous-arbres, en utilisant l'arbre s_anProfond
	// pour dpartager deux sous-arbres de mme frquence
	// et ainsi viter des dsastres
	#define IsSmaller(pTree, nEnfant1, nEnfant2) \
		(pTree[nEnfant1].Freq< pTree[nEnfant2].Freq || \
		(pTree[nEnfant1].Freq==pTree[nEnfant2].Freq && \
		 s_anProfond[nEnfant1]<=s_anProfond[nEnfant2]))
	
	const int iParent = e_aTas[nParent];
	for (;;) {
		
		int nEnfant = nParent<<1;
		if (nEnfant>e_lenTas)  break;
		
		// Rcuprer le plus petit des deux enfants
		if (nEnfant<e_lenTas &&
		    IsSmaller(pTree, e_aTas[nEnfant+1],e_aTas[nEnfant]))
			nEnfant++;
		
		// S'arrter si le parent est plus petit que ses deux enfants
		if (IsSmaller(pTree, iParent,e_aTas[nEnfant]))  break;
		
		// Exchanger le parent avec le plus petit des deux enfants
		// et poursuivre avec ce dernier
		e_aTas[nParent] = e_aTas[nEnfant];
		nParent = nEnfant;
	}
	e_aTas[nParent] = iParent;
	
	#undef IsSmaller
}



// Gnrer les codes et les nombres de bit d'un arbre donn
// * s_anbBL contient les statistiques de longueur bit de l'arbre
// * Len dfini pour tous les lments de l'arbre
// x Code dfini pour tous les lments de l'arbre
void trGenCode(CTDATA* pTree, int maxCode)
{
	int n;
	
	// Remplir le tableau des prochains codes
	// pour chaque longueur bit
	WORD codeSuiv[maxBits+1];
	WORD code = 0;
	n = 0;
	do{
		code = (WORD)((code + s_anbBL[n]) << 1);
		n++;
		codeSuiv[n] = code;
	}while (n<maxBits);
	
	ASSERT(code+s_anbBL[maxBits]-1==(1<<((WORD)maxBits))-1); // Inconsistent bit counts
	
	for (n=0; n<=maxCode; n++) {
		register int len = pTree[n].Len;
		if (len) {
			// Inverser les bits
			register WORD nCode = codeSuiv[len]++;
			register WORD nRes = 0;
			for (;;) {
				nRes  |= (nCode & 1);
				nCode >>= 1;
				if (!--len)  break;
				nRes  <<= 1;
			};
			pTree[n].Code = nRes;
		}
	}
}


// Calculer les longueurs bit optimales pour un abre et mettre  jour
// la longueur bit total pour le bloc courant
// * Freq et Dad dfinis
// * e_aTas tri par frquence
// x Len dfini avec la longueur bit optimale
// x s_anbBL contient les frquences pour chaque longueur bit
// x s_lenDyn mis  jour, s_lenSta aussi si l'arbre statique est non-nul
void trGenBL(TREEDESC& desc)
{
	ZeroMemory(s_anbBL, sizeof(s_anbBL));
	
	      CTDATA *const pTreeDyn   = desc.pTreeDyn;
	const CTDATA *const pTreeSta   = desc.pTreeSta;
	const WORD   *const aExtraBit  = desc.aExtraBit;
	const int           maxCode    = desc.maxCode;
	const int           nExtraBase = desc.nExtraBase;
	const WORD          maxLen     = desc.maxLen;
	
	WORD len;     // Longueur bit
	WORD nFreq;   // Frquence
	int  nTas;
	
	
	int i1, i2;
	
	// Premire passe
	// Calculer les longueurs bit optimales,
	// qui peuvent tre trop grandes dans le cas de l'arbre des longueurs bit
	pTreeDyn[e_aTas[e_maxTas]].Len = 0;   // Racine du tas
	
	int nbBLTropGrand = 0;   // Nombre d'lments ayant une trop grande longueur bit
	for (nTas=e_maxTas+1; nTas<lenTas; nTas++) {
		i1 = e_aTas[nTas];
		len = (WORD)(pTreeDyn[pTreeDyn[i1].Dad].Len + 1);
		if (len>maxLen) {
			len = maxLen;
			nbBLTropGrand++;
		}
		
		// On crase pTreeDyn[i1].Dad dont on n'a plus besoin
		pTreeDyn[i1].Len = len;
		
		if (i1>maxCode)  continue;   // Pas un noeud feuille
		
		s_anbBL[len]++;
		const int bitExtra = (i1<nExtraBase) ? 0 : aExtraBit[i1-nExtraBase];
		nFreq = pTreeDyn[i1].Freq;
		s_lenDyn += (DWORD)nFreq * (len + bitExtra);
		if (pTreeSta)  s_lenSta += (DWORD)nFreq * (pTreeSta[i1].Len + bitExtra);
	}
	if (!nbBLTropGrand)  return;
	
	// Trouver la premire longueur bit qu'on doit augmenter
	do{
		for (len=(WORD)(maxLen-1); !s_anbBL[len]; len--);
		s_anbBL[len]--;        // Descendre d'une feuille
		s_anbBL[len+1] += 2;   // Lui donner comme frre l'item en dpassement
		s_anbBL[maxLen]--;
		// Le frre de l'item en dpassement se dplace aussi,
		// mais cela n'affecte pas s_anbBL[maxLen]
		nbBLTropGrand -= 2;
	}while (nbBLTropGrand>0);
	
	// Recalculer toutes les longueurs bit dans l'ordre des frquences croissantes
	// nTas==lenTas au dbut
	for (len=maxLen; len; len--) {
		for (i1=s_anbBL[len]; i1; i1--) {
			do
				i2 = e_aTas[--nTas];
			while (i2>maxCode);
			if (pTreeDyn[i2].Len!=(UINT)len) {
				s_lenDyn += ((long)len-(long)pTreeDyn[i2].Len)*(long)pTreeDyn[i2].Freq;
				pTreeDyn[i2].Len = len;
			}
		}
	}
}


// Construire un arbre Huffman en assignant les chanes et longueur de code bit
// Mettre  jour la longueur bit totale pour le bloc courant
// * Freq dfini
// x Len et Code contiennent les longueurs bit optimales et le code correspondant
//   s_lenDyn et s_lenSta mis  jour, maxCode dfini
void trTreeBuild(TREEDESC& desc)
{
	      CTDATA *pTreeD = desc.pTreeDyn;
	const CTDATA *pTreeS = desc.pTreeSta;
	int maxElem          = desc.maxElem;
	int n, m;          // iterate over heap elements
	int maxCode = -1;  // largest code with non zero frequency

	// Construct the initial heap, with least frequent element in
	// e_aTas[nPlusPetit]. The sons of e_aTas[n] are e_aTas[2*n] and e_aTas[2*n+1].
	// e_aTas[0] is not used.
	e_lenTas = 0;
	e_maxTas = lenTas;
	
	for (n = 0; n < maxElem; n++) {
		if (pTreeD[n].Freq!=0) {
			e_aTas[++e_lenTas] = maxCode = n;
			s_anProfond[n] = 0;
		}else
			pTreeD[n].Len = 0;
	}
	
	// Le format PKZip demande qu'il y ait au moins un code de distance,
	// et qu'au moins un bit soit envoy mme s'il n'y a qu'un seul code
	// possible
	// Pour viter des vrifications spciales ultrieures on force
	// au moins deux codes  une frquence non-nulle
	while (e_lenTas < 2) {
		const int newT = e_aTas[++e_lenTas] = (maxCode<2) ? ++maxCode : 0;
		pTreeD[newT].Freq = 1;
		s_anProfond[newT] = 0;
		s_lenDyn--;
		if (pTreeS)  s_lenSta -= pTreeS[newT].Len;
	}
	desc.maxCode = maxCode;
	
	// les lments e_aTas[e_lenTas/2+1 .. e_lenTas]
	// sont les feuilles de l'arbre
	// L on tablit les sous-tas de longueur croissante
	for (n=e_lenTas/2; n>=1; n--)  trTasMAJ(pTreeD, n);
	
	// Construire l'arbre Huffman en conbinant  chaque fois
	// les deux noeuds les moins frquents
	int node = maxElem;
	do{
		// Enlever le plus petit lment du tas et recrer le tas
		// avec l'lment infrieure
		// Mettre  jour e_aTas et e_lenTas
		
		// n = noeud de frquence la plus basse
		n = e_aTas[nPlusPetit];
		e_aTas[nPlusPetit] = e_aTas[e_lenTas--];
		trTasMAJ(pTreeD, nPlusPetit);
		m = e_aTas[nPlusPetit];
		// m = noeud de la prochaine frquence la plus basse
		
		// Conserver l'ordre des noeuds (tris par frquence)
		e_aTas[--e_maxTas] = n;
		e_aTas[--e_maxTas] = m;
		
		// Crer un noeud pre de n et m...
		pTreeD[node].Freq = (WORD)(pTreeD[n].Freq + pTreeD[m].Freq);
		s_anProfond[node] = (BYTE) (__max(s_anProfond[n], s_anProfond[m]) + 1);
		pTreeD[n].Dad = pTreeD[m].Dad = (WORD)node;
		// ...et l'insrer dans le tas
		e_aTas[nPlusPetit] = node++;
		trTasMAJ(pTreeD, nPlusPetit);
	
	}while (e_lenTas >= 2);
	
	e_aTas[--e_maxTas] = e_aTas[nPlusPetit];
	
	//  ce point les champs Freq et Dad sont dfinis
	// on peut gnrer les longueurs bit
	trGenBL(desc);
	
	// Le champ Len est maintenant dfini:
	// on peut gnrer les codes bit
	trGenCode(pTreeD, maxCode);
}
