// Z/Huffman.h


#ifndef _Z_HUFFMAN_H_
#define _Z_HUFFMAN_H_
#if _MSC_VER>1000
#pragma once
#endif


#ifdef MUZIPEXE

// Les trois valeurs prcdentes dpendent du
// niveau de compression voulu (0~5)
// Les valeurs ci-dessous ont t proposes, mais de meilleures
// peuvent tre trouves pour certains types de fichiers
struct CONFIG
{
	WORD lenOK;     // Rduire la recherche lazy aprs cette longueur de correspondance
	WORD maxLazy;   // Ne pas effectuer de recherche lazy aprs cette longueur de correspondance
	WORD lenFin;    // Quitter la recherche aprs cette longueur de correspondance
	WORD maxChaine;
	int  vit;
	int  nFonction;
};

extern const CONFIG e_aConfig[7];

#endif


const int maxBits    =  15;  // Maximum pour tous les codes
const int maxBitsLen =   7;  // Maximum pour les codes de longueur
const int nbcodeLen  =  29;  // Nombre de codes de longueur, code spcial finBloc exclu
const int nbOctet    = 256;  // Nombre de caractres littraux possibles (0~255)
const int finBloc    = 256;  // Code de fin d'un bloc de caractres littraux
const int nbcodeL    = (nbOctet+1+nbcodeLen);  // Nombre de littraux ou de codes de longueur, code finBloc inclu
const int nbcodeD    =  30;  // Nombre de codes de distance
const int nbcodeBL   =  19;  // Nombre de codes de longueur bit

// Bits supplmentaires pour chaque code de longueur, distance et longueur bit
extern const WORD e_aExtraL [nbcodeLen+2];
extern const WORD e_aExtraD [nbcodeD];
#ifdef MUZIPEXE
extern const WORD e_aExtraBL[nbcodeBL];
#endif

// Les codes des longueurs bit sont envoyes par ordre de probabilit
// dcroissante, pour viter la transmission de longueurs de des codes
// de longueur bit inutiliss
extern const BYTE e_aOrdreBL[nbcodeBL];


// Stockage d'une valeur et de sa chane de codes
struct CTDATA
{
	union
	{
		WORD Freq;   // Frquence
		WORD Code;   // Chane de bits
	};
	union
	{
		WORD Dad;    // Noeud frre dans l'arbre Huffman
		WORD Len;    // Longueur de la chane de bits
	};
};


// Description d'arbres dynamique et statique
struct TREEDESC
{
	      CTDATA *pTreeDyn;     // L'arbre dynamique
	const CTDATA *pTreeSta;     // Arbre static correspondant ou NULL
	const WORD   *aExtraBit;    // Bits extra pour chaque code ou NULL
	int           nExtraBase;   // Index de base pour aExtraBit
	int           maxElem;      // Maximum d'lments dans l'arbre
	WORD          maxLen;       // Longueur de bit maximum des codes
	int           maxCode;      // Plus grand code de frquence non-nulle
};


#endif
