// Z/UnZ.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#ifdef MUZIPEXE
#include "../CodeRapide.h"
#include "../Muzip.h"
#include "ZArc.h"
#endif
#include "AppResource.h"
#include "Crypt.h"
#include "Huffman.h"

#include "../CRC32.h"

#include "../Zip/Zip.h"
#include "../Zip/ZipFichier.h"
#ifdef MUZIPEXE
#include "../GZip/GZipFichier.h"
#endif

#include "Cons.h"
#include "UnZ.h"
#include "../Decomp.h"
using namespace DECOMP;

#ifdef MUZIPEXE
#include "../IOFichier.h"

#include "../dlgMdp.h"
#endif


enum HUFTCONSTRUIRE
{
	hcOK,
	hcErreur,
	hcErreurNettoyer,
};


class CHuft
{
public:  // Fonctions
	
	void Free();
	
	static HUFTCONSTRUIRE Construire(const UINT* b, UINT n, UINT s,
		const WORD* d, const WORD* e, CHuft** pTree, int* m);
	
	
public:  // Donnes
	
	BYTE nbExtra;    // Nombre de bits extra ou opration
	BYTE nbBits;     // Nombre de bits dans ce code ou sous-code
	union
	{
		WORD n;       // Littral, base de longueur, ou base de distance
		CHuft *pSuiv;   // Pointeur sur le niveau suivant du tableau
	};
};


#ifdef    _DEBUG
static UINT     s_nbHuft;         // Nombre d'arbres allous
#endif // _DEBUG

static CHuft *s_tlFixed = NULL;
static CHuft *s_tdFixed;
static int    s_blFixed;
static int    s_bdFixed;
static UINT   s_posSlide;   // Position courante dans s_abSlide
static DWORD  s_bBuf;       // Buffer de bits
static UINT   s_lenbBuf;    // Bits dans le buffer de bits

static BYTE   s_abSlide[lenFen];
static int    s_lenInReste;
static BYTE * s_pbInReste;


static bool   s_fFlushOK;


static const int lbits = 9;    // bits in base literal/length lookup table
static const int dbits = 6;    // bits in base distance lookup table


// If maxCodeB needs to be larger than 16, then h and x[] should be ulg.
const UINT maxCodeB  =  16;   // maximum bit length of any code (16 for explode)
const UINT maxCodeNb = 288;   // maximum number of codes in any set

// Longueurs de copie chaque code de longueur et distance
static const WORD s_aCopyL[nbcodeL] =
{
	3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 15, 17, 19, 23, 27, 31,
	35, 43, 51, 59, 67, 83, 99, 115, 131, 163, 195, 227, 258, 0, 0
};
static const WORD s_aCopyD[nbcodeD] =
{
	1, 2, 3, 4, 5, 7, 9, 13, 17, 25, 33, 49, 65, 97, 129, 193, 257, 385,
	513, 769, 1025, 1537, 2049, 3073, 4097, 6145, 8193, 12289, 16385, 24577
};


// Masques pour toutes les plages de bits 0~n (0<=n<=16)
static const WORD s_aMask[] =
{
	0x0000, 0x0001, 0x0003, 0x0007, 0x000F, 0x001F, 0x003F, 0x007F, 0x00FF,
	0x01FF, 0x03FF, 0x07FF, 0x0FFF, 0x1FFF, 0x3FFF, 0x7FFF, 0xFFFF
};

const WORD wInvalide = 112;


#ifdef MUZIPEXE
#define CALLONREAD(len)  OnRead(len)
#else
#define CALLONREAD(len)  (void)0
#endif


#define BITS_FAUT(n) \
	{ \
		while (k<(n)) { \
			BYTE bRead; \
			e_pfr->Read(bRead); \
			CALLONREAD(1); \
			if (e_fCrypte)  bRead = keyMAJ((BYTE)(bRead ^ cryptDecrypter())); \
			b |= ((DWORD)bRead)<<k; \
			k += 8; \
		} \
	}
#define BITS_OK(n) \
	{ \
		b >>= (n); \
		k  -= (n); \
	}
#define FLUSH(len)    Flush(s_abSlide, len)


long  UNZ::e_lenComp;
BYTE *UNZ::e_pbIn;
int   UNZ::e_lenIn;


#ifdef MUZIPEXE

// Extraire un fichier Zip
bool UNZ::ExtraireZip(CZipFichier* pzf, LPCTSTR pszFichierOut)
{
	bool fOK = false;
	s_fFlushOK = true;
	
	// Ouvrir le fichier de sortie
	if (e_dm==dmNormal) {
		
		// Crer le fichier avec ses attributs
		SetFileAttributes(pszFichierOut, 0);   // Enlever lecture seule
		if (!e_fw.Open(pszFichierOut, 0, CREATE_ALWAYS, pzf->m_dwAttr))  goto Fin;
		
		// Dfinir la date de modification du fichier
		const FILETIME ftM = pzf->m_dtM;
		SetFileTime(e_fw, NULL, NULL, &ftM);
	}
	
	if (pzf->m_nFlags & locfCrypte) {
		if (pzf->m_pArc->m_sMdp.IsEmpty()) {
			// Demander le mode de passe
			if (CDlgMdp(pzf->m_pArc->m_sMdp, true).DoModal()==IDCANCEL)
				goto Fin;
		}
		keyInit(pzf->m_pArc->m_sMdp);
	}else
		e_fCrypte = false;
	
	
	e_pfr->Seek(pzf->m_off+sizeof(HEADERLOCSIG)+pzf->m_lenNom+pzf->m_lenExtra, FILE_BEGIN);
	
	// La vrification de limites d'OctetSuivant doit pouvoir marcher
	InputDeferLeftover();
	
	// Vrifier la validit du code de cryptage
	if (e_fCrypte) {
		// Rcuprer le mot de passe et prparer cls du fichier Zip
		int n;
		BYTE abEnTete[lenCryptEnTete];
		
		// Rcuprer l'en-tte une fois
		// (vider le flag "crypter" temporairement
		// pour ne pas dcrypter deux fois les mmes donnes
		e_fCrypte = false;
		InputDeferLeftover();
		BYTE b;
		for (n=0; n<lenCryptEnTete; n++) {
			e_pfr->Read(b);
			abEnTete[n] = keyMAJ((BYTE)(b ^ cryptDecrypter()));
		}
		InputUndefer();
		e_fCrypte = true;
		
		// Tester la cl
		if (!pzf->m_pArc->m_sMdp.IsEmpty() || !TesterMDP(abEnTete, pzf)) {
			// Afficher un message d'erreur
			// Demander Annuler tous
			AfxFormatString1(e_sMsg, ERR_MDP, pzf->m_pszNom);
			AfxMessageBox(e_sMsg);
			goto Fin;
		}
	}
	
	switch (pzf->m_nMethode) {
		case zmStore:
			{
				DWORD dwReste = pzf->m_dwTailleC;
				
				if (!(pzf->m_nFlags & locfCrypte)) {
					// Trs simple pour les fichiers non-crypts !
					// Une simple recopie d'un fichier  l'autre
					
					fOK = true;
					while (dwReste) {
						const DWORD dwES = (dwReste<sizeof(s_abSlide))
							? dwReste : sizeof(s_abSlide);
						if (!e_pfr->Read(s_abSlide, dwES) || !Flush(s_abSlide, dwES)) {
							fOK = false;
							break;
						}
						CALLONREAD(dwES);
						dwReste -= dwES;
					}
					
				}else{
					// Plus compliqu ici: il faut dcrypter
					
					BYTE b;
					BYTE *pbOut  = s_abSlide;
					DWORD lenOut = 0;
					e_pfr->SetVerbeux(false);
					while (dwReste) {
						if (!e_pfr->Read(b))  break;
						CALLONREAD(1);
						*pbOut++ = keyMAJ((BYTE)(b ^ cryptDecrypter()));
						dwReste--;
						if (++lenOut==lenFen) {
							fOK = Flush(s_abSlide, lenOut);
							pbOut  = s_abSlide;
							lenOut = 0;
							if (!fOK)  break;
						}
					}
					e_pfr->SetVerbeux();
					fOK = (fOK && dwReste)
						? false  // Fichier tronqu
						: Flush(s_abSlide, lenOut);
				}
			}
			break;
		
		case zmDeflate:
			fOK = Inflate();
			break;
	}
	
	InputUndefer();
	
	if (!fOK && s_fFlushOK) {
		AfxFormatString1(e_sMsg, ERR_DECOMP, pzf->m_pszNom);
		AfxMessageBox(e_sMsg);
	}
	
Fin:
	e_fw.Close();
	return fOK;
}

#else

// Extraire un fichier Zip
bool UNZ::ExtraireZip(LPCTSTR pszFichier, LPCTSTR pszFichierOut)
{
	bool fOK = s_fFlushOK = true;
	
	// Crer le fichier avec ses attributs
	SetFileAttributes(pszFichierOut, 0);   // Enlever lecture seule
	if (!e_fw.Open(pszFichierOut, 0, CREATE_ALWAYS, phCenSig->nAttrE))  return false;
	
	// Dfinir la date de modification du fichier
	FILETIME ft;
	const DWORD dt = phCenSig->dt;
	DosDateTimeToFileTime((WORD)(dt>>16), (WORD)dt, &ft);
	SetFileTime(e_fw, NULL, NULL, &ft);
	
	WORD nFlags = phCenSig->nFlags;
	
	// Demander le mot de passe si ncessaire
	if (nFlags & locfCrypte) {
		// Demander le mode de passe
		if (!*e_pszMdp && !DemanderMDP())  goto Fin;
		keyInit(e_pszMdp);
	}else
		e_fCrypte = false;
	
	
	if (nFlags & locfCrypte)
		keyInit(e_pszMdp);
	else
		e_fCrypte = false;
	
	// Se positionner  l'en-tte central
	// pour lire la longueur du champ extra
	e_pfr->Seek(phCenSig->off+lenSig+loclenExtra, FILE_BEGIN);
	WORD lenExtra;
	if (!e_pfr->Read(lenExtra))  goto Fin;
	e_pfr->Seek(phCenSig->off+sizeof(HEADERLOCSIG)+phCenSig->lenNom+lenExtra, FILE_BEGIN);
	
	// La vrification de limites d'OctetSuivant doit pouvoir marcher
	InputDeferLeftover();
	
	
	// Vrifier la validit du code de cryptage
	if (e_fCrypte) {
		// Rcuprer le mot de passe et prparer cls du fichier Zip
		int n;
		BYTE abEnTete[lenCryptEnTete];
		
		// Rcuprer l'en-tte une fois
		// (vider le flag "crypter" temporairement
		// pour ne pas dcrypter deux fois les mmes donnes
		e_fCrypte = false;
		InputDeferLeftover();
		BYTE b;
		for (n=0; n<lenCryptEnTete; n++) {
			e_pfr->Read(b);
			abEnTete[n] = keyMAJ((BYTE)(b ^ cryptDecrypter()));
		}
		InputUndefer();
		e_fCrypte = true;
		
		// Tester la cl
		if (!*e_pszMdp || !TesterMDP(abEnTete)) {
			// Afficher un message d'erreur
			// Demander Annuler tous
			Formatter(ERR_MDP, pszFichier);
			AfxMessageBox(e_psz);
			goto Fin;
		}
	}
	
	switch (phCenSig->nMethode) {
		case zmStore:
			{
				DWORD dwReste = phCenSig->nTailleC;
				
				if (!(phCenSig->nFlags & locfCrypte)) {
					// Trs simple pour les fichiers non-crypts !
					// Une simple recopie d'un fichier  l'autre
					
					fOK = false;
					for (;;) {
						if (!dwReste) {
							fOK = true;
							break;
						}
						const DWORD dwES = (dwReste<sizeof(s_abSlide))
							? dwReste : sizeof(s_abSlide);
						if (!e_pfr->Read(s_abSlide, dwES) || !Flush(s_abSlide, dwES))  break;
						CALLONREAD(dwES);
					}
					
				}else{
					// Plus compliqu ici: il faut dcrypter
					
					BYTE b;
					BYTE *pbOut  = s_abSlide;
					DWORD lenOut = 0;
					e_pfr->SetVerbeux(false);
					while (dwReste) {
						if (!e_pfr->Read(b))  break;
						CALLONREAD(1);
						*pbOut++ = keyMAJ((BYTE)(b ^ cryptDecrypter()));
						dwReste--;
						if (++lenOut==lenFen) {
							fOK = Flush(s_abSlide, lenOut);
							pbOut  = s_abSlide;
							lenOut = 0;
							if (!fOK)  break;
						}
					}
					e_pfr->SetVerbeux();
					fOK = (fOK && dwReste)
						? false  // Fichier tronqu
						: Flush(s_abSlide, lenOut);
				}
			}
			break;
		
		case zmDeflate:
			fOK = Inflate();
			break;
	}
	
	InputUndefer();
	
	if (!fOK && s_fFlushOK) {
		Formatter(ERR_DECOMP, pszFichier);
		AfxMessageBox(e_psz);
	}
	
Fin:
	e_fw.Close();
	return fOK;
}

#endif


#ifdef MUZIPEXE


// Extraire un fichier GZip
bool UNZ::ExtraireGZip(CGZipFichier* pgzf, LPCTSTR pszFichierOut)
{
	s_fFlushOK = true;
	bool fOK = true;
	
	// Ouvrir le fichier de sortie
	if (e_dm==dmNormal) {
		
		// Crer le fichier avec ses attributs
		SetFileAttributes(pszFichierOut, 0);
		if (!e_fw.Open(pszFichierOut, 0))  goto Fin;
		
		// Dfinir la date de modification du fichier
		const FILETIME ftM = pgzf->m_dtM;
		SetFileTime(e_fw, NULL, NULL, &ftM);
	}
	
	e_fCrypte = false;
	
	e_pfr->Seek(pgzf->m_off, FILE_BEGIN);
	
	// Dcompresser
	InputDeferLeftover();
	fOK = Inflate();
	InputUndefer();
	
	if (!fOK && s_fFlushOK) {
		AfxFormatString1(e_sMsg, ERR_DECOMP, pgzf->m_pszNom);
		AfxMessageBox(e_sMsg);
	}
	
Fin:
	e_fw.Close();
	return fOK;
}

#endif

// Flusher la sortie
bool UNZ::Flush(const void* pv, DWORD len)
{
	if (!len)            return true;
#ifdef MUZIPEXE
	if (e_dm==dmTester)  return true;
#endif
	VERIF(s_fFlushOK);
	
#ifdef MUZIPEXE
	if (DECOMP::Flush((const BYTE*)pv, len))  return true;
#else
	if (e_fw.Write(pv, len))  return true;
#endif
	
	return s_fFlushOK = false;
}


// Initialiser
bool UNZ::InflateInit()
{
	int  i;
	UINT aL[288];
	
	// Initialiser les arbres fixes
	
	// Table des littraux
	// Crer un jeu de codes complet mais erron
	i = 0;
	while (i<144)  aL[i++] = 8;
	while (i<256)  aL[i++] = 9;
	while (i<280)  aL[i++] = 7;
	while (i<288)  aL[i++] = 8;
	s_blFixed = 7;
	if (CHuft::Construire(aL, 288, 257, s_aCopyL, e_aExtraL, &s_tlFixed, &s_blFixed)) {
Erreur:
		s_tlFixed = NULL;
		return false;
	}
	
	// Table des distances
	// Crer un jeu de codes incomplet
	for (i=0; i<nbcodeD; i++)  aL[i] = 5;
	s_bdFixed = 5;
	if (CHuft::Construire(aL, nbcodeD, 0, s_aCopyD, e_aExtraD, &s_tdFixed, &s_bdFixed)==hcErreur) {
		s_tlFixed->Free();
		s_tdFixed = NULL;
		goto Erreur;
	}
	
	return true;
}


// Nettoyer
void UNZ::InflateClose()
{
	if (s_tdFixed) {
		s_tdFixed->Free();
		s_tlFixed->Free();
	}
}


// Librer les tableaux allous et construits par CHuft::Construire()
void CHuft::Free()
{
	CHuft *pTree = this;
	while (pTree) {
		CHuft *const pTreeSauve = (--pTree)->pSuiv;
		delete [] pTree;
		pTree = pTreeSauve;
	}
}


void UNZ::InputDeferLeftover()
{
	if ((long)e_lenIn>e_lenComp) {
		if (e_lenComp<0)  e_lenComp = 0;
		s_pbInReste  = e_pbIn + (int)e_lenComp;
		s_lenInReste = e_lenIn - (int)e_lenComp;
		e_lenIn = (int)e_lenComp;
	}else
		s_lenInReste = 0;
	e_lenComp -= e_lenIn;
}


void UNZ::InputUndefer()
{
	if (e_lenIn>0)  e_lenComp += e_lenIn;
	if (s_lenInReste>0) {
		// We know that "(csize<MAXINT)" so we can cast csize to int:
		// This condition was checked when s_lenInReste was set>0 in
		// InputDeferLeftover(), and it is NOT allowed to touch csize
		// before calling InputUndefer() when (s_lenInReste>0)
		// (single exception: see read_byte()'s  "csize<=0" handling) !!
		
		e_lenIn = s_lenInReste + (int)e_lenComp;
		e_pbIn  = s_pbInReste  - (int)e_lenComp;
		s_lenInReste = 0;
	}else if (e_lenIn<0)
		e_lenIn = 0;
}


bool UNZ::Inflate()
{
#ifdef _DEBUG
	UINT maxHuft = 0;       // maximum CHuft's allous
#endif
	
	// Initialiser la fentre et le tampon de bits
	s_posSlide = 0;
	s_lenbBuf = 0;
	s_bBuf = 0;
	
	// Dcompresser jusqu'au dernier bloc
	bool fDernier;
	do{
#ifdef    _DEBUG
		s_nbHuft = 0;
#endif // _DEBUG
		
		{
			// Utiliser des variables locales
			register DWORD b = s_bBuf;   // Tampon de bits
			register UINT  k = s_lenbBuf;   // Nombre de bits dans le tampon
			
			// Lire le bit "dernier bloc"
			BITS_FAUT(1);
			fDernier = ToBool((int)b & 1);
			BITS_OK(1);
			
			// Lire le type du bloc
			BITS_FAUT(2);
			UINT t = (UINT)b & 3;
			BITS_OK(2);
			
			// Mettre  jour les variables globales
			s_bBuf    = b;
			s_lenbBuf = k;
			
			// Inflater selon le type du bloc
			if       (t==2) {
				VERIF(InflateDynamic());
			}else if (t==0) {
				VERIF(InflateStore());
			}else if (t==1) {
				VERIF(InflateCode(s_tlFixed, s_tdFixed, s_blFixed, s_bdFixed));
			}else
				return false;
		}
		
#ifdef    _DEBUG
		if (s_nbHuft>maxHuft)  maxHuft = s_nbHuft;
#endif // _DEBUG
		
	}while (!fDernier);
	
	// Flusher s_abSlide et retourner (succs sauf si le FLUSH final choue)
	return FLUSH(s_posSlide);
}


// "decompress" an inflated type 0 (stored) block.
bool UNZ::InflateStore()
{
	// Utiliser des variables locales
	register DWORD b = s_bBuf;      // Tampon de bits
	register UINT  k = s_lenbBuf;   // Nombre de bits dans le tampon
	UINT w = s_posSlide;            // Position dans la fentre
	
	// Aligner sur un octet
	UINT n = k & 7;           // Nombre d'octets dans le bloc
	BITS_OK(n);
	
	// Rcuprer la longueur et son complment
	BITS_FAUT(16);
	n = ((UINT)b & 0xFFFF);
	BITS_OK(16);
	BITS_FAUT(16);
	VERIF(n==(UINT)((~b) & 0xFFFF));
	BITS_OK(16);
	
	// Lire et mettre en sortie les donnes compresses
	while (n--) {
		BITS_FAUT(8);
		s_abSlide[w++] = (BYTE)b;
		if (w==lenFen) {
			VERIF(FLUSH(w));
			w = 0;
		}
		BITS_OK(8);
	}
	
	// Mettre  jour les variables globales
	s_posSlide = w;
	s_bBuf = b;
	s_lenbBuf = k;
	
	return true;
}


// Dcompresse un bloc inflat de type 2 (codes Huffman dynamiques)
bool UNZ::InflateDynamic()
{
	int   i;       // temporary variables
	UINT  j;
	UINT  l;       // last length
	UINT  m;       // mask for bit lengths table
	UINT  n;       // number of lengths to get
	CHuft *tl;      // literal/length code table
	CHuft *td;      // distance code table
	int   bl;      // lookup bits for tl
	int   bd;      // lookup bits for td
	UINT  nb;      // number of bit length codes
	UINT  nl;      // number of literal/length codes
	UINT  nd;      // number of distance codes
#ifdef     PKZIP_BUG_WORKAROUND
	UINT  ll[288+32]; // literal/length and distance code lengths
#else  // !PKZIP_BUG_WORKAROUND
	UINT  ll[nbcodeD+nbcodeL]; // literal/length and distance code lengths
#endif // ?PKZIP_BUG_WORKAROUND
	
	
	// Utiliser des variables locales
	register DWORD b = s_bBuf;   // Tampon de bits
	register UINT  k = s_lenbBuf;   // Nombre de bits dans le tampon
	
	// read in table lengths
	BITS_FAUT(5);
	nl = 257 + ((UINT)b & 0x1F);      // number of literal/length codes
	BITS_OK(5);
	BITS_FAUT(5);
	nd = 1 + ((UINT)b & 0x1F);        // number of distance codes
	BITS_OK(5);
	BITS_FAUT(4);
	nb = 4 + ((UINT)b & 0x0F);         // number of bit length codes
	BITS_OK(4);
#ifdef PKZIP_BUG_WORKAROUND
	if (nl>288 || nd>32)
#else
	if (nl>nbcodeL || nd>nbcodeD)
#endif
		return false;
	
	
	// read in bit-length-code lengths
	for (j=0; j<nb; j++) {
		BITS_FAUT(3);
		ll[e_aOrdreBL[j]] = (UINT)b & 7;
		BITS_OK(3);
	}
	for (; j<19; j++)
		ll[e_aOrdreBL[j]] = 0;
	
	
	HUFTCONSTRUIRE hc;
	
	// build decoding table for trees--single level, 7 bit lookup
	bl = 7;
	hc = CHuft::Construire(ll, 19, 19, NULL, NULL, &tl, &bl);
	if (!bl)  hc = hcErreurNettoyer;
	if (hc) {
		if (hc==hcErreurNettoyer)  tl->Free();
		return false;
	}
	
	
	// read in literal and distance code lengths
	n = nl + nd;
	m = s_aMask[bl];
	i = l = 0;
	while ((UINT)i < n) {
		BITS_FAUT((UINT)bl);
		j = (td = tl + ((UINT)b & m))->nbBits;
		BITS_OK(j);
		j = td->n;
		if (j < 16)                 // length of code in bits (0..15)
			ll[i++] = l = j;          // save last length in l
		else if (j==16) {         // repeat last length 3 to 6 times
			BITS_FAUT(2);
			j = 3 + ((UINT)b & 3);
			BITS_OK(2);
			if ((UINT)i+j>n)  return false;
			while (j--)  ll[i++] = l;
		}else if (j==17) {         // 3 to 10 zero length codes
			BITS_FAUT(3);
			j = 3 + ((UINT)b & 7);
			BITS_OK(3);
			if ((UINT)i+j>n)  return false;
			while (j--)
				ll[i++] = 0;
			l = 0;
		}else{
			// j==18: 11 to 138 zero length codes
			BITS_FAUT(7);
			j = 11 + ((UINT)b & 0x7F);
			BITS_OK(7);
			if ((UINT)i+j>n)  return false;
			while (j--)  ll[i++] = 0;
			l = 0;
		}
	}
	
	
	// Librer la table de dcodate pour les arbres
	tl->Free();
	
	
	// Mettre  jour les variables globales
	s_bBuf = b;
	s_lenbBuf = k;
	
	
	// build the decoding tables for literal/length and distance codes
	bl = lbits;
	hc = CHuft::Construire(ll, nl, 257, s_aCopyL, e_aExtraL, &tl, &bl);
	if (bl==0)  hc = hcErreurNettoyer;
	if (hc) {
		if (hc==hcErreurNettoyer)  tl->Free();
		return false;
	}
	bd = dbits;
	hc = CHuft::Construire(ll + nl, nd, 0, s_aCopyD, e_aExtraD, &td, &bd);
#ifdef PKZIP_BUG_WORKAROUND
	if (hc==hcErreurNettoyer)  hc = hcOK;
#endif
	if (bd==0 && nl>257)  hc = hcErreurNettoyer;
	if (hc) {
		if (hc==hcErreurNettoyer)  td->Free();
		tl->Free();
		return false;
	}
	
	// decompress until an end-of-block code
	const bool fOK = InflateCode(tl, td, bl, bd);
	
	// free the decoding tables, return
	tl->Free();
	td->Free();
	return fOK;
}



bool UNZ::InflateCode(CHuft* tl, CHuft* td, int bl, int bd)
{
	register UINT e;  // table entry flag/number of extra bits
	UINT n, d;        // length and index for copy
	UINT w;           // current window position
	CHuft *t;       // pointer to table entry
	UINT ml, md;      // masks for bl and bd bits
	register DWORD b;       // bit buffer
	register UINT k;  // number of bits in bit buffer
	
	
	// make local copies of globals
	b = s_bBuf;                       // initialize bit buffer
	k = s_lenbBuf;
	w = s_posSlide;                       // initialize window position
	
	
	// inflate the coded data
	ml = s_aMask[bl];           // precompute masks for speed
	md = s_aMask[bd];
	for (;;) {                     // do until end of block
		BITS_FAUT((UINT)bl)
		if ((e = (t = tl + ((UINT)b & ml))->nbExtra) > 16)
			do{
				VERIF(e!=wInvalide);
				BITS_OK(t->nbBits);
				e -= 16;
				BITS_FAUT(e);
			}while ((e = (t = t->pSuiv + ((UINT)b & s_aMask[e]))->nbExtra) > 16);
		BITS_OK(t->nbBits);
		if (e==16) {
			// then it's a literal
			s_abSlide[w++] = (BYTE)t->n;
			if (w==lenFen) {
				VERIF(FLUSH(w));
				w = 0;
			}
		}else{
			// it's an EOB or a length
			
			// exit if end of block
			if (e==15)  break;
			
			// get length of block to copy
			BITS_FAUT(e)
			n = t->n + ((UINT)b & s_aMask[e]);
			BITS_OK(e);
			
			// decode distance of block to copy
			BITS_FAUT((UINT)bd)
			if ((e = (t = td + ((UINT)b & md))->nbExtra) > 16)
				do{
					VERIF(e!=wInvalide);
					BITS_OK(t->nbBits);
					e -= 16;
					BITS_FAUT(e);
				}while ((e = (t = t->pSuiv + ((UINT)b & s_aMask[e]))->nbExtra) > 16);
			BITS_OK(t->nbBits);
			BITS_FAUT(e);
			d = w - t->n - ((UINT)b & s_aMask[e]);
			BITS_OK(e);
			
			// do the copy
			do{
				n -= (e = (e = lenFen - ((d &= lenFen-1) > w ? d : w)) > n ? n : e);
				if (w-d>=e) {       // (this test assumes UINT comparison)
					memcpy(s_abSlide + w, s_abSlide + d, e);
					w += e;
					d += e;
				}else{                   // do it slowly to avoid memcpy() overlap
					do{
						s_abSlide[w++] = s_abSlide[d++];
					}while (--e);
				}
				if (w==lenFen) {
					VERIF(FLUSH(w));
					w = 0;
				}
			}while (n);
		}
	}
	
	
	// Mettre  jour les variables globales
	s_posSlide = w;
	s_bBuf = b;
	s_lenbBuf = k;
	
	return true;
}



// Given a list of code lengths and a maximum table size, make a set of
// tables to decode that set of codes.  Return zero on success, one if
// the given code set is incomplete (the tables are still built in this
// case), two if the input is invalid (all zero length codes or an
// oversubscribed set of lengths), and three if not enough memory.
// The code with value 256 is special, and the tables are constructed
// so that no bits beyond that code are fetched when that code is
// decoded.
//  b - code lengths in bits (all assumed <= maxCodeB)
//  n - number of codes (assumed <= maxCodeNb)
//  s - number of simple-valued codes (0..s-1)
//  d - list of base values for non-simple codes
//  e - list of extra bits for non-simple codes
//  t - result: starting table
//  m - maximum lookup bits, returns actual
#pragma warning (disable: 4701)
HUFTCONSTRUIRE CHuft::Construire(const UINT* b, UINT n, UINT s,
	const WORD* d, const WORD* e, CHuft** t, int* m)
{
	UINT  a;                   // counter for codes of length k
	UINT  c[maxCodeB+1];           // bit length count table
	UINT  el;                  // length of EOB code (value 256)
	UINT  f;                   // i repeats in table every f entries
	int g;                        // maximum code length
	int h;                        // table level
	register UINT i;          // counter, current code
	register UINT j;          // counter
	register int k;               // number of bits in current code
	int lx[maxCodeB+1];               // memory for l[-1..maxCodeB-1]
	int *l = lx+1;                // stack of bits per table
	register UINT *p;         // pointer into c[], b[], or v[]
	register CHuft *q;      // points to current table
	CHuft r;                // table entry for structure assignment
	CHuft *u[maxCodeB];         // table stack
	UINT v[maxCodeNb];            // values in order of bit length
	register int w;               // bits before this table==(l * h)
	UINT x[maxCodeB+1];           // bit offsets, then code stack
	UINT *xp;                 // pointer into x
	int y;                        // number of dummy codes added
	UINT z;                   // number of entries in current table
	
	
	// Generate counts for each bit length
	el = (n>256) ? b[256] : maxCodeB; // set length of EOB code, if any
	
	c[ 0] = c[ 1] = c[ 2] = c[ 3] =
	c[ 4] = c[ 5] = c[ 6] = c[ 7] =
	c[ 8] = c[ 9] = c[10] = c[11] =
	c[12] = c[13] = c[14] = c[15] =
	c[16] = 0;
	
	p = (UINT*)b;
	i = n;
	do{
		c[*p]++;   // assume all entries <= maxCodeB
		p++;
	}while (--i);
	if (c[0]==n) {    // null input--all zero length codes
		*t = NULL;
		*m = 0;
		return hcOK;
	}
	
	// Find minimum and maximum length, bound *m by those
	for (j=1; j<=maxCodeB; j++)
		if (c[j])  break;
	k = j;                        // minimum code length
	if ((UINT)*m<j)  *m = j;
	for (i=maxCodeB; i; i--)
		if (c[i])  break;
	g = i;                        // maximum code length
	if ((UINT)*m>i)  *m = i;
	
	// Adjust last length count to fill out codes, if needed
	for (y=1<<j; j<i; j++, y<<=1)
		if ((y -= c[j])<0)  return hcErreur;
	if ((y -= c[i])<0)    return hcErreur;
	c[i] += y;
	
	// Generate starting offsets into the value table for each length
	x[1] = j = 0;
	p = c + 1;  xp = x + 2;
	while (--i)                 // note that i==g from above
		*xp++ = (j += *p++);
	
	// Make a table of values in order of bit lengths
	p = (UINT*)b;  i = 0;
	do{
		j = *p++;
		if (j)  v[x[j]++] = i;
	}while (++i < n);
	n = x[g];                     // set n to length of v
	
	// Generate the Huffman codes and for each, make the table entries
	x[0] = i = 0;                 // first Huffman code is zero
	p = v;                        // grab values in bit order
	h = -1;                       // no tables yet--level -1
	w = l[-1] = 0;                // no bits decoded yet
	
	// go through the bit lengths (k already is bits in shortest code)
	for (; k <= g; k++) {
		a = c[k];
		while (a--) {
			// here i is the Huffman code of length k bits for value *p
			// make tables up to required level
			while (k>w+l[h]) {
				w += l[h++];            // add bits already decoded
				
				// compute minimum size table less than or equal to *m bits
				z = g - w;
				z = (z>(UINT)*m) ? *m : z;        // upper limit
				if ((f = 1<<(j=k-w))>a+1) {   // try a k-w bit table
				                             // too few codes for k-w bit table
					f -= a + 1;           // deduct codes from patterns left
					xp = c + k;
					while (++j < z) {     // try smaller tables up to z bits
						if ((f <<= 1) <= *++xp)
							break;            // enough codes to use up j bits
						f -= *xp;           // else deduct codes from patterns
					}
				}
				if ((UINT)w+j>el && (UINT)w<el)
					j = el - w;           // make EOB code end at table
				z = 1 << j;             // table entries for j-bit table
				l[h] = j;               // set table size in stack
				
				// allocate and link in new table
				q = new CHuft[z+1];
				
#ifdef DEBUG
				s_nbHuft += z + 1;         // track memory usage
#endif
				*t = q + 1;             // link to list for CHuft::Free()
				*(t = &(q->pSuiv)) = NULL;
				u[h] = ++q;             // table starts after link
				
				// connect to last table, if there is one
				if (h) {
					x[h] = i;             // save pattern for backing up
					r.nbBits  = (BYTE)l[h-1];    // bits to dump before this table
					r.nbExtra = (BYTE)(16 + j);  // bits in this table
					r.pSuiv   = q;            // pointer to this table
					j = (i & ((1 << w) - 1)) >> (w - l[h-1]);
					u[h-1][j] = r;        // connect to last table
				}
			}
			
			// set up table entry in r
			r.nbBits = (BYTE)(k - w);
			if (p >= v + n)
				r.nbExtra = wInvalide;               // out of values--invalid code
			else if (*p < s) {
				r.nbExtra = (*p<256) ? (BYTE)16 : (BYTE)15;  // 256 is end-of-block code
				r.n = (WORD)*p++;                // simple code is just the value
			}else{
				r.nbExtra = (BYTE)e[*p - s];   // non-simple--look up in lists
				r.n = d[*p++ - s];
			}
			
			// fill code-like entries with r
			f = 1<<(k-w);
			for (j=i>>w; j<z; j+=f)  q[j] = r;
			
			// backwards increment the k-bit code i
			for (j=1<<(k-1); i & j; j>>=1)  i ^= j;
			i ^= j;
			
			// backup over finished tables
			while ((i & ((1 << w) - 1))!=x[h])
				// On n'a pas besoin de mettre  jour q
				w -= l[--h];
		}
	}
	
	// return actual size of base table
	*m = l[0];
	
	// Return true if we were given an incomplete table
	return (y && g!=1) ? hcErreurNettoyer : hcOK;
}
#pragma warning (default: 4701)
