// Zip/ZipArc.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "../Muzip.h"
#include "../Muzip2.h"
#include "ZipArc.h"
#include "ZipFichier.h"
#include "Zip.h"

#include "../Z/Crypt.h"
#include "../Z/Huffman.h"

#include "../FichierPlus.h"
#include "../Options.h"
#include "../Exceptions.h"

#include "../IOFichier.h"

#include <GUnicode.h>
#include <FileBuf.h>
#include <sys/stat.h>


// Constructeur
CZipArc::CZipArc(CInterf* pi) : CZArc(pi)
{
	m_pszCom = NULL;
	m_lenCom = 0;
}


// Capacits
int CZipArc::GetCap() const
{
	return capNiveau | capCommentaire | capArbo |
		capAttribut | capCrypte | capRapideModifProp | capSfx | capdtM;
}

// ID de l'EXE SFX
UINT CZipArc::GetSfxID() const
{
	return IDSFX_ZIP;
}



// Ouverture
bool CZipArc::OpenDocument()
{
	CLstZipFichier lstTrouve;
	
	// Ouvrir l'archive et appeler OuvrirNormal() ou OuvrirReparer()
	CMuzipException *const pmex =
		(e_opt.m_nReparer<=repPresumer)
			? OuvrirNormal (lstTrouve)
			: OuvrirReparer(lstTrouve);
	if (pmex) {
		lstTrouve.DeleteAll();
		pmex->ReportError();
		pmex->Delete();
		return false;
	}
	
	// Afficher les commentaires s'il y en a
	// et si l'option est coche
	if (e_opt.m_fGeneralCommentaire && m_pszCom)
		oApp.m_pMainWnd->PostMessage(WM_COMMAND, ID_ARCHIVE_COMMENT, 0);
	
	Rechercher();
	return true;
}

// Ouvrir l'archive en mode normal (vrification active)
CMuzipException* CZipArc::OuvrirNormal(CLstZipFichier& lstTrouve)
{
#define phFin         ((HEADERFIN*)      abBuf)
#define phLocSig      ((HEADERLOCSIG*)   abBuf)
#define phLoc         ((HEADERLOC*)      abBuf)
#define phLocExtSig   ((HEADERLOCEXTSIG*)abBuf)
#define phCenSig      ((HEADERCENSIG*)   abBuf)
	
	BYTE abBuf[sizeof(HEADERCENSIG)];  // Tampon des en-ttes du central
	DWORD nFlags;
	
	m_offDebut = 0;
	
	BYTE *const abBufTemp = new BYTE[4096+4];
	BYTE *pb = &abBufTemp[4096];
	pb[1] = 0;
	pb[2] = 0;
	pb[3] = 0;
	
	
	// Rechercher la signature finale  partir de la fin,
	// en lisant des tranches de 4ko  la fois
	// On doit en effet ignorer ce qu'il y a aprs la fin des structures Zip,
	// et ne pas prsumer que les structures Zip se terminent avec le fichier
	//  la fin du processus, m_offDebut contiendra la position de l'en-tte final
	// (contradictoire avec le nom de la variable, mais tant pis!)
	size_t len;   // Utilis plus tard, mais les goto demandent une dclaration avant
	m_fr.SetVerbeux(false);
	if (m_fr.Seek(-4096, FILE_END)) {
		m_offDebut = m_fr.Tell();
		for (;;) {
			abBufTemp[4096] = pb[1];
			abBufTemp[4097] = pb[2];
			abBufTemp[4098] = pb[3];
			
			m_fr.Read(abBufTemp, 4096);
			m_fr.Seek(-2*4096, FILE_CURRENT);
			pb = &abBufTemp[4095];
			
			do{
				// Rechercher la signature finale
				if (*(DWORD*)pb==sigFin)  goto FinTrouveOK;
			}while (--pb>=abBufTemp);
			if (m_offDebut<4096)  break;
			m_offDebut -= 4096;
		}
		if (!m_offDebut)  goto ErreurFormat;
		m_fr.Seek(0, FILE_BEGIN);
		len = m_fr.ReadLen(abBufTemp, m_offDebut);
		
	}else{
		// Lire le maximum d'octets (4ko - 1 octet)
		len = m_fr.ReadLen(abBufTemp, 4096-1);
		m_offDebut = 0;
	}
	m_fr.SetVerbeux();
	
	// Rechercher la signature finale dans les tous-premiers octets,
	// car soit le fichier faisait moins de 4ko,
	// soit la signature se trouve dans les 4095 premiers octets (= 4ko - 1octet)
	
	abBufTemp[len  ] = pb[1];
	abBufTemp[len+1] = pb[2];
	abBufTemp[len+2] = pb[3];
	pb = &abBufTemp[len-1];
	do{
		if (*(DWORD*)pb==sigFin)  goto FinTrouveOK;
	}while (--pb>=abBufTemp);
	
	// Erreur: Signature finale non-trouve
	delete [] abBufTemp;
	
ErreurFormat:
	return new CMuzipException(ERR_FORMATZIP);
	
FinTrouveOK:  // Fin du central trouve
	delete [] abBufTemp;
	
	// Signature finale trouve
	// Calculer la position de la fin du central, et y aller
	m_offDebut += (DWORD)(pb - abBufTemp);
	m_fr.Seek(m_offDebut+lenSig, FILE_BEGIN);
	
	// Lire l'en-tte final
	if (!m_fr.Read(abBuf, sizeof(HEADERFIN))) {
ErreurFileRead:
		return new CMuzipException;
	}
	if (phFin->nDisque || phFin->nDisqueD || phFin->nbEntry!=phFin->nbEntryT)
		TRACE0("Multi-volumes ignor");
	
	// Lire les commentaires
	m_lenCom = phFin->lenCom;
	if (m_lenCom) {
		m_pszCom = new TCHAR[m_lenCom+1];
#ifdef UNICODE
		char *const pszCom = new char[m_lenCom];
		if (!m_fr.Read(pszCom, m_lenCom)) {
#else
		if (!m_fr.Read(m_pszCom, m_lenCom)) {
#endif
			delete [] m_pszCom;
			m_pszCom = NULL;
			m_lenCom = 0;
			goto ErreurFileRead;
		}
#ifdef UNICODE
		pszCom[m_lenCom] = 0;
		OemToCharA(pszCom,pszCom);
		strfromA(m_pszCom, pszCom, m_lenCom);
		delete [] pszCom;
#else
		m_pszCom[m_lenCom] = 0;
		OemToCharA(m_pszCom,m_pszCom);
#endif
	}
	
	// On prsume que le central prcde immdiatement l'en-tte final
	m_offCen = m_offDebut - phFin->nTailleT;
	const long offDecal = (e_opt.m_nReparer==repPresumer) ? 0 : m_offCen - phFin->off;
	if (!m_fr.Seek(m_offCen, FILE_BEGIN))  goto ErreurFileRead;
	
	CZipFichier *pzf;
	WORD lenNom;
	
	// Lire le central
	// La partie des en-ttes centraux qui doit tre commune
	// avec les en-ttes locaux est lue brute, pour une comparaison future
	// (cela demande que l'offset de m_pbExtra dans CZipFichier soit suprieur
	// ou gal  LOCHEAD)
	DWORD offCentral = m_offCen;   // Offset courant
	for (;;) {
		
		// Lire la signature: ce doit tre la signature des en-ttes centraux
		m_fr.SetVerbeux(false);
		const bool fOK = m_fr.Read(abBuf, sizeof(HEADERCENSIG));
		m_fr.SetVerbeux();
		if (!fOK) {
			if (m_fr.IsEOF())  break;
			return new CMuzipException(m_fr);
		}
		if (phCenSig->sig!=sigCen)  break;
		
		pzf = new CZipFichier;
		
		memcpy(&pzf->m_nVerC, abBuf+lenSig, cenlenNom-cenVerC);
		lenNom            = phCenSig->lenNom;      // Compar aprs  l'en-tte local
		pzf->m_lenExtraC  = phCenSig->lenExtraC;   // Peut tre diffrent dans l'en-tte local
		pzf->m_lenCom     = phCenSig->lenCom;
		pzf->m_nDisque    = phCenSig->nDisque;
		pzf->m_nAttrI     = phCenSig->nAttrI;
		pzf->m_nAttrE     = phCenSig->nAttrE;
		pzf->m_off        = phCenSig->off + offDecal;
		pzf->m_fDOS       = !(pzf->m_nVerC & 0xF00);
		pzf->m_offCentral = offCentral;
		
		// Ajouter le fichier  la liste
		lstTrouve.AddFin(pzf);
		
		// Lire le nom du fichier
		if (!lenNom)  goto ErreurFormat;
		pzf->m_lenNom  = lenNom;
		pzf->m_pszNomI = new char[lenNom+1];
		if (!m_fr.Read(pzf->m_pszNomI, lenNom))  goto ErreurFileRead;
		pzf->m_pszNomI[lenNom] = 0;
		
		// Lire le champ extra
		if (pzf->m_lenExtraC) {
			pzf->m_pbExtraC = new BYTE[pzf->m_lenExtraC];
			if (!m_fr.Read(pzf->m_pbExtraC, pzf->m_lenExtraC))  goto ErreurFileRead;
		}
		
		// Lire les commentaires
		const WORD lenCom = pzf->m_lenCom;
		if (lenCom) {
			pzf->m_pszCom = new TCHAR[lenCom+1];
#ifdef UNICODE
			char *const pszCom = new char[lenCom];
			if (!m_fr.Read(pszCom, lenCom))  goto ErreurFileRead;
			pszCom[lenCom] = 0;
			OemToCharA(pszCom,pszCom);
			strfromA(pzf->m_pszCom, pszCom, lenCom);
			delete [] pszCom;
#else
			if (!m_fr.Read(pzf->m_pszCom, lenCom))  goto ErreurFileRead;
			pzf->m_pszCom[lenCom] = 0;
			OemToCharA(pzf->m_pszCom,pzf->m_pszCom);
#endif
		}
		
		offCentral += sizeof(HEADERCENSIG) + lenNom + pzf->m_lenExtraC + lenCom;
		
		// Mettre  jour m_offDebut si ncessaire
		if (m_offDebut>pzf->m_off)  m_offDebut = pzf->m_off;
	}
	
	// m_offDebut pointe maintenant vers l'en-tte local
	// du premier fichier de l'archive en position
	
	// Lire les en-ttes locaux, et les comparer avec les en-ttes centraux
	POS pos = lstTrouve.GetPosDeb();
	while (pos) {
		pzf = lstTrouve.GetSuiv(pos);
		
		// Lire la signature
		if (!m_fr.Seek(pzf->m_off, FILE_BEGIN) ||
		    !m_fr.Read((DWORD&)*abBuf)) {
			goto ErreurFileRead;
		}
		
		if (phLocSig->sig!=sigLoc)
			// Erreur: en-tte local non-trouv pour pzf->m_pszNomI
			goto ErreurFormat;
		
		// Lire l'en-tte proprement dit
		if (!m_fr.Read(abBuf, sizeof(HEADERLOC)))  goto ErreurFileRead;
		pzf->m_nFlags   = phLoc->nFlags;
		lenNom          = phLoc->lenNom;
		pzf->m_lenExtra = phLoc->lenExtra;
		
		// Comparer les noms
		if (lenNom!=pzf->m_lenNom)
			// Noms de longueur diffrente dans local et central
			goto ErreurFormat;
		
		pb = new BYTE[pzf->m_lenNom];
		if (!m_fr.Read(pb, pzf->m_lenNom)) {
			delete [] pb;
			goto ErreurFileRead;
		}
		if (memcmp(pb, (BYTE*)pzf->m_pszNomI, pzf->m_lenNom)) {
			// Noms diffrents dans local et central
			delete [] pb;
			goto ErreurFormat;
		}
		delete [] pb;
		
		// Comparer les champs extra
		if (pzf->m_lenExtra) {
			pzf->m_pbExtra = new BYTE[pzf->m_lenExtra];
			if (!m_fr.Read(pzf->m_pbExtra, pzf->m_lenExtra)) {
				delete [] pzf->m_pbExtra;
				goto ErreurFileRead;
			}
			if (pzf->m_lenExtra!=pzf->m_lenExtraC &&
			    !memcmp(pzf->m_pbExtra, pzf->m_pbExtraC, pzf->m_lenExtra)) {
				// En-ttes local et central identiques:
				// tout garder dans l'en-tte central
				delete [] pzf->m_pbExtra;
				pzf->m_pbExtra = pzf->m_pbExtraC;
			}
		}
		
		// Vrifier l'en-tte local tendu s'il y en a un
		if (pzf->m_nFlags & locfEtendu) {
			char buf2[16];
			
			DWORD nTailleC = phLoc->nTailleC;
			if (!nTailleC)  nTailleC = pzf->m_nTailleC;
			if (!m_fr.Seek(pzf->m_off + sizeof(HEADERLOCSIG) +
			     pzf->m_lenNom + pzf->m_lenExtra + nTailleC, FILE_BEGIN) ||
			    !m_fr.Read(buf2, 16))
				goto ErreurFileRead;
			if (phLocExtSig->sig!=sigLocExt) {
				// En-tte local tendu non-trouv pour pzf->m_pszNomI
				goto ErreurFormat;
			}
			
			// craser les valeurs de l'en-tte local
			memcpy(&phLoc->nCRC, &phLocExtSig->nCRC, sizeof(HEADERLOCEXT));
		}
		
		// Comparer l'en-tte local avec l'en-tte central
		// (sauf les bits rservs des flags gnraux
		// et les entres dj compares)
		
		// Sauver les flags du central et masquer les bits rservs
		pb = (BYTE*)&pzf->m_nVerE;
#define phCen   ((HEADERCEN*)(pb-cenVerE))
		nFlags = phCen->nFlags;
		pb[cenFlag-cenVerE+1] &= 0x1F;
		abBuf[locFlag     +1] &= 0x1F;
		
		// Terminer l'initialisation de l'entre
		pzf->m_nTailleN  = phCen->nTailleN;
		pzf->m_nTailleC  = phCen->nTailleC;
		pzf->m_nCRC      = phCen->nCRC;
		pzf->SetUnixDateM (phCen->dt);
		pzf->m_nMethode  = phCen->nMethode;
		pzf->m_nFlags    = (WORD)nFlags;
		pzf->m_nVerE     = phCen->nVerE;
#undef phCen
	}
	
	
#if 0
	if (m_offDebut) {
		// m_nReparer==0  m_offDebut octets trouvs avant le dbut
		// m_nReparer==1 - Ajustement des offsets car dcalage de m_offDebut octets au dbut
	}
#endif
	
	// Ajouter les fichiers dans leurs dossiers
	TCHAR pszNom[MAX_PATH*2];
	const TCHAR *pcNom;
	pos = lstTrouve.GetPosDeb();
	while (pos) {
		pzf = lstTrouve.GetSuiv(pos);
		
		// Supprimer les rpertoires
		lenNom = pzf->m_lenNom;
		if (pzf->m_pszNomI[lenNom-1]=='/') {
			delete pzf;
			continue;
		}
		
		// Convertir le nom en LPTSTR avec sparateurs '\\'
		OemToChar(pzf->m_pszNomI, pszNom);
		
		// Rcuprer le dernier sparateur '/'
		pcNom = pszNom+lenNom;
		while (pcNom-- > pszNom && *pcNom!='/');
		pzf->m_pszNom = newDup(pcNom+1);
		
		// Rcuprer le dossier correspondant au chemin d'accs
		// pour y ajouter le fichier
		pzf->m_dwTailleC = pzf->m_nTailleC;
		pzf->m_dwTailleN = pzf->m_nTailleN;
		pzf->m_dwAttr    = pzf->m_nAttrE;
		pzf->MAJ();
		pzf->m_pArc      = this;
		(pzf->m_pDossier = GetDossier(pszNom, false))->m_lstFichier.AddFin(pzf);
	}
	lstTrouve.RemoveAll();
	
	return NULL;
	
#undef phFin
#undef phLocSig
#undef phLoc
#undef phLocExtSig
#undef phCenSig
}

// Ouvrir l'archive en mode rparation (vrification minimale)
CMuzipException* CZipArc::OuvrirReparer(CLstZipFichier&)
{
#define phLocSig      ((HEADERLOCSIG*)   abBuf)
#define phLoc         ((HEADERLOC*)      abBuf)
#define phLocExtSig   ((HEADERLOCEXTSIG*)abBuf)
#define phCenSig      ((HEADERCENSIG*)   abBuf)
	
	
	m_fr.SetVerbeux(false);
	
	BYTE abBuf[sizeof(HEADERCENSIG)];
	DWORD len;
	
	BYTE b;
	
	DWORD off = 0;
	m_offDebut = 0;
	
	// Trouver le dbut des structures Zip
	// 'P' est le 1er caractre commun  toutes les signatures
	m_fr.SetVerbeux(false);
	for (;;) {
		while (m_fr.Read(b) && b!='P')  off++;
		abBuf[0] = 'P';
		if (!m_fr.Read(abBuf+1, lenSig-1))  break;
#define sig  len
		sig = *(DWORD*)abBuf;
		if (sig==sigLoc || sig==sigFin)  break;
#undef sig
		if (!m_fr.Seek(-3, FILE_CURRENT)) {
ErreurFileRead:
			return new CMuzipException;
		}
		off++;
	}
	
	m_offDebut = off;
	
	// Lire les en-ttes locaux
#ifdef UNICODE
	TCHAR pszNom[MAX_PATH*2];
#else
	LPTSTR pszNom;
#endif
	const TCHAR *pcNom;
	WORD lenNom;
	
	while (*(DWORD*)abBuf==sigLoc) {
		if (!m_fr.Read(abBuf, sizeof(HEADERLOC)))  break;
		
		bool fLireSig = true;
		CZipFichier *const pzf = new CZipFichier;
		pzf->m_nVerE      = phLoc->nVerE;
		pzf->m_nVerC      = codeOS + codeVer;
		pzf->m_fDOS       = false;
		pzf->m_nFlags     = phLoc->nFlags;
		pzf->m_nMethode   = phLoc->nMethode;
		pzf->SetUnixDateM  (phLoc->dt);
		pzf->m_nCRC       = phLoc->nCRC;
		pzf->m_nTailleC   = phLoc->nTailleC;
		pzf->m_nTailleN   = phLoc->nTailleN;
		lenNom            = phLoc->lenNom;
		pzf->m_lenExtraC  =
		pzf->m_lenExtra   = phLoc->lenExtra;
		
		pzf->m_lenCom     = 0;
		pzf->m_nDisque    = 0;
		pzf->m_nFlags     = 0;
		pzf->m_offCentral = 0;
		
		// Ne pas prsumer de la taille compresse
		len = (e_opt.m_nReparer<repAltere) ? pzf->m_nTailleC : 0;
		
		// Tout mettre  NULL
		pzf->m_pszNomI  = NULL;
		pzf->m_pszNom   =
		pzf->m_pszCom   = NULL;
		pzf->m_pbExtra  =
		pzf->m_pbExtraC = NULL;
		
		// Lire le nom du fichier
		if (!lenNom) {
			// Nom vide
ErreurFormatFR:
			return new CMuzipException(ERR_REPARER);
		}
		pzf->m_lenNom  = lenNom;
		pzf->m_pszNomI = new char[lenNom+1];
		if (!m_fr.Read(pzf->m_pszNomI, lenNom))  goto ErreurFileRead;
		pzf->m_pszNomI[lenNom] = 0;
		OemToCharA(pzf->m_pszNomI,pzf->m_pszNomI);
		
		// Lire le champ extra
		if (pzf->m_lenExtra) {
			pzf->m_pbExtra = new BYTE[pzf->m_lenExtra];
			if (!m_fr.Read(pzf->m_pbExtra, pzf->m_lenExtra))  goto ErreurFileRead;
		}
		
		// Ignorer les commentaires
		
		// S'il y a un en-tte local tendu,
		// len est soit zro soit la taille compresse correcte
		pzf->m_pbExtraC = pzf->m_pbExtra;
		
		// Sauver l'offset, mettre  jour pour l'en-tte suivant
		pzf->m_off = off;
		off += sizeof(HEADERLOCSIG) + lenNom + pzf->m_lenExtra + len;
		
		if (len && !m_fr.Seek(len, FILE_CURRENT))  goto ErreurFileRead;
		
		// Sauter l'en-tte local tendu s'il y en a un
		if (pzf->m_nFlags & locfEtendu) {
			// Sauter les donnes compresses si la taille compresse est inconnue
			// Pour plus de scurit, on devrait utiliser le central
			if (!len || m_fr.Tell()!=off) {
				for (;;) {
					while (m_fr.Read(b) && b!='P');
					abBuf[0] = b;
					if (!m_fr.Read(abBuf+1, 15))    break;
					if (*(DWORD*)abBuf==sigLocExt)  break;
					if (m_fr.Seek(-15L, FILE_CURRENT))  goto ErreurFileRead;
				}
				len = phLocExtSig->nTailleC;
				off += len;
				if (m_fr.Tell()!=off+lenEtendu)  goto ErreurFormatFR;
			}else{
				// Taille compresse non-nulle, assumer qu'elle est valide
				
				if (!m_fr.Seek(off, FILE_BEGIN) ||
				    !m_fr.Read(abBuf, lenEtendu))
					goto ErreurFileRead;
				if (phLocExtSig->sig!=sigLocExt)  goto ErreurFormatFR;
			}
			
			// Rcuprer les valeurs inconnues de l'en-tte local tendu
			pzf->m_nCRC     = phLocExtSig->nCRC;
			pzf->m_nTailleC = len;
			pzf->m_nTailleN = phLocExtSig->nTailleN;
			
			off += lenEtendu;
			
		}else if (e_opt.m_nReparer==repAltere) {
			// Ne pas prsumer de la taille compresse
			
			for (;;) {
				while (m_fr.Read(b) && b!='P');
				abBuf[0] = b;
				if (!m_fr.Read(abBuf+1, 3))    break;
				if (*(DWORD*)abBuf==sigLoc)  break;
				if (!m_fr.Seek(-3, FILE_CURRENT))
					goto ErreurFileRead;
				off++;
			}
			len = off - (pzf->m_off + sizeof(HEADERLOCSIG) + lenNom + pzf->m_lenExtra);
			if (len!=pzf->m_nTailleC) {
				// Taille compresse corrige
				pzf->m_nTailleC = len;
			}
			
			// Prochain sigLoc juste l, ne pas le relire
			fLireSig = false;
		}
		
		// Supprimer les rpertoires
		if (pzf->m_pszNomI[lenNom-1]=='/') {
			delete pzf;
			goto Suite;
		}
		
		// Convertir le nom en LPTSTR avec sparateurs '\\'
#ifdef UNICODE
		strfromA(pszNom, pzf->m_pszNomI, nbArray(pszNom));
#else
		pszNom = pzf->m_pszNomI;
#endif
		
		// Rcuprer le dernier sparateur '/'
		pcNom = pszNom+lenNom;
		while (pcNom-- > pszNom && *pcNom!='/');
		pzf->m_pszNom = newDup(pcNom+1);
		
		// Rcuprer le dossier correspondant au chemin d'accs
		// pour y ajouter le fichier
		pzf->m_dwTailleC = len;
		pzf->m_dwTailleN = pzf->m_nTailleN;
		pzf->m_dwAttr    = 0;
		pzf->MAJ();
		pzf->m_pArc      = this;
		(pzf->m_pDossier = GetDossier(pszNom, false))->m_lstFichier.AddFin(pzf);
Suite:
		
		// Lire la signature suivante
		if (fLireSig && !m_fr.Read((DWORD&)*abBuf))  break;
	}
	
	// Sauver la position du central
	m_offCen = off;
	
#if 0
	if (phCen->sig!=sigCen)   // Troncation
	if (m_offDebut)           // Ajustement de m_offDebut octets au dbut du fichier
#endif
	
	return NULL;
	
#undef phLocExtSig
}

// Fermer le document
void CZipArc::CloseDocument()
{
	CZArc::CloseDocument();
	
	delete [] m_pszCom;
	m_pszCom = NULL;
}



// Reconstruire l'archive
void CZipArc::Reconstruire()
{
	if (!ReadOpen())  return;
	
	CMuzipOp mop;
	
	TCHAR pszTemp[MAX_PATH];
	if (OpenArchiveTemp (e_fw, pszTemp) &&
	    CloseArchiveTemp(e_fw, pszTemp))
		CZArc::Reconstruire();
}

// Ajouter un fichier  la liste
CFichier* CZipArc::NewFichier(LPCTSTR pszFichier, const WIN32_FIND_DATA& wfd)
{
	// Ajouter le fichier  la liste du Zip
	CZipFichier *const pzf = new CZipFichier;
	
	pzf->NewFichier(pszFichier, wfd);
	
	char pszNomI[MAX_PATH];
	strtoA(pszNomI, PathFindFileName(pszFichier), MAX_PATH);
	for (char *pc=pszNomI; *pc; pc++) {
		if (*pc=='\\')  *pc = '/';
	}
	pzf->m_pszNomI  = newDup(pszNomI);
	
	pzf->m_nTailleC = pzf->m_dwTailleC;
	pzf->m_nTailleN = pzf->m_dwTailleN;
	pzf->m_pArc     = this;
	pzf->m_pDossier = this;
	pzf->m_fDOS     = false;
	pzf->m_dt       = pzf->m_dtM;
	
	return pzf;
}


// Ajouter des fichiers  l'archive
bool CZipArc::ArchivePlus()
{
	// Crer un fichier temporaire de stockage et l'ouvrir
	TCHAR pszTemp[MAX_PATH];
	VERIF(OpenArchiveTemp(e_fw, pszTemp));
	
	bool fAuMoinsUn = false;
	
	ShowStatusAvanceFichier(true);
	UINT iFichier = 0;
	const UINT nbFichier = s_lstFichierPlus.GetNb();
	POS pos = s_lstFichierPlus.GetPosDeb();
	while (pos) {
		CFichierPlus *const pFichierPlus = s_lstFichierPlus.GetSuiv(pos);
		CZipFichier  *const pzf = (CZipFichier*)pFichierPlus->m_pFichier;
		oApp.SetProgress(STA_ARCHIVEPLUS, pzf->m_pszNomES,
			iFichier++, nbFichier);
		
		pzf->m_lenExtra  =
		pzf->m_lenExtraC =
		pzf->m_lenCom    = 0;
		
		PRGSTATUS prs = prsOK;
		
		// Si le fichier est l'archive elle-mme,
		// afficher un message d'erreur et passer au fichier suivant
		const DWORD offOld = e_lenEcrit;
		if (pzf->CompresseArchive())
			prs = prsErreur;
		else{
			pzf->m_pbExtra  =
			pzf->m_pbExtraC = NULL;
			
			if (pzf->m_fFautComp) {
				
				// Compresser le fichier
				if (!Compresser(pFichierPlus->m_ppi, pzf, e_fw, pszTemp))
					prs = ErreurArchivePlus();
				else
					// Compression russie
					fAuMoinsUn = true;
			}
		}
		
		if (0) {
			prs = prsAnnuler;
		}
		if (!ProgressArchivePlus(pzf, prs, pos, offOld))  break;
	}
	oApp.s_fMsg = true;
	oApp.HideStatusAvance2();
	oApp.HideStatusAvance();
	
	Finaliser();
	if (fAuMoinsUn)
		// Au moins un fichier a t compress:
		// remplacer l'ancien Zip par le nouveau
		CloseArchiveTemp(e_fw, pszTemp);
	else
		// Aucun fichier n'a t compress
		e_fw.Close();
	
	// Supprimer le fichier temporaire
	FichierSupprimer(pszTemp, false);
	
	return true;
}


// Ignorer un fichier
void CZipArc::IgnorerFichier(DWORD offOld, CFichier* pFichier)
{
	e_lenEcrit = offOld;
	CMuzipArc::IgnorerFichier(offOld, pFichier);
}


// Dfinir les commentaires
void CZipArc::SetComment(LPCTSTR psz)
{
	// Recopier les commentaires
	m_lenCom = (WORD)lstrlen(psz);
	delete [] m_pszCom;
	if (m_lenCom) {
		m_pszCom = new TCHAR[m_lenCom+1];
		lstrcpy(m_pszCom, psz);
	}else
		m_pszCom = NULL;
	
	// Mettre  jour
	FautReconstruire();
}

// Retourner les commentaires
LPCTSTR CZipArc::GetComment() const
{
	return m_pszCom;
}


// Initialiser la cration d'une archive temporaire
bool CZipArc::OpenArchiveTemp(CFileWrite& fw, LPTSTR pszTemp)
{
	e_lenEcrit = 0;
	return CZArc::OpenArchiveTemp(fw, pszTemp);
}


// crire le rpertoire central du Zip
// et remplacer l'ancien Zip par le fichier temporaire
bool CZipArc::CloseArchiveTemp(CFileWrite& fw, LPCTSTR pszTemp)
{
	// crire le central
	CFichier::s_nbFichier = GetNbFichier();
	CFichier::s_nFichier  = 0;
	oApp.SetMessageText(STA_PUTCENTRAL);
	oApp.ShowStatusAvance(CFichier::s_nbFichier);
	const DWORD offDeb = e_lenEcrit;
	const bool fOK = ZipPutCentral(fw);
	oApp.HideStatusAvance();
	if (!fOK) {
Erreur:
		CFichier::s_nbFichier = 0;
		fw.Close();
		return false;
	}
	
#ifdef _DEBUG  //$$$
	TRACE2("*********** Vrai: %lu   Indiqu: %lu\n", fw.Tell(), e_lenEcrit);
#endif         //$$$
	
	// crire le rpertoire final
	HEADERFINSIG headerFin;
	headerFin.sig      = sigFin;
	headerFin.nDisque  =
	headerFin.nDisqueD = 0;
	headerFin.nbEntry  =
	headerFin.nbEntryT = (WORD)CFichier::s_nbFichier;
	headerFin.nTailleT = e_lenEcrit-offDeb;
	headerFin.off      = offDeb;
	headerFin.lenCom   = m_lenCom;
	CFichier::s_nbFichier = 0;
	if (            !fw.Write(&headerFin, sizeof(headerFin)) ||
	   (m_lenCom && !fw.Write(m_pszCom,   m_lenCom)))
		goto Erreur;
	
	return CZArc::CloseArchiveTemp(fw, pszTemp);
}



// Compresser l'entre pzf dans le fichier pf
// Une "entre zip" comporte un en-tte local, un en-tte de cryptage
// (si ncessaire), les donnes compresses, et ventuellement
// un en-tte local tendu
// Met dans e_lenEcrit le nombre d'octets crits
bool CZipArc::Compresser(CPlusItem* ppi, CZipFichier* pzf, CFileWrite& fw, LPCTSTR pszFichier)
{
	WIN32_FIND_DATA wfd;
	const HANDLE hff = ::FindFirstFile(pzf->m_pszNomES, &wfd);
	if (!hff) {
		CMuzipException(ERR_OPEN, pszFichier).ReportError();
		return false;
	}
	::FindClose(hff);
	
	// Rcuprer les attributs Dos
	DWORD dwAttr = (wfd.dwFileAttributes & FILE_ATTRIBUTE_READONLY)
		? _S_IFREG | _S_IREAD
		: _S_IFREG | _S_IREAD | _S_IWRITE;
	{
		const TCHAR *pcExt = _tcsrchr(pzf->m_pszNomES, '.');
		if (pcExt) {
			pcExt++;
			if (!lstrcmpi(pcExt, _T("exe")) ||
			    !lstrcmpi(pcExt, _T("cmd")) ||
			    !lstrcmpi(pcExt, _T("bat")) ||
			    !lstrcmpi(pcExt, _T("com")))
				dwAttr |= _S_IEXEC;
		}
	}
	dwAttr |= (dwAttr & 0700)>>3;
	dwAttr |= (dwAttr & 0700)>>6;
	dwAttr  = (dwAttr<<16) | wfd.dwFileAttributes;
	
	// Convertir la date au format Dos
	DWORD dtDosM;
	FILETIME ft;
	FileTimeToLocalFileTime(&wfd.ftLastWriteTime, &ft);
	FileTimeToDosDateTime(&ft, (WORD*)&dtDosM+1, (WORD*)&dtDosM);
	SYSTEMTIME st;
	FileTimeToSystemTime(&ft, &st);
	
	pzf->m_nAttrI = attInconnu;   // Changera plus tard
	pzf->m_nAttrE = 0;            // Peut tre modifi
	
	// Librer les anciens champs extra probablement obsoltes
	if (pzf->m_lenExtra)
		delete [] pzf->m_pbExtra;
	if (pzf->m_lenExtraC && pzf->m_pbExtra!=pzf->m_pbExtraC)
		delete [] pzf->m_pbExtraC;
	pzf->m_lenExtra  =
	pzf->m_lenExtraC = 0;
	
	// Choisir la mthode en se basant sur le suffixe du fichier
	// et la mthode choisie par l'utilisateur
	ZMETHODE nMethode;
	if (ppi->CO_nNiveau==0)
		nMethode = zmStore;
	else{
		// Regarder si le fichier a une extension  ne pas compresser
		
		TCHAR pszExt[_MAX_EXT];
		lstrcpy(pszExt, PathFindExtension(pzf->m_pszNomES));
		CharLower(pszExt);
		CLstStr2 &lst = e_opt.m_lstPasCompExt;
		POS pos = lst.GetPosDeb();
		while (pos) {
			if (!lstrcmp(pszExt, lst.GetSuiv(pos))) {
				nMethode = zmStore;
				goto FinMethode;
			}
		}
		nMethode = zmDeflate;
	}
FinMethode:
	
	CFileRead fr;
	VERIF(fr.Open(pzf->m_pszNomES));
	pzf->m_dt  = dtDosM;
	pzf->m_dtM = wfd.ftLastWriteTime;
	
	if (!wfd.nFileSizeLow)  nMethode = zmStore;
	
	// Remplir les informations de l'en-tte et crire l'en-tte local
	// Cet en-tte sera rcrit plus tard car la longueur compresse
	// et le CRC sont inconnus  ce stade
	
	pzf->m_nVerC = codeOS + codeVer;
	
	// PKZIP 1.0 (mthode Store) ou 2.0 (Deflate) requis pour extraction
	pzf->m_nVerE = (nMethode==zmStore) ? (WORD)10 : (WORD)20;
	pzf->m_nCRC  = 0;   // Mis  jour plus tard
	
	// Assumer d'abord que l'on a besoin d'un en-tte local tendu
	pzf->m_nFlags2 = locfEtendu;  // Mis  jour plus tard
	if (!m_sMdp.IsEmpty()) {
		pzf->m_nFlags2 |= locfCrypte;
		// Comme on ne connat pas le CRC ici,
		// on prend la date de modification  la place
		pzf->m_nCRC = pzf->m_dt<<16;
	}
	pzf->m_nFlags    = pzf->m_nFlags2;
	pzf->m_nMethode  = (WORD)nMethode;             // Peut tre chang plus tard
	pzf->m_dwTailleC =
	pzf->m_nTailleC  = (nMethode==zmStore)   // Chang plus tard
		? wfd.nFileSizeLow : 0;
	pzf->m_dwTailleN =
	pzf->m_nTailleN  = wfd.nFileSizeLow;     // Chang plus tard
	pzf->m_nDisque   = 0;
	const bool fType = (pzf->m_nAttrI==attInconnu);
	if (fType)  pzf->m_nAttrI = attBinaire;
	
	pzf->m_nAttrE  = (pzf->m_fDOS) ? dwAttr & 0xFF : dwAttr | (pzf->m_nAttrE & 0xFF00);
	pzf->m_off     = e_lenEcrit;
	VERIF(pzf->PutLocal(fw));
	e_lenEcrit += sizeof(HEADERLOCSIG)+pzf->m_lenNom+pzf->m_lenExtra;
	
	if (fType && nMethode==zmDeflate)
		pzf->m_nAttrI = attInconnu; // Sera modifi dans Deflate() plus tard
	
	pzf->m_dwTailleC = 0;
	if (!m_sMdp.IsEmpty()) {
		cryptEnTete(m_sMdp, pzf->m_nCRC, fw);
		pzf->m_dwTailleC  = lenCryptEnTete;  // Mettre  jour plus tard
		e_lenEcrit       += lenCryptEnTete;
	}
	
	pzf->m_nFlags2 |= e_aConfig[ppi->CO_nNiveau].vit;
	
	DWORD nCRC;
	DWORD lenC = CZArc::Compresser(pzf, fr, fw,
		&nCRC, ppi->CO_nNiveau,nMethode, &pzf->m_nAttrI);
	
	fr.Close();
	
	// Vrifier la taille d'entre (annulation ou erreur d'criture)
	if ((!lenC && wfd.nFileSizeLow>0) || e_lenIn!=wfd.nFileSizeLow) {
		e_lenEcrit = fw.Tell();
		return false;
	}
	
	e_lenEcrit += lenC;
	
	DWORD off2 = e_lenEcrit; // Garder pour Seek() futur
	
	// Essayer de rcrir l'en-tte local avec les informations correctes
	pzf->m_nCRC       = nCRC;
	pzf->m_dwTailleC += lenC;
	pzf->m_nTailleC   = pzf->m_dwTailleC;
	pzf->m_dwTailleN  =
	pzf->m_nTailleN   = e_lenIn;
	
	if (!fw.Seek(pzf->m_off, FILE_BEGIN)) {
		pzf->PutEtendu(fw);
		e_lenEcrit += lenEtendu;
		pzf->m_nFlags2 = pzf->m_nFlags;  // Si m_nFlags2 modifi par inflate
		
	}else{
		// Vrifier la taille compresse
		pzf->m_nMethode = (WORD)nMethode;
		pzf->m_nVerE = (nMethode==zmStore) ? (WORD)10 : (WORD)20;   // PKUNZIP 2.0 requis pour Store
		// RAZ le flag d'en-tte local tendu
		if (!(pzf->m_nFlags2 & locfCrypte))  pzf->m_nFlags2 &= ~locfEtendu;
		pzf->m_nFlags = pzf->m_nFlags2;
		// Rcrire l'en-tte local
		VERIF(pzf->PutLocal(fw));
		fw.Seek(off2, FILE_BEGIN);
		if (pzf->m_nFlags2 & locfEtendu) {
			// Fichier crypt, en-tte tendu toujours ncessaire
			pzf->PutEtendu(fw);
			e_lenEcrit += lenEtendu;
		}
	}
	
	// Mettre  jour CFichier
	pzf->MAJ();
	
	return true;
}


// crire l'en-tte local de pzf dans un fichier
bool CZipFichier::PutLocal(CFileWrite& fw)
{
	PreparerNom();
	
	VERIF(fw.Write(sigLoc));
	
	const WORD nFlags2Old = m_nFlags2;
	m_nFlags2 = m_nFlags;
	VERIF(fw.Write(&m_nVerE, loclenExtra-locVerE));
	m_nFlags2 = nFlags2Old;
	
	return (           fw.Write(m_lenExtra) &&
	                   fw.Write(s_aszNom,  m_lenNom) &&
	   (!m_lenExtra || fw.Write(m_pbExtra, m_lenExtra)));
}

// crire l'en-tte local tendu de pzf dans le fichier pf
void CZipFichier::PutEtendu(CFileWrite& fw) const
{
	fw.Write(sigLocExt);
	fw.Write(m_nCRC);
	fw.Write(m_dwTailleC);
	fw.Write(m_dwTailleN);
}

// crire l'en-tte central de pzf dans le fichier pf
bool CZipFichier::PutCentral(CFileWrite& fw)
{
	PreparerNom();
	
	VERIF(fw.Write(sigCen));
	
	const WORD nFlags2Old = m_nFlags2;
	m_nFlags2 = m_nFlags;
	VERIF(fw.Write(&m_nVerC, cenFin-cenVerC));
	m_nFlags2 = nFlags2Old;
	
	return (            fw.Write(s_aszNom,   m_lenNom) &&
	   (!m_lenExtraC || fw.Write(m_pbExtraC, m_lenExtraC)) &&
	   (!m_lenCom    || fw.Write(m_pszCom,   m_lenCom)));
}


// Prparer m_lenNom et s_aszNom en vue d'criture dans le Zip
void CZipFichier::PreparerNom()
{
	// Rcuprer la longueur du nom, nul final compris
	m_lenNom = (WORD)(lstrlen(m_pszNom)+1);
	
	if (*m_pDossier->m_pszPath) {
		// Convertir le chemin d'accs: transformer les \ en /
		
		// Copier le chemin d'accs
		const int lenPath = lstrlen(m_pDossier->m_pszPath)+1;
		m_lenNom = (WORD)(m_lenNom+lenPath);
		strtoA(s_aszNom, m_pDossier->m_pszPath, lenPath);
		// Convertir les \ en /
		char *pcSep = s_aszNom;
		for (;;) {
			pcSep = strchr(pcSep, '\\');
			if (!pcSep)  break;
			*pcSep++ = '/';
		}
		
		// Copier le nom du fichier
		s_aszNom[lenPath-1] = '/';
		strtoA(s_aszNom+lenPath, m_pszNom, MAX_PATH);
		
	}else
		// Recopier la chane
		strtoA(s_aszNom, m_pszNom, m_lenNom);
	
	// Restaurer m_lenNom  la longueur de s_aszNom sans nul final
	m_lenNom--;
	
	// Convertir le nom du fichier de ANSI en OEM
	CharToOemA(s_aszNom, s_aszNom);
}



// Copier l'entre Zip du fichier pfS au fichier pfD
// Mettre  jour e_lenEcrit
// * s_nbFichier
// * s_nFichier
bool CZipFichier::FichierCopier(CFileRead& fr, CFileWrite& fw)
{
	if (s_nbFichier)  VERIF(CFichier::FichierCopier(fr, fw));
	
	// Aller jusqu'aux donnes compresses
	DWORD len = sizeof(HEADERLOCSIG)+(DWORD)m_lenNom+(DWORD)m_lenExtra;
	VERIF(fr.Seek(m_off+len, FILE_BEGIN));
	
	// Mettre  jour la longueur du nom de l'entre,
	// et rcrire l'en-tte local
	m_lenNom = (WORD)lstrlen(m_pszNom);
	if (*m_pDossier->m_pszPath)
		m_lenNom = (WORD)(m_lenNom+lstrlen(m_pDossier->m_pszPath)+1);
	len = sizeof(HEADERLOCSIG)+(DWORD)m_lenNom+(DWORD)m_lenExtra;
	VERIF(PutLocal(fw));
	
	m_off = e_lenEcrit;
	e_lenEcrit += len;
	len = m_dwTailleC;
	
	// Copier les donnes compresses
	if (m_nFlags & locfEtendu)  len += lenEtendu;
	e_lenEcrit += len;
	VERIF(CFichier::FichierCopier(fr, fw, m_dwTailleC));
	
	// Copier l'en-tte statique tendu
	return (m_nFlags & locfEtendu) ? fw.WriteFrom(fr, lenEtendu) : true;
}
