// dlgAjouter.cpp
//
// * Conditions initiales:
// - Muzip doit permettre l'ajout de dossiers en marquant leur nom
// - On doit utiliser CFileDialog pour la bote de dialogue d'ajout
//   car on ne veut pas rinventer la roue
//
// * Expos du problme:
//   Dans CFileDialog, quand on marque un nom de dossier et qu'on clique
//   sur OK, la bote de dialogue ne se ferme pas en validant le nom du
//   dossier comme on le voudrait, mais le dossier en question est ouvert
//
// * Solution:
//   Sous-classer la fentre qui contient le bouton OK,
//   d'o l'inner-class CWndParent


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "AppResource.h"
#include "dlgAjouter.h"

#include "PlusItem.h"

#include <GUnicode.h>
#include <Dlgs.h>


#if 0
BEGIN_MESSAGE_MAP(CDlgAjouter, CFileDialog)
	//{{AFX_MSG_MAP(CDlgAjouter)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()
#endif


// Constructeur
CDlgAjouter::CDlgAjouter(CPlusItem* ppi)
	: CFileDialog(TRUE, NULL,NULL,
		OFN_HIDEREADONLY | OFN_ALLOWMULTISELECT | OFN_PATHMUSTEXIST),
		m_ppi(ppi)
{
	m_wndParent.m_pdlgAjouter = this;
	
	const LPCTSTR pszFiltre = m_ppi->DO_sFiltre;
	
	// Charger le titre
	m_sTitle.LoadString(IDS_AJOUTER);
	m_ofn.lpstrTitle = m_sTitle;
	
	// Permettre l'ajout de nombreux fichiers
	*m_pszFichiers   = 0;
	m_ofn.nMaxFile   = sizeof(m_pszFichiers);
	m_ofn.lpstrFile  = m_pszFichiers;
	if (!*pszFiltre) {
		// Aucune spcification: *.* par dfaut
		m_pszInit = NULL;
		lstrcpy(m_pszFichiers, _T("*.*"));
	}else{
		// Former la spcification
		
		TCHAR pszDebut[MAX_PATH];
		CLstCString2 lstPartie;
		int len = m_ppi->ListerPartie(pszDebut, lstPartie);
		
		// Enregistrer le rpertoire commun
		m_ofn.lpstrInitialDir = new TCHAR[len+1];
		lstrcpyn((LPTSTR)m_ofn.lpstrInitialDir, pszDebut, len+1);
		
		TCHAR *pcOut = m_pszInit = new TCHAR[MAX_PATH*64];
		
		if (lstPartie.GetNb()==1) {
			// Une seule partie: tout simple, pas de guillemets
			if (pszDebut[len]=='\\')  len++;
			lstrcpy(pcOut, pszDebut+len);
		}else{
			// Plusieurs parties, entoures de guillemets
			// et spares par des espaces
			
			POS pos = lstPartie.GetPosDeb();
			for (;;) {
				
				// Copier la partie
				pcOut += wsprintf(pcOut, _T("\"%s\""),
					(LPCTSTR)lstPartie.GetSuiv(pos)+len);
				
				if (!pos)  break;
				
				// Rajouter un sparateur
				*pcOut++ = ' ';
			}
		}
	}
}

// Destructeur
CDlgAjouter::~CDlgAjouter()
{
	delete [] (LPTSTR)m_ofn.lpstrInitialDir;
	m_ofn.lpstrInitialDir = NULL;
	delete [] m_pszInit;
}


// Initialisation
BOOL CDlgAjouter::OnInitDialog()
{
	CFileDialog::OnInitDialog();
	
	// Sous-classer la fentre parent (voir au dbut de ce fichier)
	m_wndParent.SubclassWindow(*GetParent());
	
	// Redfinir le label du bouton OK en Ajouter
	// et masquer la liste droulante du type de fichier ainsi que son label
#ifdef UNICODE
	inittoA(pszAjouter, (LPCTSTR)CStringRes(IDS_CMDAJOUTER), 32);
	SetControlText(IDOK, pszAjouter);
#else
	SetControlText(IDOK, CStringRes(IDS_CMDAJOUTER));
#endif
	HideControl(stc2);
	HideControl(cmb1);
	
	return TRUE;
}


// Changement de spcification
void CDlgAjouter::OnFileNameChange()
{
	if (m_pszInit) {
		// Dfinir la spcification de dpart
		// viter les dpassements de pile
		// en copiant l'adresse de la chane ailleurs
		// avant de modifier la zone de texte
		const LPTSTR pszInit = m_pszInit;
		m_pszInit = NULL;
		GetParent()->SetDlgItemText(edt1, pszInit);
		delete [] pszInit;
	}
}


// Ajouter (appel par CWndParent)
void CDlgAjouter::OnOK()
{
	// Vrifier la validit de la recherche
	// Comme ::FindFirstFile() n'accepte pas les points-virgule ';'
	// (sert  effectuer des recherches multiples),
	// vrifier la validit de chaque partie indpendament des autres
	WIN32_FIND_DATA wfd;
	
	// Spcification donne par l'utilisateur
	GetParent()->GetDlgItemText(edt1, m_pszFichiers, nbArray(m_pszFichiers));
	
	// Tampon de stockage des parties
	TCHAR pszPartie[MAX_PATH];
	
	// sFiltre contiendra la spcification finale:
	// chaque partie sera un chemin d'accs absolu de chaque partie
	CString sFiltre;
	
	TCHAR *pcDeb = m_pszFichiers;
	for (;;) {
		while (isspace(*pcDeb))  pcDeb++;
		if (!*pcDeb)  break;
		
		// Dlimiter la partie
		const bool fGuillemet = (*pcDeb=='"');
		if (fGuillemet)  pcDeb++;
		const TCHAR cSep = (fGuillemet) ? '"' : ' ';
		TCHAR *pcSep = pcDeb;
		while (*pcSep && *pcSep!=cSep)  pcSep++;
		const bool fFini = !*pcSep;
		*pcSep = 0;
		
		// Rcuprer le chemin d'accs absolu de la partie
		if (!GetFullPathName(pcDeb, nbArray(pszPartie), pszPartie, NULL)) {
			// Spcification incorrecte
Erreur:
			AfxMessageBox(ERR_AJOUTERSPEC);
			return;
		}
		
		// Vrifier la validit de la spcification
		const HANDLE hff = ::FindFirstFile(pszPartie, &wfd);
		if (hff==INVALID_HANDLE_VALUE) {
			const DWORD dwErreur = GetLastError();
			if (dwErreur!=ERROR_FILE_NOT_FOUND && dwErreur!=ERROR_PATH_NOT_FOUND)
				goto Erreur;
		}else
			::FindClose(hff);
		
		// Rajouter le ';' si ncessaire
		if (!sFiltre.IsEmpty())  sFiltre += ';';
		
		// Rajouter la partie, "tendue"  la spcification complte
		sFiltre += pszPartie;
		
		if (fFini)  break;
		
		pcDeb = pcSep+1;
	}
	
	m_ppi->DO_sFiltre = sFiltre;
	
	// Spcification correcte
	((CDialog*)GetParent())->EndDialog(IDOK);
}



//*** CDlgAjouter::CWndParent

// Commande
BOOL CDlgAjouter::CWndParent::OnCommand(WPARAM wParam, LPARAM lParam)
{
	if (LOWORD(wParam)==IDOK && HIWORD(wParam)==BN_CLICKED) {
		// Bouton OK
		m_pdlgAjouter->PostMessage(WM_COMMAND, IDOK);
		return TRUE;
	}
	
	return CWnd::OnCommand(wParam, lParam);
}
