// dlgArchivePlus.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "Muzip.h"
#include "dlgArchivePlus.h"

#include "MuzipArc.h"
#include "PlusItem.h"
#include "Fichier.h"

#include "dlgAjouter.h"
#include "dlgPlusItem.h"


BEGIN_MESSAGE_MAP(CDlgArchivePlus, CDialog)
	//{{AFX_MSG_MAP(CDlgArchivePlus)
	ON_BN_CLICKED   (IDCCMD_AJOUTER,   OnAjouter)
	ON_BN_CLICKED   (IDCCMD_ENLEVER,   OnEnlever)
	ON_BN_CLICKED   (IDCCMD_SUPPRIMER, OnSupprimer)
	ON_BN_CLICKED   (IDCCMD_HAUT,      OnHaut)
	ON_BN_CLICKED   (IDCCMD_BAS,       OnBas)
	ON_LBN_SELCHANGE(IDCLST,           OnLstSelChange)
	ON_LBN_DBLCLK   (IDCLST,           OnLstDblClk)
	//}}AFX_MSG_MAP
	ON_NOTIFY_EX(TTN_NEEDTEXT, 0, OnToolTipNeedText)
END_MESSAGE_MAP()


HICON CDlgArchivePlus::s_aIcon[nbBouton];


// Constructeur
CDlgArchivePlus::CDlgArchivePlus(CMuzipArc* pArc, CArchivePlus& arPlus, CString& sMdp, CWnd* pParent)
	: CDialog(IDD, pParent), m_pArc(pArc), m_arPlus(arPlus), m_sMdp(sMdp)
{
	m_nSup     = m_arPlus.m_nSup;
	m_sDossier = m_arPlus.m_sDossier;
	
	//{{AFX_DATA_INIT(CDlgArchivePlus)
	//}}AFX_DATA_INIT
}


// Contrles
void CDlgArchivePlus::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	
	//{{AFX_DATA_MAP(CDlgArchivePlus)
	DDX_Control (pDX, IDCLST,           m_lst);
	DDX_Control (pDX, IDCCMD_AJOUTER,   m_cmdAjouter);
	DDX_Control (pDX, IDCCMD_ENLEVER,   m_cmdEnlever);
	DDX_Control (pDX, IDCCMD_SUPPRIMER, m_cmdSupprimer);
	DDX_Control (pDX, IDCCMD_HAUT,      m_cmdHaut);
	DDX_Control (pDX, IDCCMD_BAS,       m_cmdBas);
	DDX_Radio   (pDX, IDCOPT_SUP_0,     m_nSup);
	DDX_Text    (pDX, IDCTXT_DOSSIER,   m_sDossier);
	//}}AFX_DATA_MAP
}

// Initialisation
BOOL CDlgArchivePlus::OnInitDialog()
{
	CDialog::OnInitDialog();
	
	EnableToolTips();
	
	if (!s_aIcon[0]) {
		// Charger les icnes des boutons
		CImageList imlBouton;
		imlBouton.Create(IDB_ARCHIVEPLUS, cBouton, nbBouton, RGB(192,192,192));
		for (int i=0; i<nbBouton; i++)
			s_aIcon[i] = imlBouton.ExtractIcon(i);
	}
	
	// Affecter les icnes aux boutons
	for (int i=0; i<nbBouton; i++)
		(&m_cmdAjouter)[i].SetIcon(s_aIcon[i]);   // Pas beau mais simple
	
	
	// Remplir la liste
	m_lst.SetRedraw(FALSE);
	POS pos = m_arPlus.GetPosDeb();
	while (pos)
		m_lst.InsertString(-1, new CPlusItem(*m_arPlus.GetSuiv(pos)));
	m_lst.SetRedraw();
	
	// Aucune slection au dbut
	OnLstSelChange();
	
	return TRUE;
}


// OK
void CDlgArchivePlus::OnOK()
{
	if(!UpdateData())  return;
	
	m_arPlus.m_nSup     = m_nSup;
	m_arPlus.m_sDossier = m_sDossier;
	
	int nItem = m_lst.GetCount();
	if (!nItem)  OnCancel();
	
	// Lister les objets CPlusItem
	m_arPlus.DeleteAll();
	while (nItem)
		m_arPlus.AddDeb((CPlusItem*)m_lst.GetItemDataPtr(--nItem));
	
	EndDialog(IDOK);
}

// Annuler
void CDlgArchivePlus::OnCancel()
{
	// Dtruire les objets
	int nItem = m_lst.GetCount();
	while (--nItem>=0)
		delete (CPlusItem*)m_lst.GetItemDataPtr(nItem);
	
	EndDialog(IDCANCEL);
}


// Ajouter
void CDlgArchivePlus::OnAjouter()
{
	AjouterEnlever(true);
}

// Enlever
void CDlgArchivePlus::OnEnlever()
{
	AjouterEnlever(false);
}

// Ajouter/enlever
void CDlgArchivePlus::AjouterEnlever(bool fAjouter)
{
	// Crer l'item dans la liste
	CPlusItem *const ppi = new CPlusItem;
	if (!ppi->DemanderSpec()) {
		delete ppi;
		return;
	}
	
	*ppi = oApp.s_pi;
	ppi->CO_nAction = (fAjouter) ? coaPlusMAJ : coaMoins;
	
	// Ajouter l'item  la fin de la liste
	// et le slectionner uniquement lui
	m_lst.SetSel(-1, FALSE);
	m_lst.SetSel(m_lst.InsertString(-1, ppi));
	OnLstSelChange();
	
	// Demander les options
	CDlgPlusItem(m_pArc, *ppi, m_sMdp, true, this).DoModal();
	
	m_lst.SetFocus();
}


// Supprimer
void CDlgArchivePlus::OnSupprimer()
{
	m_lst.SetRedraw(FALSE);
	int nItem = 0;
	for (;;) {
		const int sel = m_lst.GetSel(nItem);
		if (sel==LB_ERR)  break;
		if (sel) {
			delete (CPlusItem*)m_lst.GetItemDataPtr(nItem);
			m_lst.DeleteString(nItem);
		}else
			nItem++;
	}
	m_lst.SetRedraw();
	OnLstSelChange();
}

// Haut
void CDlgArchivePlus::OnHaut()
{
	int nCaret = m_lst.GetCaretIndex();
	int nbItem = m_lst.GetCount();
	int nItem  = -1;
	CPlusItem *ppiSauve = NULL;
	
	m_lst.SetRedraw(FALSE);
	
	for (;;) {
		
		// Trouver le premier item slectionn
		for (;;) {
			if (++nItem==nbItem)  goto Fin;
			if (m_lst.GetSel(nItem))  break;
		}
		
		if (nItem>0) {
			// Sauver puis supprimer l'item prcdent
			// (l'item non-slectionn qu'on veut dplacer)
			nItem--;
			if      (nCaret==nItem)  nCaret = LB_ERR;
			else if (nCaret> nItem)  nCaret--;
			ppiSauve = (CPlusItem*)m_lst.GetItemDataPtr(nItem);
			m_lst.DeleteString(nItem);
			nbItem--;
		}
		
		// Aller jusqu'au premier item non-slectionn
		for (;;) {
			if (++nItem==nbItem)  goto Fin;
			if (!m_lst.GetSel(nItem))  break;
		}
		
		// Ajouter l'item sauv
		if (ppiSauve) {
			m_lst.InsertString(nItem, ppiSauve);
			if      (nCaret==LB_ERR)  nCaret = nItem;
			else if (nCaret>=nItem)   nCaret++;
			ppiSauve = NULL;
			nbItem++;
		}
	}
Fin:
	
	if (ppiSauve) {
		m_lst.InsertString(-1, ppiSauve);
		if (nCaret==LB_ERR)  nCaret = nbItem-1;
	}
	
	m_lst.SetCaretIndex(nCaret);
	m_lst.SetFocus();
	
	m_lst.SetRedraw();
}

// Bas
void CDlgArchivePlus::OnBas()
{
	int nCaret  = m_lst.GetCaretIndex();
	int nItem   = m_lst.GetCount();
	int derItem = nItem-1;
	CPlusItem *ppiSauve = NULL;
	
	m_lst.SetRedraw(FALSE);
	
	for (;;) {
		
		// Trouver le premier item slectionn
		for (;;) {
			if (--nItem<0)  goto Fin;
			if (m_lst.GetSel(nItem))  break;
		}
		
		if (nItem<derItem) {
			// Sauver puis supprimer l'item suivant
			// (l'item non-slectionn qu'on veut dplacer)
			nItem++;
			if      (nCaret==nItem)  nCaret = LB_ERR;
			else if (nCaret< nItem)  nCaret++;
			ppiSauve = (CPlusItem*)m_lst.GetItemDataPtr(nItem);
			m_lst.DeleteString(nItem);
			nItem--;
		}
		
		// Aller jusqu'au premier item non-slectionn
		for (;;) {
			if (--nItem<0)  goto Fin;
			if (!m_lst.GetSel(nItem))  break;
		}
		
		// Ajouter l'item sauv
		if (ppiSauve) {
			nItem++;
			m_lst.InsertString(nItem, ppiSauve);
			if      (nCaret==LB_ERR)  nCaret = nItem;
			else if (nCaret<=nItem)   nCaret--;
			nItem--;
			ppiSauve = NULL;
		}
	}
Fin:
	
	if (ppiSauve) {
		m_lst.InsertString(0, ppiSauve);
		if (nCaret==LB_ERR)  nCaret = 0;
	}
	
	m_lst.SetCaretIndex(nCaret);
	m_lst.SetFocus();
	
	m_lst.SetRedraw();
}


// Liste - slection modifie
void CDlgArchivePlus::OnLstSelChange()
{
	const bool fSel = (m_lst.GetSelCount()>0);
	m_cmdSupprimer.EnableWindow(fSel);
	m_cmdHaut     .EnableWindow(fSel);
	m_cmdBas      .EnableWindow(fSel);
}

// Liste - double-clic
void CDlgArchivePlus::OnLstDblClk()
{
	const int nItem = m_lst.GetCaretIndex();
	
	CPlusItem *const ppi = (CPlusItem*)m_lst.GetItemDataPtr(nItem);
	if (CDlgPlusItem(m_pArc, *ppi, m_sMdp, true, this).DoModal()==IDOK) {
		// Supprimer l'ancien item et ajouter le nouveau
		m_lst.SetRedraw(FALSE);
		m_lst.DeleteString(nItem);
		m_lst.InsertString(nItem, ppi);
		m_lst.SetRedraw();
	}
}


// Info-bulle
BOOL CDlgArchivePlus::OnToolTipNeedText(UINT, NMTTDISPINFO* pTTT, LRESULT*)
{
	if (pTTT->uFlags & TTF_IDISHWND) {
		pTTT->lpszText = MAKEINTRESOURCE(::GetDlgCtrlID((HWND)pTTT->hdr.idFrom));
		pTTT->hinst    = AfxGetResourceHandle();
		return TRUE;
	}
	return FALSE;
}
