// dlgLister.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "AppResource.h"
#include "dlgLister.h"

#include <FileBuf.h>

#include "Options.h"
#include "MuzipArc.h"
#include "Fichier.h"
#include "Dossier.h"

#ifdef UNICODE
typedef BYTE UTCHAR;
#else
typedef WORD UTCHAR;
#endif


BEGIN_MESSAGE_MAP(CDlgLister, CDialog)
	//{{AFX_MSG_MAP(CDlgLister)
	ON_CBN_SELCHANGE(IDCCBO_TYPE, OnTypeSelChange)
	ON_BN_CLICKED   (IDCCHK_NOM,  OnChkNom)
	ON_BN_CLICKED   (IDCCHK_PATH, OnChkPath)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


// Constructeur
CDlgLister::CDlgLister(CMuzipArc* pArc, CWnd* pParent)
	: CDialog(IDD, pParent), m_pArc(pArc)
{
	//{{AFX_DATA_INIT(CDlgLister)
	m_nType    = 0;
	m_fColonne = TRUE;
	m_fNom     = TRUE;
	m_fPath    = TRUE;
	m_fType    = TRUE;
	m_fTailleN = TRUE;
	m_fTailleC = TRUE;
	m_fPC      = TRUE;
	m_fDtModif = TRUE;
	m_fAttr    = TRUE;
	//}}AFX_DATA_INIT
}


// Contrles
void CDlgLister::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	
	//{{AFX_DATA_MAP(CDlgLister)
	DDX_CBIndex(pDX, IDCCBO_TYPE,    m_nType);
	DDX_Check  (pDX, IDCCHK_COLONNE, m_fColonne);
	DDX_Check  (pDX, IDCCHK_NOM,     m_fNom);
	DDX_Check  (pDX, IDCCHK_PATH,    m_fPath);
	DDX_Check  (pDX, IDCCHK_TYPE,    m_fType);
	DDX_Check  (pDX, IDCCHK_TAILLEN, m_fTailleN);
	DDX_Check  (pDX, IDCCHK_TAILLEC, m_fTailleC);
	DDX_Check  (pDX, IDCCHK_PC,      m_fPC);
	DDX_Check  (pDX, IDCCHK_DTMODIF, m_fDtModif);
	DDX_Check  (pDX, IDCCHK_ATTR,    m_fAttr);
	//}}AFX_DATA_MAP
	
	if (!pDX->m_bSaveAndValidate)  OnTypeSelChange();
}


// Nouveau format
void CDlgLister::OnTypeSelChange()
{
	const bool fPeutInfo = !GetQueUneSeuleColonne(((CComboBox*)GetDlgItem(IDCCBO_TYPE))->GetCurSel());
	
	for (int nID=IDCCHK_COLONNE; nID<=IDCCHK_ATTR; nID++)
		::EnableWindow(::GetDlgItem(*this, nID), fPeutInfo);
	
	MajChk(IDCCHK_NOM, IDCCHK_PATH);
}


// Nom/chemin d'accs
void CDlgLister::OnChkNom()
{
	MajChk(IDCCHK_NOM, IDCCHK_PATH);
}
void CDlgLister::OnChkPath()
{
	MajChk(IDCCHK_PATH, IDCCHK_NOM);
}
void CDlgLister::MajChk(int idChkClic, int idChkAutre)
{
	const bool fClic  = ToBool(IsDlgButtonChecked(idChkClic));
	const bool fAutre = ToBool(IsDlgButtonChecked(idChkAutre));
	
	if (GetQueUneSeuleColonne(((CComboBox*)GetDlgItem(IDCCBO_TYPE))->GetCurSel()))
		// Arborescence et liste: une seule case permise
		CheckDlgButton(idChkAutre, !fClic);
		
	else if (!fClic && !fAutre)
		// Autre: au moins une des deux cases
		CheckDlgButton(idChkAutre, TRUE);
}


// OK
void CDlgLister::OnOK()
{
#define VERIFOUT(f)  if (!(f))  return
	
	// Dcocher les cases dsactives
	if (GetQueUneSeuleColonne(((CComboBox*)GetDlgItem(IDCCBO_TYPE))->GetCurSel())) {
		for (int nID=IDCCHK_COLONNE; nID<=IDCCHK_ATTR; nID++)
			CheckDlgButton(nID, FALSE);
	}
	if (!UpdateData())  return;
	
	// Initialiser la bote de dialogue d'exportation
	// avec la bonne extension par dfaut et le bon filtre
	LPCTSTR pszDefExt;
	int     idsFiltre;
	if      (m_nType<=ltHTMLListe) {
		pszDefExt = _T("htm");
		idsFiltre = IDS_FILTREHTML;
	}else if (m_nType<=ltTexteTab) {
		pszDefExt = _T("txt");
		idsFiltre = IDS_FILTRETXT;
	}else /* (m_nType==ltCSV)*/ {
		pszDefExt = _T("csv");
		idsFiltre = IDS_FILTRECSV;
	}
	CFileDialog dlgExporter(FALSE, pszDefExt, NULL,
		OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, CStringRes(idsFiltre), this);
	if (dlgExporter.DoModal()==IDCANCEL)  return;
	
	
	m_nbColD =
		m_fNom +
		m_fPath;
	m_nbColF =
		m_nbColD +
		m_fType +
		m_fTailleN +
		m_fTailleC +
		m_fPC +
		m_fDtModif +
		m_fAttr;
	ZeroMemory(m_alenCol, sizeof(m_alenCol));
	
	
	//*** Initialiser l'exportation
	if (!m_fw.Open(dlgExporter.m_ofn.lpstrFile))  return;
	CDossier::s_pdlgLister = this;
	
	// Rcuprer le sparateur
	if      (m_nType==ltTexteTab)  m_cSep = '\t';   // Tabul
	else if (m_nType==ltCSV)       m_cSep = ';';    // CSV
	
	// En-tte
	if (m_nType<=ltHTMLListe) {
		// HTML
		VERIFOUT(m_fw.Write(_T("<HTML><BODY>")));
		if (m_nType==ltHTMLTableau)  VERIFOUT(m_fw.Write(_T("<TABLE>")));
	}
	
	// Liste des champs
	if (m_fColonne) {
		TCHAR psz[64];
		const CString s = MAKEINTRESOURCE(IDS_COLONNESLISTER);
		const TCHAR *pc;
		const BOOL  *pf;
		int iCol, nCol;
		
		if (m_nType==ltTexteFixe) {
			// Prendre en compte les colonnes de l'en-tte
			
			pc = s;
			pf = &m_fNom;
			for (iCol=0;; iCol++, pf++) {
				TCHAR *const pcSuite = _tcschr(pc, '\n');
				if (!pcSuite)  break;
				*pcSuite = 0;
				if (*pf)  m_alenCol[iCol] = lstrlen(pc);
				*pcSuite = '\n';
				pc = pcSuite+1;
			}
			
			// Initialiser avec les largeurs des colonnes des autres lignes
			m_nType = ltTexteFixeInit;
			m_pArc->Lister();
			m_nType = ltTexteFixe;
		}
		
		VERIFOUT(AddDeb(0));
		
		pc = s;
		pf = &m_fNom;
		for (iCol=nCol=0;; iCol++, pf++) {
			TCHAR *const pcSuite = _tcschr(pc, '\n');
			if (!pcSuite)  break;
			*pcSuite = 0;
			if (*pf) {
				if (m_nType<=ltHTMLListe) {
					// HTML - en-tte en gras
					wsprintf(psz, _T("<B>%s</B>"), pc);
					VERIFOUT(AddSep(iCol,nCol,m_nbColF, psz));
				}else
					// Texte - en-tte normal
					VERIFOUT(AddSep(iCol,nCol,m_nbColF, pc));
				nCol++;
			}
			pc = pcSuite+1;
		}
		
		VERIFOUT(AddFin());
		
	}else if (m_nType==ltTexteFixe) {
		
		// Initialiser avec les largeurs des colonnes des autres lignes
		m_nType = ltTexteFixeInit;
		m_pArc->Lister();
		m_nType = ltTexteFixe;
	}
	
	// Effectuer l'exportation
	VERIFOUT(m_pArc->Lister());
	
	// Pied de page
	if (m_nType<=ltHTMLListe) {
		if (m_nType==ltHTMLTableau)  VERIFOUT(m_fw.Write(_T("</TABLE>")));
		VERIFOUT(m_fw.Write(_T("</BODY></HTML>")));
	}
	
	// OK
	m_fw.Close();
	EndDialog(IDOK);
	
#undef VERIFOUT
}


// Ajouter un dbut de ligne
bool CDlgLister::AddDeb(int nProf)
{
	if (m_nType==ltTexteFixeInit)  return true;
	
	if (m_nType<=ltHTMLTableau)  VERIF(m_fw.Write(_T("<TR>")));
	
	// Arborescence: insrer le dcalage  gauche
	if (m_nType==ltHTMLArbo || m_nType==ltTexteArbo) {
		const LPCTSTR pszSep = (m_nType==ltHTMLArbo)
			? _T("&nbsp   &nbsp") : _T("  ");
		for (int nbReste=nProf; nbReste; nbReste--)
			VERIF(m_fw.Write(pszSep));
	}
	return true;
}

// Ajouter une fin de ligne
bool CDlgLister::AddFin()
{
	if (m_nType==ltTexteFixeInit)  return true;
	
	if (m_nType<=ltHTMLListe)
		// HTML
		return m_fw.Write((m_nType==ltHTMLTableau) ? _T("</TR>") : _T("<BR>"));
		
	else
		// Texte
		return m_fw.Write(_T("\r\n"));
}


// Lister un fichier
bool CDlgLister::Lister(const CFichier* pFichier)
{
	int nCol = 0;
	TCHAR psz[MAX_PATH];
	
	// Nom
	if (m_fNom)
		VERIF(AddSep(0,nCol++,m_nbColF, pFichier->m_pszNom));
	
	// Chemin d'accs
	if (m_fPath) {
		const LPTSTR pszPath = pFichier->m_pDossier->m_pszPath;
		lstrcpy(psz, pszPath);
		const int iDer = lstrlen(psz);
		if (iDer>0 && psz[iDer-1]!='\\') {
			psz[iDer  ] = '\\';
			psz[iDer+1] = 0;
		}
		lstrcat(psz, pFichier->m_pszNom);
		VERIF(AddSep(1,nCol++,m_nbColF, psz));
	}
	
	// Type
	if (m_fType) {
		SHFILEINFO shfi;
		SHGetFileInfo(pFichier->m_pszNom, 0, &shfi, sizeof(shfi),
			SHGFI_USEFILEATTRIBUTES | SHGFI_TYPENAME);
		VERIF(AddSep(2,nCol++,m_nbColF, shfi.szTypeName));
	}
	
	// Taille
	if (m_fTailleN) {
		StrFormatByteSize(pFichier->m_dwTailleN, psz, nbArray(psz));
		VERIF(AddSep(3,nCol++,m_nbColF, psz));
	}
	
	// Compress
	if (m_fTailleC) {
		StrFormatByteSize(pFichier->m_dwTailleC, psz, nbArray(psz));
		VERIF(AddSep(4,nCol++,m_nbColF, psz));
	}
	
	// %
	if (m_fPC) {
		wsprintf(psz, _T("%hd %%"), pFichier->m_nPC);
		VERIF(AddSep(5,nCol++,m_nbColF, psz));
	}
	
	// Modifi
	if (m_fDtModif) {
		SYSTEMTIME st = pFichier->m_dtM;
		*psz = 0;
		const int len = GetDateFormat(LOCALE_USER_DEFAULT, DATE_SHORTDATE, &st, NULL, psz, nbArray(psz));
		if (e_opt.m_fAffichageModifHeure) {
			// Rajouter l'heure  la date
			if (len)  psz[len-1] = ' ';
			GetTimeFormat(LOCALE_USER_DEFAULT, 0, &st, NULL, &psz[len], nbArray(psz)-len);
		}
		VERIF(AddSep(6,nCol++,m_nbColF, psz));
	}
	
	// Attributs
	if (m_fAttr)  VERIF(AddSep(7,nCol++,m_nbColF, pFichier->m_pszAttr));
	
	return true;
}

// Lister un dossier
bool CDlgLister::Lister(const CDossier* pDossier)
{
	int nCol = 0;
	
	if (m_nType<=ltHTMLListe)  VERIF(m_fw.Write(_T("<B>")));
	
	
	// Nom
	if (m_fNom)
		VERIF(AddSep(0,nCol++,m_nbColD, pDossier->m_pszNom));
	
	// Chemin d'accs
	if (m_fPath)
		VERIF(AddSep(1,nCol++,m_nbColD, pDossier->m_pszPath));
	
	
	return (m_nType>ltHTMLListe || m_fw.Write(_T("</B>")));
}


// Ajouter un sparateur aprs une colonne
bool CDlgLister::AddSep(int iCol, int nCol, int nbCol, LPCTSTR psz)
{
	if (m_nType==ltTexteFixeInit) {
		// Initialisation taille des colonnes en texte fixe
		const int len = lstrlen(psz);
		if (m_alenCol[iCol]<len)  m_alenCol[iCol] = len;
		return true;
	}
	
	enum{ alignLeft, alignCenter, alignRight };
	
	const LPCTSTR apszAlign[] =
	{
		_T(""),
		_T(" ALIGN=CENTER"),
		_T(" ALIGN=RIGHT"),
	};
	const int aAlign[] =
	{
		alignLeft,   alignLeft,  alignLeft,
		alignRight,  alignRight,
		alignCenter, alignCenter,
		alignLeft,
	};
	
	TCHAR pszOut[MAX_PATH+2];
	
	if (nCol>=nbCol-1) {
		// Dernire colonne
		
		if (m_nType==ltHTMLTableau)
			wsprintf(pszOut, _T("<TD%s>%s</TD>"), apszAlign[aAlign[iCol]], psz);
		else
			return m_fw.Write(psz);
		
	}else{
		// Autres colonnes
		
		if (m_nType==ltHTMLTableau)
			// HTML - tableau
			wsprintf(pszOut, _T("<TD%s>%s  &nbsp;</TD>"), apszAlign[aAlign[iCol]], psz);
			
		else{
			
			// Texte
			if (m_nType==ltTexteFixe) {
				// Longueur fixe
				
				// Calculer le nombre d'espaces  ajouter  gauche et  droite
				const int nbManque = m_alenCol[iCol]-lstrlen(psz);
				int nbG;
				if      (aAlign[iCol]==alignLeft)     nbG = 0;
				else if (aAlign[iCol]==alignCenter)   nbG = nbManque/2;
				else /* (aAlign[iCol]==alignRight)*/  nbG = nbManque;
				int nbD = nbManque-nbG + 2;
				
				// Ajouter les espaces
				TCHAR *pc = pszOut;
				while (nbG>0) {
					nbG--;
					*pc++ = ' ';
				}
				while (*psz)
					*pc++ = *psz++;
				while (nbD>0) {
					nbD--;
					*pc++ = ' ';
				}
				*pc = 0;
				
			}else // (m_nType>=ltTexteTab)
				// Sparateur
				wsprintf(pszOut, _T("%s%c"), psz, m_cSep);
		}
	}
	
	m_fw.Write(pszOut);
	m_fw.Flush();
	return true;
}
