// GUnicode.h
//
// Macros pour la gestion des conversions entre ASCII et Unicode,
// dans les deux sens
//
// toBSTR(psz)   - convertit une chane "normale" en BSTR
// fromBSTR(psz) - convertit une BSTR en chane "normale"
//
// REQUIERT
// - Dans StdAfx.h:
//   #include <Windows.h>
//   Pour toBSTR uniquement:
//   #include <OLEAuto.h>
// - Pour fromBSTR uniquement:
//   Application MFC


#ifndef _GUNICODE_H_
#define _GUNICODE_H_
#if _MSC_VER>1000
#pragma once
#endif


// Macros pour toutes les conversions possibles

#define strAtoA(pszA1, pszA2, len)   strcpy(pszA1, pszA2)
#define strWtoW(pszW1, pszW2, len)   wcscpy(pszW1, pszW2)

#define strAtoW(pszW, pszA, len) \
	MultiByteToWideChar(CP_ACP,0, pszA,-1, pszW,len)
#define strWtoA(pszA, pszW, len) \
	WideCharToMultiByte(CP_ACP,0, pszW,-1, pszA,len, NULL,NULL)

#define initAtoA( nom, pszA, len)  const LPSTR   nom = pszA
#define initAtoAC(nom, pszA, len)  const LPCSTR  nom = pszA
#define initWtoW( nom, pszW, len)  const LPWSTR  nom = pszW
#define initWtoWC(nom, pszW, len)  const LPCWSTR nom = pszW

#define initAtoW(nom, pszA, len) \
	WCHAR nom[len]; \
	strAtoW(nom, pszA, len)
#define initAtoWC  initAtoW

#define initWtoA(nom, pszW, len) \
	char nom[len]; \
	strWtoA(nom, pszW, len)
#define initWtoAC  initWtoA



#ifdef    UNICODE
// Mode Unicode

#define strtoA     strWtoA
#define strtoW     strWtoW
#define strfromA   strAtoW
#define strfromW   strWtoW

#define initA     initAtoW
#define initAC    initAtoWC
#define initW     initWtoW
#define initWC    initWtoWC

#define inittoA   initWtoA
#define inittoAC  initWtoAC
#define inittoW   initWtoW
#define inittoWC  initWtoWC


#else // !UNICODE
// Mode ASCII

#define strtoA    strAtoA
#define strtoW    strAtoW
#define strfromA  strAtoA
#define strfromW  strWtoA

#define initA     initAtoA
#define initAC    initAtoAC
#define initW     initWtoA
#define initWC    initWtoAC

#define inittoA   initAtoA
#define inittoAC  initAtoAC
#define inittoW   initAtoW
#define inittoWC  initAtoWC


#endif // UNICODE

#ifdef _OLEAUTO_H_

class toBSTR
{
public:  // Fonctions
	
	toBSTR(LPCSTR asz)
		{
			const size_t len = strlen(asz);
			if (len) {
				m_bsz = ::SysAllocStringLen(NULL, len);
				strAtoW(m_bsz, asz, len+1);
			}else
				m_bsz = NULL;
		}
	
	toBSTR(LPCWSTR wsz)
		{
			m_bsz = ::SysAllocString(wsz);
		}
	
#ifdef MFCOK
	toBSTR(UINT nID)
		{
			const int lenMax = 512;
			TCHAR psz[lenMax];
			const int len = ::LoadString(AfxGetInstanceHandle(), nID, psz, lenMax);
			if (len) {
#ifdef UNICODE
				m_bsz = ::SysAllocStringLen(psz, len);
#else
				m_bsz = ::SysAllocStringLen(NULL, len);
				strtoW(m_bsz, psz, len+1);
#endif
			}else
				m_bsz = NULL;
		}
#endif
	
	toBSTR(HINSTANCE hInst, UINT nID)
		{
			const int len = 512;
			TCHAR psz[len];
			if (::LoadString(hInst, nID, psz, len)) {
#ifdef UNICODE
				m_bsz = ::SysAllocString(psz);
#else
				WCHAR wsz[len];
				strtoW(wsz, psz, len);
				m_bsz = ::SysAllocString(wsz);
#endif
			}else
				m_bsz = NULL;
		}
	
	~toBSTR()
		{
			::SysFreeString(m_bsz);
		}
	
	operator const BSTR() const { return m_bsz; }
	
	
protected:  // Donnes
	
	BSTR m_bsz;
};

#endif // _OLEAUTO_H_


#ifdef    MFCOK

#ifdef    UNICODE
#define fromBSTR(bsz)          bsz
#else // !UNICODE
#define fromBSTR(bsz)  CString(bsz)
#endif // UNICODE

#endif // !MFCOK


#endif
