// Pile.h
//
// Classes de gestion de piles
// CPile    - classe abstraite de pile
// CPileCh  - pile chane
// CPileMax - pile avec nombre maximum d'lments


#ifndef _PILE_H_
#define _PILE_H_
#if _MSC_VER>1000
#pragma once
#endif


// Classe CPile - classe abstraite de pile
template<class T> class CPile
{
protected:  // Donnes
	
	// Longueur de la pile
	int m_nb;
	
	
public:  // Fonctions
	
	// Constructeur
	CPile()
		{
			m_nb = 0;
		}
	
	// Retourne une rfrence sur le n-ime lment de la pile
	//  partir de la fin
	virtual T& operator [] (int n) = 0;
	
	// Retourne la longueur de la pile
	int GetNb() const { return m_nb; }
	
	// Vider la pile de ses n derniers lments
	virtual void Vider(int n=1) = 0;
	
	// Vider toute la pile
	virtual void ViderTout() = 0;
	
	// Renvoie TRUE si l'on peut effectuer l'opration Pop
	virtual bool CanPop() { return m_nb>0; }
	// Retourne le dernier lment de la pile aprs l'en avoir enlev
	virtual T Pop() = 0;
	
	// Renvoie TRUE si l'on peut effectuer l'opration Push
	virtual bool CanPush() { return true; }
	// Ajoute un lement  la pile
	virtual void Push(T t) = 0;
};


// Classe CPileCh - pile chane
// T = classe des lments
template<class T> class CPileCh : public CPile<T>
{
protected:
	
	struct ELEMENT {
		T        t;        // Valeur
		ELEMENT *pAvant;   // lment prcdent
	};
	
	// Dernier lment de la pile
	ELEMENT *m_pDer;
	
	
public:
	
	// Constructeur
	CPileCh() : CPile<T>()
		{
			m_pDer = NULL;
		}
	
	// Destructeur
	~CPileCh()
		{
			ViderTout();
		}
	
	// Retourne une rfrence sur le n-ime lment de la pile
	//  partir de la fin
	T& operator [] (int n)
		{
			ELEMENT *pElement = m_pDer;
			while (n) {
				pElement = pElement->pAvant;
				n--;
			}
			
			return pElement->t;
		}
	
	// Vider la pile de ses n derniers lments
	void Vider(int n=1)
		{
			while (n) {
				// Supprimer le dernier lment
				ELEMENT *pElement = m_pDer;
				m_pDer = m_pDer->pAvant;
				m_nb--;
				delete pElement;
				
				n--;
			}
		}
	
	// Vider toute la pile
	void ViderTout()
		{
			while (m_nb) {
				ELEMENT *pElement = m_pDer;
				m_pDer = m_pDer->pAvant;
				delete pElement;
				m_nb--;
			}
		}
	
	// Retourne le dernier lment de la pile aprs l'en avoir enlev
	T Pop()
		{
			ELEMENT *pSup = m_pDer;
			T        tSup = m_pDer->t;
			m_pDer = m_pDer->pAvant;
			m_nb--;
			
			delete pSup;
			return tSup;
		}
	
	// Ajoute un lement  la pile
	void Push(T t)
		{
			ELEMENT *pElement = new ELEMENT;
			pElement->t      = t;
			pElement->pAvant = m_pDer;
			
			m_pDer = pElement;
			m_nb++;
		}
};

// Classe CPileMax - pile avec nombre maximum d'lments
// T = classe des lments
// nb = nombre maximum d'lments
template<class T, int nb> class CPileMax : public CPile<T>
{
protected:  // Donnes
	
	// lments
	T m_at[nb];
	
	
public:  // Fonctions
	
	// Retourne une rfrence sur le n-ime lment de la pile
	//  partir de la fin
	T& operator [] (int n)
		{
			return m_at[m_nb - n -1];
		}
	
	// Vider la pile de ses n derniers lments
	void Vider(int n=1)
		{
			m_nb -= n;
		}
	
	// Vider toute la pile
	void ViderTout()
		{
			m_nb = 0;
		}
	
	// Retourne le dernier lment de la pile aprs l'en avoir enlev
	T Pop()
		{
			return m_at[--m_nb];
		}
	
	// Renvoie TRUE si l'on peut effectuer l'opration Push
	virtual bool CanPush() { return m_nb<nb; }
	// Ajoute un lement  la pile
	void Push(T t)
		{
			m_at[m_nb++] = t;
		}
};


#endif
