// Temps.h
//
// Gestion du temps  l'aide de classes
// CTemps     - temps en nombre de secondes depuis le 01/01/1970
// CTempsD    - date uniquement
// CTempsH    - heure uniquement
// CTempsDH   - date et heure
// CTempsHS   - heure avec secondes
// CTempsDHS  - date, heure et secondes
// CTempsSpan - diffrence entre deux temps
//
// Les classes CTempsD,H,DH,HS,DHS maintiennent le temps en stockant
// chaque valeur sparment (jour, mois, ...)
//
// REQUIERT
// - Temps.cpp, Temps.inl
// - Application Windows
// - Application MFC (facultatif)
// - Dans StdAfx.h:
//   #pragma include_alias("AppResource.h", "../RpertoireProjet/Resource.h")


#ifndef _TEMPS_H_
#define _TEMPS_H_
#if _MSC_VER>1000
#pragma once
#endif

#include <Time.h>

const int minAn = 1970;
const int maxAn = 2106;


extern const UINT e_aNbJourMois[12];


enum DATEPART
{
	dtpOK,
	dtpJour,
	dtpMois,
	dtpAn,
	dtpHeure,
	dtpMinute,
	dtpSeconde,
};


// Type de temps reprsentant un nombre de secondes depuis de 01/01/1970
typedef ULONG TEMPS;

// Temps dcompos
struct TEMPSD
{
	UINT uJour;
	UINT uMois;
	UINT uAn;
	UINT uHeure;
	UINT uMinute;
	UINT uSeconde;
};


class CTempsD;
class CTempsH;
class CTempsDH;
class CTempsHS;
class CTempsDHS;

class CTempsSpan;


TEMPS MakeTemps(UINT uJour, UINT uMois, UINT uAn, UINT uHeure=0, UINT uMinute=0, UINT uSeconde=0);
void  MakeTempsD(TEMPS tps, TEMPSD* ptpsd);


bool IsDateValide(UINT uJour, UINT uMois, UINT uAn);
bool IsDateValide(SYSTEMTIME& st);

#ifdef _DEBUG
void FillDate(int iJour, int iMois, int iAn, SYSTEMTIME& st);
#endif


// Indiquer si une anne est bissextile ou non
inline bool IsBissextile(UINT uAn) { return !(uAn % 4) && (uAn % 100); }


// Renvoyer le nombre de jours d'une anne
// (les annes divisibles par 4 sont bissextiles,  366 jours)
inline UINT NbJoursAn(UINT uAn) { return 365 + IsBissextile(uAn); }

// Renvoyer le nombre de jours d'un mois
inline UINT NbJoursMois(UINT uMois, UINT uAn)
	{ return e_aNbJourMois[uMois-1] + (uMois==2 && IsBissextile(uAn)); }

#ifdef MFCEXTOK
void DDX_TempsD  (CDataExchange* pDX, int nIDC, CTempsD&   dt);
void DDX_TempsDHS(CDataExchange* pDX, int nIDC, CTempsDHS& dt);
void DDV_TempsD  (CDataExchange* pDX, int nIDC, CTempsD&   dt, bool fPeutNul=false);
void DDV_TempsDHS(CDataExchange* pDX, int nIDC, CTempsDHS& dt, bool fPeutNul=false);
#endif


// Fonctions communes indpendantes des MFC
#define FonctionsOK(type) \
	CTemps##type() {} \
	CTemps##type(const TEMPSD& tpsd); \
	CTemps##type(const TEMPS& tps); \
	CTemps##type(const CTemps& tps); \
	CTemps##type(const tm& otm); \
	CTemps##type(const SYSTEMTIME& st); \
	CTemps##type(const FILETIME& ft); \
	CTemps##type(const time_t& t); \
	\
	void SetNow(); \
	DATEPART PasValide() const; \
	void Add##type(long n, DATEPART dtp); \
	void FromDlg##type(HWND hDlg, int nIDC); \
	void ToDlg##type  (HWND hDlg, int nIDC) const; \
	\
	CTemps##type& operator = (const TEMPSD&       tpsd); \
	CTemps##type& operator = (const TEMPS&        tps);  \
	CTemps##type& operator = (const CTemps&       tps);  \
	CTemps##type& operator = (const CTemps##type& dt);   \
	CTemps##type& operator = (const tm&           otm);  \
	CTemps##type& operator = (const SYSTEMTIME&   st);   \
	CTemps##type& operator = (const FILETIME&     ft);   \
	CTemps##type& operator = (const time_t&       t);    \
	\
	CTemps##type  operator +  (const CTempsSpan& ts) const; \
	CTemps##type  operator -  (const CTempsSpan& ts) const; \
	CTemps##type& operator += (const CTempsSpan& ts); \
	CTemps##type& operator -= (const CTempsSpan& ts); \
	\
	CTempsSpan    operator -  (const CTemps##type& dt) const; \
	\
	bool operator == (const CTemps##type& dt) const; \
	bool operator != (const CTemps##type& dt) const; \
	bool operator <  (const CTemps##type& dt) const; \
	bool operator >  (const CTemps##type& dt) const; \
	bool operator <= (const CTemps##type& dt) const; \
	bool operator >= (const CTemps##type& dt) const; \
	\
	operator TEMPSD    () const; \
	operator TEMPS     () const; \
	operator CTemps    () const; \
	operator SYSTEMTIME() const; \
	operator FILETIME  () const; \
	operator tm        () const; \
	operator time_t    () const;



// Fonctions demandant le noyau des MFC
#define FonctionsMFCOK(type) \
	CTemps##type& operator = (const CTime& dt); \
	operator CTime() const;

// Fonctions demandant le noyau et les extensions des MFC
#define FonctionsMFCEXT(type) \
	void FromDlg##type(CWnd* pWnd, int nIDC); \
	void ToDlg##type(  CWnd* pWnd, int nIDC) const; \


// Format - date OU heure
#define dclFormat_OK(type, dwFormat) \
	void Format##type(LPTSTR psz, int len, DWORD dwFlags=dwFormat) const; \
	void Format##type(LPTSTR psz, int len, LPCTSTR pszFormat) const;
#define dclFormat_MFCOK(type, dwFormat) \
	CString Format##type(DWORD dwFlags=dwFormat) const; \
	CString Format##type(LPCTSTR pszFormat) const;


// Dfinir la macro Fonctions qui regroupe les diffrentes
// macros FonctionsXX selon les paramtres en cours
#ifdef MFCOK

#define dclFormat(type,   dwFormat) \
	dclFormat_OK(##type,    dwFormat) \
	dclFormat_MFCOK(##type, dwFormat)

#ifndef MFCEXTOK
#define Fonctions(type)   \
	FonctionsOK(##type)     \
	FonctionsMFCOK(##type)
#else
#define Fonctions(type)   \
	FonctionsOK(##type)     \
	FonctionsMFCOK(##type)  \
	FonctionsMFCEXT(##type)
#endif

#else

#define dclFormat(type, dwFormat) \
	dclFormat_OK(type, dwFormat)
#define Fonctions(type) \
	FonctionsOK(type)

#endif


// Temps
class CTemps
{
public:  // Fonctions
	
	CTemps()          : m_tps(  0) {}
	CTemps(TEMPS tps) : m_tps(tps) {}
	CTemps(UINT uJour, UINT uMois, UINT uAn, UINT uHeure=0, UINT uMinute=0, UINT uSeconde=0)
		: m_tps(MakeTemps(uJour, uMois, uAn, uHeure, uMinute, uSeconde)) {}
	
	UINT GetJour   () const;
	UINT GetMois   () const;
	UINT GetAn     () const;
	UINT GetHeure  () const { return (m_tps/3600) % 60; }
	UINT GetMinute () const { return (m_tps/  60) % 60; }
	UINT GetSeconde() const { return  m_tps       % 60; }
	
	void SetNow();
	
	CTempsSpan operator - (CTemps tps) const;
	
	CTemps  operator +  (CTempsSpan ts) const;
	CTemps  operator -  (CTempsSpan ts) const;
	CTemps& operator += (CTempsSpan ts);
	CTemps& operator -= (CTempsSpan ts);
	
	bool operator == (CTemps tps) const;
	bool operator != (CTemps tps) const;
	bool operator <  (CTemps tps) const;
	bool operator >  (CTemps tps) const;
	bool operator <= (CTemps tps) const;
	bool operator >= (CTemps tps) const;
	
	CTemps& operator = (const tm&         otm);
	CTemps& operator = (const SYSTEMTIME& st);
	CTemps& operator = (const FILETIME&   ft);
	
	operator TEMPS () const;
	operator TEMPSD() const;
	
	
protected:  // Donnes
	
	TEMPS m_tps;   // Nombre de secondes depuis le 01/01/1970
};


// Date
class CTempsD
{
public:  // Fonctions
	
	// Constructeur
	CTempsD(UINT uJour, UINT uMois, UINT uAn)
		{
			SetD(uJour, uMois, uAn);
		}
	
	// Dfinir
	void SetD(UINT uJour, UINT uMois, UINT uAn)
		{
			m_uJour = uJour;
			m_uMois = uMois;
			m_uAn   = uAn;
		}
	
	Fonctions(D)
	dclFormat(D,DATE_SHORTDATE)
	
	
public:
	
	UINT m_uJour;   // Jour
	UINT m_uMois;   // Mois
	UINT m_uAn;     // Anne
};

// Heure
class CTempsH
{
public:  // Fonctions
	
	CTempsH(UINT uHeure, UINT uMinute)
		{
			SetH(uHeure, uMinute);
		}
	
	// Dfinir
	void SetH(UINT uHeure, UINT uMinute)
		{
			m_uHeure  = uHeure;
			m_uMinute = uMinute;
		}
	
	Fonctions(H)
	dclFormat(H,TIME_NOSECONDS)
	
	void AddH(int nHeure, int nMinute=0);
	
	
public:  // Donnes
	
	UINT m_uHeure;    // Heures
	UINT m_uMinute;   // Minutes
};

// Date - Heure
class CTempsDH : public virtual CTempsD, public virtual CTempsH
{
public:  // Fonctions
	
	// Constructeur
	CTempsDH(UINT uJour, UINT uMois, UINT uAn, UINT uHeure, UINT uMinute)
		{
			SetDH(uJour, uMois, uAn, uHeure, uMinute);
		}
	
	// Dfinir
	void SetDH(UINT uJour, UINT uMois, UINT uAn, UINT uHeure, UINT uMinute)
		{
			m_uJour   = uJour;
			m_uMois   = uMois;
			m_uAn     = uAn;
			m_uHeure  = uHeure;
			m_uMinute = uMinute;
		}
	
	void AddDH(int nJour, int nMois=0, int nAn=0, int nHeure=0, int nMinute=0);
	
	Fonctions(DH)
};

// Heure - Secondes
class CTempsHS : public virtual CTempsH
{
public:  // Fonctions
	
	// Constructeur
	CTempsHS(UINT uHeure, UINT uMinute, UINT uSeconde)
		{
			SetHS(uHeure, uMinute, uSeconde);
		}
	
	// Dfinir
	void SetHS(UINT uHeure, UINT uMinute, UINT uSeconde)
		{
			m_uHeure   = uHeure;
			m_uMinute  = uMinute;
			m_uSeconde = uSeconde;
		}
	
	Fonctions(HS)
	dclFormat(HS,0)
	
	void AddHS(int nHeure, int nMinute=0, int nSeconde=0);
	
	
public:
	
	UINT m_uSeconde;   // Secondes
};

// Date - Heure - Secondes
class CTempsDHS : public virtual CTempsHS, public virtual CTempsDH
{
public:  // Fonctions
	
	// Constructeur
	CTempsDHS(UINT uJour, UINT uMois, UINT uAn, UINT uHeure, UINT uMinute, UINT uSeconde)
		{
			SetDHS(uJour, uMois, uAn, uHeure, uMinute, uSeconde);
		}
	
	// Dfinir
	void SetDHS(UINT uJour, UINT uMois, UINT uAn, UINT uHeure, UINT uMinute, UINT uSeconde)
		{
			m_uAn      = uAn;
			m_uJour    = uJour;
			m_uMois    = uMois;
			m_uHeure   = uHeure;
			m_uMinute  = uMinute;
			m_uSeconde = uSeconde;
		}
	
	Fonctions(DHS)
	
	void AddDHS(int nJour, int nMois=0, int nAn=0, int nHeure=0, int nMinute=0, int nSeconde=0);
};

#undef Fonctions

// Diffrences de dates
class CTempsSpan
{
public:  // Fonctions
	
	CTempsSpan(long lSpan=0)
		:m_lSpan(lSpan)
		{}
	CTempsSpan(const CTempsSpan& ts)
		:m_lSpan(ts.m_lSpan)
		{}
#ifdef MFCOK
	CTempsSpan(const CTimeSpan&  ts)
		:m_lSpan(ts.GetTotalSeconds())
		{}
#endif
	CTempsSpan(long lJour, int nHeure, int nMinute=0, int nSeconde=0)
		{
			Set(lJour, nHeure, nMinute, nSeconde);
		}
	
	Set(long lJour, int nHeure=0, int nMinute=0, int nSeconde=0);
	
	bool FromDlgHM (HWND hdlg,  int nIDC);
	bool FromDlgHMS(HWND hdlg,  int nIDC);
	void ToDlgHM   (HWND hdlg,  int nIDC) const;
	void ToDlgHMS  (HWND hdlg,  int nIDC) const;
#ifdef MFCEXTOK
	bool FromDlgHM (CWnd* pWnd, int nIDC);
	bool FromDlgHMS(CWnd* pWnd, int nIDC);
	void ToDlgHM   (CWnd* pWnd, int nIDC) const;
	void ToDlgHMS  (CWnd* pWnd, int nIDC) const;
#endif
	
	CTempsSpan& operator = (long lSpan)
		{
			m_lSpan = lSpan;
			return *this;
		}
	CTempsSpan& operator = (CTempsSpan ts)
		{
			m_lSpan = ts.m_lSpan;
			return *this;
		}
#ifdef MFCOK
	CTempsSpan& operator = (const CTimeSpan& ts)
		{
			m_lSpan = ts.GetTotalSeconds();
			return *this;
		}
#endif
	
	CTempsSpan  operator + (CTempsSpan ts) const {
		return CTempsSpan(m_lSpan + ts.m_lSpan); }
	CTempsSpan  operator - (CTempsSpan ts) const {
		return CTempsSpan(m_lSpan - ts.m_lSpan); }
	CTempsSpan& operator +=(CTempsSpan ts)
		{
			m_lSpan += ts.m_lSpan;
			return *this;
		}
	CTempsSpan& operator -=(const CTempsSpan& ts)
		{
			m_lSpan -= ts.m_lSpan;
			return *this;
		}
	
	bool operator == (CTempsSpan ts) const { return m_lSpan==ts.m_lSpan; }
	bool operator != (CTempsSpan ts) const { return m_lSpan!=ts.m_lSpan; }
	bool operator <  (CTempsSpan ts) const { return m_lSpan< ts.m_lSpan; }
	bool operator >  (CTempsSpan ts) const { return m_lSpan> ts.m_lSpan; }
	bool operator <= (CTempsSpan ts) const { return m_lSpan<=ts.m_lSpan; }
	bool operator >= (CTempsSpan ts) const { return m_lSpan>=ts.m_lSpan; }
	
	// Nombre de jours total
	long GetJour()    const { return  m_lSpan/86400; }
	int  GetHeure()   const { return (m_lSpan/3600) % 60; }
	int  GetMinute()  const { return (m_lSpan/  60) % 60; }
	int  GetSeconde() const { return  m_lSpan       % 60; }
	
	long GetTotalHeure()   const { return m_lSpan/3600; }
	long GetTotalMinute()  const { return m_lSpan/60; }
	long GetTotalSeconde() const { return m_lSpan; }
	
	void    FormatHM (LPTSTR psz) const;
	void    FormatHMS(LPTSTR psz) const;
#ifdef MFCOK
	CString FormatHM () const;
	CString FormatHMS() const;
#endif
	
protected:  // Donnes
	
	long m_lSpan;   // Diffrence de temps
	
	friend class CTempsD;
	friend class CTempsH;
	friend class CTempsDH;
	friend class CTempsHS;
	friend class CTempsDHS;
};

#ifdef MFCOK

// inline CArchive& AFXAPI operator << (CArchive& ar, CTempsD& dt)
// inline CArchive& AFXAPI operator >> (CArchive& ar, CTempsD& dt)
#define serialize(type) \
	inline CArchive& AFXAPI operator << (CArchive& ar, CTemps##type& dt) { \
		ar.Write(&dt, sizeof(dt));  return ar; } \
	inline CArchive& AFXAPI operator >> (CArchive& ar, CTemps##type& dt) { \
		ar.Read(&dt, sizeof(dt));  return ar; }
serialize(D  )
serialize(H  )
serialize(DH )
serialize(HS )
serialize(DHS)
#undef serialize

// CDumpContext& AFXAPI operator << (CDumpContext& dc, CTemps dt);
#ifdef _DEBUG
#define dump(type) CDumpContext& AFXAPI operator << (CDumpContext& dc, CTemps##type dt);
dump(D)
dump(H)
dump(DH)
dump(HS)
dump(DHS)
#undef dump
#endif // _DEBUG

#endif // MFCOK


// CDumpContext& AFXAPI operator << (CDumpContext& dc, CTempsSpan ts);
#ifdef MFCOK_DEBUG
CDumpContext& AFXAPI operator << (CDumpContext& dc, const CTempsSpan ts);
#endif


// int Cmp(CTemps& dt1, CTemps& dt2);
#define cmp(type)  int Cmp##type(const CTemps##type& dt1, const CTemps##type& dt2);
cmp(D)
cmp(H)
cmp(DH)
cmp(HS)
cmp(DHS)
#undef cmp

#undef Fonctions
#undef FonctionsOK
#undef FonctionsMFCOK
#undef FonctionsMFCEXT

#undef dclFormat
#undef dclFormat_OK
#undef dclFormat_MFCOK
#undef dclFormat_MFCEXT


#include "Temps.inl"

#endif
