// ToolBarEx.h
//
// Copyright (C) 1997, 1998 par Joerg Koenig
//
// Barre d'outils tendue permettant le glisser/dplacer
// des boutons ainsi qu'un mode "plat"
//
// REQUIERT
// - ToolBarEx.cpp, ControlBarEx.cpp, SysObject.cpp
// - Application MFC
// - Utiliser FrameEnableDocking() au lieu de EnableDocking()
// - Dans StdAfx.h:
//   #pragma include_alias("AppResource.h", "../RpertoireProjet/Resource.h")
//   Pour pouvoir inclure des contrles uniquement:
//   #define TOOLBAREX_CONTROL
//
// BUGS
// - Le message de notification CDRF_NEWFONT n'est pas utilis
// - Assigner du texte aux boutons peut les redimensionner horizontalement
//   sans notification de la part de CToolBar, d'o un mauvais calcul
//   dans CalcDynamicLayout() et CalcFixedLayout()


#ifndef _TOOLBAREX_H_
#define _TOOLBAREX_H_
#if _MSC_VER >= 1000
#pragma once
#endif

#include <ControlBarEx.h>
#include <MemDC.h>
#include <Liste.h>


class CToolBarEx : public CToolBar, public CBaseBarEx
{
	DECLARE_DYNAMIC(CToolBarEx)
	
public:  // Construction/destruction
	
	CToolBarEx();
	virtual ~CToolBarEx();
	
	
protected:  // Donnes
	
	CSize       m_sizeOffset;    // Point de dpart rel de l'image
	int         m_nLastBtn;      // Index du dernier bouton form
	UINT       *m_auStyle;       // Styles des boutons
	int         m_nbStyle;       // Nombre d'lments dans le tableau
	bool        m_fimlSup;       // La manire dont on a cr la liste d'images
#ifdef TOOLBAREX_CONTROL
	CListePtr2<CWnd*> m_lstControl; // Liste des contrles personnaliss
#endif
	TBBUTTON   *m_atbb;          // Sauvegarde de la liste de boutons
	int         m_nbtbb;         // Nombre d'lments dans le tableau
	TBBUTTON   *m_atbbPerso;     // Sauvegarde de la liste de boutons, sans sparateurs
	int         m_nbtbbPerso;    // Nombre d'lments dans le tableau, sans sparateurs
	bool        m_fEditable;     // TRUE si la personnalisation est autorise
	
	
public:  // Statique - ALT+glisser/dplacer
	
#ifdef TOOLBAREX_DRAG
	static HCURSOR     m_hcurOrig;
	static HCURSOR     m_hcurDrag;
	static HCURSOR     m_hcurNoDrag;
	static bool        m_fDragging;
	static bool        m_fDragCursor;
	static int         m_nDragButton;
	static CToolBarEx *m_pDropBar;
	static CPoint      m_ptDrop;
	bool               m_fDragChild;
#endif
	
	
public:  // Fonctions
	
	// Retourner les dimensions des boutons et des images
	void GetSizes(CSize& sizeButton, CSize& sizeImage) const
		{
			sizeButton = m_sizeButton;
			sizeImage  = m_sizeImage;
		}
	
	
	// Dterminer si cette barre d'outils a des boutons texte
	bool HasButtonText() const
		{
			return m_pStringMap && !m_pStringMap->IsEmpty();
		}
	
	void SetEditable(bool fEditable=true);
	bool GetEditable() const { return m_fEditable; }
	void Customize() { GetToolBarCtrl().Customize(); }
	
	void InvalidateButton(int nIndex);
	bool IsSeparator(int nIndex) const;
#ifdef TOOLBAREX_CONTROL
	bool IsControl  (int nIndex) const;
#endif
	
	BOOL LoadToolBar(UINT nID) { return LoadToolBar(MAKEINTRESOURCE(nID)); }
	BOOL LoadToolBar(LPCTSTR pszResource);
	
#ifdef TOOLBAREX_CONTROL
	CWnd* GetControl(int nIndex, bool fEstID=false) const;
	CWnd* CtrlReplace(CRuntimeClass* pClasse, CRect& rc, UINT nID, DWORD dwStyle=0);
	CWnd* CtrlInsert (CRuntimeClass* pClasse, CRect& rc, UINT nID, int nAvant=0, DWORD dwStyle=0);
#endif
	
#ifdef TOOLBAREX_CONTROL
	void RepositionControls();
#endif
	void RecalcLayout();
	
	HBITMAP GetBitmap(int ID);
	
#ifdef TOOLBAREX_REG
	// Entres/sorties avec le registre
	virtual void ToReg  (LPCTSTR pszKey) const { ToReg(pszKey, true); }
	virtual void FromReg(LPCTSTR pszKey)       { ToReg(pszKey, true); }
	void ToReg  (LPCTSTR pszKey, bool fPos) const;
	void FromReg(LPCTSTR pszKey, bool fPos);
#endif
	
	
protected:  // Hritage
	
	//{{AFX_VIRTUAL(CToolBarEx)
	//}}AFX_VIRTUAL
	virtual void OnUpdateCmdUI(CFrameWnd* pTarget, BOOL bDisableIfNoHandler);
#ifdef TOOLBAREX_DRAG
	virtual BOOL PreTranslateMessage(MSG* pMsg);
#endif
	
	
protected:  // Messages
	
	//{{AFX_MSG(CToolBarEx)
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);
	afx_msg void OnPaint();
	afx_msg void OnNcPaint();
	afx_msg void OnNcCalcSize(BOOL bCalcValidRects, NCCALCSIZE_PARAMS* pncsp);
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnMouseMove  (UINT nFlags, CPoint pt);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint pt);
	afx_msg void OnLButtonUp  (UINT nFlags, CPoint pt);
#ifdef TOOLBAREX_MENU
	afx_msg void OnContextMenu(CWnd* pWnd, CPoint pt);
#endif
	afx_msg void OnWindowPosChanging(WINDOWPOS* pwp);
#ifdef TOOLBAREX_DRAG
	afx_msg void OnCaptureChanged(CWnd *pWnd);
	afx_msg void OnParentNotify(UINT uMsg, LPARAM lParam);
	afx_msg void OnKillFocus(CWnd* pNewWnd);
#endif
	afx_msg void OnSettingChange(UINT uFlags, LPCTSTR pszSection);
	afx_msg void OnQueryDeleteInsert(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnGetButtonInfo    (NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnReset            (NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnToolBarChange    (NMHDR* pNMHDR, LRESULT* pResult);
	//}}AFX_MSG
	afx_msg LRESULT OnSetButtonSize(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnSetBitmapSize(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnAddBitmap    (WPARAM wParam, LPARAM lParam);
#ifdef TOOLBAREX_CONTROL
	afx_msg LRESULT OnDeleteButton (WPARAM wParam, LPARAM lParam);
#endif
	DECLARE_MESSAGE_MAP()
	
	
protected:  // Interne
	
	// Recalculer le point de dpart de l'image des boutons
	void CalculateOffset()
		{
			m_sizeOffset.cx = (m_sizeButton.cx-m_sizeImage.cx)/2;
			m_sizeOffset.cy = (HasButtonText())
				? m_cyTopBorder
				: (m_sizeButton.cy-m_sizeImage.cy)/2;
		}
	
	// Fonction de dessin virtuelles
	virtual void DrawDisabledButton(CImageList* piml, int nBitmap, CDC* pdc, const CRect& rc) const;
	virtual void DrawSeparator     (CDC* pdc, CRect& rc) const;
	virtual void DrawGripper       (CDC* pdc)            const;
	virtual void Draw3DBorders     (CDC* pdc, CRect& rc);
	
	HIMAGELIST GetImageList();
	
#ifdef TOOLBAREX_CONTROL
	CWnd* CreateControl(CRuntimeClass* pClass, CRect& rc, UINT ID, DWORD dwStyle);
#endif
	
	HBITMAP GetBitmap(int nIndex, const CSize& sizeBitmap);
	
#ifdef TOOLBAREX_DRAG
	// ALT+glisser/dplacer
	bool BeginDrag();
	void DragMove();
	void EndDrag(bool fDoMove=true);
	bool IsCibleValide(const CWnd* pWnd) const;
#ifdef TOOLBAREX_CONTROL
	void CheckMoveControl(CToolBarEx* ptlb, const TBBUTTON& tbButton);
#endif
	void RemoveTrailingSeparators();
	int  FindDropButton(CPoint pt);
	void SetMarker(CToolBarEx* ptlb, CPoint pt);
	void GetMarkerRect(int nButton, CRect& rc);
	void ShowMarker   (const CRect& rcMarker, CBitmap& bmpZone);
	void RestoreMarker(const CRect& rcZone,   CBitmap& bmpZone);
#endif // TOOLBAREX_DRAG
	
	bool QueryDeleteInsert(TBBUTTON& tbButton, bool fDelete, int BtnIndex);
	bool DoQueryDelete(int nButton);
	bool DoQueryInsert(TBBUTTON& tbButton, int nIndex)
		{
			return QueryDeleteInsert(tbButton, false, nIndex);
		}
};


//{{AFX_INSERT_LOCATION}}
#endif
