// FileBuf.cpp
//
// CFileBuf   - classe abstraite de base
// CFileRead  - lecture de fichier
// CFileWrite - criture dans un fichier
//
// CFileRead et CFileWrite sont deux classes permettant respectivement
// de lire et d'crire des fichiers en mode bufferis avec une gestion
// complte des erreurs
// Ces classes sont  prfrer  celles de la bibliothque standard,
// et ce pour deux raisons:
// 1) Leur code est relativement simple et reste trs proche de Win32,
//    donc trs simple  dbugger
// 2) La gestion des erreurs est automatique (pas de errno)
//
// REQUIERT
// - Application Windows
// - Fonction AfxMessageBox() dclare implmente
// - Dans StdAfx.h:
//   #pragma include_alias("AppResource.h", "../RpertoireProjet/Resource.h")


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "AppResource.h"
#include "FileBuf.h"


DWORD CFileBuf::s_dwES;


const DWORD SEEK_ERR = 0xFFFFFFFF;


//*** CFileBuf

// Fermer le fichier
void CFileBuf::Close()
{
	CloseHandle(m_hf);
	m_hf = INVALID_HANDLE_VALUE;
}


// Attacher
void CFileBuf::Attach(HANDLE hf)
{
#ifdef ASSERT
	ASSERT(m_fAttach || m_hf==INVALID_HANDLE_VALUE);
	ASSERT(hf!=INVALID_HANDLE_VALUE);
#endif
	
	m_hf      = hf;
	m_len     = 0;
#ifdef _DEBUG
	m_fAttach = true;
#endif
}

// Dtacher
HANDLE CFileBuf::Detach()
{
#ifdef ASSERT
	ASSERT(m_hf!=INVALID_HANDLE_VALUE);
#endif
	
	const HANDLE hf = m_hf;
	m_hf = INVALID_HANDLE_VALUE;
	
	return hf;
}




// Fermer le fichier en catastrophe
void CFileBuf::CloseErreur()
{
	if (!m_fPeutFermer)  return;
	CloseHandle(m_hf);
	m_hf = INVALID_HANDLE_VALUE;
}



// Afficher la dernire erreur
int CFileBuf::Erreur(UINT nID, UINT nFlags)
{
	// Rcuprer le message d'erreur de Windows
	LPTSTR pszMessage;
	FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
		NULL, GetLastError(), MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
		(LPTSTR)&pszMessage, 0, NULL);
	
#ifdef MFCOK
	// L'inclure dans le message d'erreur demand
	CString s;
	AfxFormatString2(s, nID, m_pszFichier, pszMessage);
	LocalFree(pszMessage);
	return AfxMessageBox(s, nFlags | MB_ICONEXCLAMATION);
#else
	
	const HINSTANCE hInst = GetModuleHandle(NULL);
	
	// Formatter
	TCHAR pszFormat[128], psz[512+MAX_PATH];
	LoadString(hInst, nID, pszFormat,nbArray(pszFormat));
	wsprintf(psz, pszFormat, m_pszFichier, pszMessage);
	LocalFree(pszMessage);
	
	return AfxMessageBox(psz, nFlags | MB_ICONEXCLAMATION);
#endif
}



#ifndef NO_FILEREAD

//*** CFileRead

// Ouvrir le fichier
bool CFileRead::Open(LPCTSTR pszFichier,
	DWORD dwShare, DWORD dwCreate, DWORD dwFlags)
{
#ifdef ASSERT
	ASSERT(m_fAttach || m_hf==INVALID_HANDLE_VALUE);
#endif
	
	lstrcpy(m_pszFichier, pszFichier);
	m_len     =
	m_posCur  = 0;
	m_fEOF    = false;
#ifdef _DEBUG
	m_fAttach = false;
#endif
	
	for (;;) {
		m_hf = CreateFile(pszFichier, GENERIC_READ,
			dwShare, NULL, dwCreate, dwFlags, NULL);
		if (m_hf!=INVALID_HANDLE_VALUE)  return true;
		VERIF(m_fVerbeux && Erreur(ERR_OPEN, MB_RETRYCANCEL)==IDRETRY);
	}
}


// Attacher
void CFileRead::Attach(HANDLE hf)
{
#ifdef ASSERT
	ASSERT(m_fAttach || m_hf==INVALID_HANDLE_VALUE);
	ASSERT(hf!=INVALID_HANDLE_VALUE);
#endif
	
	m_hf      = hf;
	m_len     =
	m_posCur  = 0;
	m_fEOF    = false;
#ifdef _DEBUG
	m_fAttach = true;
#endif
}


// Lire des octets dans le fichier
bool CFileRead::Read(void* pBuf, DWORD len)
{
#ifdef ASSERT
	ASSERT(m_hf!=INVALID_HANDLE_VALUE);
	ASSERT(ToBool(pBuf));
#endif
	
	if (!len)  return true;
	
	if (m_len) {
		// Utiliser le tampon
		
		if (len<=m_len) {
			// Si le tampon est assez grand
			// lire directement tout ce qu'il faut
			
			memcpy(pBuf, &m_abBuf[m_posCur], (size_t)len);
			m_len    -= (size_t)len;
			m_posCur += (size_t)len;
			if (!m_posCur || m_posCur==lenFileBuf)  m_posCur = 0;
			return true;
		}
		
		// Lire tout ce qui reste dans le tampon
		memcpy(pBuf, &m_abBuf[m_posCur], m_len);
		len     -= m_len;
		pBuf     = (BYTE*)pBuf + m_len;
		m_len    =
		m_posCur = 0;
	}
	
	//*** Tampon vide
	
	if (len>=lenFileBuf)
		// Le tampon est trop petit ou juste de la bonne taille,
		// lire directement tout ce qu'il faut
		return ReadProtect((BYTE*)pBuf, len,len);
	
	// Tampon assez grand
	if (!ReadProtect(m_abBuf, lenFileBuf,len)) {
		memcpy(pBuf, m_abBuf, s_dwES);
		m_len    =
		m_posCur = 0;
		return false;
	}
	memcpy(pBuf, m_abBuf, len);
	m_len    = (size_t)s_dwES - len;
	m_posCur = (m_len) ? len : 0;
	return true;
}

// Lire des octets dans le fichier
// et retourner le nombre d'octets lus
DWORD CFileRead::ReadLen(void* pBuf, DWORD len)
{
#ifdef ASSERT
	ASSERT(m_hf!=INVALID_HANDLE_VALUE);
	ASSERT(ToBool(pBuf));
#endif
	
	if (!len)  return 0;
	
	DWORD lenLu = 0;
	
	if (m_len) {
		// Utiliser le tampon
		
		if (len<=m_len) {
			// Si le tampon est assez grand
			// lire directement tout ce qu'il faut
			
			memcpy(pBuf, &m_abBuf[m_posCur], (size_t)len);
			m_len    -= (size_t)len;
			m_posCur += (size_t)len;
			if (!m_posCur || m_posCur==lenFileBuf)  m_posCur = 0;
			return len;
		}
		
		// Lire tout ce qui reste dans le tampon
		memcpy(pBuf, &m_abBuf[m_posCur], m_len);
		lenLu    = m_len;
		len     -= m_len;
		pBuf     = (BYTE*)pBuf + m_len;
		m_len    =
		m_posCur = 0;
	}
	
	//*** Tampon vide
	
	if (len>=lenFileBuf) {
		// Le tampon est trop petit ou juste de la bonne taille,
		// lire directement tout ce qu'il faut
		const BOOL b = ReadFile(m_hf, pBuf, len, &s_dwES, NULL);
		m_fEOF = (b && s_dwES<len);
		return lenLu+s_dwES;
	}
	
	// Tampon assez grand
	const BOOL b = ReadFile(m_hf, m_abBuf, lenFileBuf, &s_dwES, NULL);
	if (s_dwES<len) {
		m_fEOF = ToBool(b);
		memcpy(pBuf, m_abBuf, s_dwES);
		m_len    =
		m_posCur = 0;
		return lenLu+s_dwES;
	}
	memcpy(pBuf, m_abBuf, len);
	m_len    = (size_t)s_dwES - len;
	m_posCur = (m_len) ? len : 0;
	return lenLu+len;
}

// Lire un octet dans le fichier
bool CFileRead::Read(BYTE& b)
{
	if (m_len>=1) {
		b = m_abBuf[m_posCur];
		m_posCur += 1;
		m_len    -= 1;
		return true;
	}
	
	if (!ReadProtect(m_abBuf, lenFileBuf,1)) {
		m_len    =
		m_posCur = 0;
		return false;
	}
	b = m_abBuf[0];
	m_len    = (size_t)s_dwES - 1;
	m_posCur = (m_len) ? 1 : 0;
	return true;
}

// Lire un mot dans le fichier
bool CFileRead::Read(WORD& w)
{
	if (m_len>=2) {
		w = *(WORD*)&m_abBuf[m_posCur];
		m_posCur += 2;
		m_len    -= 2;
		return true;
	}
	if (!m_len) {
		// Le tampon est vide
		if (!ReadProtect(m_abBuf, lenFileBuf,2)) {
			m_len    =
			m_posCur = 0;
			return false;
		}
		w = *(WORD*)m_abBuf;
		m_len    = (size_t)s_dwES - 2;
		m_posCur = (m_len) ? 2 : 0;
		return true;
	}
	
	// Pas de chance, le tampon ne contient qu'un seul octet...
	w = m_abBuf[0];
	if (!ReadProtect(m_abBuf, lenFileBuf,1)) {
		m_len    =
		m_posCur = 0;
		return false;
	}
	w |= (WORD)(m_abBuf[0])<<8;
	m_len    = (size_t)s_dwES - 1;
	m_posCur = (m_len) ? 1 : 0;
	return true;
}

// Vider le tampon
bool CFileRead::RAZ()
{
#ifdef ASSERT
	ASSERT(m_hf!=INVALID_HANDLE_VALUE);
#endif
	
	if (m_len) {
		if (SetFilePointer(m_hf, -(long)m_len,NULL, FILE_CURRENT)==SEEK_ERR)
			return false;
		m_len    =
		m_posCur = 0;
	}
	return true;
}


// Rcuprer la position courante
DWORD CFileRead::Tell() const
{
	return SetFilePointer(m_hf, 0,NULL, FILE_CURRENT)-m_len;
}

// Dplacer le pointeur de lecture
bool CFileRead::Seek(long lMove, DWORD dwMethode)
{
#ifdef ASSERT
	ASSERT(m_hf!=INVALID_HANDLE_VALUE);
#endif
	
	RAZ();
	while (SetFilePointer(m_hf, lMove,NULL, dwMethode)==SEEK_ERR) {
		VERIF(m_fVerbeux);
		if (Erreur(ERR_READ, MB_RETRYCANCEL)!=IDRETRY) {
			CloseErreur();
			return false;
		}
	}
	return true;
}
DWORD CFileRead::SeekPos(long lMove, DWORD dwMethode)
{
#ifdef ASSERT
	ASSERT(m_hf!=INVALID_HANDLE_VALUE);
#endif
	
	RAZ();
	DWORD dwRet;
	while ((dwRet=SetFilePointer(m_hf, lMove,NULL, dwMethode))==SEEK_ERR) {
		VERIF(m_fVerbeux);
		if (Erreur(ERR_READ, MB_RETRYCANCEL)!=IDRETRY) {
			CloseErreur();
			return SEEK_ERR;
		}
	}
	return dwRet;
}


// Lecture protge: gestion des erreurs, affichage d'une bote de dialogue
bool CFileRead::ReadProtect(BYTE* pbBuf, DWORD len, DWORD lenFaut)
{
	DWORD dwES;
	s_dwES = 0;
	
	for (;;) {
		
		// Lire
		const BOOL bOK = ReadFile(m_hf, pbBuf, len-s_dwES, &dwES, NULL);
		pbBuf  += dwES;
		s_dwES += dwES;
		
		if (s_dwES==len)  return true;   // OK, tout termin
		
		if (bOK) {
			// Fin du fichier atteinte
			if (s_dwES>=lenFaut)  return true;   // On a lu le minimum requis
			
			// Afficher le message d'erreur
			m_fEOF = true;
			if (m_fVerbeux) {
				SetLastError(ERROR_HANDLE_EOF);
				Erreur(ERR_READ, MB_OK);
				CloseErreur();
			}
			return false;
		}
		
		// Demander s'il faut recommencer
		VERIF(m_fVerbeux);
		if (Erreur(ERR_READ, MB_RETRYCANCEL)!=IDRETRY) {
			CloseErreur();
			return false;
		}
	}
}

#endif



#ifndef NO_FILEWRITE

//*** CFileWrite

// Ouvrir le fichier
bool CFileWrite::Open(LPCTSTR pszFichier,
	DWORD dwShare, DWORD dwCreate, DWORD dwFlags)
{
#ifdef ASSERT
	ASSERT(m_fAttach || m_hf==INVALID_HANDLE_VALUE);
#endif
	
	lstrcpy(m_pszFichier, pszFichier);
	m_len     = 0;
#ifdef _DEBUG
	m_fAttach = false;
#endif
	
	for (;;) {
		m_hf = CreateFile(pszFichier, GENERIC_WRITE,
			dwShare, NULL, dwCreate, dwFlags, NULL);
		if (m_hf!=INVALID_HANDLE_VALUE)  return true;
		VERIF(m_fVerbeux && Erreur(ERR_OPEN, MB_RETRYCANCEL)==IDRETRY);
	}
}

// Fermer le fichier
void CFileWrite::Close()
{
	if (m_hf!=INVALID_HANDLE_VALUE) {
		Flush();
		CloseHandle(m_hf);
		m_hf = INVALID_HANDLE_VALUE;
	}
}


// Dtacher
HANDLE CFileWrite::Detach()
{
	return (Flush()) ? NULL : CFileBuf::Detach();
}


// crire des octets dans le fichier
bool CFileWrite::Write(const void* pBuf, DWORD len)
{
#ifdef ASSERT
	ASSERT(m_hf!=INVALID_HANDLE_VALUE);
	ASSERT(ToBool(pBuf));
#endif
	
	if (!len)  return true;
	
	if (m_len) {
		// Utiliser le tampon
		
		const size_t lenReste = lenFileBuf-m_len;
		if (lenReste>len) {
			// Il reste de la place dans le tampon
			memcpy(&m_abBuf[m_len], pBuf, (size_t)len);
			m_len += len;
			return true;
		}
		
		// Remplir le tampon au maximum et flusher
		memcpy(&m_abBuf[m_len], pBuf, lenReste);
		if (!WriteProtect(m_abBuf, lenFileBuf)) {
			if (s_dwES) {
				memmove(m_abBuf, &m_abBuf[s_dwES], m_len-s_dwES);
				m_len -= s_dwES;
			}
			return false;
		}
		len -= lenReste;
		pBuf = (const BYTE*)pBuf + lenReste;
	}
	
	
	// Tampon vide
	
	if (len>=lenFileBuf) {
		// Si le tampon est trop petit ou juste de la bonne taille,
		// crire directement tout ce qui reste
		m_len = 0;
		return WriteProtect((const BYTE*)pBuf, len);
	}
	
	// Copier tout le reste dans le tampon
	memcpy(m_abBuf, pBuf, len);
	m_len = len;
	return true;
}

// crire une chane dans le fichier
bool CFileWrite::Write(LPCSTR psz)
{
	return Write(psz, strlen(psz)*sizeof(psz[0]));
}
bool CFileWrite::Write(LPCWSTR psz)
{
	return Write(psz, wcslen(psz)*sizeof(psz[0]));
}

// crire un octet dans le fichier
bool CFileWrite::Write(BYTE b)
{
	if (m_len>=lenFileBuf-sizeof(b) && !Flush())  return false;
	m_abBuf[m_len] = b;
	m_len += sizeof(b);
	return true;
}

// crire un mot dans le fichier
bool CFileWrite::Write(WORD w)
{
	if (m_len>=lenFileBuf-sizeof(w) && !Flush())  return false;
	*(WORD*)&m_abBuf[m_len] = w;
	m_len += sizeof(w);
	return true;
}

// crire un double mot dans le fichier
bool CFileWrite::Write(DWORD dw)
{
	if (m_len>=lenFileBuf-sizeof(dw) && !Flush())  return false;
	*(DWORD*)&m_abBuf[m_len] = dw;
	m_len += sizeof(dw);
	return true;
}


// crire le contenu du tampon dans le fichier
bool CFileWrite::Flush()
{
#ifdef ASSERT
	ASSERT(m_hf!=INVALID_HANDLE_VALUE);
#endif
	
	if (!m_len)  return true;
	
	if (WriteProtect(m_abBuf, m_len)) {
		m_len = 0;
		return true;
	}
	if (s_dwES) {
		// Dcaler le tampon
		memmove(m_abBuf, &m_abBuf[s_dwES], m_len-s_dwES);
		m_len -= s_dwES;
	}
	return false;
}


// Rcuprer la position courante
DWORD CFileWrite::Tell() const
{
	return SetFilePointer(m_hf, 0,NULL, FILE_CURRENT)+m_len;
}

// Dplacer le pointeur de lecture
bool CFileWrite::Seek(long lMove, DWORD dwMethode)
{
#ifdef ASSERT
	ASSERT(m_hf!=INVALID_HANDLE_VALUE);
#endif
	
	VERIF(Flush());
	while (SetFilePointer(m_hf, lMove,NULL, dwMethode)==SEEK_ERR) {
		VERIF(m_fVerbeux);
		if (Erreur(ERR_WRITE, MB_RETRYCANCEL)!=IDRETRY) {
			CloseErreur();
			return false;
		}
	}
	return true;
}
DWORD CFileWrite::SeekPos(long lMove, DWORD dwMethode)
{
#ifdef ASSERT
	ASSERT(m_hf!=INVALID_HANDLE_VALUE);
#endif
	
	VERIF(Flush());
	DWORD dwRet;
	while ((dwRet=SetFilePointer(m_hf, lMove,NULL, dwMethode))==SEEK_ERR) {
		VERIF(m_fVerbeux);
		if (Erreur(ERR_WRITE, MB_RETRYCANCEL)!=IDRETRY) {
			CloseErreur();
			return SEEK_ERR;
		}
	}
	return dwRet;
}


// crit des octets provenant d'un fichier en lecture
bool CFileWrite::WriteFrom(CFileRead& fr, DWORD len)
{
	if (!len)  return true;
	
	if (len>lenFileBuf) {
		// Le nombre d'octets  crire dpasse
		// la taille du tampon d'criture
		
		// Synchroniser les deux fichiers
		if (!Flush())   goto ErreurOut;
		if (!fr.RAZ())  goto ErreurIn;
		
		// Lire/crire en direct
		while (len>lenFileBuf) {
			if (!fr.ReadProtect (m_abBuf, lenFileBuf,lenFileBuf))  goto ErreurIn;
			if (!   WriteProtect(m_abBuf, lenFileBuf))             goto ErreurOut;
			len -= lenFileBuf;
		}
		// Stocker le reste dans le tampon
		if (!fr.ReadProtect(m_abBuf, len,len))  goto ErreurIn;
		m_len = len;
		return true;
	}
	
	if (len<=fr.m_len) {
		// Le tampon de lecture contient tous les octets
		
		if (m_len+len<=lenFileBuf) {
			// Tampon d'criture assez grand: simple transfert
			memcpy(&m_abBuf[m_len], &fr.m_abBuf[fr.m_posCur], len);
			fr.m_posCur += len;
			fr.m_len    -= len;
			   m_len    += len;
			return true;
		}
		
		// Tampon d'criture trop petit
		VERIF(Flush())  goto ErreurOut;
		memcpy(m_abBuf, &fr.m_abBuf[fr.m_posCur], len);
		fr.m_posCur += len;
		fr.m_len    -= len;
		   m_len     = len;
		return true;
	}
	
	// Tampon de lecture trop petit,
	// mais CFileRead::Read fait tout le travail
	if (lenFileBuf-m_len<len && !Flush())  goto ErreurOut;
	if (!fr.Read(&m_abBuf[m_len], len))    goto ErreurIn;
	m_len += len;
	return true;
	
ErreurIn:
	CloseErreur();
	return false;
	
ErreurOut:
	fr.CloseErreur();
	return false;
}


// criture protge: gestion des erreurs, affichage d'une bote de dialogue
bool CFileWrite::WriteProtect(const BYTE* pbBuf, DWORD len)
{
	DWORD dwES;
	s_dwES = 0;
	for (;;) {
		
		// crire
		WriteFile(m_hf, pbBuf, len-s_dwES, &dwES, NULL);
		pbBuf  +=   dwES;
		s_dwES +=   dwES;
		
		if (s_dwES==len)  return true;   // OK, tout termin
		
		// Demander s'il faut recommencer
		VERIF(m_fVerbeux);
		if (Erreur(ERR_READ, MB_RETRYCANCEL)!=IDRETRY) {
			CloseErreur();
			return false;
		}
	}
}

#endif
