// ShellMenu.cpp
//
// CShellMenu - permet de crer des menus ayant le mme contenu
// que le menu contextuel affich par le shell lorsqu'on clique
// droit sur un/des fichier(s) ou un dossier
//
// REQUIERT
// - Application MFC


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "ShellMenu.h"

#include <ShlObj.h>


// Nettoyer les interfaces
void CShellMenu::Nettoyer()
{
	if (m_pcm) {
		m_pcm->Release();
		m_pcm = NULL;
	}
}


// Initialiser le menu sur des fichiers
void CShellMenu::InitFiles(IShellFolder* psf, int nb, const LPCITEMIDLIST* apidl)
{
	Nettoyer();
	
	// Rcuprer la meilleure interface IContextMenu possible
	// et s'en servir pour remplir le menu
	// On conserve son pointeur dans un membre de la classe
	// car on en aura besoin pour InvokeCommand()
	if (SUCCEEDED(psf->GetUIObjectOf(*m_pWnd,
				nb, (LPCITEMIDLIST*)apidl,
				IID_IContextMenu, NULL, (void**)&m_pcm))) {
		
		IContextMenu3 *pcm;
		if (SUCCEEDED(m_pcm->QueryInterface(IID_IContextMenu3, (void**)&pcm))) {
			m_pcm->Release();
			m_pcm = pcm;
			m_ncm = 3;
		}else if (SUCCEEDED(m_pcm->QueryInterface(IID_IContextMenu2, (void**)&pcm))) {
			m_pcm->Release();
			m_pcm = pcm;
			m_ncm = 2;
		}else
			m_ncm = 1;
		
		// Construire le menu
		m_pcm->QueryContextMenu(*m_pMenu, 0,
			idVerbeMin,idVerbeMax, CMF_EXPLORE);
	}
}


// Messages
bool CShellMenu::WindowProc(UINT uMsg, WPARAM wParam, LPARAM lParam, LRESULT* plResult)
{
	// Grer les messages des menus du shell
	// toutes versions:   WM_COMMAND
	// version 2 minimum: WM_INITMENUPOPUP, WM_DRAWITEM, WM_MEASUREITEM
	// version 3 minimum: WM_MENUCHAR
	
	if (m_pcm) {
		
		if (uMsg==WM_COMMAND && LOWORD(wParam)>=idVerbeMin && LOWORD(wParam)<=idVerbeMax) {
			CMINVOKECOMMANDINFO cmi;
			cmi.cbSize       = sizeof(cmi);
			cmi.fMask        = 0;
			cmi.hwnd         = *m_pWnd;
			cmi.lpVerb       = (LPCSTR)MAKEINTRESOURCE(LOWORD(wParam)-1);
			cmi.lpParameters = NULL;
			cmi.lpDirectory  = NULL;
			cmi.nShow        = SW_SHOWNORMAL;
			cmi.dwHotKey     = 0;
			cmi.hIcon        = NULL;
			m_pcm->InvokeCommand(&cmi);
			*plResult = 0;
			return true;
		}
		
		
		if (m_ncm>=2) {
			
			switch (uMsg) {
				
				case WM_INITMENUPOPUP:
					if (wParam>=idVerbeMin && wParam<=idVerbeMax &&
							GetMenuItemCount((HMENU)wParam)>0) {
						const UINT id = GetMenuItemID((HMENU)wParam, 0);
						if (id>=idVerbeMin && id<=idVerbeMax)  goto HandleMenuMsg;
					}
					break;
				
				// Le dbut des structures DRAWITEMSTRUCT et MEASUREITEMSTRUCT
				// est identique (du moins pour CtlType et itemID)
				case WM_DRAWITEM:
				case WM_MEASUREITEM:
					#define pdis  ((DRAWITEMSTRUCT*)lParam)
					if (pdis->CtlType==ODT_MENU &&
							pdis->itemID>=idVerbeMin && pdis->itemID<=idVerbeMax) {
HandleMenuMsg:
						m_pcm->HandleMenuMsg(uMsg, wParam, lParam);
						*plResult = TRUE;
						return true;
					}
					break;
					#undef pdis
				
				case WM_MENUCHAR:
					if (m_ncm>=3)
						return (HMENU)lParam==*m_pMenu &&
								SUCCEEDED(m_pcm->HandleMenuMsg2(uMsg, wParam, lParam, plResult));
					break;
			}
		}
	}
	
	return false;
}
