// ToolBarEx.cpp
//
// Copyright (C) 1997, 1998 par Joerg Koenig
//
// Barre d'outils tendue permettant le glisser/dplacer
// des boutons ainsi qu'un mode "plat"
//
// REQUIERT
// - ControlBarEx.cpp, SysObject.cpp
// - Application MFC
// - Utiliser FrameEnableDocking() au lieu de EnableDocking()
// - Dans StdAfx.h:
//   #pragma include_alias("AppResource.h", "../RpertoireProjet/Resource.h")
//   Pour pouvoir inclure des contrles uniquement:
//   #define TOOLBAREX_CONTROL
//
// BUGS
// - Le message de notification CDRF_NEWFONT n'est pas utilis
// - Assigner du texte aux boutons peut les redimensionner horizontalement
//   sans notification de la part de CToolBar, d'o un mauvais calcul
//   dans CalcDynamicLayout() et CalcFixedLayout()


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "AppResource.h"
#include <App.h>
#include <ToolBarEx.h>

#include <SysObject.h>
#include <afxPriv.h>


#pragma warning(disable:4706)


const UINT TIMER_TOOLBAREX = 1024;


//*** CCustomDrawInfo
//
// Gre les messages envoys  la frame possesseur de la barre d'outils
// en mode plat uniquement
//
// - Les MFC ne dfinissent pas le membre "rc" de NMCUSTOMDRAW en rapport
//   avec le bouton  dessiner, cepdendant on le fait, donc vrifier si la
//   barre d'outils est platte ou non (ou ignorer "rc" dans les deux cas)
//   Si la barre d'outils n'est pas "classique", les MFC grent tout
// - Les MFC n'envoient pas de message pour les sparateur, on ne le fait
//   donc pas non plus
// - Il semble que les barres d'outils MFC n'envoie jamais de notification
//   ERASE,  la place elles envoie TBN_QUERYDELETE
// - Le rsultat de la notification CDRF_NEWFONT est ignor en mode plat

class CCustomDrawInfo
{
public:  // Fonctions
	
	CCustomDrawInfo(CDC* pdc, CToolBarEx* ptlb);
	
	// NotifyItemPrePaint() returne TRUE si l'utilisateur
	// veut effectuer l'opration par dfaut (CDRF_DODEFAULT)
	// ou FALSE s'il veut la passer (CDRF_SKIPDEFAULT)
	// Notez que CDRF_SKIPDEFAULT n'est pas permis pour
	// CDDS_PREPAINT, CDDS_POSTPAINT, et CDDS_ITEMPOSTPAINT
	void NotifyPrePaint();
	bool NotifyItemPrePaint(int item);
	void NotifyItemPostPaint(int item);
	void NotifyPostPaint();
	
	
protected:  // Donnes
	
#if _MFC_VER >= 0x0420
	NMCUSTOMDRAW m_nmcd;            // Structure de stockage
	LRESULT      m_lPrePaint;       // Rsultat de la notification de pr-dessin
	LRESULT      m_lItemPrePaint;   // Dito pour l'item spcifique
	CToolBarEx  *m_ptlb;            // L'envoyeur rel de la notification
	CWnd        *m_pReceiver;       // Le receveur de la notification
	LRESULT      NotifyParent();
#endif
};



//*** CToolBarEx


#ifdef TOOLBAREX_DRAG

HCURSOR     CToolBarEx::m_hcurOrig    = NULL;
HCURSOR     CToolBarEx::m_hcurDrag    = NULL;
HCURSOR     CToolBarEx::m_hcurNoDrag  = NULL;
bool        CToolBarEx::m_fDragging   = false;
bool        CToolBarEx::m_fDragCursor = false;
int         CToolBarEx::m_nDragButton = -1;
CToolBarEx *CToolBarEx::m_pDropBar    = NULL;
CPoint      CToolBarEx::m_ptDrop        (0,0);

#endif // TOOLBAREX_DRAG


IMPLEMENT_DYNAMIC(CToolBarEx, CToolBar)
BEGIN_MESSAGE_MAP(CToolBarEx, CToolBar)
	//{{AFX_MSG_MAP(CToolBarEx)
	ON_WM_PAINT()
	ON_WM_NCPAINT()
	ON_WM_NCCALCSIZE()
	ON_WM_MOUSEMOVE()
	ON_WM_TIMER()
	ON_WM_WINDOWPOSCHANGING()
#ifdef TOOLBAREX_DRAG
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_CAPTURECHANGED()
	ON_WM_PARENTNOTIFY()
	ON_WM_KILLFOCUS()
#endif
#ifdef TOOLBAREX_MENU
	ON_WM_CONTEXTMENU()
#endif
	ON_WM_SETTINGCHANGE()
	ON_NOTIFY_REFLECT(TBN_QUERYDELETE,   OnQueryDeleteInsert)
	ON_NOTIFY_REFLECT(TBN_QUERYINSERT,   OnQueryDeleteInsert)
	ON_NOTIFY_REFLECT(TBN_GETBUTTONINFO, OnGetButtonInfo)
	ON_NOTIFY_REFLECT(TBN_RESET,         OnReset)
	ON_NOTIFY_REFLECT(TBN_TOOLBARCHANGE, OnToolBarChange)
	//}}AFX_MSG_MAP
	ON_MESSAGE(TB_SETBUTTONSIZE, OnSetButtonSize)
	ON_MESSAGE(TB_SETBITMAPSIZE, OnSetBitmapSize)
	ON_MESSAGE(TB_ADDBITMAP,     OnAddBitmap)
#ifdef TOOLBAREX_CONTROL
	ON_MESSAGE(TB_DELETEBUTTON,  OnDeleteButton)
#endif
	ON_WM_ERASEBKGND()
END_MESSAGE_MAP()


// Constructeur
CToolBarEx::CToolBarEx()
{
	m_nLastBtn   = -1;
	m_auStyle    = NULL;
	m_nbStyle    = 0;
#ifdef TOOLBAREX_DRAG
	m_fDragChild = false;
#endif
	m_atbb       =
	m_atbbPerso  = NULL;
	m_nbtbb      =
	m_nbtbbPerso = 0;
	m_fEditable  = false;
	
	CalculateOffset();
}

// Destructeur
CToolBarEx::~CToolBarEx()
{
	delete [] m_auStyle;
	delete [] m_atbb;
	delete [] m_atbbPerso;
#ifdef TOOLBAREX_CONTROL
	m_lstControl.DeleteAll();
#endif
}


// Dfinir "l'ditabilit"
void CToolBarEx::SetEditable(bool fEditable)
{
	m_fEditable = fEditable;
	if (m_fEditable)  ModifyStyle(0, CCS_ADJUSTABLE);
	else              ModifyStyle(CCS_ADJUSTABLE, 0);
}


// Dfinir la taille des boutons
LRESULT CToolBarEx::OnSetButtonSize(WPARAM wParam, LPARAM lParam)
{
	const LRESULT lResult = CToolBar::OnSetButtonSize(wParam, lParam);
	if (lResult)  CalculateOffset();
	return lResult;
}

// Dfinir la taille des images
LRESULT CToolBarEx::OnSetBitmapSize(WPARAM wParam, LPARAM lParam)
{
	const LRESULT lResult = CToolBar::OnSetBitmapSize(wParam, lParam);
	if (lResult)  CalculateOffset();
	return lResult;
}


// Redessiner la barre d'outils
void CToolBarEx::OnPaint()
{
	const HIMAGELIST himl = GetImageList();
	
#ifdef _DEBUG
	if (!himl)
		TRACE0("CToolBarEx::OnPaint() - Liste d'images non-trouve\n");
#endif
	
	if (himl) {
		CRect rcUpdate;
		if (!GetUpdateRect(rcUpdate))  return;
		
		// Des chanes peuvent avoir t ajoutes
		if (HasButtonText())  CalculateOffset();
		
		// Attacher la liste d'images (respect des MFC)
		CImageList iml;
		iml.Attach(himl);
		
		POINT ptCurseur;
		::GetCursorPos(&ptCurseur);
		bool fSurToolBar = WindowFromPoint(ptCurseur)==this;
		ScreenToClient(&ptCurseur);
		
		CPaintDC dcp(this);
		CMemDC dc(&dcp);
		dc.FillSolidRect(rcUpdate, CSysObject::clBtnFace);
		
		CFont *const poldFont = dc.SelectObject(&CSysObject::fntGui);
		
		// Premire notification de dessin
		CCustomDrawInfo cdrw(&dc, this);
		cdrw.NotifyPrePaint();
		
		const int nbBouton  = GetToolBarCtrl().GetButtonCount();
		for (int nBouton=0; nBouton<nbBouton; nBouton++) {
			CRect rc;
			GetItemRect(nBouton, rc);
			
			int nBitmap; UINT uID, uStyleState;
			GetButtonInfo(nBouton, uID, uStyleState, nBitmap);
			WORD wStyle = LOWORD(uStyleState);
			WORD wState = HIWORD(uStyleState);
			
			// Si le bouton est masqu, ne pas le dessiner
			if (wState & TBSTATE_HIDDEN)  continue;
			
			if (wStyle==TBSTYLE_SEP) {
				// Sparateur
				if (!(wState & TBSTATE_WRAP) || ! IsFloating())
					DrawSeparator(&dc, rc);
			}else{
				
				if (!(rcUpdate & rc))
					// Le bouton n'a pas besoin d'tre redessin
					continue;
				
				bool fBtnDown    = (wState & TBSTATE_CHECKED) || (wState & TBSTATE_PRESSED);
				bool fBtnEnabled = ToBool(GetToolBarCtrl().IsButtonEnabled((int)uID));
				bool fHasCursor  = fSurToolBar && rc.PtInRect(ptCurseur);
				COLORREF clRect  = (fBtnDown && !fHasCursor) ? CSysObject::clBtnLight : CSysObject::clBtnFace;
				
				// Le bouton a peut-tre du texte
				dc.SetTextColor(RGB(0,0,0));
				dc.SetBkColor(clRect);
				
				if (HasButtonText())
					// Il y a un bug dans CToolBar: si du texte est assign
					//  des boutons, la largeur des boutons peut changer
					// "en douce" (sans notification venant de CToolBar)
					// On doit donc recalculer le dcalage horizontal
					m_sizeOffset.cx = (rc.Width() - m_sizeImage.cx) / 2;
				
				if (!cdrw.NotifyItemPrePaint(nBouton))
					// Le parent a dj dessin le bouton
					continue;
				
				dc.FillSolidRect(rc, clRect);
				
				// Il semble que CDC::Draw3dRect()
				// change la couleur d'arrire-plan
				COLORREF clBk = dc.GetBkColor();
				
				if (fBtnDown) {
					// Dessiner un bouton enfonc
					dc.Draw3dRect(rc, CSysObject::clBtnShadow, CSysObject::clBtnHilight);
					if (!fHasCursor) {
						// Si le bouton n'a pas le curseur, le bouton
						// enfonc est un peu plus clair que les autres
						CRect rcCheck = rc;
						rcCheck.DeflateRect(1,1);
						
						// Dessiner un cadre invisible autour de la zone
						// en surbrillance
						dc.Draw3dRect(rcCheck, CSysObject::clBtnFace, CSysObject::clBtnFace);
						
						// Dessiner une trame dans un DC mmoire possdant une bitmap en COULEURS
						CDC dcTemp;
						dcTemp.CreateCompatibleDC(NULL);
						CBitmap bmp;
						bmp.CreateBitmap(m_sizeButton.cx,m_sizeButton.cy, dcTemp.GetDeviceCaps(PLANES),dcTemp.GetDeviceCaps(BITSPIXEL), NULL);
						CBitmap *const poldBitmap = dcTemp.SelectObject(&bmp);
						
						// Dessiner la trame
						rcCheck.DeflateRect(1,1);
						dcTemp.SetTextColor(CSysObject::clBtnHilight);
						dcTemp.SetBkColor  (CSysObject::clBtnFace);
						dcTemp.FillRect(CRect(0,0, rcCheck.Width(),rcCheck.Height()), CDC::GetHalftoneBrush());
						
						// Copier la bitmap du DC mmoire  l'cran
						dc.BitBlt(rcCheck.left,rcCheck.top,rcCheck.Width(),rcCheck.Height(), &dcTemp, 0,0, SRCCOPY);
						dcTemp.SelectObject(poldBitmap);
						dcTemp.DeleteDC();
					}
				}else if (fHasCursor && !fBtnDown && fBtnEnabled)
					// Dessiner un bouton normal
					dc.Draw3dRect(rc, CSysObject::clBtnHilight, CSysObject::clBtnShadow);
				else if (!fBtnDown && fBtnEnabled)
					// Dessiner un rectangle invisible autour du bouton
					// Cela nous empche d'effacer le fond si le bouton
					// a t form avant, et vite le scintillement
					dc.Draw3dRect(rc, CSysObject::clBtnFace, CSysObject::clBtnFace);
				
				dc.SetBkColor(clBk);
				
				// Point de dpart de l'image
				CPoint pt(
					rc.left + m_sizeOffset.cx + fBtnDown,
					rc.top  + m_sizeOffset.cy + fBtnDown);
				
				iml.Draw(&dc, nBitmap, pt, ILD_TRANSPARENT);
				
				CString sTexte = GetButtonText(nBouton);
				if (!sTexte.IsEmpty()) {
					CRect rcText(
						rc.left  +3+fBtnDown,
						rc.top+m_sizeOffset.cy+m_sizeImage.cy+1+fBtnDown,
						rc.right -3+fBtnDown,
						rc.bottom-3+fBtnDown
					);
					int nBkMode = dc.SetBkMode(TRANSPARENT);
					dc.DrawText(sTexte, rcText, DT_CENTER | DT_VCENTER | DT_NOCLIP);
					dc.SetBkMode(nBkMode);
				}
				
				if (!fBtnEnabled) {
					// Griser le bouton
					rc.DeflateRect(fBtnDown,fBtnDown);
					DrawDisabledButton(&iml, nBitmap, &dc, rc);
				}
				
				cdrw.NotifyItemPostPaint(nBouton);
			}
		}
		
		dc.SelectObject(poldFont);
		
		if (!m_fimlSup)  iml.Detach();
		
		// Informer le parent de la fin du dessin
		cdrw.NotifyPostPaint();
		
	}else
		// Mode classique,
		// ou si impossibilit d'avoir la liste d'image
		CToolBar::OnPaint();
}


// Dessiner un bouton dsactiv
void CToolBarEx::DrawDisabledButton(CImageList* piml, int nBitmap, CDC* pdc, const CRect& rc) const
{
	// Crer un DC mmoire monochrome et une bitmap pour le masque
	CDC dc;
	dc.CreateCompatibleDC(NULL);
	CBitmap bmp;
	bmp.CreateCompatibleBitmap(&dc, m_sizeImage.cx,m_sizeImage.cy);
	
	// Rcuprer la bitmap en monochrome, a suffit pour DrawState()
	CBitmap *const poldBitmap = dc.SelectObject(&bmp);
	dc.FillSolidRect(0,0, m_sizeImage.cx,m_sizeImage.cy, RGB(255,255,255));
	piml->Draw(&dc, nBitmap, CPoint(0,0), ILD_TRANSPARENT);
	dc.SelectObject(poldBitmap);
	
	// Dessiner l'image dsactive
	pdc->FillSolidRect(rc, CSysObject::clBtnFace);
	pdc->DrawState(CPoint(rc.left+m_sizeOffset.cx,rc.top+m_sizeOffset.cy), m_sizeImage, &bmp, DSS_DISABLED);
	
	// Nettoyer
	dc.DeleteDC();
	bmp.DeleteObject();
}


// Dessiner un sparateur
void CToolBarEx::DrawSeparator(CDC* pdc, CRect& rc) const
{
	bool fHorz = ToBool(m_dwStyle & CBRS_ORIENT_HORZ);
	
	// S'assurer que le sparateur n'est pas un emplacement
	// libre pour un autre contrle
	if (rc.Width() <= 8) {
		if (fHorz) {
			// Dessiner la barre du sparateur au milieu
			rc.left  = (rc.left + rc.right)/2 - 1;
			rc.right = rc.left + 2;
		}else{
			// Dessiner la barre du sparateur au milieu
			rc.top = (rc.bottom+rc.bottom+5)/2 - 1;
			rc.bottom = rc.top    + 2;
		}
		pdc->Draw3dRect(rc, CSysObject::clBtnShadow, CSysObject::clBtnHilight);
	}
}


// Dessiner le gripper
void CToolBarEx::DrawGripper(CDC* pdc) const
{
	// Ne pas dessiner de gripper pour les barres d'outils flottantes
	// Ou non-dockables
	if ((m_dwStyle & CBRS_FLOATING) || !m_dwDockStyle)
		return;
	
	CRect rcGripper;
	GetWindowRect(rcGripper);
	ScreenToClient(rcGripper);
	rcGripper -= rcGripper.TopLeft();
	
	rcGripper.left += 4;
	if (m_dwStyle & CBRS_ORIENT_HORZ) {
		// Gripper  gauche
		rcGripper.right   = rcGripper.left+3;
		rcGripper.top    += 3;
		rcGripper.bottom -= 3;
	}else{
		// Gripper en haut
		rcGripper.right  -= 4;
		rcGripper.top    += 4;
		rcGripper.bottom  = rcGripper.top+3;
	}
	pdc->Draw3dRect(rcGripper, CSysObject::clBtnHilight,CSysObject::clBtnShadow);
	
	if (m_dwStyle & CBRS_ORIENT_HORZ) {
		// Gripper  gauche
		rcGripper.left   += 4;
		rcGripper.right  += 4;
	}else{
		// Gripper en haut
		rcGripper.top    += 4;
		rcGripper.bottom += 4;
	}
	pdc->Draw3dRect(rcGripper, CSysObject::clBtnHilight,CSysObject::clBtnShadow);
}


// Grer la mise  jour des boutons
void CToolBarEx::OnUpdateCmdUI(CFrameWnd* pTarget, BOOL bDisableIfNoHandler)
{
	const int nbBouton = GetToolBarCtrl().GetButtonCount();
	int nBouton;
	UINT *puStyle;
	
	// Sauver les styles courants
	if (m_nbStyle!=nbBouton) {
		m_nbStyle = nbBouton;
		delete [] m_auStyle;
		m_auStyle = new UINT[nbBouton];
	}
	for (puStyle=m_auStyle, nBouton=0; nBouton<nbBouton; nBouton++)
		*puStyle++ = GetButtonStyle(nBouton);
	
	// Laisser CToolBar faire tout le travail
	CToolBar::OnUpdateCmdUI(pTarget, bDisableIfNoHandler);
	
	// Redessiner tous les boutons dont les styles
	// ont t modifis entre-temps
	for (puStyle=m_auStyle, nBouton=0; nBouton<nbBouton; nBouton++) {
		if (*puStyle++!=GetButtonStyle(nBouton))
			InvalidateButton(nBouton);
	}
}


// Calculer la taille de la zone non-cliente
void CToolBarEx::OnNcCalcSize(BOOL bCalcValidRects, NCCALCSIZE_PARAMS* pncsp)
{
	CToolBar::OnNcCalcSize(bCalcValidRects, pncsp);
	if (!m_dwDockStyle)  return;
	
	const int cDecal = (m_dwStyle & CBRS_FLOATING) ? -4 : 4;
	
	// Ajuster la taille suivant la position du gripper
	if (m_dwStyle & CBRS_ORIENT_HORZ) {
		pncsp->rgrc[0].left   += cDecal;
		pncsp->rgrc[0].right  += cDecal;
	}else{
		pncsp->rgrc[0].top    += cDecal;
		pncsp->rgrc[0].bottom += cDecal;
	}
	
	RecalcLayout();
}


// Dplacement de la souris
void CToolBarEx::OnMouseMove(UINT nFlags, CPoint pt)
{
#ifdef TOOLBAREX_DRAG
	if (m_fDragging) {
		// Grer le glisser/dplacer
		DragMove();
		return;
	}
#endif
	
	if (IsTopParentActive() && GetTopLevelParent()->IsWindowEnabled()) {
		const int nbBouton = GetToolBarCtrl().GetButtonCount();
		const int nLastBtn = m_nLastBtn;
		m_nLastBtn = -1;
		
		for (int nBouton=0; nBouton<nbBouton; nBouton++) {
			CRect rc;
			GetItemRect(nBouton, rc);
			
			const bool fBtnEnabled = ToBool(GetToolBarCtrl().IsButtonEnabled(int(GetItemID(nBouton))));
			const bool fSep = ToBool(GetButtonStyle(nBouton) & TBBS_SEPARATOR);
			
			if (fSep || !fBtnEnabled)  continue;
			
			if (rc.PtInRect(pt)) {
				if (fBtnEnabled) {
					if (nLastBtn!=nBouton) {
						// Redessiner le bouton survol
						InvalidateRect(rc, FALSE);
					}
					m_nLastBtn = nBouton;
				}
			}else if (nBouton==nLastBtn)
				// Redessiner le dernier bouton form
				InvalidateRect(rc, FALSE);
		}
		// Un problme arrive avec WM_MOUSEMOVE:
		// on ne peut pas dtecter quand la souris quitte la fentre
		// Si la souris se dplace assez-rapidement, le dernier
		// bouton form restera visible
		// Pour rsoudre ce problme, on cre un timer et on vrifie
		// si la souris est en dehors de la fentre
		KillTimer(TIMER_TOOLBAREX);
		SetTimer(TIMER_TOOLBAREX, 50, 0);
	}
	
	CToolBar::OnMouseMove(nFlags, pt);
}


// Dessiner la zone non-cliente
void CToolBarEx::OnNcPaint()
{
	// Rcuprer le DC de fentre clipp dans la zone non-cliente
	CWindowDC dc(this);
	CRect rcClient, rcWindow;
	GetClientRect(rcClient);
	GetWindowRect(rcWindow);
	ScreenToClient(rcWindow);
	rcClient -= rcWindow.TopLeft();
	dc.ExcludeClipRect(rcClient);
	
	// Dessiner les bordures dans la zone non-cliente
	rcWindow -= rcWindow.TopLeft();
	if (!m_dwDockStyle && (m_dwStyle & CBRS_ORIENT_HORZ))  rcWindow.top++;
	Draw3DBorders(&dc, rcWindow);
	dc.IntersectClipRect(rcWindow);
	
	// S'assurer que WM_ERASEBKGND ne sera pas envoy
	dc.FillSolidRect(rcWindow, CSysObject::clBtnFace);
	
	// Dessiner le gripper
	DrawGripper(&dc);
}

// Dessiner les bordures 3D
void CToolBarEx::Draw3DBorders(CDC* pdc, CRect& rc)
{
	ASSERT_VALID(this);
	ASSERT_VALID(pdc);
	
	if (!(m_dwStyle & CBRS_BORDER_ANY))  return;
	
	COLORREF cl = (m_dwStyle & CBRS_BORDER_3D) ? CSysObject::clBtnHilight : CSysObject::clBtnShadow;
	if (m_dwStyle & CBRS_BORDER_LEFT)
		pdc->FillSolidRect(0, 0, 1, rc.Height() - 1, cl);
	if (m_dwStyle & CBRS_BORDER_TOP)
		pdc->FillSolidRect(0, 0, rc.Width()-1 , 1, cl);
	if (m_dwStyle & CBRS_BORDER_RIGHT)
		pdc->FillSolidRect(rc.right, 1, -1, rc.Height() - 1, CSysObject::clBtnShadow);
	if (m_dwStyle & CBRS_BORDER_BOTTOM)
		pdc->FillSolidRect(0, rc.bottom, rc.Width()-1, -1, CSysObject::clBtnShadow);
	
	// Si une barre d'outils non-dockable est en haut de la frame,
	// ne pas salire le menu de la frame
	if (!m_pDockContext) {
		pdc->FillSolidRect(0,0, rc.Width(),1, CSysObject::clBtnShadow);
		pdc->FillSolidRect(0,1, rc.Width(),1, CSysObject::clBtnHilight);
	}
	
	if (m_dwStyle & CBRS_BORDER_LEFT)    rc.left  ++;
	if (m_dwStyle & CBRS_BORDER_TOP)     rc.top   ++;
	if (m_dwStyle & CBRS_BORDER_RIGHT)   rc.right --;
	if (m_dwStyle & CBRS_BORDER_BOTTOM)  rc.bottom--;
}


// Timer
void CToolBarEx::OnTimer(UINT nIDEvent)
{
	if (nIDEvent==TIMER_TOOLBAREX) {
		if (m_nLastBtn >= 0) {
			POINT pt;
			::GetCursorPos(&pt);
			CRect rc;
			GetWindowRect(rc);
			if (!rc.PtInRect(pt) || WindowFromPoint(pt)!=this) {
				InvalidateButton(m_nLastBtn);
				m_nLastBtn = -1;
			}
		}
		if (m_nLastBtn<0)  KillTimer(nIDEvent);
	}else
		CToolBar::OnTimer(nIDEvent);
}



// Changement de position
void CToolBarEx::OnWindowPosChanging(WINDOWPOS* pwp)
{
	if (pwp->flags & SWP_HIDEWINDOW)  return;
	CToolBar::OnWindowPosChanging(pwp);
	
	if (!(pwp->flags & SWP_NOMOVE)) {
		// Si juste dplac, forcer un recalcul de la zone cliente
		//  cause du gripper qui doit tre affich ou masqu
		// suivant si la barre d'outils est flottante ou non
		CRect rc;
		GetWindowRect(rc);
		SetWindowPos(NULL, 0,0, rc.Width()+1, rc.Height(), SWP_NOZORDER | SWP_NOMOVE | SWP_NOACTIVATE | SWP_NOREDRAW);
		SetWindowPos(NULL, 0,0, rc.Width(),   rc.Height(), SWP_NOZORDER | SWP_NOMOVE | SWP_NOACTIVATE);
	}
}


// Retourner la liste d'images de la barre d'outils
HIMAGELIST CToolBarEx::GetImageList()
{
	m_fimlSup = false;
	HIMAGELIST himl = NULL;
	
#ifdef TB_GETIMAGELIST
	// Certains anciennes versions de VC++
	// Ne connaissent pas la macro TB_GETIMAGELIST
	// (dfinie dans commctrl.h).
	himl = (HIMAGELIST)SendMessage(TB_GETIMAGELIST);
	
#ifdef _DEBUG
	if (!himl)
		TRACE0("ERREUR - CToolBarEx::GetImageList() - Liste d'images non-trouve\n");
#endif
#endif // TB_GETIMAGELIST
	
	if (!himl) {
		// comctl32.dll avant la version 4.70 ne connait rien
		// du message TB_GETIMAGELIST
		if (m_hbmImageWell) {
			// On a une image valide, mais il faut en faire
			// une copie, et sans utiliser CopyImage()
			// qui peut chouer
			
			CImageList iml;
			CBitmap bmp;
			
			// Rcuprer la taille de la bitmap
			BITMAP bmHeader;
			::GetObject(m_hbmImageWell, sizeof(BITMAP), &bmHeader);
			
			const DWORD cx = ((((bmHeader.bmBitsPixel>8)
				? bmHeader.bmWidth*24
				: bmHeader.bmWidth* 8) + 31) &~ 31)/8;
			const DWORD cy = bmHeader.bmHeight;
			
			// Copier la bitmap
			CClientDC cdc(this);
			CDC dc1, dc2;
			dc1.CreateCompatibleDC(&cdc);
			dc2.CreateCompatibleDC(&cdc);
			bmp.CreateCompatibleBitmap(&cdc, cx,cy);
			CBitmap *const poldBitmap = dc1.SelectObject(&bmp);
			const HGDIOBJ holdBitmap = ::SelectObject(dc2.GetSafeHdc(), m_hbmImageWell);
			dc1.BitBlt(0,0, cx,cy, &dc2, 0,0, SRCCOPY);
			::SelectObject(dc2.GetSafeHdc(), holdBitmap);
			dc1.SelectObject(poldBitmap);
			dc1.DeleteDC();
			dc2.DeleteDC();
			
			iml.Create(m_sizeImage.cx, m_sizeImage.cy,TRUE,cx/m_sizeImage.cx,1);
			iml.SetBkColor(CSysObject::clBtnFace);
			iml.Add(&bmp, CSysObject::clBtnFace);
			himl = iml.Detach();
			bmp.DeleteObject();
			m_fimlSup = true;
		}
	}
	return himl;
}


// Invalider un bouton d'index donn
void CToolBarEx::InvalidateButton(int nIndex)
{
	if (nIndex<0 || nIndex>=GetToolBarCtrl().GetButtonCount())
		return;
	CRect rc;
	GetItemRect(nIndex, rc);
	InvalidateRect(rc, FALSE);
}


// Retourner TRUE si un bouton d'index donn
// est un sparateur
bool CToolBarEx::IsSeparator(int nIndex) const
{
	if (nIndex>=0 && nIndex<GetToolBarCtrl().GetButtonCount() &&
	    GetButtonStyle(nIndex)==TBBS_SEPARATOR) {
		// S'assurer que c'est un "vrai" sparateur
		CRect rc;
		GetItemRect(nIndex, rc);
		return rc.Width()<=8;
	}
	return false;
}


// Retourner TRUE si un bouton d'index donn
// est un contrle
#ifdef TOOLBAREX_CONTROL
bool CToolBarEx::IsControl(int nIndex) const
{
	if (nIndex>=0 && nIndex<GetToolBarCtrl().GetButtonCount() &&
	    GetButtonStyle(nIndex)==TBBS_SEPARATOR) {
			// S'assurer que c'est un emplacement vide
			// pour un contrle
			CRect rc;
			GetItemRect(nIndex, rc);
			return rc.Width()>8;
		}
	return false;
}
#endif


// Charger une ressource Barre d'outils
BOOL CToolBarEx::LoadToolBar(LPCTSTR pszResource)
{
	int i;
	
	// Dtruire tous les boutons
	for (i=(int)DefWindowProc(TB_BUTTONCOUNT, 0,0); i; i--)
		VERIFY(DefWindowProc(TB_DELETEBUTTON, 0, 0));
	
	if (!pszResource)  return TRUE;
	
	struct TOOLBARDATA
	{
		WORD wVersion;
		WORD cx;
		WORD cy;
		WORD nbItem;
		WORD aItem[1];
	};
	
	ASSERT_VALID(this);
	ASSERT(pszResource!=NULL);
	
	// Rcuprer la ressource de la barre d'outils
	const HINSTANCE hInst = AfxFindResourceHandle(pszResource, RT_TOOLBAR);
	const HRSRC hRsrc = ::FindResource(hInst, pszResource, RT_TOOLBAR);
	if (!hRsrc)  return FALSE;
	const HGLOBAL hGlobal = LoadResource(hInst, hRsrc);
	if (!hGlobal)  return FALSE;
	TOOLBARDATA *const pData = (TOOLBARDATA*)LockResource(hGlobal);
	if (!pData)  return FALSE;
	ASSERT(pData->wVersion==1);
	
	const int buf = 128;
	TCHAR psz[buf];
	
	BOOL bRes;
	
	delete [] m_atbb;
	m_nbtbb = m_nbtbbPerso = pData->nbItem;
	TBBUTTON *ptbb = m_atbb = new TBBUTTON[m_nbtbb];
	int iImage = 0;
	for (i=0; i<m_nbtbb; i++, ptbb++) {
		ptbb->fsState = TBSTATE_ENABLED;
		if (!(ptbb->idCommand = pData->aItem[i])) {
			// Sparateur
			
			ptbb->fsStyle = TBSTYLE_SEP;
			ptbb->iBitmap = 6;
			ptbb->iString = -1;
			
			// Un bouton de moins dans le tableau sans sparateurs
			m_nbtbbPerso--;
			
		}else{
			// Bouton  image
			
			ptbb->fsStyle = TBSTYLE_BUTTON;
			ptbb->iBitmap = iImage++;
			ptbb->iString = -1;
			if (::LoadString(hInst, ptbb->idCommand, psz, buf)) {
				// La chane est charge:
				// il faut maintenant sauter le '\n'
				LPTSTR pszSep = _tcschr(psz, '\n');
				if (pszSep) {
					// Rajouter le double nul final puis ajouter la chane
					TCHAR *pc = ++pszSep;
					while (*pc)  pc++;
					*pc = 0;
					ptbb->iString = (int)SendMessage(TB_ADDSTRING, 0, (LPARAM)pszSep);
				}
			}
		}
		if (!DefWindowProc(TB_ADDBUTTONS, 1, (LPARAM)ptbb)) {
			bRes = FALSE;
			goto Fin;
		}
	}
	
	// Crer un tableau des boutons sans les sparateurs
	{
		delete [] m_atbbPerso;
		TBBUTTON *ptbbPerso = m_atbbPerso = new TBBUTTON[m_nbtbbPerso];
		ptbb = m_atbb;
		for (i=0; i<m_nbtbbPerso; ptbb++) {
			if (ptbb->fsStyle & TBSTYLE_SEP)  continue;
			i++;
			*ptbbPerso++ = *ptbb;
		}
	}
	
	// Dfinir les dimensions des boutons
	SetSizes(
		CSize(pData->cx+7, pData->cy+7),
		CSize(pData->cx,   pData->cy));
	
	// Charger la bitmap
	bRes = LoadBitmap(pszResource);
	
Fin:
	UnlockResource(hGlobal);
	FreeResource(hGlobal);
	
	return bRes;
}


// Rcuprer le contrle associ  un index donn
// Le Cwnd* retourn peut tre temporaire si vous
// n'utilisez pas les MFC pour ajouter un contrle
//  la barre d'outils
#ifdef TOOLBAREX_CONTROL
CWnd* CToolBarEx::GetControl(int nIndex, bool fEstID) const
{
	const UINT nID = (fEstID) ? (UINT)nIndex : GetItemID(nIndex);
	for (CWnd *pWnd=GetWindow(GW_CHILD); pWnd; pWnd=pWnd->GetNextWindow()) {
		if (UINT(::GetWindowLong(pWnd->GetSafeHwnd(), GWL_ID))==nID)
			return pWnd;
	}
	
	return NULL;
}
#endif

// Remplace le bouton d'ID donn par un contrle
// personnalis de classe pClasse
// Le contrle personnalis aura l'ID du bouton
// Les styles WS_CHILD et WS_VISIBLE seront ajouts
// automatiquement
// Le contrle doit supporter DECLARE_DYNCREATE,
// sauf CComboBox et CEdit qui sont traits sparment
// Ne pas "delete" la valeur retourne
#ifdef TOOLBAREX_CONTROL
CWnd* CToolBarEx::CtrlReplace(CRuntimeClass* pClasse, CRect& rc, UINT nID, DWORD dwStyle)
{
	int nIndex = CommandToIndex(nID);
	if (nIndex<0) {
		TRACE1("ERREUR - CToolBarEx::CtrlReplace() - 0x%x n'est pas un ID valide pour cette barre d'outils\n,", nID);
		return 0;
	}
	
	SetButtonInfo(nIndex, nID, TBBS_SEPARATOR, abs(rc.Width()));
	return CreateControl(pClasse, rc, nID, dwStyle);
}

// Insrer un contrle personnalis avant le contrle
// d'index donn, ou en dernier si nAvant == -1
CWnd* CToolBarEx::CtrlInsert(CRuntimeClass* pClasse, CRect& rc, UINT nID, int nAvant, DWORD dwStyle)
{
	CToolBarCtrl& tlbc = GetToolBarCtrl();
	
	TBBUTTON tbButton;
	tbButton.iBitmap   = abs(rc.Width());
	tbButton.idCommand = nID;
	tbButton.fsState   = TBSTATE_ENABLED;
	tbButton.fsStyle   = TBSTYLE_SEP;
	tbButton.dwData    = 0;
	tbButton.iString   = 0;
	if (nAvant<0 || nAvant>tlbc.GetButtonCount())
		tlbc.AddButtons(1, &tbButton);
	else
		tlbc.InsertButton(nAvant, &tbButton);
	
	return CreateControl(pClasse, rc, nID, dwStyle);
}

// Crer un nouveau contrle
// Utilis par CtrlReplace() et CtrlInsert()
CWnd* CToolBarEx::CreateControl(CRuntimeClass* pClasse, CRect& rc, UINT nID, DWORD dwStyle)
{
	if (!pClasse || !pClasse->IsDerivedFrom(RUNTIME_CLASS(CWnd))) {
		TRACE0("ERREUR - CToolBarEx::CreateControl() - Classe donne  NULL ou non-drive de CWnd\n");
		return 0;
	}
	
	CRect rect = rc;
	BOOL bCreate = FALSE;
	bool fAutoSupprimer = false;
	CWnd *pCtrl;
	
	if (pClasse->m_lpszClassName==RUNTIME_CLASS(CComboBox)->m_lpszClassName) {
		pCtrl = new CComboBox;
		bCreate = ((CComboBox*)pCtrl)->Create(WS_CHILD | WS_VISIBLE | dwStyle, rc, this, nID);
	}else if (pClasse->IsDerivedFrom(RUNTIME_CLASS(CEdit))) {
		pCtrl = new CEdit;
		bCreate = ((CEdit*)pCtrl)->Create(WS_CHILD | WS_VISIBLE | dwStyle, rc, this, nID);
	}else{
		pCtrl = (CWnd*)pClasse->CreateObject();
		if (!pCtrl) {
			TRACE1("ERREUR - CToolBarEx::CreateControl() - Cration dynamique du contrle %hs a chou\n", pClasse->m_lpszClassName);
			return 0;
		}
		fAutoSupprimer = true;
		bCreate = pCtrl->Create(0,0, WS_CHILD | WS_VISIBLE | dwStyle, rc, this, nID);
	}
	
	if (!bCreate) {
		TRACE1("ERREUR - CToolBarEx::CreateControl() - Create() a chou\n,", nID);
		if (!fAutoSupprimer)  delete pCtrl;
		return 0;
	}
	
	// On doit se souvenir de ce contrle pour le supprimer plus tard
	if (!fAutoSupprimer)  m_lstControl.AddFin(pCtrl);
	
	RepositionControls();
	
	return pCtrl;
}

// Repositionner les contrles
void CToolBarEx::RepositionControls()
{
	// Parcourir tous les descendants
	for (CWnd *pWnd=GetWindow(GW_CHILD); pWnd; pWnd=pWnd->GetNextWindow()) {
		// Si cette fentre est fille de la barre d'outils,
		// repositionner le contrle
		if (pWnd->GetParent()==this) {
			ASSERT(CommandToIndex(pWnd->GetDlgCtrlID())>=0);
			CRect rc;
			GetItemRect(CommandToIndex(pWnd->GetDlgCtrlID()), rc);
			pWnd->SetWindowPos(0, rc.left,rc.top, 0,0, SWP_NOZORDER | SWP_NOACTIVATE | SWP_NOSIZE | SWP_NOCOPYBITS);
			pWnd->ShowWindow(SW_SHOW);
		}
	}
}
#endif

// Recalculer la taille de la barre d'outils
// et mettre  jour le parent
void CToolBarEx::RecalcLayout()
{
	// Recalculer la taille de la barre d'outils
	bool fHorz = ToBool(m_dwStyle & CBRS_ORIENT_HORZ);
	if ((m_dwStyle & CBRS_FLOATING) && (m_dwStyle & CBRS_SIZE_DYNAMIC))
		CalcDynamicLayout(0, LM_HORZ | LM_MRUWIDTH | LM_COMMIT);
	else if (fHorz)
		CalcDynamicLayout(0, LM_HORZ | LM_HORZDOCK | LM_COMMIT);
	else
		CalcDynamicLayout(0, LM_VERTDOCK | LM_COMMIT);
	
#ifdef TOOLBAREX_CONTROL
	RepositionControls();
#endif
	
	// Recalculer la frame parent
	if (m_dwStyle & CBRS_FLOATING) {
		ASSERT(m_pDockBar);
		((CMiniDockFrameWnd*)m_pDockBar->GetParent())->RecalcLayout();
	}else
		CToolBar::GetParentFrame()->RecalcLayout();
}

// Rcuprer la bitmap associe  un bouton d'ID donn
// Le fond de la bitmap a la couleur COLOR_BTNFACE
// Ne pas oublier de dtruire la bitmap
HBITMAP CToolBarEx::GetBitmap(int nID)
{
	int nBitmap = SendMessage(TB_GETBITMAP, WPARAM(nID));
	return GetBitmap(nBitmap, m_sizeImage);
}

// Rcuprer la bitmap d'index nIndex et la redimensionner
// de manire  ce qu'elle tienne dans "sizeBitmap"
HBITMAP CToolBarEx::GetBitmap(int nIndex, const CSize& sizeBitmap)
{
	HIMAGELIST himl = GetImageList();
	if (!himl)  return NULL;
	
	CImageList iml;
	iml.Attach(himl);
	
	const HICON hIcon = iml.ExtractIcon(nIndex);
	CBitmap bmp;
	
	if (hIcon) {
		CClientDC cdc(this);
		CDC dc;
		dc.CreateCompatibleDC(&cdc);
		VERIFY(bmp.CreateCompatibleBitmap(&cdc, sizeBitmap.cx, sizeBitmap.cy));
		CBitmap *const poldBitmap = dc.SelectObject(&bmp);
		::DrawIconEx(
			dc.GetSafeHdc(), 0,             0,
			hIcon,           sizeBitmap.cx, sizeBitmap.cy,
			0, CBrush(CSysObject::clBtnFace), DI_NORMAL);
		dc.SelectObject(poldBitmap);
		dc.DeleteDC();
		
		// On n'a plus besoin de l'icne
		DestroyIcon(hIcon);
	}else
		TRACE1("ERREUR - CToolBarEx::GetBitmap() - Extraction de bitmap d'index %da chou\n", nIndex);
	
	if (!m_fimlSup)  iml.Detach();
	
	return hIcon ? (HBITMAP)bmp.Detach() : NULL;
}


// Une bitmap est ajoute
LRESULT CToolBarEx::OnAddBitmap(WPARAM wParam, LPARAM lParam)
{
#define nButtons    ((int)wParam)
#define pAddBitmap  ((TBADDBITMAP*)lParam)
	
	// Bug dans CToolBar:
	// Si on appelle CToolBar::GetToolBarCtrl().AddBitmap(...),
	// CToolBar ne redessine pas la barre d'outils
	
	if (pAddBitmap->hInst!=HINST_COMMCTRL) {
		// Cela ne marche pas si on met
		// HINST_COMMCTRL dans hInst, car on
		// ne peut pas accder  CommCtl32.dll
		
		TRACE0("CToolBarEx - Ajout de bitmap\n");
		
		HBITMAP hBitmap;
		if (pAddBitmap->hInst)
			// "pAddBmp->nID" contient l'ID de la bitmap
			hBitmap = ::LoadBitmap(pAddBitmap->hInst, MAKEINTRESOURCE(pAddBitmap->nID));
		else
			// "pAddBmp->nID" est le handle de la bitmap
			hBitmap = (HBITMAP)pAddBitmap->nID;
		
		if (!hBitmap)  return (LRESULT)-1;
		ASSERT(m_hbmImageWell);
		
		// Rcuprer le nombre d'images stockes dans CToolBar
		BITMAP bmHeader;
		VERIFY(::GetObject(m_hbmImageWell, sizeof(BITMAP), &bmHeader));
		int nbBitmap = bmHeader.bmWidth/m_sizeImage.cx;
		
		CClientDC cdc(this);
		CDC dcOld, dcNew;
		dcOld.CreateCompatibleDC(&cdc);
		dcNew.CreateCompatibleDC(&cdc);
		
		const HGDIOBJ holdBitmap = ::SelectObject(dcOld.GetSafeHdc(), m_hbmImageWell);
		
		// Crer la nouvelle bitmap et la rendre
		// assez grande pour contenir toutes les images
		CBitmap bmpNew;
		bmpNew.CreateCompatibleBitmap(&cdc,
			bmHeader.bmWidth + nButtons*m_sizeImage.cx,
			m_sizeImage.cy);
		CBitmap *const pbmpNew = dcNew.SelectObject(&bmpNew);
		
		dcNew.BitBlt(0,0, bmHeader.bmWidth,bmHeader.bmHeight, &dcOld, 0,0, SRCCOPY);
		
		::SelectObject(dcOld.GetSafeHdc(), hBitmap);
			dcNew.BitBlt(
				bmHeader.bmWidth,0, m_sizeImage.cx*nButtons,bmHeader.bmHeight, &dcOld, 0,0, SRCCOPY);
		
		::SelectObject(dcOld.GetSafeHdc(), holdBitmap);
		dcNew.SelectObject(pbmpNew);
		dcOld.DeleteDC();
		dcNew.DeleteDC();
		
		VERIFY(bmpNew.GetObject(sizeof(BITMAP), &bmHeader));
		
		DefWindowProc(TB_ADDBITMAP, wParam, lParam);
		
		// Synchroniser la bitmap du contrle avec la ntre
		AddReplaceBitmap((HBITMAP)bmpNew.Detach());
		
		return (LRESULT)nbBitmap;
	}
	return DefWindowProc(TB_ADDBITMAP, wParam, lParam);
	
#undef nButtons
#undef pAddBitmap
}


#ifdef TOOLBAREX_CONTROL
// Intercepter TB_DELETEBUTTON pour pouvoir aussi
// supprimer les contrles
LRESULT CToolBarEx::OnDeleteButton(WPARAM wParam, LPARAM lParam)
{
	CWnd *const pControl = GetControl((int)wParam);
	if (pControl) {
		// C'est le contrle associ au bouton  supprimer
		bool fDoitSup = false;
		
		// Supprimer le contrle
		const POS pos = m_lstControl.Find(pControl);
		if (pos) {
			m_lstControl.RemoveAt(pos);
			fDoitSup = true;
		}
		pControl->DestroyWindow();
		if (fDoitSup)  delete pControl;
	}
	return DefWindowProc(TB_DELETEBUTTON, wParam, lParam);
}
#endif


// Effacer le fond
BOOL CToolBarEx::OnEraseBkgnd(CDC*)
{
	return FALSE;
}



#ifdef TOOLBAREX_DRAG

// Alt+glisser/dplacer

// Curseurs utiliss pour le glisser/dplacer
static const BYTE ANDmaskDrop[] =
{
	0xFF, 0xFF, 0xFF, 0xFF,   // ligne 1
	0xFF, 0xFF, 0xFF, 0xFF,   // ligne 2
	0xF3, 0xFF, 0xFF, 0xFF,   // ligne 3
	0xF1, 0xFF, 0xFF, 0xFF,   // ligne 4
	0xF0, 0xFF, 0xFF, 0xFF,   // ligne 5
	0xF0, 0x7F, 0xFF, 0xFF,   // ligne 6
	0xF0, 0x3F, 0xFF, 0xFF,   // ligne 7
	0xF0, 0x1F, 0xFF, 0xFF,   // ligne 8
	0xF0, 0x0F, 0xFF, 0xFF,   // ligne 9
	0xF0, 0x07, 0xFF, 0xFF,   // ligne 10
	0xF0, 0x03, 0xFF, 0xFF,   // ligne 11
	0xF0, 0x01, 0xFF, 0xFF,   // ligne 12
	0xF0, 0x00, 0xFF, 0xFF,   // ligne 13
	0xF0, 0x0F, 0xFF, 0xFF,   // ligne 14
	0xF0, 0x0F, 0xFF, 0xFF,   // ligne 15
	0xF1, 0x07, 0xFF, 0xFF,   // ligne 16
	0xF3, 0x07, 0xFF, 0xFF,   // ligne 17
	0xF6, 0x00, 0x00, 0x3F,   // ligne 18
	0xFE, 0x00, 0x00, 0x3F,   // ligne 19
	0xFE, 0x00, 0x00, 0x3F,   // ligne 20
	0xFE, 0x00, 0x00, 0x3F,   // ligne 21
	0xFE, 0x00, 0x00, 0x3F,   // ligne 22
	0xFE, 0x00, 0x00, 0x3F,   // ligne 23
	0xFE, 0x00, 0x00, 0x3F,   // ligne 24
	0xFE, 0x00, 0x00, 0x3F,   // ligne 25
	0xFE, 0x00, 0x00, 0x3F,   // ligne 26
	0xFF, 0xFF, 0xFF, 0xFF,   // ligne 27
	0xFF, 0xFF, 0xFF, 0xFF,   // ligne 28
	0xFF, 0xFF, 0xFF, 0xFF,   // ligne 29
	0xFF, 0xFF, 0xFF, 0xFF,   // ligne 30
	0xFF, 0xFF, 0xFF, 0xFF,   // ligne 31
	0xFF, 0xFF, 0xFF, 0xFF    // ligne 32
};
 
static const BYTE XORmaskDrop[] =
{
	0x00, 0x00, 0x00, 0x00,   // ligne 1
	0x00, 0x00, 0x00, 0x00,   // ligne 2
	0x00, 0x00, 0x00, 0x00,   // ligne 3
	0x04, 0x00, 0x00, 0x00,   // ligne 4
	0x06, 0x00, 0x00, 0x00,   // ligne 5
	0x07, 0x00, 0x00, 0x00,   // ligne 6
	0x07, 0x80, 0x00, 0x00,   // ligne 7
	0x07, 0xC0, 0x00, 0x00,   // ligne 8
	0x07, 0xE0, 0x00, 0x00,   // ligne 9
	0x07, 0xF0, 0x00, 0x00,   // ligne 10
	0x07, 0xF8, 0x00, 0x00,   // ligne 11
	0x07, 0xFC, 0x00, 0x00,   // ligne 12
	0x07, 0xE0, 0x00, 0x00,   // ligne 13
	0x07, 0x60, 0x00, 0x00,   // ligne 14
	0x06, 0x60, 0x00, 0x00,   // ligne 15
	0x04, 0x30, 0x00, 0x00,   // ligne 16
	0x00, 0x30, 0x00, 0x00,   // ligne 17
	0x00, 0x18, 0x00, 0x00,   // ligne 18
	0x00, 0xDD, 0xFF, 0x00,   // ligne 19
	0x00, 0xAC, 0xAA, 0x00,   // ligne 20
	0x00, 0xCD, 0x55, 0x00,   // ligne 21
	0x00, 0xA0, 0xAA, 0x00,   // ligne 22
	0x00, 0xD5, 0x55, 0x00,   // ligne 23
	0x00, 0xAA, 0xAA, 0x00,   // ligne 24
	0x00, 0x00, 0x00, 0x00,   // ligne 25
	0x00, 0x00, 0x00, 0x00,   // ligne 26
	0x00, 0x00, 0x00, 0x00,   // ligne 27
	0x00, 0x00, 0x00, 0x00,   // ligne 28
	0x00, 0x00, 0x00, 0x00,   // ligne 29
	0x00, 0x00, 0x00, 0x00,   // ligne 30
	0x00, 0x00, 0x00, 0x00,   // ligne 31
	0x00, 0x00, 0x00, 0x00    // ligne 32
};

static const BYTE ANDmaskNoDrop[] =
{
	0xFF, 0xFF, 0xFF, 0xFF,   // ligne 1
	0xFF, 0xFF, 0xFF, 0xFF,   // ligne 2
	0xF3, 0xFF, 0xFF, 0xFF,   // ligne 3
	0xF1, 0xFF, 0xFF, 0xFF,   // ligne 4
	0xF0, 0xFF, 0xFF, 0xFF,   // ligne 5
	0xF0, 0x7F, 0xFF, 0xFF,   // ligne 6
	0xF0, 0x3F, 0xFF, 0xFF,   // ligne 7
	0xF0, 0x1F, 0xFF, 0xFF,   // ligne 8
	0xF0, 0x0F, 0xFF, 0xFF,   // ligne 9
	0xF0, 0x07, 0xFF, 0xFF,   // ligne 10
	0xF0, 0x03, 0xFF, 0xFF,   // ligne 11
	0xF0, 0x01, 0xFF, 0xFF,   // ligne 12
	0xF0, 0x00, 0xFF, 0xFF,   // ligne 13
	0xF0, 0x0F, 0xFF, 0xFF,   // ligne 14
	0xF0, 0x0F, 0xFF, 0xFF,   // ligne 15
	0xF1, 0x07, 0xFF, 0xFF,   // ligne 16
	0xF3, 0x07, 0xFF, 0xFF,   // ligne 17
	0xF6, 0x00, 0x00, 0x3F,   // ligne 18
	0xFE, 0x00, 0x00, 0x3F,   // ligne 19
	0xFE, 0x00, 0x00, 0x3F,   // ligne 20
	0xFE, 0x00, 0x00, 0x3F,   // ligne 21
	0xFE, 0x00, 0x00, 0x0F,   // ligne 22
	0xFE, 0x00, 0x00, 0x0F,   // ligne 23
	0xFE, 0x00, 0x00, 0x0F,   // ligne 24
	0xFE, 0x00, 0x00, 0x0F,   // ligne 25
	0xFE, 0x00, 0x00, 0x0F,   // ligne 26
	0xFF, 0xFF, 0x80, 0x0F,   // ligne 27
	0xFF, 0xFF, 0x80, 0x0F,   // ligne 28
	0xFF, 0xFF, 0x80, 0x0F,   // ligne 29
	0xFF, 0xFF, 0x80, 0x0F,   // ligne 30
	0xFF, 0xFF, 0x80, 0x0F,   // ligne 31
	0xFF, 0xFF, 0x80, 0x0F    // ligne 32
};
 
static const BYTE XORmaskNoDrop[] =
{
	0x00, 0x00, 0x00, 0x00,   // ligne 1
	0x00, 0x00, 0x00, 0x00,   // ligne 2
	0x00, 0x00, 0x00, 0x00,   // ligne 3
	0x04, 0x00, 0x00, 0x00,   // ligne 4
	0x06, 0x00, 0x00, 0x00,   // ligne 5
	0x07, 0x00, 0x00, 0x00,   // ligne 6
	0x07, 0x80, 0x00, 0x00,   // ligne 7
	0x07, 0xC0, 0x00, 0x00,   // ligne 8
	0x07, 0xE0, 0x00, 0x00,   // ligne 9
	0x07, 0xF0, 0x00, 0x00,   // ligne 10
	0x07, 0xF8, 0x00, 0x00,   // ligne 11
	0x07, 0xFC, 0x00, 0x00,   // ligne 12
	0x07, 0xE0, 0x00, 0x00,   // ligne 13
	0x07, 0x60, 0x00, 0x00,   // ligne 14
	0x06, 0x60, 0x00, 0x00,   // ligne 15
	0x04, 0x30, 0x00, 0x00,   // ligne 16
	0x00, 0x30, 0x00, 0x00,   // ligne 17
	0x00, 0x18, 0x00, 0x00,   // ligne 18
	0x00, 0xDD, 0xFF, 0x00,   // ligne 19
	0x00, 0xAC, 0xAA, 0x00,   // ligne 20
	0x00, 0xCD, 0x55, 0x00,   // ligne 21
	0x00, 0xA0, 0x80, 0x00,   // ligne 22
	0x00, 0xD5, 0x3F, 0xE0,   // ligne 23
	0x00, 0xAA, 0xA7, 0x20,   // ligne 24
	0x00, 0x00, 0x22, 0x20,   // ligne 25
	0x00, 0x00, 0x30, 0x60,   // ligne 26
	0x00, 0x00, 0x38, 0xE0,   // ligne 27
	0x00, 0x00, 0x30, 0x60,   // ligne 28
	0x00, 0x00, 0x22, 0x20,   // ligne 29
	0x00, 0x00, 0x27, 0x20,   // ligne 30
	0x00, 0x00, 0x3F, 0xE0,   // ligne 31
	0x00, 0x00, 0x00, 0x00    // ligne 32
};

// Bouton gauche enfonc
void CToolBarEx::OnLButtonDown(UINT nFlags, CPoint pt)
{
	if (::GetAsyncKeyState(VK_MENU) & 0x8000)
		// Une des touches ALT est presse - commencer l'opration
		// Commencer le glisser/dplacer
		if (BeginDrag())  return;
	
	if (::GetAsyncKeyState(VK_SHIFT) & 0x8000)
		// Dsactiver l'ancien mode de glisser/dplacer
		return;
	
	CToolBar::OnLButtonDown(nFlags, pt);
}

// Bouton gauche relch
void CToolBarEx::OnLButtonUp(UINT nFlags, CPoint pt)
{
	if (m_fDragging)
		EndDrag();
	else
		CToolBar::OnLButtonUp(nFlags, pt);
}


// Commencer le glisser/dplacer
bool CToolBarEx::BeginDrag()
{
	if (!m_fEditable)  return false;
	
	VERIFY(!m_hcurDrag);
	VERIFY(!m_hcurNoDrag);
	
	if (!(GetToolBarCtrl().GetStyle() & CCS_ADJUSTABLE))
		return false;  // La barre n'est pas ajustable
	
	const int nbBouton = GetToolBarCtrl().GetButtonCount();
	m_nLastBtn = -1;
	CPoint pt;
	::GetCursorPos(&pt);
	ScreenToClient(&pt);
	
	m_nDragButton = -1;
	
	// Il faut un look si le bouton est valide:
	// dessiner une bordure autour
	for (int nBouton=0; nBouton<nbBouton; nBouton++) {
		if (IsSeparator(nBouton))
			continue;  // Les sparateurs rels ne sont pas dplaables
		
		CRect rc;
		GetItemRect(nBouton, rc);
		
		if (rc.PtInRect(pt)) {
			// On a trouv le bouton
			// Demander la suppression
			if (!DoQueryDelete(nBouton))
				// L'application ne permet pas
				return false;
			
			m_nDragButton = nBouton;
			CClientDC cdc(this);
			cdc.DrawDragRect(rc, CSize(2,2),0,CSize(0,0));
			break;
		}
	}
	
	// S'il n'y a rien  dplacer, s'en aller
	if (m_nDragButton<0)  return false;
	
	// Crer les curseurs
	VERIFY(m_hcurDrag   = ::CreateCursor(0, 4, 2, 32,32, ANDmaskDrop,   XORmaskDrop));
	VERIFY(m_hcurNoDrag = ::CreateCursor(0, 4, 2, 32,32, ANDmaskNoDrop, XORmaskNoDrop));
	
	// Capturer la souris et le clavier
	SetCapture();
	SetFocus();
	
	m_hcurOrig = ::SetCursor(m_hcurDrag);
	m_fDragCursor = true;
	m_fDragging   = true;
	
	::GetCursorPos(&pt);
	SetMarker(m_pDropBar=this, m_ptDrop=pt);
	
	return m_fDragging;
}

// Retourner TRUE si la fentre spcifie
// est une destination valide de glisser/dplacer,
// c'est  dire une barre d'outils ajustable
bool CToolBarEx::IsCibleValide(const CWnd* pWnd) const
{
	return pWnd &&
		pWnd->IsKindOf(RUNTIME_CLASS(CToolBarEx)) &&
		(::GetWindowLong(((CToolBarEx*)pWnd)->GetToolBarCtrl().GetSafeHwnd(), GWL_STYLE) & CCS_ADJUSTABLE) &&
		pWnd->GetParentFrame()==GetParentFrame();
}

// Glisser/dplacer
void CToolBarEx::DragMove()
{
	CPoint pt;
	::GetCursorPos(&pt);
	const CWnd *pWnd = WindowFromPoint(pt);
	
	// Le curseur est-il sur une barre d'outils ajustable ?
	bool fToolBar = IsCibleValide(pWnd);
	
	// Si la fentre pointe n'est pas une barre d'outils,
	// vrifier si ce n'est pas une fille de la barre d'outils
	while (!fToolBar && (pWnd = pWnd->GetParent()))
		fToolBar = IsCibleValide(pWnd);
	
	// Vrifier si on doit changer de curseur
	if (fToolBar && ! m_fDragCursor) {
		::SetCursor(m_hcurDrag);
		m_fDragCursor = true;
	}else if (!fToolBar && m_fDragCursor) {
		::SetCursor(m_hcurNoDrag);
		m_fDragCursor = false;
	}
	
	SetMarker(m_pDropBar = (fToolBar ? (CToolBarEx*)pWnd : NULL), m_ptDrop = pt);
}

// Fin du glisser/dplacer
void CToolBarEx::EndDrag(bool fDoMove)
{
	// Effacer le marqueur
	SetMarker(0, CPoint(0,0));
	
	VERIFY(::SetCursor(m_hcurOrig));
	::DestroyCursor(m_hcurDrag);
	::DestroyCursor(m_hcurNoDrag);
	m_hcurDrag   = NULL;
	m_hcurNoDrag = NULL;
	
	if (m_nDragButton>=0) {
		CToolBarCtrl &tlbc = GetToolBarCtrl();
		const int nbBouton = tlbc.GetButtonCount();
		
		if (m_fDragCursor && fDoMove) {
			// Dplacer le bouton  une position diffrente
			
			// S'assurer que le dernier "DragMove()" a fonctionn correctement
			ASSERT(m_pDropBar);
			ASSERT_KINDOF(CToolBarEx, m_pDropBar);
			ASSERT(::GetWindowLong(m_pDropBar->GetToolBarCtrl().GetSafeHwnd(), GWL_STYLE) & CCS_ADJUSTABLE);
			
			// Il faut avoir un endroit o dplacer le bouton
			int nDropBtn = m_pDropBar->FindDropButton(m_ptDrop);
			TBBUTTON tbButton;
			ZeroMemory(&tbButton, sizeof(TBBUTTON));
			
			if (m_pDropBar==this) {
				// Dplacer le bouton sur CETTE barre d'outils
				if (nDropBtn==m_nDragButton+1 || (nDropBtn<0 && m_nDragButton==nbBouton-1)) {
					// Insrer un sparateur avant le bouton dplac,
					// s'il n'y en a pas dj un
					if (m_nDragButton>0 && !IsSeparator(m_nDragButton-1)) {
						tbButton.iBitmap = 8;
						tbButton.fsState = TBSTATE_ENABLED;
						tbButton.fsStyle = TBSTYLE_SEP;
						if (DoQueryInsert(tbButton, m_nDragButton))
							tlbc.InsertButton(m_nDragButton, &tbButton);
					}
				}else if (nDropBtn==m_nDragButton && m_nDragButton>0) {
					// Supprimer le sparateur plac immdiatement avant
					// le bouton dplac
					// S'il n'y en a pas, ne rien faire
					if (IsSeparator(nDropBtn-1) && DoQueryDelete(nDropBtn-1))
						tlbc.DeleteButton(nDropBtn-1);
				}else{
					tlbc.GetButton(m_nDragButton, &tbButton);
					if (DoQueryInsert(tbButton, (nDropBtn>=0) ? nDropBtn : nbBouton)) {
#ifdef TOOLBAREX_CONTROL
						CWnd *pControl = NULL;
						if (IsControl(m_nDragButton)) {
							// Attention: le message TB_DELETEBUTTON
							// supprime le contrle associ
							// Pour viter cela, on met temporairement le parent  NULL
							pControl = GetControl(m_nDragButton);
							VERIFY(pControl);
							pControl->SetParent(NULL);
						}
#endif
						
						if (nDropBtn>=0)
							// On doit insrer
							tlbc.InsertButton(nDropBtn, &tbButton);
						else
							// Ajouter le bouton
							tlbc.AddButtons(1, &tbButton);
						
						// Supprimer le bouton de son ancienne position
						// On n'a pas besoin de le demander au parent
						// car on l'a dj fait dans BeginDrag()
						tlbc.DeleteButton(
							(m_nDragButton<nDropBtn || nDropBtn<0)
								? m_nDragButton
								: m_nDragButton+1
						);
						
#ifdef TOOLBAREX_CONTROL
						// Reconnecter le contrle
						if (pControl)  pControl->SetParent(this);
#endif
						
						if (m_nDragButton==tlbc.GetButtonCount()-1)
							// Supprimer les sparateurs en trop
							RemoveTrailingSeparators();
					}
				}
			}else{
				// Dplacer le bouton sur une AUTRE barre d'outils
				
				tlbc.GetButton(m_nDragButton, &tbButton);
				CToolBarCtrl& tlbcDrop = m_pDropBar->GetToolBarCtrl();
				
				if (m_pDropBar->DoQueryInsert(tbButton, (nDropBtn>=0) ? nDropBtn : tlbcDrop.GetButtonCount())) {
					// Rcuprer la bitmap du bouton dplac et la redimensionner
					//  la taille de la barre d'outils de destination
					int nBitmapDest = 0;
#ifdef TOOLBAREX_CONTROL
					if (IsControl(m_nDragButton)) {
						CRect rc;
						GetItemRect(m_nDragButton, rc);
						nBitmapDest = rc.Width();
					}else{
#endif
						const HBITMAP hBmp = GetBitmap(tbButton.iBitmap, m_pDropBar->m_sizeImage);
						if (hBmp) {
							CBitmap bmp;
							bmp.Attach(hBmp);
							nBitmapDest = tlbcDrop.AddBitmap(1, &bmp);
							bmp.Detach();
						}
#ifdef TOOLBAREX_CONTROL
					}
#endif
					tbButton.iBitmap = nBitmapDest;
					tbButton.iString = -1;
					if ((nDropBtn>=0)
							? tlbcDrop.InsertButton(nDropBtn, &tbButton)
							: tlbcDrop.AddButtons(1, &tbButton)) {
						// Transfrer la chane aussi, s'il y en a
						// (vrifier la cible d'abord)
						if (m_pDropBar->HasButtonText() && HasButtonText()) {
							// Laisser CToolBar faire tout le travail
							m_pDropBar->SetButtonText(
								m_pDropBar->CommandToIndex(tbButton.idCommand),
								GetButtonText(m_nDragButton));
						}
						
						// Vrifier si le bouton dplac tait un contrle en vie
						// et si c'est le cas dplacer ce contrle  son nouveau parent
#ifdef TOOLBAREX_CONTROL
						CheckMoveControl(m_pDropBar, tbButton);
#endif
						tlbc.DeleteButton(m_nDragButton);
						
						if (m_nDragButton==tlbc.GetButtonCount()-1)
							// Supprimer les sparateurs en trop
							RemoveTrailingSeparators();
						
						m_pDropBar->RecalcLayout();
					}
				}
			}
		}else{
			// Supprimer le bouton de la barre d'outils
			
			if (fDoMove) {
				tlbc.DeleteButton(m_nDragButton);
				
				if (m_nDragButton == tlbc.GetButtonCount())
					// Supprimer les sparateurs en trop
					RemoveTrailingSeparators();
			}else
				// L'utilisateur a arrt l'opration
				// Supprimer la bordure de dplacement
				InvalidateButton(m_nDragButton);
		}
		
		// Recalculer la taille de la barre et du parent
		RecalcLayout();
	}
	
	m_fDragging = false;
	
	// Librer la souris
	ReleaseCapture();
}

#ifdef TOOLBAREX_CONTROL
// Dplacer un contrle sur une nouvelle barre d'outils
void CToolBarEx::CheckMoveControl(CToolBarEx* ptlb, const TBBUTTON& tbButton)
{
	ASSERT_VALID(ptlb);
	
	CWnd *const pControl = GetControl(tbButton.idCommand, TRUE);
	if (pControl) {
		// Changer le parent du contrle, de manire
		//  ce qu'il passe dans l'autre barre d'outils
		pControl->SetParent(ptlb);
		
		// Supprimer le contrle de notre liste, s'il y tait
		// et l'ajouter dans la liste de la cible
		const POS pos = m_lstControl.Find(pControl);
		if (pos) {
			m_lstControl.RemoveAt(pos);
			ptlb->m_lstControl.AddFin(pControl);
		}
	}
}
#endif

// Supprimer les sparateurs en trop
void CToolBarEx::RemoveTrailingSeparators()
{
	CToolBarCtrl &tlbc = GetToolBarCtrl();
	const int nbBouton = tlbc.GetButtonCount();
	int nBouton = nbBouton;
	while(nBouton && IsSeparator(--nBouton)) {
		if (DoQueryDelete(nBouton))  tlbc.DeleteButton(nBouton);
	}
}

// Rechercher un bouton d'aprs sa position
int CToolBarEx::FindDropButton(CPoint pt)
{
	ScreenToClient(&pt);
	
	CRect rc;
	
	// Trouver le bouton vis
	const int nbBouton = GetToolBarCtrl().GetButtonCount();
	for (int nBouton=0; nBouton<nbBouton; nBouton++) {
		GetItemRect(nBouton, rc);
		if (rc.PtInRect(pt))
			// Insrer le bouton pour lcher avant ce bouton
			return (pt.x-rc.left<rc.right-pt.x)
				? nBouton
				: ((nBouton==nbBouton-1)
					? -1
					: nBouton+1);
	}
	
	// On doit ajouter le bouton
	return -1;
}

// Rcuprer la marque
void CToolBarEx::GetMarkerRect(int nButton, CRect& rc)
{
	const int nbBouton = GetToolBarCtrl().GetButtonCount();
	if (nButton<0 || nButton>nbBouton) {
		// Mettre le marqueur avant le dernier bouton
		GetItemRect(nbBouton-1, rc);
		rc.right += 3;
		rc.left   = rc.right-6;
	}else{
		// Positionner le marqueur au bouton donn
		GetItemRect(nButton, rc);
		rc.left -= 3;
		rc.right = rc.left+6;
	}
	rc.DeflateRect(0,1);
}

// Afficher le marqueur
void CToolBarEx::ShowMarker(const CRect& rcMarker, CBitmap& bmpZone)
{
	ASSERT(bmpZone.GetSafeHandle()==NULL);
	
	CClientDC  dcWin(this);
	CDC        dcMem; dcMem.CreateCompatibleDC(&dcWin);
	
	bmpZone.CreateCompatibleBitmap(&dcWin, rcMarker.Width(), rcMarker.Height());
	dcMem.SelectObject(&bmpZone);
	
	CPen pen(PS_SOLID, 1, RGB(0,0,0));
	CPen *const poldPen = dcWin.SelectObject(&pen);
	
	// Sauver la zone originale
	dcMem.BitBlt(
		0,0,    rcMarker.Width(),rcMarker.Height(),
		&dcWin, rcMarker.left, rcMarker.top, SRCCOPY);
	
	dcWin.MoveTo(rcMarker.left,    rcMarker.top     );
	dcWin.LineTo(rcMarker.right,   rcMarker.top     );
	dcWin.MoveTo(rcMarker.left +1, rcMarker.top   +1);
	dcWin.LineTo(rcMarker.right-1, rcMarker.top   +1);
	
	dcWin.MoveTo(rcMarker.left +2, rcMarker.top   +2);
	dcWin.LineTo(rcMarker.left +2, rcMarker.bottom-2);
	dcWin.MoveTo(rcMarker.left +3, rcMarker.top   +2);
	dcWin.LineTo(rcMarker.left +3, rcMarker.bottom-2);
	
	dcWin.MoveTo(rcMarker.left,    rcMarker.bottom-1);
	dcWin.LineTo(rcMarker.right,   rcMarker.bottom-1);
	dcWin.MoveTo(rcMarker.left +1, rcMarker.bottom-2);
	dcWin.LineTo(rcMarker.right-1, rcMarker.bottom-2);
	
	dcMem.SelectObject(dcMem);
	dcMem.DeleteDC();
	dcWin.SelectObject(poldPen);
}

// Restaurer le marqueur
void CToolBarEx::RestoreMarker(const CRect& rcZone, CBitmap& bmpZone)
{
	if (!bmpZone.GetSafeHandle())  return;
	
	CClientDC dcWin(this);
	CDC       dcMem;
	dcMem.CreateCompatibleDC(&dcWin);
	
	CBitmap *const poldBitmap = dcMem.SelectObject(&bmpZone);
	dcWin.BitBlt(rcZone.left,rcZone.top, rcZone.Width(),rcZone.Height(), &dcMem, 0,0, SRCCOPY);
	
	dcMem.SelectObject(poldBitmap);
	dcMem.DeleteDC();
}

// Positionner le marqueur
void CToolBarEx::SetMarker(CToolBarEx* pBar, CPoint pt)
{
	static CToolBarEx *s_ptlbDernier = NULL;
	static CRect       s_rcDernier(0,0, 0,0);
	static CBitmap     s_bmpZoneDernier;
	
	CRect rcMarker;
	
	// Rcuprer le rectangle propos pour le marqueur
	if (pBar) {
		int nDropBtn = pBar->FindDropButton(pt);
		pBar->GetMarkerRect(nDropBtn, rcMarker);
		
		// On n'a pas besoin de dessiner
		// si les rectangles sont identiques
		if (rcMarker==s_rcDernier)  return;
	}
	
	// Restaurer l'ancienne zone marque
	if (s_ptlbDernier) {
		s_ptlbDernier->RestoreMarker(s_rcDernier, s_bmpZoneDernier);
		s_bmpZoneDernier.DeleteObject();
	}
	
	// Dessiner le marqueur
	if (pBar) {
		pBar->ShowMarker(rcMarker, s_bmpZoneDernier);
		s_rcDernier = rcMarker;
	}
	s_ptlbDernier = pBar;
}

#endif // TOOLBAREX_DRAG


// Suppression de bouton demande
bool CToolBarEx::DoQueryDelete(int nButton)
{
	ASSERT(nButton>=0);
	
	TBBUTTON tbButton;
	if (!GetToolBarCtrl().GetButton(nButton, &tbButton)) {
		TRACE1("ERREUR - CToolBarEx::DoQueryDelete() - Rcupration du bouton %d a chou\n", nButton);
		return false;
	}
	return QueryDeleteInsert(tbButton, true, nButton);
}

// Suppression ou insertion demande
bool CToolBarEx::QueryDeleteInsert(TBBUTTON& tbButton, bool fDelete, int nIndex)
{
	TBNOTIFY tbn;
	tbn.hdr.hwndFrom = m_hWnd;
	tbn.hdr.idFrom   = (UINT)::GetWindowLong(tbn.hdr.hwndFrom, GWL_ID);
	tbn.hdr.code     = fDelete ? TBN_QUERYDELETE : TBN_QUERYINSERT;
	tbn.iItem        = nIndex;
	tbn.tbButton     = tbButton;
	
	CString sTexte;
	if (fDelete) {
		GetButtonText(nIndex, sTexte);
		tbn.cchText = sTexte.GetLength();
		tbn.pszText = sTexte.GetBuffer(tbn.cchText);
	}else{
		tbn.cchText = 0;
		tbn.pszText = NULL;
	}
	
	const bool fOK = ToBool(GetParentFrame()->SendMessage(
		WM_NOTIFY, (WPARAM)tbn.hdr.idFrom, (LPARAM)&tbn));
	
	if (fDelete)  sTexte.ReleaseBuffer();
	
	return fOK;
}


#ifdef TOOLBAREX_DRAG

// Changement de capture
void CToolBarEx::OnCaptureChanged(CWnd* pWnd)
{
	if (m_fDragging)
		// Sans capture souris on ne peut pas
		// terminer l'opration
		EndDrag(false);
	
	CToolBar::OnCaptureChanged(pWnd);
}


// Boucle de messages
BOOL CToolBarEx::PreTranslateMessage(MSG* pMsg)
{
	if (m_fDragging &&
		(pMsg->message==WM_KEYDOWN || pMsg->message==WM_KEYUP) &&
		(int)pMsg->wParam==VK_ESCAPE) {
		
		// L'utilisateur a press la touche CHAPP pour annuler
		// le glisser/dplacer
		EndDrag(false);
		return TRUE;
	}
	return CToolBar::PreTranslateMessage(pMsg);
}


// Notification du parent
void CToolBarEx::OnParentNotify(UINT uMsg, LPARAM lParam)
{
	if (LOWORD(uMsg)==WM_LBUTTONDOWN &&
	   (::GetAsyncKeyState(VK_MENU) & 0x8000)) {
		m_fDragChild = true;
		SetFocus();
	}
	CToolBar::OnParentNotify(uMsg, lParam);
}


// Perte de focus
void CToolBarEx::OnKillFocus(CWnd* pNewWnd)
{
	CToolBar::OnKillFocus(pNewWnd);
	
	if (m_fDragChild) {
		m_fDragChild = false;
		if (GetCapture()!=this)  BeginDrag();
	}
}

#endif // TOOLBAREX_DRAG


#ifdef TOOLBAREX_MENU

// Menu contextuel
void CToolBarEx::OnContextMenu(CWnd*, CPoint)
{
	OnToolBarMenu(GetParentFrame());
}

#endif

// Changement de paramtre systme
void CToolBarEx::OnSettingChange(UINT, LPCTSTR)
{
	CSize sizeButton, sizeImage;
	GetSizes(sizeButton, sizeImage);
	CToolBar::OnSettingChange(0,NULL);
	SetSizes(sizeButton, sizeImage);
}

// Confirmation d'ajout/suppression
void CToolBarEx::OnQueryDeleteInsert(NMHDR*, LRESULT* pResult)
{
	*pResult = m_fEditable;
}

// Demande des boutons supprims de la barre d'outils
void CToolBarEx::OnGetButtonInfo(NMHDR* pNMHDR, LRESULT* pResult)
{
#define pNMTLB  ((TBNOTIFY*)pNMHDR)
	
	if (!(*pResult = (pNMTLB->iItem<m_nbtbbPerso)))
		return;
	
	// Charger la chane d'aprs l'ID de commande du bouton
	const LPTSTR psz = new TCHAR[pNMTLB->cchText];
	pNMTLB->tbButton = m_atbbPerso[pNMTLB->iItem];
	if (!(*pResult = ::LoadString(AfxGetResourceHandle(),
	    pNMTLB->tbButton.idCommand, psz, pNMTLB->cchText))) {
		delete [] psz;
		return;
	}
	
	// La chane est charge:
	// il faut maintenant sauter le '\n'
	const TCHAR *pcSep = _tcschr(psz, '\n');
	lstrcpy(pNMTLB->pszText, (pcSep)
		? ++pcSep : psz);
	delete [] psz;
	pNMTLB->cchText = lstrlen(pNMTLB->pszText);
	
#undef pNMTLB
}

// Rinitialisation
void CToolBarEx::OnReset(NMHDR*, LRESULT*)
{
	int i;
	
	// Dtruire tous les boutons
	for (i=(int)DefWindowProc(TB_BUTTONCOUNT, 0,0); i; i--)
		VERIFY(DefWindowProc(TB_DELETEBUTTON, 0, 0));
	
	const TBBUTTON *ptbb = m_atbb;
	for (i=0; i<m_nbtbb; i++, ptbb++)
		DefWindowProc(TB_ADDBUTTONS, 1, (LPARAM)ptbb);
	
	// Redessiner la barre d'outils
	OnToolBarChange(NULL,NULL);
}

// Barre d'outils modifie
void CToolBarEx::OnToolBarChange(NMHDR*, LRESULT*)
{
	OnUpdateCmdUI(GetParentFrame(), TRUE);
}



//*** CCustomDrawInfo

#if _MFC_VER>=0x420

// Notifier le parent du message dcrit par la structure
LRESULT CCustomDrawInfo::NotifyParent()
{
	return (m_pReceiver)
		? m_pReceiver->SendMessage(
			WM_NOTIFY, (WPARAM)m_nmcd.hdr.idFrom, (LPARAM)&m_nmcd)
		: CDRF_DODEFAULT;
}

// Remplir la structure avec les informations fournies
CCustomDrawInfo::CCustomDrawInfo(CDC* pdc, CToolBarEx* pBar)
{
	VERIFY(m_ptlb = pBar);
	VERIFY(m_nmcd.hdc = pdc->GetSafeHdc());
	
	m_lPrePaint     = 0;
	m_lItemPrePaint = 0;
	
	HWND hwnd = pBar->GetSafeHwnd();
	VERIFY(::IsWindow(hwnd));
	
	// Initialiser les membres de la structure
	m_nmcd.hdr.hwndFrom = hwnd;
	m_nmcd.hdr.idFrom   = (UINT)::GetWindowLong(hwnd, GWL_ID);
	m_nmcd.hdr.code     = NM_CUSTOMDRAW;
	
	// Ne pas utiliser CControlBar::GetDockingFrame() ni
	// CWnd::GetParent() pour rcuprer le parent
	// Ces deux fonctions ne marchent pas si la barre d'outils
	// est flottante
	m_pReceiver = pBar->CToolBar::GetParentFrame();
}

// Notifier le parent que le dessin va commencer
void CCustomDrawInfo::NotifyPrePaint()
{
	// Remplir la structure de dessin personnalis
	// avec les valeurs pour CDDS_PREPAINT
	m_nmcd.dwDrawStage = CDDS_PREPAINT;
	// Le reste de la structure reste indfini  ce stade du dessin
	m_lPrePaint = NotifyParent();
}

// Notifier le parent du dessin d'un item
bool CCustomDrawInfo::NotifyItemPrePaint(int nItem)
{
	if (!(m_lPrePaint & CDRF_NOTIFYITEMDRAW))  return true;
	
	m_nmcd.dwDrawStage = CDDS_ITEMPREPAINT;
	m_ptlb->GetItemRect(nItem, &m_nmcd.rc);
	m_nmcd.dwItemSpec = DWORD(m_ptlb->GetItemID(nItem));
	const UINT uStyle = m_ptlb->GetButtonStyle(nItem);
	m_nmcd.uItemState =
		(m_ptlb->GetToolBarCtrl().IsButtonEnabled(m_nmcd.dwItemSpec) ? 0 : CDIS_DISABLED) |
		(((uStyle & TBBS_PRESSED) || (uStyle & TBBS_CHECKED))
			? CDIS_CHECKED
			: 0);
	m_nmcd.lItemlParam = 0;
	m_lItemPrePaint = NotifyParent();
	return !(m_lItemPrePaint & CDRF_SKIPDEFAULT);
}

// Notifier le parent qu'un item a t dessin
void CCustomDrawInfo::NotifyItemPostPaint(int nItem)
{
	UNUSED(nItem);
	
	if (m_lItemPrePaint & CDRF_NOTIFYPOSTPAINT) {
		m_nmcd.dwDrawStage = CDDS_ITEMPOSTPAINT;
		// Le reste des donnes n'a pas t chang depuis ITEMPREPAINT
		// S'assurer que cela est bien vrai
		ASSERT(m_ptlb->GetItemID(nItem)==m_nmcd.dwItemSpec);
		NotifyParent();
	}
}

// Notifier le parent que le dessin est termin
void CCustomDrawInfo::NotifyPostPaint()
{
	if (m_lPrePaint & CDRF_NOTIFYPOSTPAINT) {
		m_nmcd.dwDrawStage = CDDS_POSTPAINT;
		NotifyParent();
	}
}

#else   // _MFC_VER<4.2

CCustomDrawInfo::CCustomDrawInfo(CDC& dc, CToolBarEx* pParent) {}
void CCustomDrawInfo::NotifyPrePaint() {}
void CCustomDrawInfo::NotifyPostPaint() {}
bool CCustomDrawInfo::NotifyItemPrePaint(int)
{
	// On fait toujours le dessin nous-mme
	return TRUE;
}
void CCustomDrawInfo::NotifyItemPostPaint(int) {}

#endif  // _MFC_VER



#ifdef TOOLBAREX_REG

const LPCTSTR eszBarreOutilsNbBoutons = _T("Nombre");
const LPCTSTR eszBarreOutilsBoutons   = _T("Boutons");

// Enregistrer l'tat de la barre d'outils
void CToolBarEx::ToReg(LPCTSTR pszKey, bool fPos) const
{
	if (fPos) {
		::ToReg(this, pszKey);
		if (!m_fEditable)  return;
	}
	
	// Lister les boutons
	CToolBarCtrl &tlb = GetToolBarCtrl();
	const int nb = tlb.GetButtonCount();
	TBBUTTON *const atbb = new TBBUTTON[nb];
	TBBUTTON *ptbb = atbb;
	for (int i=0; i<nb; i++, ptbb++)
		tlb.GetButton(i, ptbb);
	
	// Enregistrer dans le registre
	pApp->WriteProfileInt   (pszKey, eszBarreOutilsNbBoutons, nb);
	pApp->WriteProfileBinary(pszKey, eszBarreOutilsBoutons,   (BYTE*)atbb, nb*sizeof(TBBUTTON));
	delete [] atbb;
}

// Charger l'tat de la barre d'outils
void CToolBarEx::FromReg(LPCTSTR pszKey, bool fPos)
{
	if (fPos) {
		::FromReg(this, pszKey);
		if (!m_fEditable)  return;
	}
	
	// Rcuprer le nombre de boutons et les boutons eux-mmes
	const int nb = (int)pApp->GetProfileInt(pszKey, eszBarreOutilsNbBoutons, -1);
	if (nb<=0)  return;
	TBBUTTON *atbb;
	UINT      len;
	pApp->GetProfileBinary(pszKey, eszBarreOutilsBoutons, (BYTE**)&atbb, &len);
	if (len!=nb*sizeof(TBBUTTON)) {
		delete [] atbb;
		return;
	}
	
	// Dtruire tous les boutons
	int i;
	for (i=(int)DefWindowProc(TB_BUTTONCOUNT, 0,0); i; i--)
		VERIFY(DefWindowProc(TB_DELETEBUTTON, 0, 0));
	
	// Ajouter les nouveaux boutons
	TBBUTTON *ptbb = atbb;
	for (i=0; i<nb; i++, ptbb++)
		DefWindowProc(TB_ADDBUTTONS, 1, (LPARAM)ptbb);
	
	delete [] atbb;
}

#endif // TOOLBAREX_REG
